package AnaAspDotNet;
# les modules importes
use strict;
use warnings;

use Erreurs;
use StripAspDotNet;
use AnaUtils;
use Vues; # dumpvues_filter_line
use Timeout;
use IsoscopeDataFile;
use CS::ParseCS;
use VbDotNet::ParseVbDotNet;

# prototypes publics
sub Strip ($$$$);
sub Count ($$$$$);
sub Analyse ($$$$);
sub FileTypeRegister ($$);

# prototypes prives

#-------------------------------------------------------------------------------
# DESCRIPTION: module de lancement du strip
#-------------------------------------------------------------------------------
sub Strip ($$$$)
{
    my ($fichier, $vue, $options, $compteurs) = @_;
    my $status = 0;

    eval
    {
        $status = StripAspDotNet::StripAspDotNet($fichier, $vue, $options, $compteurs);
    };

    if ($@) {
        Timeout::DontCatchTimeout(); # propagate timeout errors
        print STDERR "Error in Strip: $@ \n";
        $status = Erreurs::COMPTEUR_STATUS_PB_STRIP;
    }

    if (Erreurs::isAborted($status)) {
        # si le strip genere une erreur, on ne continue pas
        return $status;
    }

    return $status;
}

sub Parse($$$$)
{
    my ($fichier, $vue, $options, $compteurs) = @_;
    my $status = 0;

    my $ASPNET_TECHNO = StripAspDotNet::getTechnoAspNet();

    eval
    {
        if ($ASPNET_TECHNO eq 'C#') {
            $status |= CS::ParseCS::Parse($fichier, $vue, $compteurs, $options);
        }
        elsif ($ASPNET_TECHNO eq 'VB') {
            $status |= VbDotNet::ParseVbDotNet::Parse($fichier, $vue, $compteurs, $options);
        }
    };
    if ($@) {
        Timeout::DontCatchTimeout(); # propagate timeout errors
        print STDERR "Error in Parsing: $@ \n";
        $status = Erreurs::COMPTEUR_STATUS_PB_STRIP;
    }

    return $status;
}


#-------------------------------------------------------------------------------
# DESCRIPTION: module de lancement des comptages
#-------------------------------------------------------------------------------
sub Count ($$$$$)
{
    my ($fichier, $vue, $options, $compteurs, $r_TableFonctions) = @_;
    my $status = 0;

    $status |= AnaUtils::Count($fichier, $vue, $options, $compteurs, $r_TableFonctions);
    return $status;
}

# Ces variables doivent etre globales dans le cas nominal (dit nocrashprevent)
my $r_TableMnemos = [];
my $r_TableFonctions = [];
my $confStatus = 0;

my %MODULE_LOADED = ();

# Module d'enregistrement des compteurs pour la sortie .csv
sub FileTypeRegister($$) {
    my $options = shift;
    my $ConfigModul = shift;
    my $merge = shift;
    
    # load ASP in anycase.
    if (! exists $MODULE_LOADED{'AspDotNet'}) {
		my ($r_TableMnemos_asp, $r_TableFonctions_asp);
        ($r_TableMnemos_asp, $r_TableFonctions_asp, $confStatus) = AnaUtils::Read_ConfAnalyse('AspDotNet_Conf', $options);
        $MODULE_LOADED{'AspDotNet'} = [$r_TableMnemos_asp, $r_TableFonctions_asp];
        
        #------------------ register AspDotNet mnemos -----------------------
        if (defined $options->{'--o'}) {
            IsoscopeDataFile::csv_file_type_register("AspDotNet", $r_TableMnemos_asp);
        }
        
    }

	my $techno = $ConfigModul;
	$techno =~ s/_Conf$//m;

	if ($techno eq "") {
		# Get default AspDotNet model;
		@{$r_TableMnemos} = @{$MODULE_LOADED{'AspDotNet'}->[0]};
		@{$r_TableFonctions} = @{$MODULE_LOADED{'AspDotNet'}->[1]};
	}
	else {
		if (! exists $MODULE_LOADED{$techno}) {
			#------------------ Chargement des comptages a effectuer -----------------------
			#~ if (defined $options->{'--conf'}) {
				#~ $ConfigModul = $options->{'--conf'};
			#~ }

			# Get AspDotNet model;
			@{$r_TableMnemos} = @{$MODULE_LOADED{'AspDotNet'}->[0]};
			@{$r_TableFonctions} = @{$MODULE_LOADED{'AspDotNet'}->[1]};
		
			# import configuration of techno inside AspDotNet
			# (disable Green if any, in order not to retrieve related conf for VB & C#  ...)
			my $GreenIT_opt = $options->{'--GreenIT'};
			$options->{'--GreenIT'} = undef;
			$confStatus |= AnaUtils::mergeConf($r_TableMnemos, $r_TableFonctions, $techno, $options);
			$options->{'--GreenIT'} = $GreenIT_opt;

			$MODULE_LOADED{$techno} = [$r_TableMnemos, $r_TableFonctions];

			# delete Alias_LinesOfCode from CS_conf or VbDotNet_conf
			#delete_element_from_array($r_TableFonctions_tmp, 'CountCommun::CountLinesOfCode');
			#delete_element_from_array($r_TableFonctions_tmp, 'CountCommun::CountLinesOfCode_SansPrepro');

			#~ @{$r_TableMnemos} = (@{$r_TableMnemos}, @{$r_TableMnemos_tmp});
			#~ @{$r_TableFonctions} = (@{$r_TableFonctions}, @{$r_TableFonctions_tmp});
		}
		else {
			@{$r_TableMnemos} = @{$MODULE_LOADED{$techno}->[0]};
			@{$r_TableFonctions} = @{$MODULE_LOADED{$techno}->[1]};
		}
	}
    
    AnaUtils::load_ready();
}

# sub delete_element_from_array($$) {
#     my $r_array = shift;
#     my $element = shift;
#
#     my @array = @{$r_array};
#     my @del_indexes = grep { $array[$_]->[1] eq $element } 0..$#array;
#     if (scalar @del_indexes > 0) {
#         splice(@array, $del_indexes[0], 1);
#     }
#
#    # return \@array;
# }

#-------------------------------------------------------------------------------
# DESCRIPTION: module de lancement de l'analyse
#-------------------------------------------------------------------------------
sub Analyse ($$$$) {
    my ($fichier, $vues, $options, $couples) = @_;
    my $status = 0;

    # init arrays for performances reasons
    $r_TableMnemos = [];
    $r_TableFonctions = [];

	AnaUtils::load_ready();

	# FIXME : Maybe useless
    my $ConfigModul = 'AspDotNet_Conf';
    #~ FileTypeRegister($options, $ConfigModul);
    #~ $status |= $confStatus;

    # launching only StripAspDotNet without compute of counters
    my $analyseur_callbacks = [ \&Strip, undef, undef, undef ];
    $status |= AnaUtils::Analyse($fichier, $vues, $options, $couples, $analyseur_callbacks);

    my $ASPNET_TECHNO = StripAspDotNet::getTechnoAspNet();

    if ($ASPNET_TECHNO eq 'C#') {
        $ConfigModul = 'CS_Conf';
    }
    elsif ($ASPNET_TECHNO eq 'VB') {
        $ConfigModul = 'VbDotNet_Conf';
    }
    else {
        $ConfigModul = '';
    }
	
    FileTypeRegister($options, $ConfigModul);

    if ($ConfigModul ne '') {
        # setting the Parse sub regarding the asp.net techno
        # the Count sub is retrieved from the CS/VBDotNet config file
        $analyseur_callbacks = [ undef, \&Parse, \&Count, $r_TableFonctions ];

        $status |= AnaUtils::Analyse($fichier, $vues, $options, $couples, $analyseur_callbacks);
    }
    else {
        print STDERR "[ASPNET] ERROR: unknown ASP.NET code-behind technology\n";
        #$status |= Couples::counter_add($couples, undef, Erreurs::COMPTEUR_ERREUR_VALUE );
        
        # launch analysis for ASP only ...
        $analyseur_callbacks = [ undef, undef, \&Count, $r_TableFonctions ];
        $status |= AnaUtils::Analyse($fichier, $vues, $options, $couples, $analyseur_callbacks);
    }

    return $status;
}

1;
