package AspDotNet::CountASP;

use strict;
use warnings;

use Erreurs;

use Lib::NodeUtil;
use AspDotNet::AspDotNetScriptNode;

use JSP::CountJSP;

my $MissingJSPComment__mnemo = Ident::Alias_MissingJSPComment();
my $LogicalScriptlet__mnemo = Ident::Alias_LogicalScriptlet();
my $ScriptletLocAverage__mnemo = Ident::Alias_ScriptletLocAverage();
my $ScriptLOC__mnemo = Ident::Alias_ScriptLOC();
my $UnsecureHtmlInjection__mnemo = Ident::Alias_UnsecureHtmlInjection();
my $StdScriptlet__mnemo = Ident::Alias_StdScriptlet();
my $StdExpression_mnemo = Ident::Alias_StdExpression();
my $StdDirective__mnemo = Ident::Alias_StdDirective();
my $JSPComment__mnemo = Ident::Alias_JSPComment();
my $MissingSpaceInTag__mnemo = Ident::Alias_MissingSpaceInTag();
my $MissingShortTag__mnemo = Ident::Alias_MissingShortTag();
my $HtmlComment__mnemo = Ident::Alias_HtmlComment();
my $MultipleComments__mnemo = Ident::Alias_MultipleComments();
my $TabIndentation__mnemo = Ident::Alias_TabIndentation();

my $nb_MissingJSPComment = 0;
my $nb_LogicalScriptlet = 0;
my $nb_ScriptletLocAverage = 0;
my $nb_ScriptLOC = 0;
my $nb_UnsecureHtmlInjection = 0;
my $nb_StdScriptlet = 0;
my $nb_StdExpression = 0;
my $nb_StdDirective = 0;
my $nb_JSPComment = 0;
my $nb_MissingSpaceInTag = 0;
my $nb_MissingShortTag = 0;
my $nb_HtmlComment = 0;
my $nb_MultipleComments = 0;
my $nb_TabIndentation = 0;


sub CountLinesOfCode($$$) {
	my ($fichier, $view, $compteurs) = @_ ;
	my $ret = 0;
	my $nb_LinesOfCode = 0;
	
	# script view contain all ASP code (tag + embedded code). Do not contain HTML tags...
	my $code = $view->{'script'};
	my $strip_info = $view->{'strip_info'};
	
	if (! defined $code) {
		$ret |= Couples::counter_add($compteurs, Ident::Alias_LinesOfCode, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	$nb_LinesOfCode = () = $code =~ /^\s*\S/mg;

	$ret |= Couples::counter_add($compteurs, Ident::Alias_LinesOfCode, $nb_LinesOfCode );
	return $ret;
}

sub CountUnsecureHtmlInjection($$$) {
	my ($fichier, $vue, $compteurs) = @_ ;
    my $ret = 0;

	$nb_UnsecureHtmlInjection = 0;
	
	my $ScriptHtml = $vue->{'script_html'};
	
	if ( ! defined $ScriptHtml ) {
		$ret |= Couples::counter_add($compteurs, $UnsecureHtmlInjection__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	$nb_UnsecureHtmlInjection = () = $ScriptHtml =~ /(>\s*<\%=)/g;

	$ret |= Couples::counter_add($compteurs, $UnsecureHtmlInjection__mnemo, $nb_UnsecureHtmlInjection );
	
	return $ret;
}

sub CountMissingJSPComment($$$) {
	my ($fichier, $vue, $compteurs) = @_ ;
    my $ret = 0;

	$nb_MissingJSPComment = 0;
    
	my $Mix = $vue->{'MixBloc2'};
    
    if ( ! defined $Mix ) {
		$ret |= Couples::counter_add($compteurs, $MissingJSPComment__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	
	# C# comments
	$Mix =~ s/\/\*(?:\*[^\/]|[^*])*\*\//__HIGHLIGHT_COMMENT__/g;
	# Vb.net comments
	$Mix =~ s/'(?:%[^\n>]|[^\n%])*(\n|%>|%\n)/__HIGHLIGHT_COMMENT__ $1/g;

	$nb_MissingJSPComment = () = $Mix =~ /<%[=!]?\s*(?:__HIGHLIGHT_COMMENT__\s*)*%>/g;

	$ret |= Couples::counter_add($compteurs, $MissingJSPComment__mnemo, $nb_MissingJSPComment );
	
	return $ret;
}


sub CountMissingCatch($$$) {
	my ($fichier, $view, $compteurs) = @_ ;
	my $ret = 0;
	my $nb_MissingCatch = 0;
	my $techno;
	
	my $strip_info = $view->{'strip_info'};
	if (defined $strip_info) {
		$techno = $strip_info->{'techno'};
	}

	# FIXME : TBD

	$ret |= Couples::counter_add($compteurs, Ident::Alias_MissingCatch, $nb_MissingCatch );
	return $ret;
}

sub _cb_get_TextBox_ID($$$) {
	my $node = shift;
	my $context = shift;
	my $level = shift;

	my $statement = GetStatement($node);

	if (IsKind($node, "AspDotNet_TAG_LIB")) {
		if ($$statement =~ /<asp:TextBox\b/i) {
			if ($$statement =~ /\bid\b\s*=\s*(AspDotNet_CHAINE_\d+)/i) {
				my $textboxName = $context->{'strings'}->{$1};
				$textboxName =~ s/^["']//m;
				$textboxName =~ s/["']$//m;
				push @{$context->{'TextBox'}}, $textboxName;
			}
		}
	}

	return undef;
}

sub CountUntrustedInputData($$$) {
	my ($fichier, $view, $compteurs) = @_ ;
	my $ret = 0;
	my $nb_UntrustedInputData = 0;
	my $techno;
	
	my $strip_info = $view->{'strip_info'};
	my $script = $view->{'script'};
	my $script_string = $view->{'script_string'}->{'strings_values'};
	my $code = \$view->{'code'};
	my $tag_tree = $view->{'tag_tree'};
	
	if (!defined $script_string) {
		$ret |= Couples::counter_add($compteurs, Ident::Alias_UntrustedInputData, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	if (defined $strip_info) {
		if (defined $strip_info->{'techno'}) {
			$techno = $strip_info->{'techno'};
		}
	}

	# get list of textbox names
	my $context = { 'TextBox' => [], 'strings' => $script_string};
	Lib::Node::Iterate($tag_tree, 0, \&_cb_get_TextBox_ID, $context);
	
	if ($techno eq "C#") {
		for my $textbox (@{$context->{'TextBox'}}) {
			if ($$code =~ /\w+\.Text\s*\+?=\s*$textbox\.Text/) {
				$nb_UntrustedInputData++;
				Erreurs::VIOLATION(Ident::Alias_UntrustedInputData, "Untrusted data for textbox $textbox");
			}
		}
	}
	elsif ($techno eq "VB") {
		for my $textbox (@{$context->{'TextBox'}}) {
			if ($$code =~ /\w+\.Text\s*[\+\&]?=\s*$textbox\.Text/i) {
				$nb_UntrustedInputData++;
				Erreurs::VIOLATION(Ident::Alias_UntrustedInputData, "Untrusted data for textbox $textbox");
			}
		}
	}
	
	$ret |= Couples::counter_add($compteurs, Ident::Alias_UntrustedInputData, $nb_UntrustedInputData );
	return $ret;
}

sub removeTechnoComments($$) {
	my $code = shift;
	my $techno = shift;
	
	if ($techno eq 'C#') {
		$$code =~ s/\/\*([^\*]|\*[^\/])*\*\///g;
		$$code =~ s/\/\/[^\n]*//g;
	}
	elsif ($techno eq 'VB') {
		$$code =~ s/'[^\n]*//g;
	}
	
}

sub CountLinesOfEmbeddedCode($$$) {
	my ($fichier, $view, $compteurs) = @_ ;
	my $ret = 0;
	my $nb_LinesOfEmbeddedCode = 0;
	
	my $code = $view->{'code'};
	my $strip_info = $view->{'strip_info'};
	
	if (! defined $code) {
		$ret |= Couples::counter_add($compteurs, Ident::Alias_LinesOfEmbeddedCode, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	$nb_LinesOfEmbeddedCode = () = $code =~ /^\s*\S/mg;

	if (defined $strip_info) {
		if (defined $strip_info->{'addedLinesOfCode'}) {
			$nb_LinesOfEmbeddedCode -= $strip_info->{'addedLinesOfCode'};
		}
	}

	$ret |= Couples::counter_add($compteurs, Ident::Alias_LinesOfEmbeddedCode, $nb_LinesOfEmbeddedCode );
	return $ret;
}

sub countScriptletLocAverage($$) {
	my $statement = shift;
	my $context = shift;
	
	my $scriptletCode = $$statement;
	$scriptletCode =~ s/\A<%[=:]?//;
	$scriptletCode =~ s/%>$//;
	removeTechnoComments(\$scriptletCode, $context->{'techno'});
	
	$context->{'nb_CodeRenderBlocks'} ++;
	$context->{'nb_ScriptletLocAverage'} += () = $scriptletCode =~ /(?:^|\n)((?:[^\n\S])*\S)/g;
}

sub countScriptLOC($$) {
	my $statement = shift;
	my $context = shift;

	my $scriptCode = $$statement;
	$scriptCode =~ s/\A<script\b[^>]*>//;
	$scriptCode =~ s/<\/script>//;
	removeTechnoComments(\$scriptCode, $context->{'techno'});

	$nb_ScriptLOC += () = $scriptCode =~ /(?:^|\n)((?:[^\n\S])*\S)/g;
}

sub checkShortTag($$) {
	my $node = shift;
	my $stmt = shift;
	my $children = Lib::NodeUtil::GetChildren($node);

	my $should_be_short = 0;

	# if not a short tag ( not in the form <...../> )
	if ($$stmt !~ /\/>\Z/) {
		$should_be_short = 1;
		
		# if a child different from JSP_STRING can be found, then the tag is not considered empty
		# and so can not be written in short format.
		for my $child (@$children) {
			my $childKind = Lib::NodeUtil::GetKind($child);
			if ( $childKind ne AspDotNet_STRING) {
				$should_be_short=0;
			}
		}
	}
	return $should_be_short;
}

sub _cb_TagTree($$$) {
	my $node = shift;
	my $context = shift;
	my $level = shift;
	
	my $statement = GetStatement($node);
	my $kind = GetKind($node);
	
	pos($$statement) = 0;
	my $line = GetLine($node);

	if ($kind eq "STD_AspDotNet_TAG") {
		
		if ($$statement =~ /\A<%\s*([\@=:]|--|[^\@=:\-])/) {
			if ($1 eq '@') {
				$nb_StdDirective++;
			}
			elsif (($1 eq "=") || ($1 eq ":")) {
				$nb_StdExpression++;
			}
			else {
				$nb_StdScriptlet += 1 ;
			}
		}

		my $code = $$statement;
		$code =~ s/\A<%[=:]?//;
		$code =~ s/%>$//;
		removeTechnoComments(\$code, $context->{'techno'});
		
		# do not take into account expressions (<% and <:), binding expression (<%#) or expression builder (<%$)
		if ($$statement !~ /\A<%[=:#\$]/) {
			countScriptletLocAverage(\$code, $context);
		}
		
		if ($context->{'techno'} eq "C#") {
			if ($code =~ /\A\s*(if|while|for|foreach)\b/) {
				my $nb_open = () = $code =~ /(\{)/sg;
				my $nb_close = () = $code =~ /(\})/sg;
				if ($nb_open != $nb_close) {
					$nb_LogicalScriptlet++;
					Erreurs::VIOLATION(Ident::Alias_LogicalScriptlet, "Logical scriptlet at line $line");
				}
			}
		}
		elsif ($context->{'techno'} eq "VB") {
			if ($code =~ /\A\s*(if|while|for)\b/i) {
				my $nb_open = () = $code =~ /\b(if|while|for)\b/sgi;
				my $nb_close = () = $code =~ /\bend\s+(if|while|next)\b/sgi;
				if ($nb_open != $nb_close) {
					$nb_LogicalScriptlet++;
					Erreurs::VIOLATION(Ident::Alias_LogicalScriptlet, "Logical scriptlet at line $line");
				}
			}
		}
	}
	elsif ($kind eq "XML_SCRIPT") {
		countScriptLOC($statement, $context);
	}
	elsif ($kind eq "AspDotNet_COMMENT") {
		$nb_JSPComment++;
	}

	# Do not check space in JSP comments.
	if (($kind ne HTML) && ($kind ne JSP_COMMENT)) {
		# Search patterns:
		#     <%xxx  OR <%[@=!:#$]xxx  OR  xxx%>  OR  "xxx"%>
		# NOTE: for the last pattern, "CHAINE_\d+ %>" is searched because strings are encoded and a space is added after them !!
		if ($$statement =~ /(\A<\%[^\@=!:#\$\s\-]|\A<\%\s*[\@=!:#\$]\S|[^\s\-]\%>|CHAINE_\d+ %>)/) {
			$nb_MissingSpaceInTag++;
#print STDERR "MISSING SPACE in $kind : $$statement -> ($1)\n";
		}
	}

	if ($kind eq AspDotNet_TAG_LIB) {
		# XML tags can have a short format. Check it ...
		if (checkShortTag($node, $statement)) {
			$nb_MissingShortTag++;
		}
	}
	elsif ($kind eq HTML_COMMENT) {
		$nb_HtmlComment++;
	}

	return undef;
}


sub CountTagTree($$$) {
	my ($fichier, $view, $compteurs) = @_ ;
	my $ret = 0;
	$nb_LogicalScriptlet = 0;
	$nb_ScriptletLocAverage = 0;
	$nb_ScriptLOC = 0;
	$nb_StdScriptlet = 0;
	$nb_StdExpression = 0;
	$nb_StdDirective = 0;
	$nb_JSPComment = 0;
	$nb_MissingSpaceInTag = 0;
	$nb_MissingShortTag = 0;
	$nb_HtmlComment = 0;
	
	my $techno;
	
	my $strip_info = $view->{'strip_info'};
	my $tag_tree = $view->{'tag_tree'};
	
	if (!defined $tag_tree) {
		$ret |= Couples::counter_add($compteurs, Ident::Alias_LogicalScriptlet, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_ScriptletLocAverage, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_ScriptLOC, Erreurs::COMPTEUR_ERREUR_VALUE );
		
		$ret |= Couples::counter_add($compteurs, Ident::Alias_StdScriptlet, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_StdExpression, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_StdDirective, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_JSPComment, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_MissingSpaceInTag, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_MissingShortTag, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, Ident::Alias_HtmlComment, Erreurs::COMPTEUR_ERREUR_VALUE );
		
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	if (defined $strip_info) {
		if (defined $strip_info->{'techno'}) {
			$techno = $strip_info->{'techno'};
		}
	}
	
	my $context = { 'techno' => $techno, 'nb_CodeRenderBlocks' => 0, 'nb_ScriptletLocAverage' => 0};
	Lib::Node::Iterate($tag_tree, 0, \&_cb_TagTree, $context);
	
	# Compute ratio of ScriptletLocAverage
	if ($context->{'nb_CodeRenderBlocks'}) {
		$nb_ScriptletLocAverage = int($context->{'nb_ScriptletLocAverage'} / $context->{'nb_CodeRenderBlocks'});
	}
	
	Erreurs::VIOLATION(Ident::Alias_ScriptletLocAverage, "METRIC : scriptlet average LOC is $nb_ScriptletLocAverage");
	Erreurs::VIOLATION(Ident::Alias_ScriptLOC, "METRIC : script LOC is $nb_ScriptLOC");

	$ret |= Couples::counter_add($compteurs, Ident::Alias_LogicalScriptlet, $nb_LogicalScriptlet );
	$ret |= Couples::counter_add($compteurs, Ident::Alias_ScriptletLocAverage, $nb_ScriptletLocAverage );
	$ret |= Couples::counter_add($compteurs, Ident::Alias_ScriptLOC, $nb_ScriptLOC );
	
	$ret |= Couples::counter_add($compteurs, Ident::Alias_StdScriptlet, $nb_StdScriptlet );
	$ret |= Couples::counter_add($compteurs, Ident::Alias_StdExpression, $nb_StdExpression );
	$ret |= Couples::counter_add($compteurs, Ident::Alias_StdDirective, $nb_StdDirective );
	$ret |= Couples::counter_add($compteurs, Ident::Alias_JSPComment, $nb_JSPComment );

	$ret |= Couples::counter_add($compteurs, Ident::Alias_MissingSpaceInTag, $nb_MissingSpaceInTag );
	$ret |= Couples::counter_add($compteurs, Ident::Alias_MissingShortTag, $nb_MissingShortTag );
	$ret |= Couples::counter_add($compteurs, Ident::Alias_HtmlComment, $nb_HtmlComment );
	
	return $ret;	
}

sub CountMultipleComments($$$) {
	my ($fichier, $vue, $compteurs) = @_ ;
    my $ret = 0;

	$nb_MultipleComments = 0;

	my $techno;

	my $strip_info = $vue->{'strip_info'};
	my $script = $vue->{'script_html'};

	if ( ! defined $script ) {
		$ret |= Couples::counter_add($compteurs, $MultipleComments__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	if (defined $strip_info) {
		if (defined $strip_info->{'techno'}) {
			$techno = $strip_info->{'techno'};
		}
	}

	$nb_MultipleComments = () = $script =~ /--%>\s*<%--/sg;

	if ($techno eq "C#") {
		# Normalization of // comment to /**/
		# 1 - replace // ....... %>    with  /**/%> 
		$script =~ s/\/\/[^\n%]*%>/\/\*\*\/%>/sg;
		# 2 - replace // ........\n    with  /**/\n
		$script =~ s/\/\/[^\n]*\n/\/\*\*\/\n/sg;
	}
	
	$nb_MultipleComments += () = $script =~ /\*\/\s*\/\*/sg;

	$ret |= Couples::counter_add($compteurs, $MultipleComments__mnemo, $nb_MultipleComments );
	
	return $ret;
}

sub CountUselessControlReference($$$) {
	my ($fichier, $vue, $compteurs) = @_ ;
    my $ret = 0;

	my $nb_UselessTaglib = 0;

	my $script = $vue->{'script'};
	my $strings = $vue->{'script_string'}->{'strings_values'};

	if ( ! defined $script ) {
		$ret |= Couples::counter_add($compteurs, $Ident::Alias_UselessTaglib, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	while ($script =~ /<\%\s*\@\s*Register([^\%]*)\%>/ig) {
		my $contentRef = $1;
		my ($TagPrefix) = $contentRef =~ /\bTagPrefix\b\s*=\s*([^\s\%]+)/;
		my ($TagName) = $contentRef =~ /\bTagName\b\s*=\s*([^\s\%]+)/;
		my ($Assembly) = $contentRef =~ /\bAssembly\b\s*=\s*([^\s\%]+)/;
		
		$TagPrefix = $strings->{$TagPrefix} if (defined $TagPrefix);
		$TagName = $strings->{$TagName} if (defined $TagName);
		$Assembly = $strings->{$Assembly} if (defined $Assembly);
		
		if (defined $TagPrefix) {
			$TagPrefix =~ s/["']//g;
			if (defined $TagName) {
				$TagName =~ s/["']//g;
				if ($script !~ /<$TagPrefix:$TagName\b/) {
#print STDERR "USELESS TAG REFERENCE CONTROL : $TagPrefix:$TagName\n";
					$nb_UselessTaglib++;
				}
			}
			elsif (defined $Assembly) {
				if ($script !~ /<$TagPrefix:/) {
#print STDERR "USELESS TAG REFERENCE ASSEMBLY : $TagPrefix\n";
					$nb_UselessTaglib++;
				}
			}
		} 
	}

	$ret |= Couples::counter_add($compteurs, Ident::Alias_UselessTaglib, $nb_UselessTaglib );
	
	return $ret;
}

sub CountIndentation($$$) {
	my ($fichier, $vue, $compteurs) = @_ ;
    my $ret = 0;
	
	$nb_TabIndentation = 0;
	my $line = 1;
	while ($vue->{'script'} =~ /^( *(\t)?[\t ]*(<(?:script|\%[\@=:\-]?)|\S))/mg) {
		if (defined $2) {
			if (($3 ne "<%=") && ($3 ne "<%:")) {
				$nb_TabIndentation++;
			}
		}
		$line++;
	}

	$ret |= Couples::counter_add($compteurs, $TabIndentation__mnemo, $nb_TabIndentation );
	return $ret;
}

sub countBreakVB($) {
	my $view = shift;
	my $nb_Break = 0;
	
	if (defined $view->{'code_lc'}) {
		$nb_Break = () = $view->{'code_lc'} =~ /(exit\s+(?:do|for|while))/g;
	}
	
	return $nb_Break;
}

sub CountDefault(@) {
	my ($fichier, $view, $compteurs, $mnemo) = @_ ;
	my $ret = 0;
	my $nb = 0;
	
	if ($mnemo eq Ident::Alias_Break) {
		$ret = countBreakVB($view);
	}
	
#print STDERR "CountDefault : mnemo = $mnemo\n";
	$ret |= Couples::counter_add($compteurs, $mnemo, $nb );
	return $ret;
}

sub NeedOverride_CountCommentBlocs($$$) 		{ return CountDefault(@_, Ident::Alias_CommentBlocs()); }
sub NeedOverride_CountSuspiciousComments($$$) 	{ return CountDefault(@_, Ident::Alias_SuspiciousComments()); }
sub NeedOverride_CountBadMethodNames($$$)		{ return CountDefault(@_, Ident::Alias_BadMethodNames()); }
sub NeedOverride_CountCommentedOutCode($$$) 	{ return CountDefault(@_, Ident::Alias_CommentedOutCode()); }
sub NeedOverride_CountComplexConditions($$$) 	{ return CountDefault(@_, Ident::Alias_ComplexConditions()); }
sub NeedOverride_CountUntrustedInputData($$$) 	{ return CountDefault(@_, Ident::Alias_UntrustedInputData()); }
sub NeedOverride_CountMissingCatch($$$) 		{ return CountDefault(@_, Ident::Alias_MissingCatch()); }
sub NeedOverride_CountAssignmentsInConditionalExpr($$$) { return CountDefault(@_, Ident::Alias_AssignmentsInConditionalExpr()); }
sub NeedOverride_CountMissingBreakInSwitch($$$)	{ return CountDefault(@_, Ident::Alias_MissingBreakInSwitch()); }
sub NeedOverride_CountBugPatterns($$$) 			{ return CountDefault(@_, Ident::Alias_BugPatterns()); }
sub NeedOverride_CountRiskyFunctionCalls($$$) 	{ return CountDefault(@_, Ident::Alias_RiskyFunctionCalls()); }
sub NeedOverride_CountIllegalThrows($$$) 		{ return CountDefault(@_, Ident::Alias_IllegalThrows()); }
sub NeedOverride_CountMissingBraces($$$) 		{ return CountDefault(@_, Ident::Alias_MissingBraces()); }
sub NeedOverride_CountMissingDefaults($$$) 		{ return CountDefault(@_, Ident::Alias_MissingDefaults()); }
sub NeedOverride_CountEmptyCatches($$$) 		{ return CountDefault(@_, Ident::Alias_EmptyCatches()); }
sub NeedOverride_CountMagicNumbers($$$) 		{ return CountDefault(@_, Ident::Alias_MagicNumbers()); }
sub NeedOverride_CountContinue($$$) 			{ return CountDefault(@_, Ident::Alias_Continue()); }
sub NeedOverride_CountGoto($$$) 				{ return CountDefault(@_, Ident::Alias_Goto()); }
sub NeedOverride_CountGosub($$$) 				{ return CountDefault(@_, Ident::Alias_Gosub()); }
sub NeedOverride_CountWhile($$$) 				{ return CountDefault(@_, Ident::Alias_While()); }
sub NeedOverride_CountFor($$$) 					{ return CountDefault(@_, Ident::Alias_For()); }
sub NeedOverride_CountForEach($$$) 				{ return CountDefault(@_, Ident::Alias_ForEach()); }
sub NeedOverride_CountIf($$$) 					{ return CountDefault(@_, Ident::Alias_If()); }
sub NeedOverride_CountElse($$$)					{ return CountDefault(@_, Ident::Alias_Else()); }
sub NeedOverride_CountCase($$$)					{ return CountDefault(@_, Ident::Alias_Case()); }
sub NeedOverride_CountDefault($$$)				{ return CountDefault(@_, Ident::Alias_Default()); }
sub NeedOverride_CountCatch($$$)				{ return CountDefault(@_, Ident::Alias_Catch()); }
sub NeedOverride_CountLoop($$$)					{ return CountDefault(@_, Ident::Alias_Loop()); }
sub NeedOverride_CountBreak($$$) 				{ return CountDefault(@_, Ident::Alias_Break()); }
sub NeedOverride_CountShortMethodNamesLT($$$) 	{ return CountDefault(@_, Ident::Alias_ShortMethodNamesLT()); }
sub NeedOverride_CountFunctionMethodImplementations($$$) 	{ return CountDefault(@_, Ident::Alias_FunctionMethodImplementations()); }
sub NeedOverride_CountRiskyCatches($$$)		 	{ return CountDefault(@_, Ident::Alias_RiskyCatches()); }



#~ sub CountMissingCatch($$$) {
	#~ my ($fichier, $view, $compteurs) = @_ ;
	#~ my $ret = 0;
	
	#~ $nb_MissingCatch = 0;

	#~ if ( ! defined $view->{'code'} ) {
		#~ $ret |= Couples::counter_add($compteurs, $UntrustedInputData__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		#~ return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	#~ }

	#~ my $code = \$view->{'code'};
	
	#~ my %TabAccess = ();
	#~ while ($$code =~ /(\w+)\s*\[\s*([a-zA-Z_]\w*)/sg) {
		#~ $TabAccess{$1."[".$2."]"} = 1;
#~ #print "Found tab acces : $1\[$2\]\n";
	#~ }
	#~ my $nb_TabAccess = scalar keys %TabAccess;
	#~ my $nb_ParseInt = () = $$code =~ /\.parse(?:Int|Float|Double)\s*\(/sg;
	#~ my $nb_While = () = $$code =~ /\bwhile\b/sg;
	#~ my $nb_Catch = () = $$code =~ /\bcatch\b/sg;

#~ #print "TabAcess : $nb_TabAccess\n";
#~ #print "ParseInt : $nb_ParseInt\n";
#~ #print "While : $nb_While\n";
#~ #print "Catch : $nb_Catch\n";

	#~ if (! $nb_Catch){
		#~ $nb_MissingCatch+= 2*$nb_ParseInt + 2*$nb_TabAccess + $nb_While + $nb_While * $nb_TabAccess; 
	#~ }
#~ #print "MISSING CATCH = $nb_MissingCatch\n";
	#~ $ret |= Couples::counter_add($compteurs, $MissingCatch__mnemo, $nb_MissingCatch );
	#~ return $ret;
#~ }

1;


