package CS::CountLoop;

use strict;
use warnings;

use Erreurs;
use Lib::NodeUtil;
use CS::CSNode;
use CS::CSConfig;
use GreenIT::lib::KeepPortalData;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $Break_mnemo = Ident::Alias_Break();
my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $InstantiationInLoop__mnemo = Ident::Alias_InstantiationInLoop();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $LoopCondComparedToNonZero__mnemo = Ident::Alias_LoopCondComparedToNonZero();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();

my $nb_Break = 0;
my $nb_ConcatInLoop = 0;
my $nb_InstantiationInLoop = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_LoopCondComparedToNonZero = 0;
my $nb_ToManyNestedLoop = 0;

sub CountLoop($$$$)
{
    my ($fichier, $views, $compteurs, $options) = @_;
    
	my $status = 0;
	
	$nb_Break = 0;
	$nb_FunctionCallInLoopTest = 0;
	$nb_LoopCondComparedToNonZero = 0;
	$nb_ToManyNestedLoop = 0;
	
	my $KindsLists = $views->{'KindsLists'};

	if ( ! defined $KindsLists ) {
		$status |= Couples::counter_add($compteurs, $Break_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $LoopCondComparedToNonZero__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );

		return $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my $whiles = $KindsLists->{&WhileKind};
	my $Fors = $KindsLists->{&ForKind};
	my $Foreachs = $KindsLists->{&ForeachKind};
	my $dos = $KindsLists->{&DoKind};
	
	my @loops = ( @$whiles, @$Fors, @$Foreachs, @$dos);
	
	for my $loop (@loops) {
		
		my $condNode = GetChildren($loop)->[0];		
		my $line = GetLine($loop);
		my $kind = GetKind($loop);
		
		my @breaks = GetNodesByKindList_StopAtBlockingNode($loop, [BreakKind], [SwitchKind, WhileKind, ForKind, ForeachKind]);
		
		$nb_Break += scalar @breaks;
		
		if ($kind ne ForeachKind) {
			# detect a function call
			my $condition = GetStatement($condNode);
			while ( $$condition =~ /(\w+)\s*\(/g ) {
				$nb_FunctionCallInLoopTest++;
				my $numline = $condNode->[5];
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
				Erreurs::VIOLATION($FunctionCallInLoopTest__mnemo, "Function call in loop condition at line ".GetLine($condNode));
			}
			
			#~ # check all expressions using operators <, >, <=, >=, ==, !=
			my $bool_comparisonToZero;
			if ($$condition !~ /\&\&|\|\|/) {
				if ( $$condition =~ /([^<>!=\s]+)\s*(?:[<>!=]=|<|>)\s*([^<>!=\s]+)/ ) {
					$bool_comparisonToZero = 0;
					if (($1 =~ /(?:^|[^\.\w])0$/m) || ($2 =~ /^0(?:$|[^\.\w])/m)) {
						$bool_comparisonToZero = 1;
					}
				}
				if ((defined $bool_comparisonToZero) && ( $bool_comparisonToZero == 0)) {
					$nb_LoopCondComparedToNonZero++;
					my $numline = $condNode->[5];
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_LoopCondComparedToNonZero, $rulesDescription, "LoopCondComparedToNonZero");
					Erreurs::VIOLATION($LoopCondComparedToNonZero__mnemo, "Loop condition is not comparison to zero at line ".GetLine($condNode));
				}
			}
		}
		
		# find nested loops ...
		my @subloop = GetNodesByKindList($loop, [WhileKind, ForKind, ForeachKind, DoKind], 1);
		for my $sloop (@subloop) {
			$nb_ToManyNestedLoop++;
			my $numline = $sloop->[5];
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
			Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "sub loop at line ".GetLine($sloop));
		}
	}

	$status |= Couples::counter_add($compteurs, $Break_mnemo, $nb_Break);
	$status |= Couples::dispatch($FunctionCallInLoopTest__mnemo, $nb_FunctionCallInLoopTest);
	$status |= Couples::dispatch($LoopCondComparedToNonZero__mnemo, $nb_LoopCondComparedToNonZero);
	$status |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);
	
	return $status;
} 

sub CountPatternInLoop($$$) {
	my ($file, $vue, $compteurs) = @_;

	my $ret = 0;

	$nb_ConcatInLoop = 0;
	$nb_InstantiationInLoop = 0;
	$nb_OpenCloseFileInLoop = 0;

	my $root = $vue->{'structured_code'};
	my $code = \$vue->{'code'};

	if (!defined $root) {
		$ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my @loops = GetNodesByKindList($root, [ WhileKind, ForKind, DoKind, ForeachKind ], 1);

	for my $loop (@loops) {
		my $codeBody = Lib::NodeUtil::GetKindData($loop)->{'codeBody'};
		my $bodyBeginLine = Lib::NodeUtil::GetKindData($loop)->{'bodyBeginLine'};

		# HL-2022 Avoid OPEN/CLOSE inside loops
		my $regexOpen = qr/(?:\bFile\.(?:Open|Create)\b|\bnew\s+(?:FileStream|StreamReader|StreamWriter)\b)/i;
		my $regexClose = qr/\.Close\b/i;
		# check presence of literal strings concaténations (variables strings cannot be detected !)
		my $lineOffset = 0;
		while ($$codeBody =~ /(\n+)|(\bCHAINE_\d+)|(\+=)|(\+)|(\bnew\b)|($regexOpen)|($regexClose)/gc) {
			if (defined $1) {
				$lineOffset += () = $1 =~ /\n/g;
			}
			elsif (defined $5) {
				$nb_InstantiationInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
				Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation in loop at line " . ($bodyBeginLine + $lineOffset));
			}
			elsif (defined $2) {
				# CHAINE_XX
				if ($$codeBody =~ /\G(\s*)\+/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
				}
			}
			elsif (defined $6 || defined $7) {
				$lineOffset += () = $6 =~ /\n/g if (defined $6);
				$lineOffset += () = $7 =~ /\n/g if (defined $7);
				$nb_OpenCloseFileInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				# print "Open/close file inside loop at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
				Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
			}
			else {
				# + first or +=
				if ($$codeBody =~ /\G(\s*)\bCHAINE_\d+(\s*)(\+)?/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));

					$lineOffset += () = $2 =~ /\n/g;
					if (defined $3) {
						$nb_ConcatInLoop++;
						my $numline = $bodyBeginLine + $lineOffset;
						CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
						Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
					}
				}
			}
		}
	}

	$ret |= Couples::dispatch($ConcatInLoop__mnemo, $nb_ConcatInLoop);
	$ret |= Couples::dispatch($InstantiationInLoop__mnemo, $nb_InstantiationInLoop);
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);

	return $ret;
}

1;
