package Clojure::CountLoop;
# les modules importes
use strict;
use warnings;
use Carp::Assert; # traces_filter_line
use Erreurs;
use Couples;
use Lib::NodeUtil;

use Clojure::ClojureNode;
use Clojure::Config;
use GreenIT::lib::KeepPortalData;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $While__mnemo = Ident::Alias_While();
my $Loop__mnemo = Ident::Alias_Loop();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $InstantiationInLoop__mnemo = Ident::Alias_InstantiationInLoop();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();

my $nb_While = 0;
my $nb_Loop = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_ConcatInLoop = 0;
my $nb_InstantiationInLoop = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_ToManyNestedLoop = 0;

sub CountLoop($$$) {
	my ($file, $vue, $compteurs) = @_ ;
	
	my $ret = 0;
    $nb_Loop = 0;
    $nb_While = 0;

	my $KindsLists = $vue->{'KindsLists'};
  
	if ( ! defined $KindsLists )
	{
		$ret |= Couples::counter_add($compteurs, $Loop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $While__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my $loops = $KindsLists->{&LoopKind};
	my $whiles = $KindsLists->{&WhileKind};
	
	for my $loop (@$loops) {
		$nb_Loop++;
	}
	
	for my $while (@$whiles) {
		$nb_While++;
	}
	
	Erreurs::VIOLATION($Loop__mnemo, "[METRIC] number of 'loop' is $nb_Loop");
	Erreurs::VIOLATION($While__mnemo, "[METRIC] number of 'while' is $nb_While");
	
	$ret |= Couples::counter_update($compteurs, $Loop__mnemo, $nb_Loop );
	$ret |= Couples::counter_update($compteurs, $While__mnemo, $nb_While );
	
    return $ret;
}

sub checkLoopBody($) {
	my $loop = shift;

	my $codeBody = Clojure::ClojureNode::getClojureKindData($loop, 'codeBody');
	my $bodyBeginLine = Clojure::ClojureNode::getClojureKindData($loop, 'lineBodyBegin');
	# HL-2022 Avoid OPEN/CLOSE inside loops
	# codeBody is in lowercase
	my $regexOpen = qr/\b(?:with\-open|slurp|spit)\b|\*in\*|\*out\*/i;
	my $regexClose = qr/\.close\b/i;
	my $lineOffset = 0;
	while ($$codeBody =~ /(\n+)|(\(\s*str\b)|(\(\s*(?:new\b|\w+\.[^\w]))|($regexOpen)|($regexClose)/g) {
		if (defined $1) {
			$lineOffset += () = $1 =~ /\n/g;
		}
		elsif (defined $2) {
			$lineOffset += () = $2 =~ /\n/g;
			$nb_ConcatInLoop++;
			my $numline = $bodyBeginLine + $lineOffset;
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
			Erreurs::VIOLATION($ConcatInLoop__mnemo, "Concatenation inside loop at line " . $numline);
		}
		elsif (defined $3) {
			$lineOffset += () = $3 =~ /\n/g;
			my $numline = $bodyBeginLine + $lineOffset;
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
			$nb_InstantiationInLoop++;
			Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation inside loop at line " . $numline);
		}
		elsif (defined $4 || defined $5) {
			$lineOffset += () = $4 =~ /\n/g if (defined $4);
			$lineOffset += () = $5 =~ /\n/g if (defined $5);
			$nb_OpenCloseFileInLoop++;
			my $numline = $bodyBeginLine + $lineOffset;
			# print "Open/close file inside loop at line $numline\n";
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
			Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
		}
	}
}

sub checkNestedLoop($$$);
sub checkNestedLoop($$$) {
	my $loop = shift;
	my $level = shift;
	my $recursive_fct = shift;

	# find nested loops ...
	my @subloop = GetNodesByKindList($loop, [WhileKind, ForKind, LoopKind, DoseqKind, DotimesKind, FunctionKind, AnonymousKind, FunctionArityKind], 1);
	
	my $subloop = 0;
	for my $sloop (@subloop) {
		my $kind = GetKind($sloop);
#print STDERR "KIND $kind at line ".GetLine($sloop).", LEVEL = $level\n";
		$subloop = 1;
		if (($kind eq FunctionKind) || ($kind eq AnonymousKind) || ($kind eq FunctionArityKind)) {
			if (! exists $recursive_fct->{$sloop}) {
#print STDERR "        ---> simple function\n";
				$subloop = 0;
			}
		}

		if ($subloop) {
			if ($level > 0) {
				$nb_ToManyNestedLoop++;
				my $numline = $sloop->[5];
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
				Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "sub loop at line ".GetLine($sloop));
			}
		}
		
		checkNestedLoop($sloop, $level+$subloop, $recursive_fct);
	}
}

sub CountPatternInLoop($$$) {
	my ($file, $vue, $compteurs) = @_ ;
	
	my $ret = 0;
    $nb_ConcatInLoop = 0;
    $nb_InstantiationInLoop = 0;
	$nb_OpenCloseFileInLoop = 0;
	$nb_ToManyNestedLoop = 0;

	my $KindsLists = $vue->{'KindsLists'};
	my $root = $vue->{'structured_code'};
  
	if ( ! defined $KindsLists )
	{
		$ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my $loops = $KindsLists->{&LoopKind};
	my $whiles = $KindsLists->{&WhileKind};
	my $fors = $KindsLists->{&ForKind};
	my $doseqs = $KindsLists->{&DoseqKind};
	my $dotimes = $KindsLists->{&DotimesKind};
	
	# check loops ...
	for my $loop (@$loops, @$whiles, @$fors, @$doseqs, @$dotimes) {
		checkLoopBody($loop);
	}
	
	# check recursives functions acting as loops ... 
	my $fcts = $KindsLists->{&FunctionKind};
	my $anos = $KindsLists->{&AnonymousKind};
	my $farity = $KindsLists->{&FunctionArityKind};

	my %recursive_fct=();

	for my $fct (@$fcts, @$anos, @$farity) {
		my @recurs = GetNodesByKind($fct, RecurKind);
		
		for my $recur (@recurs) {

			my $parent = GetParent($recur);
			my $kind = GetKind($parent);
			#my $if;
			while (($kind ne RootKind) && ($kind ne LoopKind) && ($kind ne FunctionKind) && ($kind ne AnonymousKind) && ($kind ne FunctionArityKind)){
				#if (($kind eq IfKind) && (! defined $if)) {
				#	$if = GetChildren($parent)->[0];
				#}
				$parent = GetParent($parent);
				$kind = GetKind($parent);
			}
			
			if (($kind eq FunctionKind) || ($kind eq AnonymousArityKind) || ($kind eq FunctionArityKind)) {
				checkLoopBody($parent);
				$recursive_fct{$fct}=1;
				#if (defined $if) {
				#	my $condition = GetChildren($if)->[0];
#print STDERR "CONDITION : ".${GetStatement($condition)}."\n";
				#}
			}
		}
	}
	
	checkNestedLoop($root, 0, \%recursive_fct);
	
	$ret |= Couples::dispatch($ConcatInLoop__mnemo, $nb_ConcatInLoop );
	$ret |= Couples::dispatch($InstantiationInLoop__mnemo, $nb_InstantiationInLoop );
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);
	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);

	return $ret;
}

1;
