package CloudReady::CountCobol;

use strict;
use warnings;

use Erreurs;
use CloudReady::detection;
use CloudReady::config;
use CloudReady::Ident;
use CloudReady::lib::HardCodedIP;
use CloudReady::lib::HardCodedPath;
use CloudReady::lib::HardCodedURL;
use CloudReady::lib::URINotSecured;
use CloudReady::lib::SensitiveDataString;
use CloudReady::lib::KeepPortalData;

use constant BEGINNING_MATCH => 0;
use constant FULL_MATCH => 1;

my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

sub checkPatternSensitive($$$) {
    my $reg = shift;
    my $code = shift;
    my $mnemo = shift;

    return CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $reg, $mnemo, $rulesDescription);
    #return () = ($$code =~ /$reg/gm);
}

sub checkPatternInsensitiveInString($$$) {
    my $text = shift;
    my $PatternInString = shift;
    my $mnemo = shift;
    my $reg = qr/["']$PatternInString["']/i;

    return CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $reg, $mnemo, $rulesDescription);
    #return $nb_detection;
}


sub CountCobol($$$$) {
    my ($fichier, $vue, $compteurs, $techno) = @_;
    my $ret = 0;

    my $text;
    my $HString;

    my $checkPattern = \&checkPatternSensitive;

    $text = \$vue->{'text'};
    $HString = $vue->{'HString'};

    my $code = \$vue->{'code'};

    if ((!defined $text) || (!defined $code)) {
        return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
    }

    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedIP(), CloudReady::lib::HardCodedIP::checkHardCodedIP($HString, $code, $techno, $text));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HexaConstants(), $checkPattern->(qr/\b(?:VALUE|MOVE)\s+XCHAINE\_/i, $code, CloudReady::Ident::Alias_HexaConstants()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DISPLAY_statement(), $checkPattern->(qr/\bDISPLAY\b/i, $code, CloudReady::Ident::Alias_DISPLAY_statement()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_CodeSkipped(), $checkPattern->(qr/\bSOURCE\-COMPUTER\s+IBM\-370\s+WITH\s+DEBUGGING\s+MODE\b/i, $code, CloudReady::Ident::Alias_CodeSkipped()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_ALTER_statement(), $checkPattern->(qr/\bALTER\b/i, $code, CloudReady::Ident::Alias_ALTER_statement()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_PackedDecimal(), $checkPattern->(qr/\b(?:PACKED\-DECIMAL|[^\w\-]COMP\-3)\b/i, $code, CloudReady::Ident::Alias_PackedDecimal()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Binary(), $checkPattern->(qr/(?:\bBINARY\b|[^\w\-]COMP(?:[^\w\-]|\z)|[^\w\-]COMP\-4(?:[^\w\-]|\z)|[^\w\-]COMP\-5(?:[^\w\-]|\z))/i, $code, CloudReady::Ident::Alias_Binary()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_CAPanvaletCommand(), $checkPattern->(qr/[^\w\-]\+\+(?:WRITE|INCLUDE)\b/i, $code, CloudReady::Ident::Alias_CAPanvaletCommand()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DLICalls(), checkPatternInsensitiveInString($text, '\b(?:CBLTDLI|AIBTDLI|CEETDLI|PLITDLI)\b', CloudReady::Ident::Alias_DLICalls()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DoubleByteCharacter(), $checkPattern->(qr/\bMOVE\s+[NX]\s*CHAINE\_|\bPIC\s+[NG]\b|\bNATIONAL\-OF\b|\bDISPLAY\-OF\b|\bUSAGE\s+NATIONAL\b|\bUSAGE\s+DISPLAY\-1\b/i, $code, CloudReady::Ident::Alias_DoubleByteCharacter()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GOTO_statement(), $checkPattern->(qr/\bGO\s+TO\b/i, $code, CloudReady::Ident::Alias_GOTO_statement()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DecimalPointComma(), $checkPattern->(qr/\bDECIMAL\-POINT\s+IS\s+COMMA\b/i, $code, CloudReady::Ident::Alias_DecimalPointComma()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_CurrencySign(), $checkPattern->(qr/\bCURRENCY\s+SIGN\s+IS\b/i, $code, CloudReady::Ident::Alias_CurrencySign()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_OccurClause(), $checkPattern->(qr/\bOCCURS?\b/i, $code, CloudReady::Ident::Alias_OccurClause()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_RedefinesClause(), $checkPattern->(qr/\bREDEFINES\b/i, $code, CloudReady::Ident::Alias_RedefinesClause()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DummyTable(), $checkPattern->(qr/\bSYSIBM\.SYSDUMMY1\b/i, $code, CloudReady::Ident::Alias_DummyTable()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_JSONParsing(), $checkPattern->(qr/\bJSON\s+PARSE\b/i, $code, CloudReady::Ident::Alias_JSONParsing()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_XMLParsing(), $checkPattern->(qr/\bXML\s+PARSE\b/i, $code, CloudReady::Ident::Alias_XMLParsing()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_JSONGenerate(), $checkPattern->(qr/\bJSON\s+GENERATE\b/i, $code, CloudReady::Ident::Alias_JSONGenerate()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UROption(), $checkPattern->(qr/\bWITH\s+UR\b/i, $code, CloudReady::Ident::Alias_UROption()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SubsetRow(), $checkPattern->(qr/\bFETCH\s+FIRST\s+[0-9]+\s+ROWS\s+ONLY\b/i, $code, CloudReady::Ident::Alias_SubsetRow()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_CICSWebservice(), $checkPattern->(qr/\bEXEC\s+CICS\s+INVOKE\s+WEBSERVICE\b/i, $code, CloudReady::Ident::Alias_CICSWebservice()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MQCallBatch(), checkPatternInsensitiveInString($text, '\b(?:CSQBBACK|CSQBBFMH|CSQBCB|CSQBCLOS|CSQBCOMM|CSQBCONN|CSQBCONX|CSQBCTMH|CSQBCTL|CSQBDISC|CSQBDTMH|CSQBDTMP|CSQBGET|CSQBINQ|CSQBIQMP|CSQBMHBF|CSQBOPEN|CSQBPUT|CSQBPUT1|CSQBSET|CSQBSTMP|CSQBSTAT|CSQBSUB|CSQBSUBR)\b', CloudReady::Ident::Alias_MQCallBatch()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MQCall(), checkPatternInsensitiveInString($text, '\b(?:MQBACK|MQBUFMH|MQCB|MQCLOSE|MQCMIT|MQCONN|MQCONNX|MQCRTMH|MQCTL|MQDISC|MQDLTMH|MQDLTMP|MQGET|MQINQ|MQINQMP|MQMHBUF|MQOPEN|MQPUT|MQPUT1|MQSET|MQSETMP|MQSTAT|MQSUB|MQSUBRQ)\b', CloudReady::Ident::Alias_MQCall()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DB2Connect(), $checkPattern->(qr/\bEXEC\s+SQL\b/i, $code, CloudReady::Ident::Alias_DB2Connect()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_CASE_statement(), $checkPattern->(qr/\bEXEC\s+SQL\b.*\bCASE\b.*\bEND-EXEC\b/si, $code, CloudReady::Ident::Alias_CASE_statement()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SensitiveDataString(), CloudReady::lib::SensitiveDataString::CountSensitiveDataString($code, $HString, $techno));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedPath(), CloudReady::lib::HardCodedPath::checkHardCodedPath($HString, $techno, $text, $rulesDescription));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HTTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURL($fichier, $code, $HString, $techno, $text));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'FTP', $text));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LDAPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'LDAP', $text));
    CloudReady::lib::URINotSecured::checkURINotSecured($fichier, $code, $HString, $techno, $text);

    my $parameters = AnalyseUtil::recuperer_options();
    my $dbgMatchPatternDetail = $parameters->[0]->{'--dbgMatchPatternDetail'};
    if (defined $dbgMatchPatternDetail) {
        # replacing SHA by filenames in log
        CommonOutput::ElaborateLog::ElaborateLogPartTwo($fichier);
    }

    return $ret;
}

1;
