package CloudReady::CountGo;

use strict;
use warnings;

use Erreurs;
use CloudReady::detection;
use CloudReady::config;
use CloudReady::Ident;
use CloudReady::lib::HardCodedIP;
use CloudReady::lib::HardCodedPath;
use CloudReady::lib::HardCodedURL;
use CloudReady::lib::URINotSecured;
use CloudReady::lib::SensitiveDataString;

use constant BEGINNING_MATCH => 0;
use constant FULL_MATCH => 1;

my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

sub checkPatternSensitive($$$) {
	my $reg = shift;
	my $code = shift;
	my $mnemo = shift;

	return CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $reg, $mnemo, $rulesDescription);
	#return () = ($$code =~ /$reg/gm);
}

sub checkImport($$$$$) {
	my $code = shift;
	my $HString = shift;
	my $pattern = shift;
	my $text = shift;
	my $mnemo = shift;
	pos($$code) = undef;

	my $nb_string_content = 0;
	while ($$code =~ /\bimport\s*\((.*)\)/sgc || $$code =~ /\bimport\s+(.*)$/mgc) {
		my $importContent = $1;
		while ($importContent =~ /\b(CHAINE_\d+)\b/gc) {
			my $stringID = $1;
			if ($HString->{$stringID} =~ /$pattern/) {
				$nb_string_content++;
				my $string = quotemeta($HString->{$stringID});
				my $regex = qr/$string/;
				CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);
			}
		}
	}
	return $nb_string_content;
}

sub checkImportAndCode($$$$$$) {
	my $code = shift;
	my $HString = shift;
	my $patternImport = shift;
	my $patternCode = shift;
	my $text = shift;
	my $mnemo = shift;
	pos($$code) = undef;

	my $nb_detect = 0;
	while ($$code =~ /\bimport\s*\((.*)\)/sgc || $$code =~ /\bimport\s+(.*)$/mgc) {
		my $importContent = $1;
		while ($importContent =~ /\b(CHAINE_\d+)\b/gc) {
			my $stringID = $1;
			if ($HString->{$stringID} =~ /$patternImport/) {
				$nb_detect++;
				my $string = quotemeta($HString->{$stringID});
				my $regex = qr/$string/;
				CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);
			}
		}
	}

	# search pattern in code
	my $regex = qr/$patternCode/;
	$nb_detect += CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

	return $nb_detect;
}

sub CountGo($$$$) {
	my ($fichier , $vue , $compteurs , $techno) = @_;
	my $ret = 0;

	my $code;
	my $HString;
	my $MixBloc;
	my $text;
	$text = \$vue->{'text'};
	my $checkPattern = \&checkPatternSensitive;

	$code = \$vue->{'code'};
	$HString = $vue->{'HString'};
	$MixBloc = \$vue->{'MixBloc'};

	if ((!defined $code) || (!defined $$code)) {
		#$ret |= Couples::counter_add($compteurs, $MissingJSPComment__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	# BLOCKERS
	# HL-2312 20/03/2023 Detect the usage of local files manipulation
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileCreate(), $checkPattern->(qr/os\.Create\b/, $code, CloudReady::Ident::Alias_FileCreate()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Open(), $checkPattern->(qr/os\.Open(?:File)?\b/, $code, CloudReady::Ident::Alias_Open()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileCopy(), $checkPattern->(qr/io\.Copy\b/, $code, CloudReady::Ident::Alias_FileCopy()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileMove(), $checkPattern->(qr/os\.Rename\b/, $code, CloudReady::Ident::Alias_FileMove()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileDelete(), $checkPattern->(qr/os\.Remove\b/, $code, CloudReady::Ident::Alias_FileDelete()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileRead(), $checkPattern->(qr/\.(?:NewReader|Read)\b/, $code, CloudReady::Ident::Alias_FileRead()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Write(), $checkPattern->(qr/\.(?:NewWriter|Write(?:File)?)\b/, $code, CloudReady::Ident::Alias_Write()));
	# HL-2313 20/03/2023 Detect the usage of local directory manipulation
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryOpen(), $checkPattern->(qr/\.ReadDir\b/, $code, CloudReady::Ident::Alias_DirectoryOpen()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryDelete(), $checkPattern->(qr/os\.RemoveAll\b/, $code, CloudReady::Ident::Alias_DirectoryDelete()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryExists(), $checkPattern->(qr/os\.ErrNotExist\b/, $code, CloudReady::Ident::Alias_DirectoryExists()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryCreate(), $checkPattern->(qr/os\.Mkdir(?:All)?\b/, $code, CloudReady::Ident::Alias_DirectoryCreate()));
	# HL-2314 21/03/2023 Detect the usage of environment variables
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_EnvironmentVariable(), $checkPattern->(qr/\.(?:Setenv|Getenv|Unsetenv|Clearenv|Environ|ExpandEnv|LookupEnv)\b/, $code, CloudReady::Ident::Alias_EnvironmentVariable()));
	# HL-2315 21/03/2023 Detect the usage of hardcoded path
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedPath(), CloudReady::lib::HardCodedPath::checkHardCodedPath($HString, $techno, $text, $rulesDescription));
	# HL-2316 21/03/2023 Detect the usage of hardcoded IP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedIP(), CloudReady::lib::HardCodedIP::checkHardCodedIP($HString, $code, $techno, $text));
	# HL-2317 21/03/2023 Detect the usage of hardcoded URL
	# Use_SecuredProtocolsHTTP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HTTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURL($fichier, $code, $HString, $techno, $text));
	# Use_SecuredProtocolsFTP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'FTP', $text));
	# Use_LDAPProtocol
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LDAPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'LDAP', $text));
	# HL-2318 21/03/2023 Detect the usage of Access Control List
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_New_AccessControlList(), checkImport($code, $HString, qr /\bgo\-acl["'\/\\]/, $text, CloudReady::Ident::Alias_New_AccessControlList()));
	# HL-2319 21/03/2023 Detect the usage of system calls or processes management
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LaunchSubProcess(), $checkPattern->(qr/\bexec\.(?:Command|LookPath)\b/, $code, CloudReady::Ident::Alias_LaunchSubProcess()));
	# HL-2320 21/03/2023 Detect the usage of unsecured URI
	CloudReady::lib::URINotSecured::checkURINotSecured($fichier, $code, $HString, $techno, $text);
	# HL-2321 22/03/2023 Detect the usage of dll files system
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DllImport, $checkPattern->(qr /\.LoadLibrary\b/, $code, CloudReady::Ident::Alias_DllImport));
	# HL-2322 22/03/2023 Detect the usage of a local temporary file
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GetTempPath, $checkPattern->(qr /\.(?:TempDir|TempFile)\b/, $code, CloudReady::Ident::Alias_GetTempPath));
	# HL-2323 22/03/2023 Detect the usage of an unsecured data string
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SensitiveDataString(), CloudReady::lib::SensitiveDataString::CountSensitiveDataString($code, $HString, $techno));

	# BOOSTERS
	# HL-2294 22/03/2023 Detect the usage of a cloud-based database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_AmazonRedshift(), checkImport($code, $HString, qr /\bredshiftdata["'\/\\]/, $text, CloudReady::Ident::Alias_Using_AmazonRedshift()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsAthena(), checkImportAndCode($code, $HString, qr /\bathena["'\/\\]/, qr /\bathena\.New\b/, $text, CloudReady::Ident::Alias_AwsAthena()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigQuery(), checkImportAndCode($code, $HString, qr /\bbigquery["'\/\\]/, qr /\bbigquery\.NewClient\b/, $text, CloudReady::Ident::Alias_UsingBigQuery()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPCloudSQL(), checkImportAndCode($code, $HString, qr /\bcloudsqlconn["'\/\\]/, qr /\bcloudsqlconn\.NewDialer\b/, $text, CloudReady::Ident::Alias_GCPCloudSQL()));
	# HL-2295 23/03/2023 Detect the usage of a cloud-based storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsS3Storage(), checkImportAndCode($code, $HString, qr /\bs3["'\/\\]/, qr /\bs3manager\.NewUploader\b/, $text, CloudReady::Ident::Alias_AmazonawsS3Storage()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MicrosoftAzureBlobStorage(), checkImportAndCode($code, $HString, qr /\bazblob["'\/\\]/, qr /\bazblob\.NewClient\b/, $text, CloudReady::Ident::Alias_MicrosoftAzureBlobStorage()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPStorage(), checkImportAndCode($code, $HString, qr /\bcloud\.google\.com[\/\\]go[\/\\]storage["'\/\\]/, qr /\bstorage\.NewClient\b/, $text, CloudReady::Ident::Alias_GCPStorage()));
	# HL-2296 24/03/2023 Detect the usage of a cloud-based search engine
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Elasticsearch(), checkImportAndCode($code, $HString, qr /\bgo\-elasticsearch["'\/\\]/, qr /\belasticsearch\.NewDefaultClient\b/, $text, CloudReady::Ident::Alias_Elasticsearch()));
	# HL-2297 24/03/2023 Detect the usage of a cloud-based function as a service (serverless)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsLambda(), checkImportAndCode($code, $HString, qr /\baws\-lambda-go[\/\\]lambda["'\/\\]/, qr /\blambda\.Start\b/, $text, CloudReady::Ident::Alias_AwsLambda()));
	# CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureFunctions(), $checkPattern->(qr/\bimport\s+com\.microsoft\.azure\.functions\b.*/, $code));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPFunctions(), checkImportAndCode($code, $HString, qr /\bGoogleCloudPlatform[\/\\]functions\-framework\-go[\/\\](?:functions|funcframework)["'\/\\]/, qr /\b(?:functions\.HTTP|funcframework\.Start)\b/, $text, CloudReady::Ident::Alias_GCPFunctions()));
	# HL-2298 24/03/2023 Detect the usage of a cloud-based NoSQL database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_DynamoDB(), checkImportAndCode($code, $HString, qr /\baws\-sdk\-go[\/\\]service[\/\\]dynamodb["'\/\\]/, qr /\bdynamodb\.New\b/, $text, CloudReady::Ident::Alias_DBMS_DynamoDB()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MongoDB(), checkImportAndCode($code, $HString, qr /\bgo\.mongodb\.org[\/\\]mongo-driver[\/\\]mongo["'\/\\]/, qr /\bmongo\.(?:NewClient|Connect)\b/, $text, CloudReady::Ident::Alias_DBMS_MongoDB()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_CosmosDB(), checkImportAndCode($code, $HString, qr /\bazure\-sdk\-for\-go[\/\\]sdk[\/\\]data[\/\\]azcosmos["'\/\\]/, qr /\bazcosmos\.NewClient\b/, $text, CloudReady::Ident::Alias_DBMS_CosmosDB()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingFireBase(), checkImportAndCode($code, $HString, qr /\b(?:cloud|firebase)\.google\.com[\/\\]go(?:[\/\\]firestore)?["'\/\\]/, qr /\b(?:firebase\.NewApp|firestore\.NewClient)\b/, $text, CloudReady::Ident::Alias_UsingFireBase()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigTable(), checkImportAndCode($code, $HString, qr /\bcloud\.google\.com[\/\\]go[\/\\]bigtable["'\/\\]/, qr /\bbigtable\.NewAdminClient\b/, $text, CloudReady::Ident::Alias_UsingBigTable()));
	# HL-2299 24/03/2023 Detect the usage of cloud-based relational database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_PostgreSQL(), checkImportAndCode($code, $HString, qr /\blib[\/\\]pq["'\/\\]/, qr /\bsql\.Open\s*\(\s*["']postgres["']/, $text, CloudReady::Ident::Alias_DBMS_PostgreSQL()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MySQL(), checkImportAndCode($code, $HString, qr /\bgo\-sql\-driver[\/\\]mysql["'\/\\]/, qr /\bsql\.Open\s*\(\s*["']mysql["']/, $text, CloudReady::Ident::Alias_DBMS_MySQL()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_Oracle(), checkImportAndCode($code, $HString, qr /\bgo\-ora["'\/\\]/, qr /\bgo\_ora\.NewConnection\b/, $text, CloudReady::Ident::Alias_DBMS_Oracle()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_SQLServer(), checkImportAndCode($code, $HString, qr /\bgo\-mssqldb[\/\\]azuread["'\/\\]/, qr /\bsql\.Open\s*\(\s*azuread\b/, $text, CloudReady::Ident::Alias_DBMS_SQLServer()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_AmazonRDS(), checkImport($code, $HString, qr /\baws[\/\\].*[\/\\]rds["'\/\\]/, $text, CloudReady::Ident::Alias_Using_AmazonRDS()));
	# HL-2300 24/03/2023 Detect the usage of a cloud-based Identity and Access Management (IAM)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsCloudIAM(), checkImportAndCode($code, $HString, qr /\baws-sdk-go[\/\\]service[\/\\]iam["'\/\\]/, qr /\biam\.New\b/, $text, CloudReady::Ident::Alias_AwsCloudIAM()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureDirectoryService(), checkImportAndCode($code, $HString, qr /\bazure\-sdk\-for\-go[\/\\]sdk[\/\\]azidentity["'\/\\]/, qr /\bazidentity\.NewDefaultAzureCredential\b/, $text, CloudReady::Ident::Alias_AzureDirectoryService()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudIAM(), checkImportAndCode($code, $HString, qr /\bcloud\.google\.com[\/\\]go[\/\\]iam["'\/\\]/, qr /\biam\.NewIamPolicyClient\b/, $text, CloudReady::Ident::Alias_UsingCloudIAM()));
	# HL-2301 24/03/2023 Detect the usage of a cloud-based key service encryption
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsServicesKms(), checkImportAndCode($code, $HString, qr /\baws[\/\\].*[\/\\]kms["'\/\\]/, qr /\bkms\.NewFromConfig\b/, $text, CloudReady::Ident::Alias_AmazonawsServicesKms()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MicrosoftAzureKeyVault(), checkImportAndCode($code, $HString, qr /\bazure\-sdk\-for\-go[\/\\]sdk[\/\\]keyvault[\/\\]azsecrets["'\/\\]/, qr /\bazsecrets\.NewClient\b/, $text, CloudReady::Ident::Alias_MicrosoftAzureKeyVault()));
	# HL-2302 27/03/2023 Detect the usage of a cloud-based container service
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsElasticContainer(), checkImportAndCode($code, $HString, qr /\baws\-sdk\-go[\/\\]service[\/\\]ec2["'\/\\]/, qr /\bec2\.New\b/, $text, CloudReady::Ident::Alias_AwsElasticContainer()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Docker(), checkImportAndCode($code, $HString, qr /\bdocker[\/\\](?:client|api)["'\/\\]/, qr /\bclient\.NewClientWithOpts\b/, $text, CloudReady::Ident::Alias_Docker()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Kubernetes(), checkImportAndCode($code, $HString, qr /\bclient\-go[\/\\]kubernetes["'\/\\]/, qr /\bkubernetes\.NewForConfig\b/, $text, CloudReady::Ident::Alias_Kubernetes()));
	# HL-2303 27/03/2023 Detect the usage of a cloud-based middleware application
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_AmazonSQS(), checkImportAndCode($code, $HString, qr /\baws\-sdk\-go[\/\\]service[\/\\]sqs["'\/\\]/, qr /\bsqs\.New\b/, $text, CloudReady::Ident::Alias_Using_AmazonSQS()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingMicrosoftServiceBusMessaging(), checkImportAndCode($code, $HString, qr /\bazure\-sdk\-for\-go[\/\\]sdk[\/\\]messaging[\/\\]azservicebus["'\/\\]/, qr /\bazservicebus\.NewClient\b/, $text, CloudReady::Ident::Alias_UsingMicrosoftServiceBusMessaging()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudPubSub(), checkImportAndCode($code, $HString, qr /\bcloud\.google\.com[\/\\]go[\/\\]pubsub["'\/\\]/, qr /\bpubsub\.NewClient\b/, $text, CloudReady::Ident::Alias_UsingCloudPubSub()));
	# HL-2304 28/03/2023 Detect the usage of a cloud-based Big Data technology
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsEMRBigData(), checkImportAndCode($code, $HString, qr /\baws\-sdk\-go[\/\\]service[\/\\]emr["'\/\\]/, qr /\bemr\.New\b/, $text, CloudReady::Ident::Alias_AwsEMRBigData()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPDataproc(), checkImportAndCode($code, $HString, qr /\bgoogle\.golang\.org[\/\\]api[\/\\]dataproc["'\/\\]/, qr /\bdataproc\.NewService\b/, $text, CloudReady::Ident::Alias_GCPDataproc()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Kafka(), checkImportAndCode($code, $HString, qr /\bkafka\-go["'\/\\]/, qr /\bkafka\.DialLeader\b/, $text, CloudReady::Ident::Alias_Kafka()));
	# HL-2305 28/03/2023 Detect the usage of JSON to facilitate interactions with cloud services
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_JSONParsing(), checkImportAndCode($code, $HString, qr /\bencoding[\/\\]json["'\/\\]/, qr /\bjson\.Unmarshal\b/, $text, CloudReady::Ident::Alias_JSONParsing()));
	# HL-2310 28/03/2023 Detect the usage of XML to facilitate interactions with cloud services
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_XMLParsing(), $checkPattern->(qr/\bxml\.Unmarshal\b/, $code, CloudReady::Ident::Alias_XMLParsing));
	# HL-2311 28/03/2023 Detect the usage of a cloud-based blockchain technology
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Blockchain(), checkImportAndCode($code, $HString, qr /\b(?:hyperledger|go\-ethereum[\/\\]ethclient)["'\/\\]/, qr /\bethclient\.Dial\b/, $text, CloudReady::Ident::Alias_Blockchain()));
	# # HL-2331 28/03/2023 Detect the usage of in-memory database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_InMemoryRedis(), checkImportAndCode($code, $HString, qr /\bgo\-redis["'\/\\]/, qr /\bredis\.NewClient\b/, $text, CloudReady::Ident::Alias_InMemoryRedis()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_InMemoryRedisGCP(), checkImportAndCode($code, $HString, qr /\bcloud\.google\.com[\/\\]go[\/\\]redis["'\/\\]/, qr /\bredis\.NewCloudRedisClient\b/, $text, CloudReady::Ident::Alias_Using_InMemoryRedisGCP()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_InMemoryMemcached(), checkImportAndCode($code, $HString, qr /\bgomemcache[\/\\]memcache["'\/\\]/, qr /\bmemcache\.New\b/, $text, CloudReady::Ident::Alias_InMemoryMemcached()));

	my $parameters = AnalyseUtil::recuperer_options();
	my $dbgMatchPatternDetail = $parameters->[0]->{'--dbgMatchPatternDetail'};
	if (defined $dbgMatchPatternDetail) {
		# replacing SHA by filenames in log
		CommonOutput::ElaborateLog::ElaborateLogPartTwo($fichier);
	}

	return $ret;
}

1;
