package CloudReady::CountPHP;

use strict;
use warnings;

use Erreurs;
use CloudReady::detection;
use CloudReady::config;
use CloudReady::Ident;
use CloudReady::lib::HardCodedIP;
use CloudReady::lib::HardCodedPath;
use CloudReady::lib::HardCodedURL;
use CloudReady::lib::URINotSecured;
use CloudReady::lib::SensitiveDataString;
use CloudReady::lib::KeepPortalData;

use constant BEGINNING_MATCH => 0;
use constant FULL_MATCH => 1;

my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

sub checkPatternSensitive($$$) {
	my $reg = shift;
	my $code = shift;
	my $mnemo = shift;

	return CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $reg, $mnemo, $rulesDescription);
	# return () = ($$code =~ /$reg/gm);
}


sub oneStringContains($$) {
	my $HString = shift;
	my $REG = shift;
	
	for my $stringId (keys %$HString) 
    {
		if ($HString->{$stringId} =~ /$REG/) 
        {
            return 1;
		}
	}
	return 0;
}

sub checkPatternInStrings($$$) {
	my $text = shift;
	my $regex = shift;
	my $mnemo = shift;
	my $detect = 0;

	$detect = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

	return $detect;
}

sub checkDBMSMySQL($$$) {
	my $code = shift;
	my $HString = shift;
	my $text = shift;
	my $nb_DBMSMySQL = 0;

	my $mnemo = CloudReady::Ident::Alias_DBMS_MySQL;

	# HL-597
	if ($$code =~ /\b(?:mysql_connect|mysqli|mysqli_connect)\s*\(/) {
		my $regex = qr/\b(?:mysql_connect|mysqli|mysqli_connect)\s*\(/;
		$nb_DBMSMySQL = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);
	}

	if (oneStringContains($HString, qr/\bmysql:host\b/)) {
		my $regex = qr/\bmysql:host\b/;
		$nb_DBMSMySQL += CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);
	}
	# HL-954
	my $regex = qr/\bgetenv\s*\(\s*["'](?:MYSQL_DSN|MYSQL_USER|MYSQL_PASSWORD)["']/;
	$nb_DBMSMySQL += CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

	return $nb_DBMSMySQL;
}

sub checkMongoDB($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_DBMS_MongoDB;
	my $nb_DBMSMongoDB = 0;

	my $regex = qr/\bnew\s+MongoClient\s*\(/;
	$nb_DBMSMongoDB = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_DBMSMongoDB;
}


sub checkCosmosDB($$) {
	my $code = shift;
	my $text = shift;
	my $mnemo = CloudReady::Ident::Alias_DBMS_CosmosDB;
	my $nb_DBMSCosmosDB = 0;

	if ($$code =~ /\bnew\s+DocumentDB\b/) {
		my $regex = qr/\bphpdocumentdb\.php\b/;
		$nb_DBMSCosmosDB = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);
	}

	return $nb_DBMSCosmosDB;
}

sub checkDynamoDB($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_DBMS_DynamoDB;
	my $nb_DBMSDynamoDB = 0;

	my $regex = qr/\bDynamoDbClient\b/;
	$nb_DBMSDynamoDB = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_DBMSDynamoDB;
}

sub checkPostgreSQL($$$) {
	my $code = shift;
	my $HString = shift;
	my $text = shift;
	my $mnemo = CloudReady::Ident::Alias_DBMS_PostgreSQL;
	my $nb_DBMSPostgreSQL = 0;

	my $regex = qr/\bpg_connect\s*\(/;
	$nb_DBMSPostgreSQL = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	# HL-955
	$regex = qr/\bgetenv\s*\(\s*["'](?:POSTGRES_DSN|POSTGRES_USER|POSTGRES_PASSWORD)["']/;
	$nb_DBMSPostgreSQL += CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

	return $nb_DBMSPostgreSQL;
}

sub checkAmazonawsServicesKms($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_AmazonawsServicesKms();
	my $nb_AmazonawsServicesKms = 0;

	my $regex = qr/\bKmsClient\b/;
	$nb_AmazonawsServicesKms += CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AmazonawsServicesKms;
}

sub checkAzureBlobStorage($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_MicrosoftAzureBlobStorage();
	my $nb_AzureBlobStorage = 0;

	my $regex = qr/\buse\s+MicrosoftAzure(?:\\|\/)Storage\b/;
	$nb_AzureBlobStorage = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AzureBlobStorage;
}

sub checkAmazonawsS3Storage($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_AmazonawsS3Storage();
	my $nb_AmazonawsS3Storage = 0;

	my $regex = qr/\b(?:use\s+Aws(?:\\|\/)S3|S3Client)\b/;
	$nb_AmazonawsS3Storage = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AmazonawsS3Storage;
}

sub checkInMemoryRedisAzure($)
{
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_Import_InMemoryRedisAzure();
	my $nb_InMemoryRedisAzure = 0;

	my $regex = qr/\bnew\s+Redis\b/;
	$nb_InMemoryRedisAzure = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_InMemoryRedisAzure;
}

sub checkInMemoryRedisAws($$) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_Import_InMemoryRedisAws();
	my $nb_InMemoryRedisAws = 0;

	my $regex = qr/\bElastiCacheClient\b/;
	$nb_InMemoryRedisAws = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_InMemoryRedisAws;
}

sub checkAzureDirectoryService($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_AzureDirectoryService();
	my $nb_AzureDirectoryService = 0;

	my $regex = qr/\bGraphServiceAccessHelper\b/;
	$nb_AzureDirectoryService = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AzureDirectoryService;
}

sub checkAwsDirectoryService($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_AmazonawsDirectoryService();
	my $nb_AwsDirectoryService = 0;

	my $regex = qr/\bconnectDirectory\b/;
	$nb_AwsDirectoryService = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AwsDirectoryService;
}

sub checkAmazonawsBatch($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_AmazonawsBatch();
	my $nb_AwsDirectoryService = 0;

	my $regex = qr/\bBatchClient\b|\bnamespace\s+Aws\\Batch\b/;
	$nb_AwsDirectoryService = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AwsDirectoryService;
}

sub checkAzureServiceBusMessaging($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_AzureServiceBusMessaging();
	my $nb_AzureServiceBusMessaging = 0;

	my $regex = qr/\bnamespace\s+WindowsAzure\\ServiceBus\b|\bServicesBuilder\b/;
	$nb_AzureServiceBusMessaging = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AzureServiceBusMessaging;
}

sub checkAmazonAwsAccessManagement($) {
	my $code = shift;
	my $mnemo = CloudReady::Ident::Alias_AmazonAwsAccessManagement();
	my $nb_AmazonAwsAccessManagement = 0;

	my $regex = qr/\bIamClient\b/;
	$nb_AmazonAwsAccessManagement = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

	return $nb_AmazonAwsAccessManagement;
}

sub CountPHP($$$$) {
	my ($fichier, $vue, $compteurs, $techno) = @_ ;
	my $ret = 0;

	my $code;
	my $HString;
	my $text;
	$text = \$vue->{'text'};
    my $MixBloc;
	my $checkPattern = \&checkPatternSensitive;
	
	$code = \$vue->{'code'};
	$HString = $vue->{'HString'};
    $MixBloc = \$vue->{'MixBloc'};
    
	if ((! defined $code ) || (!defined $$code)) {
		#$ret |= Couples::counter_add($compteurs, $MissingJSPComment__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedIP(), CloudReady::lib::HardCodedIP::checkHardCodedIP($HString, $code, $techno, $text));
    # HL-576 Launching subprocess
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LaunchSubProcess(), $checkPattern->(qr/\bproc_open\s*\(|\bpopen\s*\(|\bsystem\s*\(|\bexec\s*\(/, $code, CloudReady::Ident::Alias_LaunchSubProcess()));
	# HL-579 Perform Directory Manipulation 
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryCreateDirectory(), $checkPattern->(qr/\bmkdir\s*\(/, $code, CloudReady::Ident::Alias_DirectoryCreateDirectory()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryMove(), $checkPattern->(qr/\brename\s*\(/, $code, CloudReady::Ident::Alias_DirectoryMove()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryDelete(), $checkPattern->(qr/\brmdir\s*\(/, $code, CloudReady::Ident::Alias_DirectoryDelete()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryExists(), $checkPattern->(qr/\b(?:scandir|readdir|is_dir)\s*\(/, $code, CloudReady::Ident::Alias_DirectoryExists()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryCurrent(), $checkPattern->(qr/\bgetcwd\s*\(/, $code, CloudReady::Ident::Alias_DirectoryCurrent()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryChange(), $checkPattern->(qr/\bchdir\s*\(/, $code, CloudReady::Ident::Alias_DirectoryChange()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryAccess(), $checkPattern->(qr/\b(?:chmod|chown)\s*\(/, $code, CloudReady::Ident::Alias_DirectoryAccess()));
    # HL-583 Perform File Manipulation PHP
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Open(), $checkPattern->(qr/\b(?:fopen|file)\s*\(/, $code, CloudReady::Ident::Alias_Open()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Write(), $checkPattern->(qr/\b(?:fwrite|fputs)\s*\(/, $code, CloudReady::Ident::Alias_Write()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileExists(), $checkPattern->(qr/\bfile_exists\s*\(/, $code, CloudReady::Ident::Alias_FileExists()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileMove(), $checkPattern->(qr/\brename\s*\(/, $code, CloudReady::Ident::Alias_FileMove()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileDelete(), $checkPattern->(qr/\bunlink\s*\(/, $code, CloudReady::Ident::Alias_FileDelete()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileCopy(), $checkPattern->(qr/\bcopy\s*\(/, $code, CloudReady::Ident::Alias_FileCopy()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_New_FileInputStream(), $checkPattern->(qr/\b(?:fgetc|fgets|fgetss|fgetcsv|readfile|tempnam|tmpfile)\s*\(/, $code, CloudReady::Ident::Alias_New_FileInputStream()));
    # HL-586 Access to environment variable from PHP
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_EnvironmentVariable(), $checkPattern->(qr/(?:\$_ENV\b|\$HTTP_ENV_VARS\b|\bgetenv\s*\()/, $code, CloudReady::Ident::Alias_EnvironmentVariable()));
	# HL-593 PHP & Python (HardCodedPath)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedPath(), CloudReady::lib::HardCodedPath::checkHardCodedPath($HString, $techno, $text, $rulesDescription));
    # HL-597 MySQL PHP & Python
	# HL-954 08/10/2019 [GCP Boosters] Using MySQL database
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MySQL, checkDBMSMySQL($code, $HString, $text));
    # HL-598 MongoDB PHP & Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MongoDB, checkMongoDB($code));
    # HL-599 NoSQL CosmosDB Azure Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_CosmosDB, checkCosmosDB($code, $text));
    # HL-600 NoSQL DynamoDB AWS PHP & Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_DynamoDB, checkDynamoDB($code));
    # HL-601 NoSQL PostgreSQL PHP & Python
	# HL-955 10/10/2019 [GCP Boosters] Using PostgreSQL database	
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_PostgreSQL, checkPostgreSQL($code, $HString, $text));
    #HL-608 BUG Fix HL-247 unsecured URL without XML namespace context
	#HL-870 21/05/2019 Split FTP/HTTP CloudReady pattern in 2 separate patterns
	CloudReady::lib::URINotSecured::checkURINotSecured($fichier, $code, $HString, $techno, $text);
    #HL-615 MySQL database access deprecated PHP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBAccessMySQL(), $checkPattern->(qr/\b(?:mysql_connect|mysql_select_db|mysql_close)\s*\(/, $code, CloudReady::Ident::Alias_DBAccessMySQL()));
    #HL-630 KMS service encryption AWS PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsServicesKms(), checkAmazonawsServicesKms($code));
    #HL-634 Azure blob storage PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MicrosoftAzureBlobStorage(), checkAzureBlobStorage($code)); 
    #HL-636 AWS S3 storage PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsS3Storage(), checkAmazonawsS3Storage($code)); 
    #HL-640 Azure elasticache PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Import_InMemoryRedisAzure(), checkInMemoryRedisAzure($code));
    #HL-641 AWS elasticache PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Import_InMemoryRedisAws(), checkInMemoryRedisAws($code, $HString)); 
    #HL-646 Azure Active Directory access permission PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureDirectoryService(), checkAzureDirectoryService($code)); 
    #HL-647 AWS Active Directory access permission PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsDirectoryService(), checkAwsDirectoryService($code)); 
    #HL-651 Detect Usage of AWS Batch PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsBatch(), checkAmazonawsBatch($code)); 
    #HL-654 Azure service bus messaging PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureServiceBusMessaging(), checkAzureServiceBusMessaging($code)); 
    #HL-659 AWS Cloud Identity and Access Management PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonAwsAccessManagement(), checkAmazonAwsAccessManagement($code)); 
	# HL-952 17/09/2019 [GCP Boosters] Using Kubernetes
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingKubernetes(), $checkPattern->(qr/\bnew\s+KubernetesClient\b|\buse\s+(?:[\w\\]+)?\bKubernetes(?:Runtime)?\b/, $code, CloudReady::Ident::Alias_UsingKubernetes()));
	# HL-953 07/10/2019 [GCP Boosters] Using BigQuery
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigQuery(), $checkPattern->(qr/\bnew\s+BigQueryClient\b|\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]BigQuery\b/, $code, CloudReady::Ident::Alias_UsingBigQuery()));
	# HL-957 10/10/2019 [GCP Boosters] Using a Cloud-Based Storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudDataStore(), $checkPattern->(qr/\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]Datastore\b/, $code, CloudReady::Ident::Alias_UsingCloudDataStore()));
	# HL-958 11/10/2019 [GCP Boosters] Using BigTable
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigTable(), $checkPattern->(qr/\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]Bigtable\b/, $code, CloudReady::Ident::Alias_UsingBigTable()));
	# HL-959 15/10/2019 [GCP Boosters] Using Cloud Spanner
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudSpanner(), $checkPattern->(qr/\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]Spanner\b/, $code, CloudReady::Ident::Alias_UsingCloudSpanner()));
	# HL-960 16/10/2019 [GCP Boosters] Using Cloud in-memory database (redis)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_InMemoryRedisGCP(), $checkPattern->(qr/\bnew\s+CloudRedisClient\b|\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]Redis\b/, $code, CloudReady::Ident::Alias_Using_InMemoryRedisGCP()));
	# HL-961 17/10/2019 [GCP Boosters] Using Cloud IAM (Identity and Access Management)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudIAM(), $checkPattern->(qr/\bnew\s+Iam\b|\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]Core[\\]Iam\b/, $code, CloudReady::Ident::Alias_UsingCloudIAM()));
	# HL-962 18/10/2019 [GCP Boosters] Using Firebase
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingFireBase(), $checkPattern->(qr/\buse\s+(?:[\w\\]+)?\bKreait[\\]Firebase\b/, $code, CloudReady::Ident::Alias_UsingFireBase()));
	# HL-963 28/10/2019 [GCP Boosters] Using Cloud IAP (Identity Aware Proxy)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudIAP(), $checkPattern->(qr/\bnamespace\s+Google[\\]Cloud[\\]Samples[\\]Iap\b|\bfunction\s+make_iap_request\b/, $code, CloudReady::Ident::Alias_UsingCloudIAP()));
	# HL-964 28/10/2019 [GCP Boosters] Using a Cloud-based Key storage (KMS)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPServicesKms(), $checkPattern->(qr/\bnew\s+Google_Service_CloudKMS\b/, $code, CloudReady::Ident::Alias_GCPServicesKms()));
	# HL-965 28/10/2019 [GCP Boosters] Using a Cloud-Based Storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPStorage(), $checkPattern->(qr/\bnew\s+StorageClient\b|\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]Storage\b/, $code, CloudReady::Ident::Alias_GCPStorage()));
	# HL-966 29/10/2019 [GCP Boosters] Using a Cloud-based task scheduling service
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPScheduler(), $checkPattern->(qr/\bnew\s+Google_Service_CloudScheduler\b/, $code, CloudReady::Ident::Alias_GCPScheduler()));
	# HL-968 30/10/2019 [GCP Boosters] Using a Cloud-based middleware application
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudPubSub(), $checkPattern->(qr/\buse\s+(?:[\w\\]+)?\bGoogle[\\]Cloud[\\]PubSub\b|\bnew\s+PubSubClient\b/, $code, CloudReady::Ident::Alias_UsingCloudPubSub()));
	# 25/11/2021 [Blocker] servlet PHP (like java blocker HttpSession_setAttribute)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_ServletPHP(), $checkPattern->(qr/\bsession_start\s*\(\)/, $code, CloudReady::Ident::Alias_ServletPHP()));
	# 14/01/2022 [Blocker] Azure Database Driver incompatibility PDO_DBLIB
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DbDriverDbLib(), checkPatternInStrings($text, qr/\bdblib\s*\:/, CloudReady::Ident::Alias_DbDriverDbLib()));
	# 14/01/2022 [Blocker] can't use sendmail utility on Paas
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SendMail(), $checkPattern->(qr/\bmail\s*\(/, $code, CloudReady::Ident::Alias_SendMail()));
	# Use_Unsecured_Data_Strings
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SensitiveDataString(), CloudReady::lib::SensitiveDataString::CountSensitiveDataString($code, $HString, $techno));
	# Use_SecuredProtocolsHTTP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HTTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURL($fichier, $code, $HString, $techno, $text));
	# Use_SecuredProtocolsFTP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'FTP', $text));
	# Use_LDAPProtocol
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LDAPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'LDAP', $text));

	my $parameters = AnalyseUtil::recuperer_options();
	my $dbgMatchPatternDetail = $parameters->[0]->{'--dbgMatchPatternDetail'};
	if (defined $dbgMatchPatternDetail) {
		# replacing SHA by filenames in log
		CommonOutput::ElaborateLog::ElaborateLogPartTwo($fichier);
	}

	return $ret;
}

1;
