package CloudReady::CountPlSql;

use strict;
use warnings;

use Erreurs;
use CloudReady::detection;
use CloudReady::config;
use CloudReady::Ident;
use CloudReady::lib::HardCodedIP;
use CloudReady::lib::HardCodedPath;
use CloudReady::lib::HardCodedURL;
use CloudReady::lib::URINotSecured;
use CloudReady::lib::SensitiveDataString;

use constant BEGINNING_MATCH => 0;
use constant FULL_MATCH => 1;

my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

sub checkPatternCodeStrings($$$$$) {
    my $code = shift;
    my $text = shift;
    my $HString = shift;
    # my $patternImport = shift;
    my $pattern = shift;
    my $mnemo = shift;
    pos($$code) = undef;

    my $nb_detect = 0;
    for my $stringID (keys %{$HString}) {
        if ($HString->{$stringID} =~ /$pattern/) {
            $nb_detect++;
            my $string = quotemeta($HString->{$stringID});
            my $regex = qr/$string/;
            CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);
        }
    }

    # search pattern in code
    my $regex = qr/$pattern/;
    $nb_detect += CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_detect;
}

sub CountPlSql($$$) {
    my ($fichier, $vue, $compteurs) = @_;
    my $ret = 0;

    my $code;
    my $HString;
    my $MixBloc;
    my $text;
    $text = \$vue->{'text'};
    my $checkPattern = \&checkPatternCodeStrings;

    $code = \$vue->{'code'};
    $HString = $vue->{'HString'};
    $MixBloc = \$vue->{'MixBloc'};

    if ((!defined $code) || (!defined $$code)) {
        #$ret |= Couples::counter_add($compteurs, $MissingJSPComment__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
        return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
    }

    # BLOCKERS
    # HL-2493 17/08/2023 Ensure unsupported PL/SQL packages DBMS are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_PLSQL(), $checkPattern->($code, $text, $HString, qr/\bDBMS\_(?:DEBUG|LDAP)[._]/i, CloudReady::Ident::Alias_DBMS_PLSQL()));
    # HL-2494 17/08/2023 Ensure MERGE statement is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Merge(), $checkPattern->($code, $text, $HString, qr/\bmerge\b/i, CloudReady::Ident::Alias_Merge()));
    # HL-2495 17/08/2023 Ensure CLOB data type is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Clob(), $checkPattern->($code, $text, $HString, qr/\bCLOB\b|\bTO\_CLOB\b/i, CloudReady::Ident::Alias_Clob()));
    # HL-2496 17/08/2023 Ensure PL/SQL package UTL_FILE is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Utl_File(), $checkPattern->($code, $text, $HString, qr/\bUTL\_FILE\b/i, CloudReady::Ident::Alias_Utl_File()));
    # HL-2497 18/08/2023 Ensure PL/SQL package DBMS_SCHEDULER is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_Scheduler(), $checkPattern->($code, $text, $HString, qr/\bDBMS\_SCHEDULER\b/i, CloudReady::Ident::Alias_DBMS_Scheduler()));
    # HL-2498 18/08/2023 Ensure PL/SQL package DBMS_RANDOM is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_Random(), $checkPattern->($code, $text, $HString, qr/\bDBMS\_RANDOM\b/i, CloudReady::Ident::Alias_DBMS_Random()));
    # HL-2499 18/08/2023 Ensure PL/SQL package DBMS_RANDOM is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_Output(), $checkPattern->($code, $text, $HString, qr/\bDBMS\_OUTPUT\b/i, CloudReady::Ident::Alias_DBMS_Output()));
    # HL-2500 18/08/2023 Ensure PL/SQL package DBMS_AUTO_INDEX is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_Auto_Index(), $checkPattern->($code, $text, $HString, qr/\bDBMS\_AUTO\_INDEX\b/i, CloudReady::Ident::Alias_DBMS_Auto_Index()));
    # HL-2501 18/08/2023 Ensure PL/SQL packages UTL_HTTP, UTL_SMTP, UTL_TCP, UTL_INADDR are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Utl(), $checkPattern->($code, $text, $HString, qr/\bUTL\_(?:HTTP|SMTP|TCP|INADDR)\b/i, CloudReady::Ident::Alias_Utl()));
    # HL-2502 18/08/2023 Ensure PL/SQL procedures DBMS_LOGMNR.ADD_LOGFILE and DBMS_LOGMNR.REMOVE_LOGFILE are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_LOGMNR(), $checkPattern->($code, $text, $HString, qr/\bDBMS\_LOGMNR\.(?:ADD|REMOVE)\_LOGFILE\b/i, CloudReady::Ident::Alias_DBMS_LOGMNR()));
    # HL-2503 18/08/2023 Ensure LOB data types are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Lob(), $checkPattern->($code, $text, $HString, qr/\b(?:NCLOB|TO\_NCLOB|DBMS\_LOB)\b/i, CloudReady::Ident::Alias_Lob()));
    # HL-2504 18/08/2023 Ensure you don't have unsupported option GENERATED BY DEFAULT for identity columns
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Generated_Default(), $checkPattern->($code, $text, $HString, qr/\bGENERATED\s+BY\s+DEFAULT\b/i, CloudReady::Ident::Alias_Generated_Default()));
    # HL-2505 18/08/2023 Ensure SHARDED tables are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Sharded_Tbl(), $checkPattern->($code, $text, $HString, qr/\bSHARDED\b/i, CloudReady::Ident::Alias_Sharded_Tbl()));
    # HL-2506 18/08/2023 Ensure XMLType columns are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_XMLType(), $checkPattern->($code, $text, $HString, qr/\bXMLTYPE\b/i, CloudReady::Ident::Alias_XMLType()));
    # HL-2507 18/08/2023 Ensure CLUSTER clause in CREATE TABLE statements is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Cluster(), $checkPattern->($code, $text, $HString, qr/\bcreate\s+table\b.+?cluster\b/is, CloudReady::Ident::Alias_Cluster()));
    # HL-2508 18/08/2023 Ensure INDEX-ORGANIZED TABLEs (IOT) are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Iot(), $checkPattern->($code, $text, $HString, qr/\bORGANIZATION\s+INDEX\b/i, CloudReady::Ident::Alias_Iot()));
    # HL-2509 18/08/2023 Ensure ROWID and ROWNUM are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Row(), $checkPattern->($code, $text, $HString, qr/\bROW(?:ID|NUM)\b/i, CloudReady::Ident::Alias_Row()));
    # HL-2510 18/08/2023 Ensure EXTERNAL TABLEs are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_External_Tbl(), $checkPattern->($code, $text, $HString, qr/\bEXTERNAL\b/i, CloudReady::Ident::Alias_External_Tbl()));
    # HL-2511 18/08/2023 Ensure Database Links are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Db_Links(), $checkPattern->($code, $text, $HString, qr/\bDATABASE\s+LINK\b/i, CloudReady::Ident::Alias_Db_Links()));
    # HL-2512 18/08/2023 Ensure Unused Column is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Unused_Column(), $checkPattern->($code, $text, $HString, qr/\bUNUSED\b/i, CloudReady::Ident::Alias_Unused_Column()));
    # HL-2513 18/08/2023 Ensure READ ONLY Table is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Read_Only_Tbl(), $checkPattern->($code, $text, $HString, qr/\bREAD\s+ONLY\b/i, CloudReady::Ident::Alias_Read_Only_Tbl()));
    # HL-2514 18/08/2023 Ensure Invisible Index is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Invisible_Index(), $checkPattern->($code, $text, $HString, qr/\bINVISIBLE\b/i, CloudReady::Ident::Alias_Invisible_Index()));
    # HL-2515 18/08/2023 Ensure DUAL Table is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Dual_Tbl(), $checkPattern->($code, $text, $HString, qr/\bFROM\s+dual\b/i, CloudReady::Ident::Alias_Dual_Tbl()));
    # HL-2516 18/08/2023 Ensure VARCHAR2 data type is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Varchar2(), $checkPattern->($code, $text, $HString, qr/\bVARCHAR2\b/i, CloudReady::Ident::Alias_Varchar2()));
    # HL-2517 18/08/2023 Ensure NVL function is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Nvl(), $checkPattern->($code, $text, $HString, qr/\bNVL\s*\(/i, CloudReady::Ident::Alias_Nvl()));
    # HL-2518 18/08/2023 Ensure DECODE function is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Decode(), $checkPattern->($code, $text, $HString, qr/\bDECODE\s*\(/i, CloudReady::Ident::Alias_Decode()));
    # HL-2519 18/08/2023 Ensure BLOB data type is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Blob(), $checkPattern->($code, $text, $HString, qr/\bBLOB\b/i, CloudReady::Ident::Alias_Blob()));
    # HL-2520 18/08/2023 Ensure Oracle hints are not used
    # diag must be in text vue
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Oracle_Hints(), $checkPattern->($text, $text, $HString, qr/\b(?:DELETE|INSERT|SELECT|UPDATE)\s+\/\*/i, CloudReady::Ident::Alias_Oracle_Hints()));
    # HL-2521 21/08/2023 Ensure SYSDATE function is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SysDate(), $checkPattern->($code, $text, $HString, qr/\bSYSDATE\b/i, CloudReady::Ident::Alias_SysDate()));
    # HL-2522 21/08/2023 Ensure IS NULL operator is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_IsNull(), $checkPattern->($code, $text, $HString, qr/\bIS\s+NULL\b/i, CloudReady::Ident::Alias_IsNull()));
    # HL-2523 21/08/2023 Ensure SYNONYM alias is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Synonym(), $checkPattern->($code, $text, $HString, qr/\bSYNONYM\b/i, CloudReady::Ident::Alias_Synonym()));
    # HL-2524 21/08/2023 Ensure BITMAP INDEX is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Bitmap_Index(), $checkPattern->($code, $text, $HString, qr/\bBITMAP\s+INDEX\b/i, CloudReady::Ident::Alias_Bitmap_Index()));
    # HL-2525 21/08/2023 Ensure outer joins (+) are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Outer_Join(), $checkPattern->($code, $text, $HString, qr/\(\+\)/, CloudReady::Ident::Alias_Outer_Join()));
    # HL-2526 21/08/2023 Ensure CONNECT BY statement is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Connect_By(), $checkPattern->($code, $text, $HString, qr/\bCONNECT\s+BY\b/i, CloudReady::Ident::Alias_Connect_By()));
    # HL-2527 21/08/2023 Ensure BULK COLLECT statement is not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Bulk_Collect(), $checkPattern->($code, $text, $HString, qr/\bBULK\s+COLLECT\b/i, CloudReady::Ident::Alias_Bulk_Collect()));
    # HL-2528 21/08/2023 Ensure %FOUND, %NOTFOUND, %ISOPEN and %ROWCOUNT for cursors are not used
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Cursors(), $checkPattern->($code, $text, $HString, qr/\b\%(?:(?:NOT)?FOUND|ISOPEN|ROWCOUNT)\b/i, CloudReady::Ident::Alias_Cursors()));

    my $parameters = AnalyseUtil::recuperer_options();
    my $dbgMatchPatternDetail = $parameters->[0]->{'--dbgMatchPatternDetail'};
    if (defined $dbgMatchPatternDetail) {
        # replacing SHA by filenames in log
        CommonOutput::ElaborateLog::ElaborateLogPartTwo($fichier);
    }

    return $ret;
}

1;
