package CloudReady::CountPython;

use strict;
use warnings;

use Erreurs;
use CloudReady::detection;
use CloudReady::config;
use CloudReady::Ident;
use CloudReady::lib::HardCodedIP;
use CloudReady::lib::HardCodedPath;
use CloudReady::lib::HardCodedURL;
use CloudReady::lib::URINotSecured;
use CloudReady::lib::SensitiveDataString;
use CloudReady::lib::KeepPortalData;

use constant BEGINNING_MATCH => 0;
use constant FULL_MATCH => 1;

my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

sub checkPatternSensitive($$$) {
	my $reg = shift;
	my $code = shift;
    my $mnemo = shift;

    return CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $reg, $mnemo, $rulesDescription);
    # return () = ($$code =~ /$reg/gm);
}

sub checkDBMSMySQL($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_DBMS_MySQL;
    my $nb_DBMSMySQL = 0;

    my $regex = qr/\b(?:mysql\.connector\b|MySQLConnection\s*\(|MySQLdb\.connect\b)/;
    $nb_DBMSMySQL = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_DBMSMySQL;
}

sub checkMongoDB($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_DBMS_MongoDB;
    my $nb_DBMSMongoDB = 0;

    my $regex = qr/\b(?:pymongo\b|MongoClient)\b/;
    $nb_DBMSMongoDB = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_DBMSMongoDB;
}

sub checkCosmosDB($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_DBMS_CosmosDB;
    my $nb_DBMSCosmosDB = 0;

    my $regex = qr/\bpydocumentdb\b/;
    $nb_DBMSCosmosDB = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_DBMSCosmosDB;
}

sub checkDynamoDB($) {
    my $text = shift;
    my $mnemo = CloudReady::Ident::Alias_DBMS_DynamoDB;
    my $nb_DBMSDynamoDB = 0;

    my $regex = qr/\bboto3\.(?:client|resource)\b.*["']dynamodb["']/;
    $nb_DBMSDynamoDB = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_DBMSDynamoDB;
}

sub checkPostgreSQL($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_DBMS_PostgreSQL;
    my $nb_DBMSPostgreSQL = 0;

    my $regex = qr/\b(?:psycopg2|pg8000)\b/;
    $nb_DBMSPostgreSQL = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_DBMSPostgreSQL;
}

sub checkAzureCloudEncryption($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_AzureCloudEncryption;
    my $nb_AzureCloudEncryption = 0;

    my $regex = qr/\b(?:KeyWrapper|KeyResolver)\s*\(/;
    $nb_AzureCloudEncryption = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_AzureCloudEncryption;
}

# sub checkAmazonCloudEncryption
# {
# 	my $code = shift;
# 	my $nb_AmazonCloudEncryption = 0;
#
#     if ($$code =~ /\b(?:Crypto\.Cipher|import\s+AES|import\s+aws_encryption_sdk)\b/) {
#         $nb_AmazonCloudEncryption = 1;
#     }
#
# 	return $nb_AmazonCloudEncryption;
#
# }

sub checkAzureKeyVault($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_MicrosoftAzureKeyVault;
    my $nb_AzureKeyVault = 0;

    my $regex = qr/\b(?:azure\.keyvault|KeyVaultClient|KeyVaultId)\b/;
    $nb_AzureKeyVault = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_AzureKeyVault;
}

sub checkAzureBlobStorage($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_MicrosoftAzureBlobStorage;
    my $nb_AzureBlobStorage = 0;

    my $regex = qr/\b(?:azure\.storage|BlobService)\b/;
    $nb_AzureBlobStorage = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_AzureBlobStorage;
}

sub checkAmazonawsS3Storage($) {
    my $text = shift;
    my $HString = shift;
    my $mnemo = CloudReady::Ident::Alias_AmazonawsS3Storage;
    my $nb_AmazonawsS3Storage = 0;

    my $regex = qr/\bboto3\.(?:client|resource)\b.*["']s3["']/;
    $nb_AmazonawsS3Storage = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_AmazonawsS3Storage;
}

sub checkInMemoryRedisAzure($) {
    my $text = shift;
    my $mnemo = CloudReady::Ident::Alias_Import_InMemoryRedisAzure;
    my $nb_InMemoryRedisAzure = 0;

    my $regex = qr/\bimport\s+redis\b|\bredis\.cache\.windows\.net\b/i;
    $nb_InMemoryRedisAzure = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_InMemoryRedisAzure;
}

sub checkInMemoryRedisAws($) {
    my $text = shift;
    my $mnemo = CloudReady::Ident::Alias_Import_InMemoryRedisAws;
    my $nb_InMemoryRedisAws = 0;

    my $regex = qr/\bboto3\.client\b.*["']elasticache["']|["'].*\bcache\.amazonaws\.com["']/i;
    $nb_InMemoryRedisAws = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_InMemoryRedisAws;
}


sub checkAzureDirectoryService($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_AzureDirectoryService;
    my $nb_AzureDirectoryService = 0;

    my $regex = qr/\badal\b/i;
    $nb_AzureDirectoryService = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_AzureDirectoryService;
}

sub checkAwsDirectoryService($) {
    my $text = shift;
    my $mnemo = CloudReady::Ident::Alias_AmazonawsDirectoryService;
    my $nb_AwsDirectoryService = 0;

    my $regex = qr/\bboto3\.client\b.*["']ds["']/;
    $nb_AwsDirectoryService = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_AwsDirectoryService;
}

sub checkAmazonawsBatch($) {
    my $text = shift;
    my $mnemo = CloudReady::Ident::Alias_AmazonawsBatch;
    my $nb_AmazonawsBatch = 0;

    my $regex = qr/\bboto3\.client\b.*["']batch["']/;
    $nb_AmazonawsBatch = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_AmazonawsBatch;
}

sub checkAzureBatch($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_AzureBatch;
    my $nb_AzureBatch = 0;

    my $regex = qr/\bimport\s+azure\.batch\b/;
    $nb_AzureBatch = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_AzureBatch;
}

sub checkAzureServiceBusMessaging($) {
    my $code = shift;
    my $mnemo = CloudReady::Ident::Alias_AzureServiceBusMessaging;
    my $nb_AzureServiceBusMessaging = 0;

    my $regex = qr/\bazure\.servicebus\b|\bServiceBusService\b/;
    $nb_AzureServiceBusMessaging = CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $regex, $mnemo, $rulesDescription);

    return $nb_AzureServiceBusMessaging;
}

# sub checkAzureAccessManagement($)
# {
# 	my $code = shift;
# 	# my $HString = shift;
# 	my $nb_AzureAccessManagement = 0;
#
#     if ($$code =~ /\bazure\.mgmt\.authorization\b|\bAuthorizationManagementClient\b|\bazure\.common\.credentials\b|\bUserPassCredentials\b/g)
#     {
#         $nb_AzureAccessManagement = 1;
#     }
#
# 	return $nb_AzureAccessManagement;
#
# }

sub checkAmazonAwsAccessManagement($) {
    my $text = shift;
    my $mnemo = CloudReady::Ident::Alias_AmazonAwsAccessManagement;
    my $nb_AmazonAwsAccessManagement = 0;

    my $regex = qr/\bboto3\.client\b.*["']iam["']/;
    $nb_AmazonAwsAccessManagement = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_AmazonAwsAccessManagement;
}

sub checkPatternInCodeAndString($$$$) {
    my $text = shift;
    my $PatternInCode = shift;
    my $PatternInString = shift;
    my $mnemo = shift;
    my $nb_detection = 0;

    my $regex = qr/$PatternInCode.*["']$PatternInString["']/;
    $nb_detection = CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, $mnemo, $rulesDescription);

    return $nb_detection;
}

sub CountPython($$$$) {
	my ($fichier, $vue, $compteurs, $techno) = @_ ;
	my $ret = 0;
	
	my $code;
	my $HString;
    my $text;
    $text = \$vue->{'text'};
    my $MixBloc;
	my $checkPattern = \&checkPatternSensitive;
	
	$code = \$vue->{'code_with_prepro'};
	$HString = $vue->{'HString'};
    $MixBloc = \$vue->{'MixBloc'};
    
	if ((! defined $code ) || (!defined $$code)) {
		#$ret |= Couples::counter_add($compteurs, $MissingJSPComment__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedIP(), CloudReady::lib::HardCodedIP::checkHardCodedIP($HString, $code, $techno, $text));
	# HL-575 Launching subprocess
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LaunchSubProcess(), $checkPattern->(qr/\bos\.(?:system|popen)\b|\bsubprocess\.(?:call|run|Popen)\b|\bshell\s*\=\s*True\b/, $code, CloudReady::Ident::Alias_LaunchSubProcess()));
	# HL-578 Perform Directory Manipulation 
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryCreateDirectory(), $checkPattern->(qr/\bos\.(?:mkdir|makedirs)\b/, $code, CloudReady::Ident::Alias_DirectoryCreateDirectory()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryMove(), $checkPattern->(qr/\bos\.rename\b|\bshutil\.move\b/, $code, CloudReady::Ident::Alias_DirectoryMove()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryDelete(), $checkPattern->(qr/\bos\.(?:remove|removedirs|rmdir)\b|\bshutil\.rmtree\b/, $code, CloudReady::Ident::Alias_DirectoryDelete()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryExists(), $checkPattern->(qr/\bos\.(?:listdir|is_dir|path\.isdir|path\.dirname|path\.exists)\b/, $code, CloudReady::Ident::Alias_DirectoryExists()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryCurrent(), $checkPattern->(qr/\bos\.(?:getcwd|getcwdb|fchdir|chdir)\b/, $code, CloudReady::Ident::Alias_DirectoryCurrent()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryChange(), $checkPattern->(qr/\bos\.chdir\b/, $code, CloudReady::Ident::Alias_DirectoryChange()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryAccess(), $checkPattern->(qr/\bos\.(?:access|chmod|chown)\b/, $code, CloudReady::Ident::Alias_DirectoryAccess()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryCopy(), $checkPattern->(qr/\bshutil\.copytree\b/, $code, CloudReady::Ident::Alias_DirectoryCopy()));
    # HL-582 Perform File Manipulation Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Open(), $checkPattern->(qr/\bopen\s*\(/, $code, CloudReady::Ident::Alias_Open()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Write(), $checkPattern->(qr/\bwrite\s*\(|\bwin32file\.WriteFile\b/, $code, CloudReady::Ident::Alias_Write()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileCreate(), $checkPattern->(qr/\bos\.mknod\b|\bwin32file\.CreateFile\b/, $code, CloudReady::Ident::Alias_FileCreate()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileMove(), $checkPattern->(qr/\bos\.rename\b|\bshutil\.move\b/, $code, CloudReady::Ident::Alias_FileMove()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileDelete(), $checkPattern->(qr/\bos\.(?:remove|unlink)\b/, $code, CloudReady::Ident::Alias_FileDelete()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileCopy(), $checkPattern->(qr/\bshutil\.(?:copy(?:file|2|fileobj)?)\b/, $code, CloudReady::Ident::Alias_FileCopy()));
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_New_FileInputStream(), $checkPattern->(qr/\bimport\s+(?:fileinput|tempfile)\b/, $code, CloudReady::Ident::Alias_New_FileInputStream()));
    # HL-585 Access to environment variable from Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_EnvironmentVariable(), $checkPattern->(qr/\bos\.(?:environ|putenv|unsetenv|getenv)\b/, $code, CloudReady::Ident::Alias_EnvironmentVariable()));
	# HL-593 PHP & Python (HardCodedPath)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedPath(), CloudReady::lib::HardCodedPath::checkHardCodedPath($HString, $techno, $text, $rulesDescription));
    # HL-597 MySQL PHP & Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MySQL, checkDBMSMySQL($code));
    # HL-598 MongoDB PHP & Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MongoDB, checkMongoDB($code));
    # HL-599 NoSQL CosmosDB Azure Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_CosmosDB, checkCosmosDB($code));
    # HL-600 NoSQL DynamoDB AWS PHP & Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_DynamoDB, checkDynamoDB($text));
    # HL-601 NoSQL PostgreSQL PHP & Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_PostgreSQL, checkPostgreSQL($code));
    # HL-605 Azure encryption Python
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureCloudEncryption, checkAzureCloudEncryption($code));
    # HL-606 AWS encryption Python
#    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonCloudEncryption, checkAmazonCloudEncryption($code));
    #HL-608 BUG Fix HL-247 unsecured URL without XML namespace context
	#HL-870 21/05/2019 Split FTP/HTTP CloudReady pattern in 2 separate patterns
	CloudReady::lib::URINotSecured::checkURINotSecured($fichier, $code, $HString, $techno, $text);
    #HL-629 KMS service encryption Azure PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MicrosoftAzureKeyVault(), checkAzureKeyVault($code)); 
    #HL-630 KMS service encryption AWS PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Import_ComAmazonawsServicesKms(), $checkPattern->(qr/\bimport\s+boto3\b/, $code, CloudReady::Ident::Alias_Import_ComAmazonawsServicesKms()));
    #HL-634 Azure blob storage Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MicrosoftAzureBlobStorage(), checkAzureBlobStorage($code)); 
    #HL-636 AWS S3 storage PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsS3Storage(), checkAmazonawsS3Storage($text));
    #HL-640 Azure elasticache PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Import_InMemoryRedisAzure(), checkInMemoryRedisAzure($text));
    #HL-641 AWS elasticache PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Import_InMemoryRedisAws(), checkInMemoryRedisAws($text));
    #HL-646 Azure Active Directory access permission PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureDirectoryService(), checkAzureDirectoryService($code)); 
    #HL-647 AWS Active Directory access permission PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsDirectoryService(), checkAwsDirectoryService($text));
    #HL-651 Detect Usage of AWS Batch PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsBatch(), checkAmazonawsBatch($text));
    #HL-652 Detect Usage of Azure Batch PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureBatch(), checkAzureBatch($code)); 
    #HL-654 Azure service bus messaging PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureServiceBusMessaging(), checkAzureServiceBusMessaging($code)); 
    #HL-658 Azure Cloud Identity and Access Management
#	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureAccessManagement(), checkAzureAccessManagement($code)); 
    #HL-659 AWS Cloud Identity and Access Management PHP & Python
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonAwsAccessManagement(), checkAmazonAwsAccessManagement($text));
	#HL-952 17/09/2019 [GCP Boosters] Using Kubernetes
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingKubernetes(), $checkPattern->(qr/\b(?:import|from)\s+kubernetes\b/, $code, CloudReady::Ident::Alias_UsingKubernetes()));
	# HL-953 07/10/2019 [GCP Boosters] Using BigQuery
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigQuery(), $checkPattern->(qr/\b(?:import|from)\s+bigquery\b|\bbigquery\.Client\b/, $code, CloudReady::Ident::Alias_UsingBigQuery()));
	# HL-957 10/10/2019 [GCP Boosters] Using a Cloud-Based Storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudDataStore(), $checkPattern->(qr/\bfrom\s+google\.cloud\s+import\s+datastore\b|\bdatastore\.Client\b/, $code, CloudReady::Ident::Alias_UsingCloudDataStore()));
	# HL-958 11/10/2019 [GCP Boosters] Using BigTable
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigTable(), $checkPattern->(qr/\bfrom\s+google\.cloud\s+import\s+bigtable\b|\bbigtable\.Client\b/, $code, CloudReady::Ident::Alias_UsingBigTable()));
	# HL-959 15/10/2019 [GCP Boosters] Using Cloud Spanner
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudSpanner(), $checkPattern->(qr/\bfrom\s+google\.cloud\s+import\s+spanner\b|\bspanner\.Client\b/, $code, CloudReady::Ident::Alias_UsingCloudSpanner()));
	# HL-960 16/10/2019 [GCP Boosters] Using Cloud in-memory database (redis)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_InMemoryRedisGCP(), $checkPattern->(qr/\bimport\s+redis\b|\bredis\.StrictRedis\b/, $code, CloudReady::Ident::Alias_Using_InMemoryRedisGCP()));
	# HL-961 17/10/2019 [GCP Boosters] Using Cloud IAM (Identity and Access Management)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudIAM(), checkPatternInCodeAndString($text, '\bgoogleapiclient\.discovery\.build\s*\(\s*', '(?i)iam', CloudReady::Ident::Alias_UsingCloudIAM()));
	# HL-962 18/10/2019 [GCP Boosters] Using Firebase
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingFireBase(), $checkPattern->(qr/\bfrom\s+firebase(?:_admin)?\s+import\s+\b(?:auth|firebase)\b/, $code, CloudReady::Ident::Alias_UsingFireBase()));
	# HL-963 28/10/2019 [GCP Boosters] Using Cloud IAP (Identity Aware Proxy)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudIAP(), $checkPattern->(qr/\bdef\s+make_iap_request\b/, $code, CloudReady::Ident::Alias_UsingCloudIAP()));
	# HL-964 28/10/2019 [GCP Boosters] Using a Cloud-based Key storage (KMS)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPServicesKms(), checkPatternInCodeAndString($text, '\bgoogleapiclient\.discovery\.build\s*\(\s*', 'cloudkms', CloudReady::Ident::Alias_GCPServicesKms()));
	# HL-965 28/10/2019 [GCP Boosters] Using a Cloud-Based Storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPStorage(), $checkPattern->(qr/\bfrom\s+google\.cloud\s+import\s+storage\b|\bstorage\.Client\b/, $code, CloudReady::Ident::Alias_GCPStorage()));
	# HL-966 29/10/2019 [GCP Boosters] Using a Cloud-based task scheduling service
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPScheduler(), checkPatternInCodeAndString($text, '\bdiscovery\.build\s*\(\s*', 'cloudscheduler', CloudReady::Ident::Alias_GCPScheduler()));
	# HL-967 29/10/2019 [GCP Boosters] Using a Cloud-based Stream and Batch data processing
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudDataflow(),  $checkPattern->(qr/\bimport\s+apache_beam\b/, $code, CloudReady::Ident::Alias_UsingCloudDataflow()));
	# HL-968 30/10/2019 [GCP Boosters] Using a Cloud-based middleware application
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudPubSub(),  $checkPattern->(qr/\bfrom\s+google\.cloud\s+import\s+pubsub_v[0-9]+\b/, $code, CloudReady::Ident::Alias_UsingCloudPubSub()));
    # Use_Unsecured_Data_Strings
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SensitiveDataString(), CloudReady::lib::SensitiveDataString::CountSensitiveDataString($code, $HString, $techno));
    # Use_SecuredProtocolsHTTP
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HTTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURL($fichier, $code, $HString, $techno, $text));
    # Use_SecuredProtocolsFTP
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'FTP', $text));
    # Use_LDAPProtocol
    CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LDAPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'LDAP', $text));

    my $parameters = AnalyseUtil::recuperer_options();
    my $dbgMatchPatternDetail = $parameters->[0]->{'--dbgMatchPatternDetail'};
    if (defined $dbgMatchPatternDetail) {
        # replacing SHA by filenames in log
        CommonOutput::ElaborateLog::ElaborateLogPartTwo($fichier);
    }

	return $ret;
}

1;
