package CloudReady::CountRuby;

use strict;
use warnings;

use Erreurs;
use CloudReady::detection;
use CloudReady::config;
use CloudReady::Ident;
use CloudReady::lib::HardCodedIP;
use CloudReady::lib::HardCodedPath;
use CloudReady::lib::HardCodedURL;
use CloudReady::lib::URINotSecured;
use CloudReady::lib::SensitiveDataString;

use constant BEGINNING_MATCH => 0;
use constant FULL_MATCH => 1;

my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

sub checkPatternSensitive($$$) {
	my $reg = shift;
	my $code = shift;
	my $mnemo = shift;

	return CommonOutput::ElaborateLog::ElaborateLogPartOne($code, $reg, $mnemo, $rulesDescription);
	#return () = ($$code =~ /$reg/gm);
}

sub CountRuby($$$$) {
	my ($fichier , $vue , $compteurs , $techno) = @_;
	my $ret = 0;

	my $code;
	my $HString;
	my $MixBloc;
	my $text;
	$text = \$vue->{'text'};
	my $checkPattern = \&checkPatternSensitive;

	$code = \$vue->{'code'};
	$HString = $vue->{'HString'};
	$MixBloc = \$vue->{'MixBloc'};

	if ((!defined $code) || (!defined $$code)) {
		#$ret |= Couples::counter_add($compteurs, $MissingJSPComment__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	# BLOCKERS
	# HL-2384 21/04/2023 Detect the usage of local files manipulation
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileCreate(), $checkPattern->(qr/\bFile\.new\b/, $code, CloudReady::Ident::Alias_FileCreate()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Open(), $checkPattern->(qr/\bFile\.open\b/, $code, CloudReady::Ident::Alias_Open()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileCopy(), $checkPattern->(qr/\bFileUtils\.cp\b/, $code, CloudReady::Ident::Alias_FileCopy()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileMove(), $checkPattern->(qr/\bFileUtils\.mv\b/, $code, CloudReady::Ident::Alias_FileMove()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileDelete(), $checkPattern->(qr/\bFile\.delete\b|\bFileUtils\.rm\b/, $code, CloudReady::Ident::Alias_FileDelete()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FileRead(), $checkPattern->(qr/\.sysread\s*\(|\.readlines\b/, $code, CloudReady::Ident::Alias_FileRead()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Write(), $checkPattern->(qr/\.syswrite\b/, $code, CloudReady::Ident::Alias_Write()));
	# HL-2385 21/04/2023 Detect the usage of local directory manipulation
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryOpen(), $checkPattern->(qr/\bDir\.(?:entries|glob)\b|\bFileUtils\.ln(?:\_(?:sf?))?\b/, $code, CloudReady::Ident::Alias_DirectoryOpen()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryDelete(), $checkPattern->(qr/\bFileUtils\.(?:rm_rf?|rmdir|rmtree)\b/, $code, CloudReady::Ident::Alias_DirectoryDelete()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryExists(), $checkPattern->(qr/\bDir\.exists\?/, $code, CloudReady::Ident::Alias_DirectoryExists()));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DirectoryCreate(), $checkPattern->(qr/\bDir\.mkdir\b|\bFileUtils\.mkdir(?:\_p)?\b/, $code, CloudReady::Ident::Alias_DirectoryCreate()));
	# HL-2386 21/04/2023 Detect the usage of environment variables
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_EnvironmentVariable(), $checkPattern->(qr/\bENV[.\[]["']?/, $code, CloudReady::Ident::Alias_EnvironmentVariable()));
	# HL-2387 02/05/2023 Detect the usage of hardcoded path
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedPath(), CloudReady::lib::HardCodedPath::checkHardCodedPath($HString, $techno, $text, $rulesDescription));
	# HL-2388 02/05/2023 Detect the usage of hardcoded IP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HardCodedIP(), CloudReady::lib::HardCodedIP::checkHardCodedIP($HString, $code, $techno, $text));
	# HL-2389 02/05/2023 Detect the usage of hardcoded URL
	# Use_SecuredProtocolsHTTP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_HTTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURL($fichier, $code, $HString, $techno, $text));
	# Use_SecuredProtocolsFTP
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_FTPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'FTP', $text));
	# Use_LDAPProtocol
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LDAPProtocol(), CloudReady::lib::HardCodedURL::checkHardCodedURLProtocol($fichier, $code, $HString, 'LDAP', $text));
	# HL-2390 02/05/2023 Detect the usage of Access Control List
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_New_AccessControlList(), $checkPattern->(qr/\bACL(?:\.new\b|\:\:)/, $code, CloudReady::Ident::Alias_New_AccessControlList()));
	# HL-2391 02/05/2023 Detect the usage of system calls or processes management
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_LaunchSubProcess(), $checkPattern->(qr/[`].+?[`]|(?:\%x|\bexec|\bspawn|\bsystem)\b|\b(?:capture|popen)[23]?[e]?\b/, $code, CloudReady::Ident::Alias_LaunchSubProcess()));
	# HL-2392 03/05/2023 Detect the usage of unsecured URI
	CloudReady::lib::URINotSecured::checkURINotSecured($fichier, $code, $HString, $techno, $text);
	# HL-2393 03/05/2023 Detect the usage of dll files system
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DllImport, $checkPattern->(qr /\bWin32API\.new\b/, $code, CloudReady::Ident::Alias_DllImport));
	# HL-2394 03/05/2023 Detect the usage of a local temporary file
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GetTempPath, $checkPattern->(qr /\bTempfile\.(?:create|new)\b/, $code, CloudReady::Ident::Alias_GetTempPath));
	# HL-2395 03/05/2023 Detect the usage of an unsecured data string
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_SensitiveDataString(), CloudReady::lib::SensitiveDataString::CountSensitiveDataString($code, $HString, $techno));

	# BOOSTERS
	# HL-2369 04/05/2023 Detect the usage of a cloud-based database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_AmazonRedshift(), $checkPattern->(qr /\bAws\:\:Redshift\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_Using_AmazonRedshift));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsAthena(), $checkPattern->(qr /\bAws\:\:Athena\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_AwsAthena));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigQuery(), $checkPattern->(qr /\bGoogle\:\:Cloud\:\:Bigquery\.new\b/, $code, CloudReady::Ident::Alias_UsingBigQuery));
	# CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPCloudSQL(), checkImportAndCode($code, $HString, qr /\bcloudsqlconn["'\/\\]/, qr /\bcloudsqlconn\.NewDialer\b/, $text, CloudReady::Ident::Alias_GCPCloudSQL()));
	# HL-2370 05/05/2023 Detect the usage of a cloud-based storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsS3Storage(), $checkPattern->(qr /\bAws\:\:S3\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_AmazonawsS3Storage));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MicrosoftAzureBlobStorage(), $checkPattern->(qr /\bAzure\:\:Storage\:\:Blob\:\:BlobService\.(?:create|new)\b/, $code, CloudReady::Ident::Alias_MicrosoftAzureBlobStorage));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPStorage(), $checkPattern->(qr /\bGoogle\:\:Cloud\:\:Storage\.new\b/, $code, CloudReady::Ident::Alias_GCPStorage));
	# HL-2371 05/05/2023 Detect the usage of a cloud-based search engine
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Elasticsearch(), $checkPattern->(qr /\bElasticsearch\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_Elasticsearch));
	# HL-2372 05/05/2023 Detect the usage of a cloud-based function as a service (serverless)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsLambda(), $checkPattern->(qr /\bAws\:\:Lambda\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_AwsLambda));
	# CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureFunctions(), $checkPattern->(qr/\bimport\s+com\.microsoft\.azure\.functions\b.*/, $code));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPFunctions(), $checkPattern->(qr /\brequire\s+["']functions_framework["']|\bFunctionsFramework\./, $text, CloudReady::Ident::Alias_GCPFunctions));
	# HL-2373 05/05/2023 Detect the usage of a cloud-based NoSQL database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_DynamoDB(), $checkPattern->(qr /\bAws\:\:DynamoDB\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_DBMS_DynamoDB));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MongoDB(), $checkPattern->(qr /\bMongo\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_DBMS_MongoDB));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_CosmosDB(), $checkPattern->(qr /\bDBI\.connect\s*\(["'].*?\bCosmosDB\b.*?["']/, $text, CloudReady::Ident::Alias_DBMS_CosmosDB));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingFireBase(), $checkPattern->(qr /\bGoogle\:\:Cloud\:\:Firestore\.new\b|\bFirebase\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_UsingFireBase));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingBigTable(), $checkPattern->(qr /\bGoogle\:\:Cloud\:\:Bigtable\.new\b/, $code, CloudReady::Ident::Alias_UsingBigTable));
	# HL-2374 05/05/2023 Detect the usage of cloud-based relational database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_PostgreSQL(), $checkPattern->(qr /\brequire\s+["']pg["']|\bPG\.connect\b/, $text, CloudReady::Ident::Alias_DBMS_PostgreSQL));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_MySQL(), $checkPattern->(qr /\brequire\s+["']mysql["']|\bMysql\.connect\b/, $text, CloudReady::Ident::Alias_DBMS_MySQL));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_Oracle(), $checkPattern->(qr /\bDBI\.connect\s*\(["'].*?\bOracleOCI\b.*?["']/, $text, CloudReady::Ident::Alias_DBMS_Oracle));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_DBMS_SQLServer(), $checkPattern->(qr /\bTinyTds\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_DBMS_SQLServer));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_AmazonRDS(), $checkPattern->(qr /\bAws\:\:RDS\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_Using_AmazonRDS));
	# HL-2375 09/05/2023 Detect the usage of a cloud-based Identity and Access Management (IAM)
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsCloudIAM(), $checkPattern->(qr /\bAws\:\:IAM\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_AwsCloudIAM));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureDirectoryService(), $checkPattern->(qr /\brequire\s+["']adal["']|\bADAL\:\:(?:ClientCredential|UserCredential)\.new\b|\bADAL\:\:Logger\b/, $text, CloudReady::Ident::Alias_AzureDirectoryService));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudIAM(), $checkPattern->(qr /\brequire\s+["']google\/apis\/iamcredentials.*?["']|\bGoogle\:\:Apis\:\:Iamcredentials/, $text, CloudReady::Ident::Alias_UsingCloudIAM));
	# HL-2376 09/05/2023 Detect the usage of a cloud-based key service encryption
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AmazonawsServicesKms(), $checkPattern->(qr /\bAws\:\:KMS\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_AmazonawsServicesKms));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_MicrosoftAzureKeyVault(), $checkPattern->(qr /\bAzure\:\:ARM\:\:KeyVault\:\:KeyVaultManagementClient\.new\b/, $code, CloudReady::Ident::Alias_MicrosoftAzureKeyVault));
	# HL-2377 09/05/2023 Detect the usage of a cloud-based container service
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsElasticContainer(), $checkPattern->(qr /\bAws\:\:EC2\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_AwsElasticContainer));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Docker(), $checkPattern->(qr /\brequire\s+["']docker["']|\bDocker\:\:(?:Container|Image)\.create\b/, $text, CloudReady::Ident::Alias_Docker));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Kubernetes(), $checkPattern->(qr /\bK8s\.client\b|\bK8s\:\:Client\.config\b/, $code, CloudReady::Ident::Alias_Kubernetes));
	# HL-2378 09/05/2023 Detect the usage of a cloud-based middleware application
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_AmazonSQS(), $checkPattern->(qr /\bAws\:\:SQS\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_Using_AmazonSQS));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingMicrosoftServiceBusMessaging(), $checkPattern->(qr /\bAzure\:\:ServiceBusService\.new\b/, $code, CloudReady::Ident::Alias_UsingMicrosoftServiceBusMessaging));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_UsingCloudPubSub(), $checkPattern->(qr /\brequire\s+["']google\/cloud\/pubsub["']|\bGoogle\:\:Cloud\:\:PubSub\.new\b/, $text, CloudReady::Ident::Alias_UsingCloudPubSub));
	# HL-2379 09/05/2023 Detect the usage of a cloud-based Big Data technology
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AwsEMRBigData(), $checkPattern->(qr /\bAws\:\:EMR\:\:Client\.new\b/, $code, CloudReady::Ident::Alias_AwsEMRBigData));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_AzureHDInsights(), $checkPattern->(qr /\brequire\s+["']application_insights["']|\bApplicationInsights\:\:TelemetryClient\.new\b/, $text, CloudReady::Ident::Alias_AzureHDInsights));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_GCPDataproc(), $checkPattern->(qr /\bGoogle\:\:Cloud\:\:Dataproc\b/, $code, CloudReady::Ident::Alias_GCPDataproc));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Hadoop(), $checkPattern->(qr /\brequire\s+["'](?:hbase|webhdfs)["']|\b(?:HBase|WebHDFS)\:\:Client\.new\b/, $text, CloudReady::Ident::Alias_Hadoop));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Spark(), $checkPattern->(qr /\brequire\s+["']ruby\-spark["']|\bSpark\.start\b/, $text, CloudReady::Ident::Alias_Spark));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Kafka(), $checkPattern->(qr /\brequire\s+["']kafka["']|\bKafka\.new\b/, $text, CloudReady::Ident::Alias_Kafka));
	# HL-2380 10/05/2023 Detect the usage of JSON to facilitate interactions with cloud services
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_JSONParsing(), $checkPattern->(qr /\brequire\s+["']json["']|\bJSON\.(?:parse|generate)\b/, $text, CloudReady::Ident::Alias_JSONParsing));
	# HL-2381 10/05/2023 Detect the usage of XML to facilitate interactions with cloud services
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_XMLParsing(), $checkPattern->(qr/\brequire\s+["']nokogiri["']|\bNokogiri\:\:XML\b/, $text, CloudReady::Ident::Alias_XMLParsing));
	# HL-2382 10/05/2023 Detect the usage of a cloud-based blockchain technology
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Blockchain(), $checkPattern->(qr/\brequire\s+["']hyperledger\-fabric\-sdk["']|\b(?:FabricCA\.client|Fabric\:\:Identity\.new|Ethereum\:\:Contract\.create)\b/, $text, CloudReady::Ident::Alias_Blockchain));
	# HL-2383 10/05/2023 Detect the usage of in-memory database storage
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_InMemoryRedis(), $checkPattern->(qr/\brequire\s+["']redis["']|\bRedis\.new\b/, $text, CloudReady::Ident::Alias_InMemoryRedis));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_Using_InMemoryRedisGCP(), $checkPattern->(qr/\brequire\s+["']google\/cloud\/redis\b|\bCloudRedis\:\:Client\.new\b/, $text, CloudReady::Ident::Alias_Using_InMemoryRedisGCP));
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_InMemoryMemcached(), $checkPattern->(qr/\brequire\s+["']memcached["']|\bMemcached\:\:Client\.new\b/, $text, CloudReady::Ident::Alias_InMemoryMemcached));

	my $parameters = AnalyseUtil::recuperer_options();
	my $dbgMatchPatternDetail = $parameters->[0]->{'--dbgMatchPatternDetail'};
	if (defined $dbgMatchPatternDetail) {
		# replacing SHA by filenames in log
		CommonOutput::ElaborateLog::ElaborateLogPartTwo($fichier);
	}

	return $ret;
}

1;
