package CloudReady::lib::HardCodedPath;

use strict;
use warnings;
use CommonOutput::ElaborateLog;
use Lib::SHA;
use AnalyseUtil;

sub checkHardCodedPath($$$$;$;$) {
	my $HString = shift;
	my $techno = shift;
	my $text = shift;
	my $rulesDescription = shift;
	my $fileConfig = shift;
	my $category = shift;

	my $nb_hardcoded_path = 0;
	my $regexpr_path;

	# LIST EXCEPTIONS
	my $exceptionPatterns = undef;
	if (defined $techno && $techno eq "CS") {
		push(@{$exceptionPatterns}, qr/\bnew\s+Regex\b/);
	}


	# searching absolute path not relative

	# Path format supported
	# WINDOWS
	#   c:/Users/jm/AppData/bar-foo.json
	#   Z:\Documents and Settings\frenzi\
	#   \\Server2\Share\Test\Foo.txt
	# ---
	# LINUX
	#   ref: http://www.linux-france.org/article/sys/fichiers/fichiers-2.html
	#   /bin | /boot | /dev | /etc | /home | /lib | /lib64 | /mnt | /opt | /proc | /root | /run | /bin | /srv | /sys | /tmp | /usr | /var
	#   /home/olivier_test/scripts
	#   /var/mysql/mysql.db

	$regexpr_path = qr/	# Windows OS
		((\b[a-z]\:|[\\])([\\\/]+[\w \.\-]+)+
		# Linux OS
		|[\/]
		(bin|boot|dev|etc|home|lib|lib64|mnt|opt|proc|root|run|bin|srv|sys|tmp|usr|var)\b
		# path ending for both OS
		([\\\/]+[\w \.\-]+)*)[\\\/]*
	/xim;

	my %stringsToSearch;

	for my $stringId (keys %$HString) {
		my $string = $HString->{$stringId};
		if (defined $string) {
			$string =~ s/^["']//m;
			$string =~ s/["']$//m;
			# # managing file path format prefix : file:///
			# $string =~ s/file\:[\/\\]+(?=[a-z]\:)//i; # windows format
			# $string =~ s/file\:[\/\\]+/\//i;          # linux format

			# 1RST FILTER
			# /!\ exclude url and pom.xml files
			if ($string =~ /^(.*)($regexpr_path)/ && ($string !~ /\b(?:http\:|www\.|pom\.xml$)/m)) {
				my $previousPattern = $1;
				my $path = $2;

				# 2ND FILTER
				if (defined $path && $path =~ /[a-z]{3,}/i && $path !~ /[*]/) {
					my $result = 0;
					if (defined $1) {
						# 3RD FILTER
						$result = checkExceptionInPreviousPattern($previousPattern, $exceptionPatterns);
					}

					if ($result == 0) {
						$stringsToSearch{$string} = 1;
					}
				}
			}
		}
	}

	if (scalar %stringsToSearch > 0) {
		my $mnemoId;
		if (!defined $category) {
			$mnemoId = CloudReady::Ident::Alias_HardCodedPath;
		}
		# $category = GREEN
		else {
			$mnemoId = Ident::Alias_HardCodedPathNotGreen;
		}

		$nb_hardcoded_path += CommonOutput::ElaborateLog::ElaborateLogPartOneFromStringList(\$text, \%stringsToSearch, $mnemoId, $rulesDescription, $fileConfig);
	}

	#print "nb_hardcoded_path=$nb_hardcoded_path\n";
	return $nb_hardcoded_path;
}

sub checkExceptionInPreviousPattern($$) {
	my $previousPattern = shift;
	my $exceptionPatterns = shift;

		foreach my $exceptionPattern (@$exceptionPatterns) {
			if ($previousPattern =~ /$exceptionPattern/m) {
				return 1;
			}
		}

	return 0;
}

1;
