package CloudReady::lib::URINotSecured;

use strict;
use warnings;
use CloudReady::detection;
use CloudReady::lib::HardCodedURL;

# exceptions for all technos
my $domain_exclusion = qr/\b(?:www\.w3\.org)\b/;
my $exceptionPatternsInURL = [ qr/\b(?:xml)?ns\b/i ];
my $exceptionPatterns = [ qr/(?:\bQName\b|\b\w*namespaces?\b|\b(?:xml)?ns\b)/i];

sub checkURINotSecured($$$$$) {
	my $fichier = shift;
	my $code = shift;
	my $HString = shift;
	my $techno = shift;
	my $text = shift;
	# my $nb_hardcoded_URI = 0;
	my $nb_hardcoded_URI_HTTP = 0;
	my $nb_hardcoded_URI_FTP = 0;
	my $regexpr_uri;

	my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

	if (defined $techno) {
		# print "TECHNO= $techno\n";
		# add exceptions
		if ($techno eq "CS") {
			push(@$exceptionPatterns, qr/\b(?:Action|ReplyAction)\b\s*\=\s*/i);
		}
		elsif ($techno eq "Java") {
			push(@$exceptionPatterns, qr/\blogger\./i);
			# ticket 46707 false positive
			push(@$exceptionPatterns, qr/\.(?:s|g)etFeature/);
		}

		# add domain exclusion
		if ($techno eq "VB") {
			$regexpr_uri = qr/^((http|ftp)\:(\/\/))(?!$domain_exclusion)[^"']/im;
		}
		# domain exclusion for all technos
		else {
			$regexpr_uri = qr/^["']((http|ftp)\:(\/\/))(?!$domain_exclusion)[^"']/im;
		}

		for my $stringId (keys %$HString) {
			if ($HString->{$stringId} =~ /$regexpr_uri/) {
				my $prefixURI = $2;
				($nb_hardcoded_URI_HTTP, $nb_hardcoded_URI_FTP) = checkContextDetectionURI($code, $text, $exceptionPatterns, $HString, $stringId, $prefixURI, $nb_hardcoded_URI_HTTP, $nb_hardcoded_URI_FTP, $rulesDescription);
			}
		}
	}

	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_URINotSecuredHTTP(), $nb_hardcoded_URI_HTTP);
	CloudReady::detection::addFileDetection($fichier, CloudReady::Ident::Alias_URINotSecuredFTP(), $nb_hardcoded_URI_FTP);

	# Counter not necessary for analyzer, mnemonic is splitted in 2 patterns HTTP & FTP
	# return $nb_hardcoded_URI;
}

sub checkContextDetectionURI ($$$$$$$$$) {
	my $code = shift;
	my $text = shift;
	my $exceptionPatterns = shift;
	my $HString = shift;
	my $stringId = shift;
	my $prefixURI = shift;
	my $nb_hardcoded_URI_HTTP = shift;
	my $nb_hardcoded_URI_FTP = shift;
	my $rulesDescription = shift;

	my $string = quotemeta($HString->{$stringId});
	my $regex = qr/$string/;

	# check the context of detection for exceptions
	# "context" matches with the line where opening parenthesis or opening bracket begins
	# and if exists the contents of the parenthesis or brackets
	# if context contains more than 2 lines, get only the last two lines
	# for instance:
		# in following instructions, context is delimited with []
			# [  exception_pattern ( exception_pattern   ] "http|ftp://...  "
			# [  exception_pattern { exception_pattern   ] "http|ftp://...  "
	while ($$code =~ /([^\n]*[^({]*)\b$stringId\b/sg) {
		if (defined $1) {
			my $previousPattern = $1;
			# threshold of previous pattern based on number of lines
			my @x = split(/\n/, $previousPattern);
			# get only last two lines
			if (defined $x[-2] && defined $x[-1]) {
				$previousPattern = $x[-2]."\n".$x[-1];
			}
			elsif (defined $x[-1]) {
				$previousPattern = $x[-1];
			}
			my $result = checkExceptionsUri($previousPattern, $exceptionPatterns);
			if ($result == 0) {
				if (defined($prefixURI) and lc($prefixURI) eq "http") {
					my $res = CloudReady::lib::HardCodedURL::checkExceptionsURL($code, $stringId, $HString, $exceptionPatternsInURL);
					if ($res == 0) {
						$nb_hardcoded_URI_HTTP++;
						# print "URI not secured $HString->{$stringId} \n";
						# display Alias_URINotSecured in log
						CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, CloudReady::Ident::Alias_URINotSecured, $rulesDescription);
					}
				}
				else {
					$nb_hardcoded_URI_FTP++;
					# display Alias_URINotSecured in log
					CommonOutput::ElaborateLog::ElaborateLogPartOne($text, $regex, CloudReady::Ident::Alias_URINotSecured, $rulesDescription);
				}
				# print 'detection of unsecured URI ' . $HString->{$stringId} ."\n";
			}
		}
	}
	return ($nb_hardcoded_URI_HTTP, $nb_hardcoded_URI_FTP);
}

sub checkExceptionsUri($$) {
	my $previousPattern = shift;
	my $exceptionPatterns = shift;

	foreach my $exceptionPattern (@$exceptionPatterns) {
		if ($previousPattern =~ /$exceptionPattern/m) {
			return 1;
		}
	}

	return 0;
}

1;
