package CloudReady::projectFiles;

use strict;
use warnings;
use Lib::XML;
use Lib::SHA;

use CloudReady::Ident;
use CloudReady::detection;
use CloudReady::lib::KeepPortalData;

my $rulesDescription = CloudReady::lib::KeepPortalData::keepPortalData();

my $MASTER_TECHNO = undef;
my %boolConfigXML;

my $MINIMAL_JAVA_VERSION = 1.8; # cloud blocker: Use of deprecated framework => JAVA under 1.8 is unsupported
my $MINIMAL_NET_VERSION = 4.7; # .NET under 4.7 is unsupported

my @deprecated_Tfm = (
	'dotnet', 'dotnet50', 'dotnet51', 'dotnet52' , 'dotnet53', 'dotnet54', 'dotnet55', 'dotnet56',
	'aspnet50', 'aspnetcore50', 'dnxcore50', 'dnx', 'dnx45', 'dnx451', 'dnx452',
	# .NET frameworks
	'net11', 'net20', 'net35', 'net40', 'net403', 'net45', 'net451', 'net452', 'net46', 'net461', 'net462',
	# .NET core
	'netcore45', 'netcore451', 'netcore50',
	# .NET core app
	'netcoreapp1.0', 'netcoreapp1.1', 'netcoreapp2.0', 'netcoreapp2.1', 'netcoreapp2.2'
);

sub setMasterTechno($) {
	$MASTER_TECHNO = shift;
	# if $techno is initialized with the name of the analyseur, retrieves the name of the techno by removing the suffix "Ana" !! 
	$MASTER_TECHNO =~ s/\AAna//;
}

sub checkWebConfig($$) {
	my $webconf = shift;
	my $webconfContent = shift;
	# print "CloudReady : checking web.config\n";

	# Loading xml content
	# my $webconfContent = Lib::XML::load_xml($webconf);
	# my $webconfContentWithNoComment = $webconfContent;

	return if (! defined $webconfContent);

	# WebConfContainsConnectionStrings rule inactivated 04/03/2023
	# # # .NET 3.5 (and above)
	# #if (defined $webconfContent->first_elt('connectionStrings')) {
	# if (scalar $webconfContent->getElementsByLocalName('connectionStrings')->get_nodelist() ) {
	#     # print "web.config contains connectionStrings !!\n";
	# 	CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfContainsConnectionStrings, []);
	# }

	#
	# # 15/02/18 HL-365 Connection String defined inside project files
	# # .Net 2.0
	#if (scalar $webconfContentWithNoComment->findnodes("//appSettings") ) {
	# if (scalar $webconfContent->getElementsByLocalName('appSettings')->get_nodelist() ) {
	#
	#     my $cpt_SQLDatabaseUnsecureAuthentication = 0;
	#     my $regexp_ConnectionString = qr/
	#     \b(?:
	#     Persist\s+Security\s+Info\s*\=
	#     | Integrated\s+Security\s*\=
	#     | Data\s+Source\s*\=
	#     | User\s+ID\s*\=
	#     | uid\s*\=
	#     | Password\s*\=
	#     | Initial\s+Catalog\s*\=
	#     | Provider\s*\=
	#     )\b/xi;
	#
	#     #foreach ($webconfContentWithNoComment->findnodes('//appSettings/add[@value =~ /'.$regexp_ConnectionString.'/]'))
	#     foreach (Lib::XML::findnodes_AttrMatch($webconfContent, q{//appSettings/add}, "value", $regexp_ConnectionString) )
	#     {
	#         # get the node content
	#         my $ConnectionString = $_->toString;
	#
	#         my $count_StringPattern = 0;
	#         # my $bool_MicrosoftAuthenticate;
	#
	#         while ($ConnectionString =~ /$regexp_ConnectionString/g)
	#         {
	# 			$count_StringPattern++;
	#
	#             ###############
	#             # TO BE FIXED:#
	# 			###############
	# 			# 19/02/18 HL-474 Microsoft authentication
	#             # my $regexp_MicrosoftAuthenticate = qr/\b(?:Integrated\s+Security | Trusted_Connection\s*\=)\b/xi;
	#
	#             # if ($ConnectionString =~ /$regexp_MicrosoftAuthenticate/)
	#             # {
	#                 # $bool_MicrosoftAuthenticate = 1;
	#
	#                 #if ($ConnectionString =~ /\bIntegrated\s+Security\s*\=\s*(?!true|yes|SSPI)|\bTrusted_Connection\s*\=\s*(?!true|yes)/i)
	#                 #{
	#                 #    # print '++++ Violation detection Microsoft authentication in string '."\n";
	#                 #    $cpt_SQLDatabaseUnsecureAuthentication++;
	#                 #}
	#             # }
	#         }
	#
	#         if ($count_StringPattern >= 3)
	#         {
	#             # print '++++ Connection string detected in web.config file'."\n";
	#             CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfContainsConnectionStrings, []);
	#
	#             #if ($cpt_SQLDatabaseUnsecureAuthentication > 0)
	#             #{
	#             #    # print '++++ Violation detection Microsoft authentication in web.config file'."\n";
	#             #    CloudReady::detection::addFileDetection($webconf, CloudReady::Ident::Alias_SQLDatabaseUnsecureAuthentication(), 1);
	#             #
	#             #}
	#         }
	#     }
	# }


	# count elsewhere in the document all "authentication" nodes :
	#	* having a parent "system.web"
	#	* having an attribute "mode" whose value is "Forms"
	#if (scalar $webconfContent->findnodes('//system.web/authentication[@mode="Forms"]') ) {
	if (scalar Lib::XML::findnodes($webconfContent, '//system.web/authentication[@mode="Forms"]')) {
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfigAuthenticationForm, []);
	}

	# count elsewhere in the document all "authentication" nodes :
	#	* having a parent "system.web"
	#	* having an attribute "mode" whose value is "Forms"
	#if (scalar $webconfContent->findnodes('//system.web/authentication[@mode="Windows"]') ) {
	if (scalar Lib::XML::findnodes($webconfContent, '//system.web/authentication[@mode="Windows"]') ) {
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfigAuthenticationWindows, []);
	}

	# count elsewhere in the document all "add" nodes :
	#	* having a parent "appSettings"
	#	* having an attribute "key" whose value matches /^ida:.*/
	#if (scalar $webconfContent->findnodes('//appSettings/add[@key =~ /^ida:.*/]') ) {
	if (scalar Lib::XML::findnodes_AttrMatch($webconfContent, '//appSettings/add', "key", qr/^ida:.*/) ) {
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfigActiveDirectory, []);
	}

	# count elsewhere in the document all "add" nodes :
	#	* having a parent "connectionStrings"
	#	* having an attribute "name" whose value matches /^AzureWebJobs.*/
	#if (scalar $webconfContent->findnodes('//connectionStrings/add[@name =~ /AzureWebJobs\w*/]') ) {
	if (scalar Lib::XML::findnodes_AttrMatch($webconfContent, '//connectionStrings/add', "name", qr/AzureWebJobs\w*/) ) {
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfigAzureWebJobs, []);
	}

	# count elsewhere in the document all "identity" nodes :
	#	* having an attribute "impersonate" to "true"
	#if (scalar $webconfContent->findnodes('//identity[@impersonate="true"]') ) {
	if (scalar Lib::XML::findnodes($webconfContent, '//identity[@impersonate="true"]') ) {
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfigIdentityImpersonateTrue, []);
	}
}

# Common sub for web & app.config files
sub checkWebAppConfig($$)
{
	my $fileNameConf = shift;
	my $WebAppContent = shift;

	# Loading xml content
	return if (! defined $WebAppContent);

	# HL-940 28/06/2019 Detect usage of Log4Net
	# configSections
	if (scalar Lib::XML::findnodes_AttrMatch($WebAppContent, '//configSections/section', "name", qr/\blog4net\b/)
		or scalar Lib::XML::findnodes_AttrMatch($WebAppContent, '//configSections/section', "type", qr/\blog4net\b/)
	)
	{
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_Log4NetConfig, [$fileNameConf]);
	}
	# log4net
	elsif (scalar Lib::XML::findnodes($WebAppContent, '//log4net'))
	{
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_Log4NetConfig, [$fileNameConf]);
	}
	# HL-939 24/06/2019 Detect WCF services
	if (scalar Lib::XML::findnodes($WebAppContent, '//system.serviceModel'))
	{
		if ($fileNameConf =~ /\bweb\.config\b$/im) {
			# print "[Web.config] WCF service found \n";
			CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WCFServiceWebConf, []);
		}
		if ($fileNameConf =~ /\bapp\.config\b$/im) {
			CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WCFServiceAppConf, []);
		}
	}
}

sub checkCsVbProj($$$) {
	my $csVbproj = shift;
	my $buffText = shift;
	my $webconfContent = shift;

	return if (!defined $webconfContent);

	# C# / VB.Net
	# COMReference with "include" attribute
	if (scalar Lib::XML::findnodes($webconfContent, '//COMReference[@Include]')) {
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_CsprojCOMReferenceInclude, [ $csVbproj ]);
	}
	# HL-2802 Cloud Maturity - Detect deprecated Java and .NET versions
	# following TFM are assumed to be deprecated:
	# dotnet, dotnet50, dotnet51, dotnet52 , dotnet53, dotnet54, dotnet55, dotnet56
	# aspnet50, aspnetcore50, dnxcore50, dnx, dnx45, dnx451, dnx452
	# .NET frameworks < 4.7 (net11, net20, net35, net40, net403, net45, net451, net452, net46, net461, net462)
	# .NET core < 6.0 (netcore45, netcore451, netcore50)
	# Ticket #49432
	# .NET core app < 3.0 (netcoreapp1.0, netcoreapp1.1, netcoreapp2.0, netcoreapp2.1, netcoreapp2.2)
	my @tag = Lib::XML::findnodes($webconfContent, '//TargetFramework');
	push(@tag, Lib::XML::findnodes($webconfContent, '//TargetFrameworks'));

	if (scalar @tag) {
		my $valueTargetFrameworks = $tag[0]->textContent;
		for my $tfm (@deprecated_Tfm) {
			if ($valueTargetFrameworks =~ /$tfm/i) {
				# print "deprecated .Net framework found <$tfm>\n";
				CommonOutput::ElaborateLog::ElaborateLogPartOne(\$buffText, $valueTargetFrameworks, CloudReady::Ident::Alias_DeprecatedFw, $rulesDescription, "XML");
				CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_DeprecatedFw, [ $csVbproj ]);
				# last;
			}
		}
	}
	@tag = Lib::XML::findnodes($webconfContent, '//TargetFrameworkVersion');
	if (scalar @tag) {
		my $valueTargetFrameworkVersion = $tag[0]->textContent;
		# $valueTargetFrameworkVersion must be numeric
		if ($valueTargetFrameworkVersion =~ /v([0-9]+(?:\.[0-9]+)?)/) {
			if ($1 < $MINIMAL_NET_VERSION) {
				# print "deprecated version of .Net framework found <$1>\n";
				CommonOutput::ElaborateLog::ElaborateLogPartOne(\$buffText, $valueTargetFrameworkVersion, CloudReady::Ident::Alias_DeprecatedFw, $rulesDescription, "XML");
				CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_DeprecatedFw, [ $csVbproj ]);
			}
		}
	}

	# C# only
	# HL-962
	if ($MASTER_TECHNO eq "CS"
		&& scalar Lib::XML::findnodes($webconfContent, '//ItemGroup/PackageReference[@Include]')) {
		my $nodes = Lib::XML::findnodes($webconfContent, '//ItemGroup/PackageReference');
		for my $node (@{$nodes}) {
			if ($node->getAttribute("Include") eq "FirebaseAdmin") {
				CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_UsingFireBase, [ $csVbproj ]);
			}
		}
	}
	# replacing SHA by filenames in log
	CommonOutput::ElaborateLog::ElaborateLogPartTwo($csVbproj);
}

sub checkPomXmlDependencies($$$$$) {
	my $pom = shift;
	my $pomContent = shift;
	my $groupId = shift;
	my $artifactId = shift;
	my $aliasMnemo = shift;

	return if (!defined $pomContent);

	my @nodesDependencies = Lib::XML::findnodes($pomContent, '//dependencies/dependency');

	for my $node (@nodesDependencies) {
		my @artifactId = Lib::XML::findnodes($node, './artifactId');

		if ((scalar @artifactId) && ($artifactId[0]->textContent =~ /^${artifactId}$/mi)) {
			my @groupId = Lib::XML::findnodes($node, './groupId');
			if ((scalar @groupId) && ($groupId[0]->textContent eq $groupId)) {
				# print "PATTERN <$artifactId> found in pom.xml\n";
				CloudReady::detection::addAppliDetection(&$aliasMnemo, [ $pom ]);
				return;
			}
		}
	}
}

sub checkPomXmlProperties($$$$$) {
	my $pom = shift;
	my $buffText = shift;
	my $pomContent = shift;
	my $tagNameList = shift;
	my $aliasMnemo = shift;

	return if (!defined $pomContent);

	my @nodesProperties = Lib::XML::findnodes($pomContent, '//properties');
	my @tagNameList = split(/\|/, $tagNameList);

	for my $node (@nodesProperties) {
		for my $tagName (@tagNameList) {
			my @tag = Lib::XML::findnodes($node, "./$tagName");
			# HL-2802 Cloud Maturity - Detect deprecated Java and .NET versions
			if ($MASTER_TECHNO eq "Java") {
				if (scalar @tag) {
					my $valueProperty = $tag[0]->textContent;
					# valueProperty must be numeric
					if ($valueProperty =~ /([0-9.]+)/) {
						if ($1 < $MINIMAL_JAVA_VERSION) {
							# print "java version <$valueProperty> found in properties of pom.xml\n";
							my $regex = quotemeta("<$tagName>$valueProperty</$tagName>");
							$regex = qr/$regex/;
							CommonOutput::ElaborateLog::ElaborateLogPartOne(\$buffText, $regex, &$aliasMnemo, $rulesDescription, "XML");
							CloudReady::detection::addAppliDetection(&$aliasMnemo, [ $pom ]);
							return;
						}
					}
				}
			}
		}
	}
}

sub checkPomXmlPlugIns($$$$$$) {
	my $pom = shift;
	my $buffText = shift;
	my $pomContent = shift;
	my $artifactId = shift;
	my $tagNameList = shift;
	my $aliasMnemo = shift;

	return if (!defined $pomContent);

	my @nodesPlugins = Lib::XML::findnodes($pomContent, '//build/plugins/plugin');
	my @tagNameList = split(/\|/, $tagNameList);

	for my $node (@nodesPlugins) {
		my @artifactId = Lib::XML::findnodes($node, './artifactId');

		if (scalar @artifactId && $artifactId[0]->textContent =~ /$artifactId/i) {
			for my $tagName (@tagNameList) {
				my @tag = Lib::XML::findnodes($node, "./configuration/$tagName");
				if (scalar @tag) {
					my $valuePlugIn = $tag[0]->textContent;
					# HL-2802 Cloud Maturity - Detect deprecated Java and .NET versions
					if ($MASTER_TECHNO eq "Java") {
						if ($valuePlugIn =~ /([0-9.]+)/ && $valuePlugIn !~ /[a-z]/i) {
							if ($1 < $MINIMAL_JAVA_VERSION) {
								# print "java version <$valuePlugIn> found in plugins of pom.xml\n";
								my $regex = quotemeta("<$tagName>$valuePlugIn</$tagName>");
								$regex = qr/$regex/;
								CommonOutput::ElaborateLog::ElaborateLogPartOne(\$buffText, $regex, &$aliasMnemo, $rulesDescription, "XML");
								CloudReady::detection::addAppliDetection(&$aliasMnemo, [ $pom ]);
								return;
							}
						}
					}
				}
			}
		}
	}
}

sub checkBuildXml($$$$) {
	my $file = shift;
	my $buildXmlContent = shift;
	my $bufftext = shift;
	my $aliasMnemo = shift;

	return if (!defined $buildXmlContent);

	my @targetNode = Lib::XML::findnodes($buildXmlContent, '//target[@name="compile"]');
	my $regex = qr/\btarget\s*\=\s*["']([0-9.]+)["']/;
	for my $node (@targetNode) {
		if ($node =~ /$regex/) {
			if ($1 < $MINIMAL_JAVA_VERSION) {
				CommonOutput::ElaborateLog::ElaborateLogPartOne(\$bufftext, $regex, &$aliasMnemo, $rulesDescription, "XML");
				CloudReady::detection::addAppliDetection(&$aliasMnemo, [ $file ]);
			}
		}
	}

	return;
}

sub checkContentConfig($$$$) {
	my $fileConfig = shift;
	my $buff = shift;
	my $patternMatching = shift;
	my $aliasMnemo = shift;

	if ($buff =~ /(${patternMatching})/g) {
		if (defined $1) {
			# print "PATTERN <$patternMatching> FOUND IN $fileConfig\n";
			CloudReady::detection::addAppliDetection(&$aliasMnemo, [ $fileConfig ]);
		}
	}
}

sub checkJavaDeprecatedVersion($$$$) {
	my $fileGradle = shift;
	my $buff = shift;
	my $patternMatching = shift;
	my $aliasMnemo = shift;

	while ($buff =~ /$patternMatching/gc) {
		my $version = $1;
		if ($version =~ /([0-9.]+)/) {
			if ($1 < $MINIMAL_JAVA_VERSION) {
				my $posInit = pos($buff);
				my $regex = qr/\b(?:source|target)Compatibility\s*\=\s*$version/;
				CommonOutput::ElaborateLog::ElaborateLogPartOne(\$buff, $regex, &$aliasMnemo, $rulesDescription);
				CloudReady::detection::addAppliDetection(&$aliasMnemo, [ $fileGradle ]);
				pos($buff) = $posInit;
			}
		}
	}
}

sub checkFileProjectElaborateLog($$$$) {
	my $fileConfig = shift;
	my $buff = shift;
	my $patternMatching = shift;
	my $aliasMnemo = shift;

	if ($buff =~ /(${patternMatching})/g) {
		if (defined $1) {
			# print "PATTERN <$patternMatching> FOUND IN $fileConfig\n";
			CommonOutput::ElaborateLog::ElaborateLogPartOne(\$buff, $patternMatching, $aliasMnemo, $rulesDescription, "XML");
			CloudReady::detection::addAppliDetection(\&{$aliasMnemo}, [ $fileConfig ]);
		}
	}
}

sub checkSpringScopes($$) {
	my $webconf = shift;
	my $webconfContent = shift;

	return if (!defined $webconfContent);

	# count elsewhere in the document all "bean" nodes :
	#	* having a parent "beans"
	#	* having an attribute "scope" whose value matches /^(session|globalSession)$/
	#if (scalar $webconfContent->findnodes('//beans/bean[@scope =~ /^(session|globalSession)$/]') ) {
	if (scalar Lib::XML::findnodes_AttrMatch($webconfContent, '//beans/bean', "scope", qr/^(session|globalSession)$/)) {
		CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_Spring_XmlConfForStatefulCompliantScopes, [ $webconf ]);
	}
}


sub checkConfigXMLExists($$) {
	my $NameFileXML = shift;
	my $ConfigName = shift;

	# print 'name of file xml=<'.$NameFileXML."> config $ConfigName\n";

	my @ConfigXMLFilesDetect = ();
	if ($ConfigName eq 'Weblogic') {
		@ConfigXMLFilesDetect =
			qw(
				weblogic.xml
				weblogic-cmp-rdbms-jar.xml
				weblogic-ejb-jar.xml
				weblogic-ra.xml
				persistence-configuration.xml
				weblogic-webservices.xml
				weblogic-wsee-clientHandlerChain.xml
				webservice-policy-ref.xml
				weblogic-wsee-standaloneclient.xml
				weblogic-application.xml
			);
	}
	elsif ($ConfigName eq 'Websphere') {
		@ConfigXMLFilesDetect =
			qw(
				client-resource.xmi
				ibm-application-bnd.xmi
				ibm-application-bnd.xml
				ibm-application-client-bnd.xmi
				ibm-application-client-bnd.xml
				ibm-application-client-ext.xmi
				ibm-application-client-ext.xml
				ibm-application-ext.xmi
				ibm-application-ext.xml
				ibm-ejb-access-bean.xml
				ibm-ejb-jar-bnd.xmi
				ibm-ejb-jar-bnd.xml
				ibm-ejb-jar-ext.xmi
				ibm-ejb-jar-ext.xml
				ibm-ejb-jar-ext-pme.xmi
				ibm-ejb-jar-ext-pme.xml
				ibm-webservices-bnd.xmi
				ibm-webservices-ext.xml
				ibm-web-bnd.xmi
				ibm-web-bnd.xml
				ibm-web-ext.xmi
				ibm-web-ext.xml
				ibm-web-ext-pme.xmi
				ibm-web-ext-pme.xml
				j2c_plugin.xml
			);
	}
	elsif ($ConfigName eq 'JBoss') {
		@ConfigXMLFilesDetect =
			qw(
				jaws.xml
				jboss.xml
				jbosscmp-jdbc.xml
				jboss-service.xml
				jboss-web.xml
			);
	}
	elsif ($ConfigName eq 'JEE') {
		@ConfigXMLFilesDetect =
			qw(
				application.xml
				application-client.xml
				ejb-jar.xml
				ra.xml
				webservices.xml
			);
	}

	foreach my $NameFileConfig (@ConfigXMLFilesDetect) {
		if ($NameFileXML eq $NameFileConfig) {
			# print $NameFileXML ." detected\n";
			return 1;
		}
	}

	return 0;
}

sub checkXpathAttributeValues($$$$$$) {
	my $File = shift;
	my $xmlContent = shift;
	my $XpathOrigin = shift;
	my $AttributeName = shift;
	my $Ref_HashValues = shift;
	my $Alias = shift;

	return if (!defined $xmlContent);

	foreach my $key (keys %{$Ref_HashValues}) {

		if (scalar Lib::XML::findnodes_AttrMatch($xmlContent, $XpathOrigin, $AttributeName, qr/\b$key\b/)) {
			# print '+++++++'. $AttributeName ." found with value containing $key\n";
			CloudReady::detection::addAppliDetection(&$Alias, [ $File ]);
		}
	}
}


sub stripXML($) {
	my $buf = shift;
	my $codeXML;
	my $MIX;
	my $countString = 0;
	my $countTag = 0;
	my %HString = ();
	while ($$buf =~ /(\<\!\-\-)|(\-\-\>)|(["'])|\>(.*?)\<|([^-<>"']+|[-<>"']+)/gc) {
		if (defined $1) {
			$MIX .= '/*';
			my $bufComment = "";
			while ($$buf =~ /(-->)|([^->]+)/gc) {
				if (defined $1) {
					# end of comment
					last;
				}
				elsif (defined $2) {
					# it is a comment => consumes buffer
					$bufComment .= $2;
				}
			}
			$MIX .= $bufComment;
			$MIX .= '*/';
		}
		elsif (defined $2) {
			# it is a comment => consumes buffer
		}
		elsif (defined $3) {
			# it is a string
			my $bufString = $buf;
			my $string;
			while ($$bufString =~ /([^"']+)|(["']+)/gc) {
				if (defined $1) {
					$string .= $1;
				}
				else {
					last;
				}
			}
			my $stringID = "CHAINE_" . ++$countString;
			$HString{$stringID} = $string;
			$codeXML .= $stringID;
			$MIX .= $stringID;
		}
		elsif (defined $4) {
			# it is a tag value
			my $stringID = "TAG_" . ++$countTag;
			my $tagValue = $4;
			$tagValue =~ s/^["']//m;
			$tagValue =~ s/["']$//m;
			$HString{$stringID} = $tagValue;
			$codeXML .= '>' . $stringID . '<';
			$MIX .= '>' . $stringID . '<';
		}
		else {
			$codeXML .= $5;
			$MIX .= $5;
		}
	}

	return \$codeXML, \%HString, $MIX;
}

sub checkHardCodedPathProject($$$$$) {

	# Use algo CloudReady::lib::HardCodedIP::checkHardCodedIP
	# Use algo CloudReady::lib::HardCodedPath::checkHardCodedPath

	my $fileConfig = shift;
	my $buff = shift;
	my $aliasMnemo = shift;
	my $alias = shift;
	my $techno = shift;

	my %HStrings;

	# Do a simple code view and a hash string...
	my $parameters = ();
	# retrieving the 'bin' view
	my $vue = SourceLoader::mainLoadFile($fileConfig, $parameters);

	if (defined $buff) {
		my $refHStrings;
		my $MIX;
		($vue->{'code'}, $refHStrings, $MIX) = stripXML(\$buff);
		%HStrings = %{$refHStrings};
		$vue->{'agglo'} = "";
		StripUtils::agglomerate_C_Comments(\$MIX, \$vue->{'agglo'});
		CommonOutput::ManageVue::setVue($vue);

		# 07/03/2022 deactivate HardCodedIP
		# if ($alias eq 'Alias_HardCodedIP') {
		# 	# if checkHardCodedIP result > 0
		# 	if (CloudReady::lib::HardCodedIP::checkHardCodedIP(\%HStrings, \$codeXML, $techno, \$buff)) {
		# 		CloudReady::detection::addAppliDetection($aliasMnemo, [ $fileConfig ]);
		# 	}
		# }

		# project files are analyzed only for CloudReady and not for Green
		if ($alias eq 'Alias_HardCodedPath') {
			my $nb_detection = CloudReady::lib::HardCodedPath::checkHardCodedPath(\%HStrings, $techno, $buff, $rulesDescription, $fileConfig);
			if ($nb_detection > 0) {
				CloudReady::detection::addAppliDetection($aliasMnemo, [ $fileConfig ]);
			}
		}
		# replacing SHA by filenames in log
		CommonOutput::ElaborateLog::ElaborateLogPartTwo($fileConfig);
	}
}

sub detect($) {
	my $file = shift;

	if ($file =~ /(.*[\\\/])?([^\\\/]+)$/) {
		my $path = $1 || "";
		my $basename = lc($2);

		if ($basename eq "dockerfile") {
			if ($MASTER_TECHNO eq "Docker") {
				print "Analyzing $file ...\n";
				my $buffText;
				my $status = open(FILE, "<", $file);
				if ($status) {
					local $/ = undef;
					$buffText = <FILE>;
					close FILE;
				}

				# remove comment lines
				$buffText =~ s/^[^\n]*\#.*//mg;

				if ($buffText =~ /\G\s*FROM/) {
					print "dockerfile identified\n";
					CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_Docker_File, [ $file ]);
				}
			}
		}
		elsif ($basename =~ /(.+)\.([^\.]*)$/) {
			my $name = $1;
			my $ext = $2;
			if (($MASTER_TECHNO eq "CS") || ($MASTER_TECHNO eq "VbDotNet")) {
				if ($ext eq 'config') {
					print "Analyzing $file ...\n";
					my $buffText;
					my $status = open(FILE, "<", $file);
					if ($status) {
						local $/ = undef;
						$buffText = <FILE>;
						close FILE;
					}
					# /!\ Lib::XML::load_xml could change the file content
					my $bufferXML = Lib::XML::load_xml($file);
					if (!defined $bufferXML) {
						print STDERR "ERROR: malformed XML for file $file\n";
					}
					# 09/06/2020 HardCodedIP
					# 13/07/2021 HardCodedIP disabled for config files
					# checkHardCodedIPPath($file, $buff, CloudReady::Ident::Alias_HardCodedIP, 'Alias_HardCodedIP');
					# 10/06/2020 HardCodedPath
					# 13/07/2021 HardCodedPath disabled for config files
					# checkHardCodedIPPath($file, $buff, CloudReady::Ident::Alias_HardCodedPath, 'Alias_HardCodedPath');
					# HL-1628 07/03/2022 deactivate HTTP, FTP, LDAP
					# 11/06/2020 HTTPProtocol
					# checkContentConfig($file, $buff, "(?i)\\bhttps?\:\/\/", \&{'CloudReady::Ident::Alias_HTTPProtocol'});
					# 11/06/2020 FTPProtocol
					# checkContentConfig($file, $buff, "(?i)\\bs?ftps?\:\/\/", \&{'CloudReady::Ident::Alias_FTPProtocol'});
					# 11/06/2020 LDAPProtocol
					# checkContentConfig($file, $buff, "(?i)\\bldaps?\:\/\/", \&{'CloudReady::Ident::Alias_LDAPProtocol'});

					if ($name eq 'web') {
						CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebConfFile, [ $file ]);
						checkWebConfig($file, $bufferXML);
						checkWebAppConfig($file, $bufferXML);
					}
					elsif ($name eq 'app') {
						checkWebAppConfig($file, $bufferXML);
					}
					# HL-940 28/06/2019 Detect usage of Log4Net
					elsif ($name eq 'log4net') {
						CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_Log4NetConfig, [ $file ]);
					}
					elsif ($name !~ /\b(?:nuget|packages)\b|\bweb\b.*/) {
						CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_ExtraConfFile, [ $file ]);
						#print "Id_ExtraConfFile => $file\n";
					}

					# 28/02/2018 HL-489 .Net EventLog
					my $Ref_HashValues = {
						"System.Diagnostics.EventLogTraceListener" => 1
					};
					my $Alias = \&{'CloudReady::Ident::Alias_Config_EventLogTraceListener'};

					checkXpathAttributeValues($file, $bufferXML, '//system.diagnostics/trace/listeners/add', 'type', $Ref_HashValues, $Alias);

					# 01/03/2018 HL-497 Machine Key use
					$Ref_HashValues = {
						"AutoGenerate" => 1
					};
					$Alias = \&{'CloudReady::Ident::Alias_MachineKey_ValidationKey_AutoGenerate'};

					checkXpathAttributeValues($file, $bufferXML, '//machineKey', 'validationKey', $Ref_HashValues, $Alias);
					# //machineKey[@validationKey="... AutoGenerate ...]
				}
				# HL-1708
				elsif (($MASTER_TECHNO eq "CS" && $ext eq 'csproj')
					|| ($MASTER_TECHNO eq "VbDotNet" && $ext eq 'vbproj')) {
					print "Analyzing $file ...\n";
					my $buffText;
					my $status = open(FILE, "<", $file);
					if ($status) {
						local $/ = undef;
						$buffText = <FILE>;
						close FILE;
					}
					# /!\ Lib::XML::load_xml could change the file content
					my $bufferXML = Lib::XML::load_xml($file);
					if (!defined $bufferXML) {
						print STDERR "ERROR: malformed XML for file $file\n";
					}
					checkCsVbProj($file, $buffText, $bufferXML);
				}
				elsif ($ext eq "xml") {
					print "Analyzing $file ...\n";
					my $buff;
					my $status = open(FILE, "<", $file);
					if ($status) {
						local $/ = undef;
						$buff = <FILE>;
						close FILE;
					}

					# 09/06/2020 HardCodedIP
					# 07/03/2022 deactivate HardCodedIP for xml
					# checkHardCodedIPPath($file, $buff, CloudReady::Ident::Alias_HardCodedIP, 'Alias_HardCodedIP', $MASTER_TECHNO);
					# 10/06/2020 HardCodedPath
					checkHardCodedPathProject($file, $buff, CloudReady::Ident::Alias_HardCodedPath, 'Alias_HardCodedPath', $MASTER_TECHNO);
					# HL-1628
					# HTTPProtocol 20/12/2021 deactivate rule for xml
					# checkContentConfig($file, $buff, "(?i)\\bhttps?\:\/\/", \&{'CloudReady::Ident::Alias_HTTPProtocol'});
					# FTPProtocol 20/12/2021 deactivate rule for xml
					# checkContentConfig($file, $buff, "(?i)\\bs?ftps?\:\/\/", \&{'CloudReady::Ident::Alias_FTPProtocol'});
					# LDAPProtocol 20/12/2021 deactivate rule for xml
					# checkContentConfig($file, $buff, "(?i)\\bldaps?\:\/\/", \&{'CloudReady::Ident::Alias_LDAPProtocol'});
				}
			}
			elsif ($MASTER_TECHNO eq "Python" or $MASTER_TECHNO eq "PHP") {
				if ($ext eq "yaml") {
					if ($name eq 'app') {
						print "Analyzing $file ...\n";
						my $buff;
						my $status = open(FILE, "<", $file);
						if ($status) {
							local $/ = undef;
							$buff = <FILE>;
							close FILE;
						}

						# HL-954 08/10/2019 [GCP Boosters] Using MySQL database
						checkContentConfig($file, $buff, "\\bmysql(?:[+]pymysql)?\\b", \&{'CloudReady::Ident::Alias_DBMS_MySQL'});
						# HL-955 09/10/2019 [GCP Boosters] Using PostgreSQL database
						checkContentConfig($file, $buff, "\\bpostgresql|pgsql\\b", \&{'CloudReady::Ident::Alias_DBMS_PostgreSQL'});
					}
				}
			}
			elsif ($MASTER_TECHNO eq "Java") {
				if ($ext eq "xml") {
					print "Analyzing $file ...\n";
					my $buffText;
					my $status = open(FILE, "<", $file);
					if ($status) {
						local $/ = undef;
						$buffText = <FILE>;
						close FILE;
					}
					# /!\ Lib::XML::load_xml could change the file content
					my $bufferXML = Lib::XML::load_xml($file);
					if (!defined $bufferXML) {
						print STDERR "ERROR: malformed XML for file $file\n";
					}
					# 09/06/2020 HardCodedIP
					# 07/03/2022 deactivate HardCodedIP for xml
					# checkHardCodedIPPath($file, $buff, CloudReady::Ident::Alias_HardCodedIP, 'Alias_HardCodedIP', $MASTER_TECHNO);
					# 10/06/2020 HardCodedPath
					checkHardCodedPathProject($file, $buffText, CloudReady::Ident::Alias_HardCodedPath, 'Alias_HardCodedPath', $MASTER_TECHNO);
					# HL-1628
					# HTTPProtocol 20/12/2021 deactivate rule for xml
					# checkContentConfig($file, $buff, "(?i)\\bhttps?\:\/\/", \&{'CloudReady::Ident::Alias_HTTPProtocol'});
					# FTPProtocol 20/12/2021 deactivate rule for xml
					# checkContentConfig($file, $buff, "(?i)\\bs?ftps?\:\/\/", \&{'CloudReady::Ident::Alias_FTPProtocol'});
					# LDAPProtocol 20/12/2021 deactivate rule for xml
					# checkContentConfig($file, $buff, "(?i)\\bldaps?\:\/\/", \&{'CloudReady::Ident::Alias_LDAPProtocol'});

					if ($name eq "web") {
						CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_WebXml, [ $file ]);
					}
					elsif ($name eq "pom") {
						checkPomXmlDependencies($file, $bufferXML, "com.microsoft.azure", "adal4j", \&{'CloudReady::Ident::Alias_PomXmlAdal4jDependency'});
						# HL-953 04/10/2019 [GCP Boosters] Using BigQuery
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-bigquery", \&{'CloudReady::Ident::Alias_UsingBigQuery'});
						# HL-954 08/10/2019 [GCP Boosters] Using MySQL database
						my $status = checkPomXmlDependencies($file, $bufferXML, "mysql", "mysql\-connector\-java", \&{'CloudReady::Ident::Alias_DBMS_MySQL'});
						if (!$status) {
							checkPomXmlDependencies($file, $bufferXML, "com.google.cloud.sql", "mysql\-socket\-factory\-connector\-j\-[0-9]+", \&{'CloudReady::Ident::Alias_DBMS_MySQL'});
						}
						# HL-955 09/10/2019 [GCP Boosters] Using PostgreSQL database
						$status = checkPomXmlDependencies($file, $bufferXML, "org.postgresql", "postgresql", \&{'CloudReady::Ident::Alias_DBMS_PostgreSQL'});
						if (!$status) {
							checkPomXmlDependencies($file, $bufferXML, "com.google.cloud.sql", "postgres\-socket\-factory", \&{'CloudReady::Ident::Alias_DBMS_PostgreSQL'});
						}
						# HL-957 10/10/2019 [GCP Boosters] Using a Cloud-Based Storage
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-datastore", \&{'CloudReady::Ident::Alias_UsingCloudDataStore'});
						# HL-958 11/10/2019 [GCP Boosters] Using BigTable
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-bigtable", \&{'CloudReady::Ident::Alias_UsingBigTable'});
						# HL-959 15/10/2019 [GCP Boosters] Using Cloud Spanner
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-spanner", \&{'CloudReady::Ident::Alias_UsingCloudSpanner'});
						# HL-960 16/10/2019 [GCP Boosters] Using Cloud in-memory database (redis)
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-redis", \&{'CloudReady::Ident::Alias_Using_InMemoryRedisGCP'});
						# HL-961 17/10/2019 [GCP Boosters] Using Cloud IAM (Identity and Access Management)
						checkPomXmlDependencies($file, $bufferXML, "com.google.apis", "google\-api\-services\-iam", \&{'CloudReady::Ident::Alias_UsingCloudIAM'});
						# HL-962 18/10/2019 [GCP Boosters] Using Firebase
						checkPomXmlDependencies($file, $bufferXML, "com.google.firebase", "firebase\-admin", \&{'CloudReady::Ident::Alias_UsingFireBase'});
						# HL-963 28/10/2019 [GCP Boosters] Using Cloud IAP (Identity Aware Proxy)
						checkPomXmlDependencies($file, $bufferXML, "com.google.apis", "google\-api\-services\-iap", \&{'CloudReady::Ident::Alias_UsingCloudIAP'});
						# HL-964 28/10/2019 [GCP Boosters] Using a Cloud-based Key storage (KMS)
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-kms", \&{'CloudReady::Ident::Alias_GCPServicesKms'});
						# HL-968 30/10/2019 [GCP Boosters] Using a Cloud-based middleware application
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-pubsub", \&{'CloudReady::Ident::Alias_UsingCloudPubSub'});
						# HL-1724 10/05/2021
						checkPomXmlDependencies($file, $bufferXML, "com.google.apis", "google\-api\-services\-cloudfunctions", \&{'CloudReady::Ident::Alias_GCPFunctions'});
						# HL-1723 10/05/2021
						checkPomXmlDependencies($file, $bufferXML, "com.azure.resourcemanager", "azure\-resourcemanager\-recoveryservices", \&{'CloudReady::Ident::Alias_AzureArchiveStorage'});
						checkPomXmlDependencies($file, $bufferXML, "com.google.apis", "google\-api\-services\-file", \&{'CloudReady::Ident::Alias_GCPFilestore'});
						# HL-1725 11/05/2021
						checkPomXmlDependencies($file, $bufferXML, "com.azure.resourcemanager", "azure\-resourcemanager\-datafactory", \&{'CloudReady::Ident::Alias_AzureDataFactory'});
						checkPomXmlDependencies($file, $bufferXML, "com.google.apis", "google\-api\-services\-datafusion", \&{'CloudReady::Ident::Alias_GCPCloudDataFusion'});
						# HL-1726 17/05/2021
						checkPomXmlDependencies($file, $bufferXML, "com.azure.resourcemanager", "azure\-resourcemanager\-deploymentmanager", \&{'CloudReady::Ident::Alias_AzureDeploymentManager'});
						checkPomXmlDependencies($file, $bufferXML, "com.google.apis", "google\-api\-services\-deploymentmanager", \&{'CloudReady::Ident::Alias_GCPDeploymentManager'});
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-build", \&{'CloudReady::Ident::Alias_GCPCloudBuild'});
						checkPomXmlDependencies($file, $bufferXML, "com.google.apis", "google\-api\-services\-composer", \&{'CloudReady::Ident::Alias_GCPCloudComposer'});
						# HL-1727 18/05/2021
						checkPomXmlDependencies($file, $bufferXML, "com.azure.resourcemanager", "azure\-resourcemanager\-streamanalytics", \&{'CloudReady::Ident::Alias_AzureStreamAnalytics'});
						checkPomXmlDependencies($file, $bufferXML, "com.azure", "azure\-messaging\-eventhubs", \&{'CloudReady::Ident::Alias_AzureEventHub'});
						checkPomXmlDependencies($file, $bufferXML, "com.microsoft.azure", "azure\-eventgrid", \&{'CloudReady::Ident::Alias_AzureEventGrid'});
						# HL-1730 18/05/2021
						checkPomXmlDependencies($file, $bufferXML, "com.google.cloud", "google\-cloud\-monitoring", \&{'CloudReady::Ident::Alias_GCPCloudMonitoring'});
						# HL-2802 Cloud Maturity - Detect deprecated Java and .NET versions
						checkPomXmlProperties($file, $buffText, $bufferXML, 'java.version|maven.compiler.source|maven.compiler.release|maven.compiler.target', \&{'CloudReady::Ident::Alias_DeprecatedFw'});
						checkPomXmlPlugIns($file, $buffText, $bufferXML, qr/.*\bmaven\b.*/, "release|source|target", \&{'CloudReady::Ident::Alias_DeprecatedFw'});

						# replacing SHA by filenames in log
						CommonOutput::ElaborateLog::ElaborateLogPartTwo($file);
					}
					# 13/02/2018 HL-472 CDI Beans
					elsif ($name eq "beans") {

						my $Ref_HashValues = {
							"http://java.sun.com/xml/ns/javaee"  => 1,
							"http://xmlns.jcp.org/xml/ns/javaee" => 1
						};
						my $Alias = \&{'CloudReady::Ident::Alias_CDIBeansConfiguration'};

						checkXpathAttributeValues($file, $bufferXML, '//beans', 'xmlns', $Ref_HashValues, $Alias);
					}
					elsif ($name eq 'build') {
						# Ant java
						# HL-2868 Detect deprecated Java versions from Ant
						checkBuildXml($file, $bufferXML, $buffText, \&{'CloudReady::Ident::Alias_DeprecatedFw'});

						# replacing SHA by filenames in log
						CommonOutput::ElaborateLog::ElaborateLogPartTwo($file);
					}
					else {
						# HL-251
						if ($path =~ /WEB-INF[\\\/]$/m) {
							checkSpringScopes($file, $bufferXML);
						}
						CloudReady::detection::addAppliDetection(CloudReady::Ident::Alias_ExtraXmlFile, [ $file ]);
					}
				}
				elsif ($ext eq 'gradle') {
					if ($name eq 'build') {
						print "Analyzing $file ...\n";
						my $buff;
						my $status = open(FILE, "<", $file);
						if ($status) {
							local $/ = undef;
							$buff = <FILE>;
							close FILE;
						}

						# HL-953 04/10/2019 [GCP Boosters] Using BigQuery
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-bigquery.*?[\"']", \&{'CloudReady::Ident::Alias_UsingBigQuery'});
						# HL-954 08/10/2019 [GCP Boosters] Using MySQL database
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud.sql:mysql-socket-factory-connector-j.*?[\"']", \&{'CloudReady::Ident::Alias_DBMS_MySQL'});
						# HL-957 10/10/2019 [GCP Boosters] Using a Cloud-Based Storage
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-datastore.*?[\"']", \&{'CloudReady::Ident::Alias_UsingCloudDataStore'});
						# HL-958 11/10/2019 [GCP Boosters] Using BigTable
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-bigtable.*?[\"']", \&{'CloudReady::Ident::Alias_UsingBigTable'});
						# HL-959 15/10/2019 [GCP Boosters] Using Cloud Spanner
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-spanner.*?[\"']", \&{'CloudReady::Ident::Alias_UsingCloudSpanner'});
						# HL-960 16/10/2019 [GCP Boosters] Using Cloud in-memory database (redis)
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-redis.*?[\"']", \&{'CloudReady::Ident::Alias_Using_InMemoryRedisGCP'});
						# HL-962 18/10/2019 [GCP Boosters] Using Firebase
						checkContentConfig($file, $buff, "[\"'].*?com.google.firebase:firebase-admin.*?[\"']", \&{'CloudReady::Ident::Alias_UsingFireBase'});
						# HL-963 28/10/2019 [GCP Boosters] Using Cloud IAP (Identity Aware Proxy)
						checkContentConfig($file, $buff, "[\"'].*?com.google.apis:google-api-services-iap.*?[\"']", \&{'CloudReady::Ident::Alias_UsingCloudIAP'});
						# HL-964 28/10/2019 [GCP Boosters] Using a Cloud-based Key storage (KMS)
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-kms.*?[\"']", \&{'CloudReady::Ident::Alias_GCPServicesKms'});
						# HL-968 30/10/2019 [GCP Boosters] Using a Cloud-based middleware application
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-pubsub.*?[\"']", \&{'CloudReady::Ident::Alias_UsingCloudPubSub'});
						# HL-1724 10/05/2021
						checkContentConfig($file, $buff, "[\"'].*?com.google.apis:google-api-services-cloudfunctions.*?[\"']", \&{'CloudReady::Ident::Alias_GCPFunctions'});
						# HL-1723 10/05/2021
						checkContentConfig($file, $buff, "[\"'].*?com.google.apis:google-api-services-file.*?[\"']", \&{'CloudReady::Ident::Alias_GCPFilestore'});
						# HL-1725 11/05/2021
						checkContentConfig($file, $buff, "[\"'].*?com.google.apis:google-api-services-datafusion.*?[\"']", \&{'CloudReady::Ident::Alias_GCPCloudDataFusion'});
						# HL-1726 17/05/2021
						checkContentConfig($file, $buff, "[\"'].*?com.google.apis:google-api-services-deploymentmanager.*?[\"']", \&{'CloudReady::Ident::Alias_GCPDeploymentManager'});
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-build.*?[\"']", \&{'CloudReady::Ident::Alias_GCPCloudBuild'});
						checkContentConfig($file, $buff, "[\"'].*?com.google.apis:google-api-services-composer.*?[\"']", \&{'CloudReady::Ident::Alias_GCPCloudComposer'});
						# HL-1730 18/05/2021
						checkContentConfig($file, $buff, "[\"'].*?com.google.cloud:google-cloud-monitoring.*?[\"']", \&{'CloudReady::Ident::Alias_GCPCloudMonitoring'});
						# HL-2868 Detect deprecated Java versions from Gradle
						checkJavaDeprecatedVersion($file, $buff, qr/\b(?:source|target)Compatibility\s*\=\s*(.*)/, \&{'CloudReady::Ident::Alias_DeprecatedFw'});

						# replacing SHA by filenames in log
						CommonOutput::ElaborateLog::ElaborateLogPartTwo($file);
					}
				}
				elsif ($ext eq "yaml") {
					if ($name eq 'app') {
						print "Analyzing $file ...\n";
						my $buff;
						my $status = open(FILE, "<", $file);
						if ($status) {
							local $/ = undef;
							$buff = <FILE>;
							close FILE;
						}

						# HL-954 08/10/2019 [GCP Boosters] Using MySQL database
						checkContentConfig($file, $buff, "\\bmysql(?:[+]pymysql)?\\b", \&{'CloudReady::Ident::Alias_DBMS_MySQL'});
					}
				}
				elsif ($ext eq "properties") {
					print "Analyzing $file ...\n";
					my $buff;
					my $status = open(FILE, "<", $file);
					if ($status) {
						local $/ = undef;
						$buff = <FILE>;
						close FILE;
					}

					# 09/06/2020 HardCodedIP
					# 07/03/2022 deactivate HardCodedIP for properties
					# checkHardCodedIPPath($file, $buff, CloudReady::Ident::Alias_HardCodedIP, 'Alias_HardCodedIP', $MASTER_TECHNO);
					# 10/06/2020 HardCodedPath
					checkHardCodedPathProject($file, $buff, CloudReady::Ident::Alias_HardCodedPath, 'Alias_HardCodedPath', $MASTER_TECHNO);
					# HL-1628 07/03/2022 deactivate HTTP, FTP, LDAP for properties
					# 11/06/2020 HTTPProtocol
					# checkContentConfig($file, $buff, "(?i)\\bhttps?\:\/\/", \&{'CloudReady::Ident::Alias_HTTPProtocol'});
					# 11/06/2020 FTPProtocol
					# checkContentConfig($file, $buff, "(?i)\\bs?ftps?\:\/\/", \&{'CloudReady::Ident::Alias_FTPProtocol'});
					# 11/06/2020 LDAPProtocol
					# checkContentConfig($file, $buff, "(?i)\\bldaps?\:\/\/", \&{'CloudReady::Ident::Alias_LDAPProtocol'});
				}
				if ($ext eq "xml" or $ext eq "xmi") {
					# 07/02/2018 HL-460 Weblogic configuration
					# 08/02/2018 HL-462 Websphere configuration
					# 08/02/2018 HL-463 JBoss configuration
					# 09/02/2018 HL-464 JEE configuration
					my %checkConfig = (
						'Weblogic'  => \&checkConfigXMLExists("$name.$ext", 'Weblogic'),
						'Websphere' => \&checkConfigXMLExists("$name.$ext", 'Websphere'),
						'JBoss'     => \&checkConfigXMLExists("$name.$ext", 'JBoss'),
						'JEE'       => \&checkConfigXMLExists("$name.$ext", 'JEE'),
					);

					foreach my $ConfigKind (keys %checkConfig) {
						if (not exists $boolConfigXML{$ConfigKind} and ${$checkConfig{$ConfigKind}} == 1) {
							# print '+++++++ ' . $ConfigKind.' config detected!!!!'."\n";
							$boolConfigXML{$ConfigKind} = 1;

							my $Alias = \&{'CloudReady::Ident::Alias_' . $ConfigKind . 'Configuration'};
							CloudReady::detection::addAppliDetection(&$Alias, [ $file ]);
						}
					}
				}
			}
			elsif ($MASTER_TECHNO eq "Swift") {
				if ($name eq "package" && $ext eq "swift") {
					print "Analyzing $file ...\n";
					my $buff;
					my $status = open(FILE, "<", $file);
					if ($status) {
						local $/ = undef;
						$buff = <FILE>;
						close FILE;
					}

					# HL-1337 Detect the usage of a cloud-based Data Warehouse
					checkContentConfig($file, $buff, qr/(?i)url\:\s*\"https\:\/\/github\.com\/swift\-aws\/S3\.git\"/, \&{'CloudReady::Ident::Alias_AmazonawsS3Storage'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/vapor\-community\/google\-cloud\.git\"/, \&{'CloudReady::Ident::Alias_GCPStorage'});
					# HL-1340 Detect the usage of a cloud-based NoSQL database storage
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/swift\-aws\/DynamoDB\.git\"/, \&{'CloudReady::Ident::Alias_DBMS_DynamoDB'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https:\/\/github\.com\/mongodb\/mongo\-swift\-driver\.git\"/, \&{'CloudReady::Ident::Alias_DBMS_MongoDB'});
					# HL-1342 08/09/2020 Detect the usage of cloud-based relational database storage
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/PerfectlySoft\/Perfect\-PostgreSQL\.git\"/, \&{'CloudReady::Ident::Alias_DBMS_PostgreSQL'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/stepanhruda\/PostgreSQL\-Swift\.git\"/, \&{'CloudReady::Ident::Alias_DBMS_PostgreSQL'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/IBM\-Swift\/Swift\-Kuery\-PostgreSQL\.git\"/, \&{'CloudReady::Ident::Alias_DBMS_PostgreSQL'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/PerfectlySoft\/Perfect\-MySQL\.git\"/, \&{'CloudReady::Ident::Alias_DBMS_MySQL'});
					# HL-1343 09/09/2020 Detect the usage of cloud-based cache in-memory database
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/Zewo\/Redis\"/, \&{'CloudReady::Ident::Alias_InMemoryRedis'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/IBM\-Swift\/Kitura\-redis\.git\"/, \&{'CloudReady::Ident::Alias_InMemoryRedis'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https:\/\/github\.com\/orungrau\/SwiftMemcached\.git\"/, \&{'CloudReady::Ident::Alias_InMemoryMemcached'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github.com\/orungrau\/libMemcachedSwift\.git\"/, \&{'CloudReady::Ident::Alias_InMemoryMemcached'});
					# HL-1344 09/09/2020 Detect the usage of a cloud-based Identity and Access Management (IAM)
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/swift\-aws\/IAM\.git\"/, \&{'CloudReady::Ident::Alias_AwsCloudIAM'});
					# HL-1345 09/09/2020 Detect the usage of a cloud-based Active Directory service
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/Azure\-Samples\/active\-directory\-ios\.git\"/, \&{'CloudReady::Ident::Alias_AzureDirectoryService'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/Azure\-Samples\/ms\-identity\-mobile\-apple\-swift\-objc\.git\"/, \&{'CloudReady::Ident::Alias_AzureDirectoryService'});
					# HL-1348 10/09/2020 Detect the usage of a cloud Container Registry
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/spprichard\/K8s\.git\"/, \&{'CloudReady::Ident::Alias_Kubernetes'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/iainsmith\/swift\-docker\.git\"/, \&{'CloudReady::Ident::Alias_Docker'});
					# HL-1352 10/09/2020 Detect the usage of a cloud-based blockchain technology
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/shu223\/BlockchainSwift\.git\"/, \&{'CloudReady::Ident::Alias_Blockchain'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/appcoda\/BlockchainDemo\.git\"/, \&{'CloudReady::Ident::Alias_Blockchain'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/hyperledger\/sawtooth\-sdk\-swift\.git\"/, \&{'CloudReady::Ident::Alias_Blockchain'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/matter\-labs\/web3swift\.git\"/, \&{'CloudReady::Ident::Alias_Blockchain'});
					# HL-1360 11/09/2020 Detect the usage of system calls or processes management
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/jamf\/Subprocess\.git\"/, \&{'CloudReady::Ident::Alias_LaunchSubProcess'});
					checkContentConfig($file, $buff, qr/(?i)url:\s*\"https\:\/\/github\.com\/marcoconti83\/morione\.git\"/, \&{'CloudReady::Ident::Alias_LaunchSubProcess'});

				}
			}
			elsif ($MASTER_TECHNO eq "CCpp") {
				if ($ext eq 'h') {
					print "Analyzing $file ...\n";
					my $buff;
					my $status = open(FILE, "<", $file);
					if ($status) {
						local $/ = undef;
						$buff = <FILE>;
						close FILE;
					}

					# HL-1923 10/01/2022 Using dynamic libraries (dll, so...)
					checkFileProjectElaborateLog($file, $buff, qr/\_\_declspec\s*\(\s*dll(?:imp|exp)ort\s*\)/, CloudReady::Ident::Alias_DllImport);
					# replacing SHA by filenames in log
					CommonOutput::ElaborateLog::ElaborateLogPartTwo($file);
				}
			}
		}
	}
}

1;
