package Cobol::GreenCobol;

use strict;
use warnings;
use Erreurs;
use GreenIT::lib::KeepPortalData;
use GreenIT::lib::HardCodedPathNotGreen;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $mnemo_EmptyCatches = Ident::Alias_EmptyCatches();
my $mnemo_HardCodedPathNotGreen = Ident::Alias_HardCodedPathNotGreen();
my $mnemo_ConcatInLoop = Ident::Alias_ConcatInLoop();
my $mnemo_FunctionCallInLoopTest = Ident::Alias_FunctionCallInLoopTest();
my $mnemo_InstantiationInLoop = Ident::Alias_InstantiationInLoop();
my $mnemo_OpenCloseFileInLoop = Ident::Alias_OpenCloseFileInLoop();
my $mnemo_LoopCondComparedToNonZero = Ident::Alias_LoopCondComparedToNonZero();
my $mnemo_ToManyNestedLoop = Ident::Alias_ToManyNestedLoop();

my $nb_EmptyCatches = 0;
my $nb_HardCodedPathNotGreen = 0;
my $nb_ConcatInLoop = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_InstantiationInLoop = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_LoopCondComparedToNonZero = 0;
my $nb_ToManyNestedLoop = 0;

sub CountCobol($$$$) {
    my ($fichier, $vue, $couples, $options) = @_;

    my $status = 0;
    $nb_EmptyCatches = 0;
    $nb_ConcatInLoop = 0;
    $nb_FunctionCallInLoopTest = 0;
    $nb_InstantiationInLoop = 0;
    $nb_OpenCloseFileInLoop = 0;
    $nb_LoopCondComparedToNonZero = 0;
    $nb_ToManyNestedLoop = 0;

    my $codeView = $vue->{'code'};
    if (not defined $vue->{'code'}) {
        $status |= Couples::counter_add($couples, $mnemo_EmptyCatches, Erreurs::COMPTEUR_ERREUR_VALUE);
        $status |= Couples::counter_add($couples, $mnemo_ConcatInLoop, Erreurs::COMPTEUR_ERREUR_VALUE);
        $status |= Couples::counter_add($couples, $mnemo_FunctionCallInLoopTest, Erreurs::COMPTEUR_ERREUR_VALUE);
        $status |= Couples::counter_add($couples, $mnemo_InstantiationInLoop, Erreurs::COMPTEUR_ERREUR_VALUE);
        $status |= Couples::counter_add($couples, $mnemo_OpenCloseFileInLoop, Erreurs::COMPTEUR_ERREUR_VALUE);
        $status |= Couples::counter_add($couples, $mnemo_LoopCondComparedToNonZero, Erreurs::COMPTEUR_ERREUR_VALUE);
        $status |= Couples::counter_add($couples, $mnemo_ToManyNestedLoop, Erreurs::COMPTEUR_ERREUR_VALUE);

        $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
        return $status;
    }

    # delete line numbers at the line begining
    AnaCobol::createCode($fichier, $vue, $options, $couples);

    my $procDivBody = $vue->{'code'};
    Cobol::Vue::ProcDiv(\$procDivBody, $couples);
    # get begining line of PRODEDURE DIVISION
    my $beginLineProcDiv = $couples->{Dat_ProcDivLine};

    my $lineOffset = 0;

    while ($procDivBody =~ /(\n+)|\bcatch\s+(?:[^\n]+)\n(.*?)(?:\bfinally\b|\bend\-try\b)|\b(perform)\b|\b(open|close)\b/sig) {
        #my $catch1 = $1;
        #my $catch2 = $2;
        #my $catch3 = $3;
        #my $catch4 = $4;
        #my $catch5 = $5;
        #my $catch6 = $6;
        #my $catch7 = $7;

        if (defined $1) {
            $lineOffset += () = $1 =~ /\n/g;
        }
        # TRY/CATCH
        elsif (defined $2) {
            my $catchBody = $2;
            # HL-2782 Avoid empty catch blocks
            if ($catchBody !~ /\S/) {
                # print "EMPTY CATCH FOUND\n";
                $nb_EmptyCatches++;
                CommonOutput::ElaborateLog::SimpleElaborateLog($beginLineProcDiv + $lineOffset, Ident::Alias_EmptyCatches, $rulesDescription, "Avoid_EmptyCatches");
            }
            $lineOffset += () = $catchBody =~ /\n/g;
        }
        # LOOPS
        # syntax supported with END-PERFORM statement
        # PERFORM xxx <TIMES|UNTIL|VARYING...UNTIL|THRU|FROM> xxx END-PERFORM.
        # assuming syntax with final points are not used in recent versions of COBOL
        elsif (defined $3) {
            my $countTimeOption = 0;
            my $numline = $beginLineProcDiv + $lineOffset;
            #print "PERFORM at line $numline\n";

            my $pos = pos($procDivBody);
            my $bodyLoop = testParsingPotentialLoop(\$procDivBody);

            if ($bodyLoop ne "") {
                while ($bodyLoop =~ /(\n+)|^\s*(string\b)|^\s*(call\b)|^\s*(invoke\b)|^\s*(open|close)\b|\bif\s+(.*?)\bend\-if\b|\b(times|(?:varying).*?until|thru|from)\b/msig) {
                    #my $catch1 = $1;
                    #my $catch2 = $2;
                    #my $catch3 = $3;
                    #my $catch4 = $4;
                    #my $catch5 = $5;
                    #my $catch6 = $6;

                    if (defined $1) {
                        $lineOffset += () = $1 =~ /\n/g;
                    }
                    # HL-2784 Avoid String concatenation in loops
                    elsif (defined $2) {
                        $nb_ConcatInLoop++;
                        CommonOutput::ElaborateLog::SimpleElaborateLog($beginLineProcDiv + $lineOffset, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
                    }
                    # HL-2785 Avoid calling a function in a condition loop
                    elsif (defined $3) {
                        $nb_FunctionCallInLoopTest++;
                        CommonOutput::ElaborateLog::SimpleElaborateLog($beginLineProcDiv + $lineOffset, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
                    }
                    # HL-2786 Avoid instantiations inside loops
                    elsif (defined $4) {
                        $nb_InstantiationInLoop++;
                        CommonOutput::ElaborateLog::SimpleElaborateLog($beginLineProcDiv + $lineOffset, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
                    }
                    # HL-2787 Avoid opening/closing files inside a loop
                    elsif (defined $5) {
                        $nb_OpenCloseFileInLoop++;
                        CommonOutput::ElaborateLog::SimpleElaborateLog($beginLineProcDiv + $lineOffset, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
                    }
                    # HL-2790 Prefer comparison-to-0 in loop conditions
                    elsif (defined $6) {
                        my $condition = $6;
                        # check all expressions using operators <, >, <=, >=, =
                        # value 0, ZERO, ZEROS, ZEROES
                        my $bool_comparisonToZero;
                        if ($condition !~ /\b(?:AND|OR)\b/i) {
                            if ($condition =~ /([^<>=\s]+)\s*(?:[<>=]=?|<|>)\s*([^<>=\s]+)/) {
                                $bool_comparisonToZero = 0;
                                #my $catch1 = $1;
                                #my $catch2 = $2;
                                if (($1 =~ /(?:^|[^\.\w])(?:0|ZEROE?S?)$/im) || ($2 =~ /^(?:0|ZEROE?S?)(?:$|[^\.\w])/m)) {
                                    $bool_comparisonToZero = 1;
                                }
                            }
                            if ((defined $bool_comparisonToZero) && ($bool_comparisonToZero == 0)) {
                                $nb_LoopCondComparedToNonZero++;
                                CommonOutput::ElaborateLog::SimpleElaborateLog($beginLineProcDiv + $lineOffset, Ident::Alias_LoopCondComparedToNonZero, $rulesDescription, "LoopCondComparedToNonZero");
                            }
                        }
                        $lineOffset += () = $condition =~ /\n/g;
                    }
                    # HL-2791 Avoid nested loops
                    elsif (defined $7) {
                        my $numline = $beginLineProcDiv + $lineOffset;
                        #print "time option $7 at line $numline\n";

                        $countTimeOption++;

                        if ($countTimeOption > 1) {
                            $nb_ToManyNestedLoop++;
                            #print "counter++\n";
                            CommonOutput::ElaborateLog::SimpleElaborateLog($beginLineProcDiv + $lineOffset, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
                        }
                    }
                }
            }
            else {
                # only 'PERFORM' is consumed, it's a simple PERFORM statement not a loop
                pos($procDivBody) = $pos;
            }
        }
    }


    $status |= Couples::counter_add($couples, $mnemo_EmptyCatches, $nb_EmptyCatches);
    $status |= Couples::counter_add($couples, $mnemo_ConcatInLoop, $nb_ConcatInLoop);
    $status |= Couples::counter_add($couples, $mnemo_FunctionCallInLoopTest, $nb_FunctionCallInLoopTest);
    $status |= Couples::counter_add($couples, $mnemo_InstantiationInLoop, $nb_InstantiationInLoop);
    $status |= Couples::counter_add($couples, $mnemo_OpenCloseFileInLoop, $nb_OpenCloseFileInLoop);
    $status |= Couples::counter_add($couples, $mnemo_LoopCondComparedToNonZero, $nb_LoopCondComparedToNonZero);
    $status |= Couples::counter_add($couples, $mnemo_ToManyNestedLoop, $nb_ToManyNestedLoop);

    return $status;
}

sub CountString($$$$) {
    my ($fichier, $vue, $compteurs, $options) = @_;
    my $status = 0;
    $nb_HardCodedPathNotGreen = 0;

    my $code = \$vue->{'code'};
    my $HString = $vue->{'HString'};
    my $text = \$vue->{'text'};

    if (!defined $code) {
        $status |= Couples::counter_add($compteurs, $mnemo_HardCodedPathNotGreen, Erreurs::COMPTEUR_ERREUR_VALUE);
    }

    my $parameters = AnalyseUtil::recuperer_options();
    my $techno = $parameters->[0]->{'--language'};

    # HL-2783 Use a virtualized environment where possible
    $nb_HardCodedPathNotGreen += GreenIT::lib::HardCodedPathNotGreen::checkHardCodedPathNotGreen($HString, $techno, $text, $rulesDescription);

    $status |= Couples::counter_add($compteurs, $mnemo_HardCodedPathNotGreen, $nb_HardCodedPathNotGreen);
    return $status;
}


sub IsLoop ($) {
    my $pattern = shift;

    if ($pattern =~ /\b(?:times|until|from)\b/i) {
        return 1;
    }
    else {
        return 0;
    }
}

sub testParsingPotentialLoop($) {
    my $buffer = shift;
    my $bodyLoop = "";

    my $countPerform = 1;
    while ($$buffer =~ /(.*?(?:\bend\-perform\.))/sig) {

        my $catch = $1;

        if (defined $catch && $catch ne "") {
            $bodyLoop .= $catch;
            $countPerform += () = $catch =~ /(?<!end\-)perform\b/ig;
            $countPerform -= () = $catch =~ /\bend\-perform\b/ig;
            #$countPerform -= () = $catch =~ /\./g;

            if ($countPerform == 0) {
                last;
            }
        }

    }

    if ($countPerform == 0) {
        # test if it's a real loop
        my $result = IsLoop($bodyLoop);
        if ($result == 1) {
            return $bodyLoop;
        }
    }
}

1;



