package CommonOutput::ElaborateLog;

use strict;
use warnings;
use File::Basename;
use File::Spec;

use AnalyseUtil;
use CommonOutput::ManageVue;
use Lib::SHA;

# list of technos which support bookmark patterns in IDE
my $expectedTechnosForIDE = {
    'JAVA'  => "", 'CSHARP' => "", 'JS' => "", 'PHP' => "", 'VB' => "", 'PYTHON' => "", 'TYPESCRIPT' => "",
    'SCALA' => "", 'CLOJURE' => "", 'CCPP' => "", 'COBOL' => "", 'GO' => "", 'RUBY' => "", 'PLSQL' => "",
    'KOTLIN' => ""
};

my %SCAN_LOG = ();
my $DEBUG = 0;

my $techno;
my $parameters = AnalyseUtil::recuperer_options();
my $ideOutputOpt = $parameters->[0]->{'--dbgMatchPatternDetail'};
my $technoLabelInPortal = '';

# Agent GUI is not using the --language option
# techno name is retrieved from GetCurrentAnaName() in Analyse.pl
sub init($) {
    my $options = shift;
    $techno = $options->{'--language'};
    $technoLabelInPortal = getTechnoLabelInPortal($techno);
}

# option --dbgMatchPatternDetail must be activated for IDE export

sub ElaborateLogPartOne ($$$$;$) {
    my $view = shift;
    my $regex = shift;
    my $mnemoID = shift;
    my $rulesDescription = shift;
    my $fileProjectExtension = shift; # in case of project file

    if (!defined $view || !defined $mnemoID) {
        print STDERR "[ElaborateLogPartOne] Error missing view or mnemo!!!\n";
    }

    # /!\ avoid the following issue:
    # "strings with code points over 0xFF may not be mapped into in-memory file handles"
    #    search & replace non ascii characters (may be encountered in comments)
    $$view =~ s/[^[:ascii:]]/ /g;

    # count occurrence pattern with more precision
    # search scope is multiline
    # position reset by default
    pos($$view) = undef;
    my $nb_patterns = () = ($$view =~ /$regex/gm);
    pos($$view) = undef;

    my $aggloView;
    my $SHA;

    # IDE EXPORT OPTION ACTIVATED & TECHNO IS EXPECTED
    if (defined $ideOutputOpt && exists $expectedTechnosForIDE->{$technoLabelInPortal}) {
        $SHA = getSHA();

        $aggloView = CommonOutput::ManageVue::getAggloView();

        if (!defined $$aggloView) {
            print STDERR "[ElaborateLog::isCommentLine] WARNING: agglo View not defined (comment lines are not detecting properly)\n";
        }

        open my $handle, '<', $view;
        # bookmark search is done line by line
        while (<$handle>) {
            # use "while" statement in case of multiple detection by line
            while (defined $_ && /$regex/g) {

                my $numline = $.;
                my $boolComment = 0;

                # project files
                if (defined $fileProjectExtension) {
                    if ($fileProjectExtension eq 'XML' && $_ =~ /^\<\!\-\-/m) {
                        $boolComment = 1;
                    }
                }
                # techno files
                else {
                    if (defined $$aggloView) {
                        $boolComment = isCommentLineNew($aggloView, $numline);
                    }
                }

                if ($boolComment == 0) {
                    # $-[0] position before matching
                    # $+[0] position after matching
                    my $previousPos = $-[0];
                    my $lastpos = $+[0];

                    # print "regex: $regex\n";
                    # print "pattern found at line $numline between $previousPos and $lastpos\n";

                    # rule should be present in portal
                    if (exists $rulesDescription->{$mnemoID}->{$technoLabelInPortal}) {

                        foreach my $ruleName (keys %{$rulesDescription->{$mnemoID}->{$technoLabelInPortal}}) {
                            my $ruleLabel = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[0];
                            my $urlDoc = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[1];
                            my $formula = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[2];
                            my $kindIndicator = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[3];
                            # print "ruleLabel=$ruleLabel\n";
                            # print "$ruleLabel : add to scanlog: $numline\[$previousPos,$lastpos]\|\n";

                            # avoid duplicates
                            if (exists $SCAN_LOG{$ruleLabel}{$SHA}[0]) {
                                if ($SCAN_LOG{$ruleLabel}{$SHA}[0] !~ /\b$numline\[$previousPos,$lastpos]\|/) {
                                    $SCAN_LOG{$ruleLabel}{$SHA}[0] .= "$numline\[$previousPos,$lastpos]\|";
                                }
                            }
                            else {
                                $SCAN_LOG{$ruleLabel}{$SHA}[0] = "$numline\[$previousPos,$lastpos]\|";
                            }
                            # urldoc
                            $SCAN_LOG{$ruleLabel}{$SHA}[1] = $urlDoc;
                            # multiPattern formula
                            if ($formula) {
                                $SCAN_LOG{$ruleLabel}{$SHA}[2] = $formula;
                                # avoid duplicates
                                if (exists $SCAN_LOG{$ruleLabel}{$SHA}[3]) {
                                    if ($SCAN_LOG{$ruleLabel}{$SHA}[3] !~ /$mnemoID\|/) {
                                        $SCAN_LOG{$ruleLabel}{$SHA}[3] .= $mnemoID . "\|";
                                    }
                                    else {
                                        # print "[ElaborateLogPartOne] Same string discovered at same line $numline\n";
                                    }
                                }
                                else {
                                    # print "$ruleLabel : add to formula: $mnemoID\n";
                                    $SCAN_LOG{$ruleLabel}{$SHA}[3] = $mnemoID . "\|";
                                }
                            }
                            # kind (booster/blocker)
                            $SCAN_LOG{$ruleLabel}{$SHA}[4] = $kindIndicator;
                        }
                    }
                }
            }
        }
        close $handle;
    }

    return $nb_patterns;
}

# sub actually used for project files (xml...)
# same as ElaborateLogPartOne but with performance improvement
sub ElaborateLogPartOneFromStringList ($$$$;$) {
    my $view = shift;
    my $stringsToSearch = shift;
    my $mnemoID = shift;
    my $rulesDescription = shift;
    my $fileConfig = shift;

    my $nb_patterns = 0;

    if (!defined $view || !defined $mnemoID) {
        print STDERR "[ElaborateLogPartOneByList] Error missing view or mnemo!!!\n";
    }

    my $handleView = $view;
    my $is_ref = ref($handleView);

    if ($is_ref eq 'REF') {
        $handleView = $$handleView;
    }

    my $aggloView;
    my $SHA;

    # /!\ avoid the following issue:
    # "strings with code points over 0xFF may not be mapped into in-memory file handles"
    #    search & replace non ascii characters (may be encountered in comments)
    $handleView =~ s/[^[:ascii:]]/ /g;

    if (exists $expectedTechnosForIDE->{$technoLabelInPortal}) {
        $SHA = getSHA();

        $aggloView = CommonOutput::ManageVue::getAggloView();

        if (!defined $$aggloView) {
            print STDERR "[ElaborateLog::isCommentLine] WARNING: agglo View not defined (comment lines are not detecting properly)\n";
        }
    }

    for my $strToSearch (keys %{$stringsToSearch}) {
        #print "searching $strToSearch\n";
        $strToSearch = quotemeta($strToSearch);

        # in text view scope of search is string ("') or tag in XML (<)
        my $regex = qr/["'>]$strToSearch["'<]/;

        open my $handle, '<', $handleView;
        # bookmark search is done line by line
        while (<$handle>) {

            # use "while" statement in case of multiple detection by line
            while (defined $_ && /$regex/g) {

                $nb_patterns++;

                # IDE EXPORT OPTION ACTIVATED & TECHNO IS EXPECTED
                if (defined $ideOutputOpt && exists $expectedTechnosForIDE->{$technoLabelInPortal}) {

                    my $numline = $.;

                    my $boolComment = 0;
                    # project files
                    if (defined $fileConfig) {
                        if ($fileConfig =~ /\.xml$/im && $_ =~ /^\<\!\-\-/m) {
                            $boolComment = 1;
                        }
                    }
                    # techno files
                    else {
                        if (defined $$aggloView) {
                            $boolComment = isCommentLineNew($aggloView, $numline);
                        }
                    }

                    if ($boolComment == 0) {
                        # $-[0] position before matching
                        # $+[0] position after matching
                        my $previousPos = $-[0];
                        my $lastpos = $+[0];
                        $nb_patterns++;

                        # rule should be present in portal
                        if (exists $rulesDescription->{$mnemoID}->{$technoLabelInPortal}) {

                            foreach my $ruleName (keys %{$rulesDescription->{$mnemoID}->{$technoLabelInPortal}}) {
                                my $ruleLabel = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[0];
                                my $urlDoc = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[1];
                                my $formula = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[2];
                                my $kindIndicator = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleName}->[3];

                                # print "ruleName = $ruleName\n";
                                # print "$numline\[$previousPos,$lastpos\]\n";
                                $SCAN_LOG{$ruleLabel}{$SHA}[0] .= "$numline\[$previousPos,$lastpos]\|";

                                # urldoc
                                $SCAN_LOG{$ruleLabel}{$SHA}[1] = $urlDoc;

                                # multiPattern formula
                                if ($formula) {
                                    $SCAN_LOG{$ruleLabel}{$SHA}[2] = $formula;
                                    $SCAN_LOG{$ruleLabel}{$SHA}[3] .= $mnemoID . "\|";
                                }
                                # kind (booster/blocker)
                                $SCAN_LOG{$ruleLabel}{$SHA}[4] = $kindIndicator;
                            }
                        }
                    }
                }
            }
        }
        close $handle;

    }

    #print "nb_patterns=$nb_patterns\n";
    return $nb_patterns;
}

sub ElaborateLogPartTwo ($) {
    my $file = shift;

    # IDE EXPORT OPTION ACTIVATED & TECHNO IS EXPECTED
    if (defined $ideOutputOpt && exists $expectedTechnosForIDE->{$technoLabelInPortal}) {

        my $SHA = getSHA();
        # check formula for blockers multi pattern
        foreach my $alert (keys %SCAN_LOG) {
            if (exists $SCAN_LOG{$alert}{$SHA} && $SCAN_LOG{$alert}{$SHA}[3]) {
                my @split_values = split(/\|/, $SCAN_LOG{$alert}{$SHA}[3]);
                foreach my $val (@split_values) {
                    $SCAN_LOG{$alert}{$SHA}[2] =~ s/\b$val\b/1/g;
                }
                $SCAN_LOG{$alert}{$SHA}[2] =~ s/Id_[0-9]+/0/g;
                $SCAN_LOG{$alert}{$SHA}[2] =~ s/\&\&/\*/g;
                $SCAN_LOG{$alert}{$SHA}[2] =~ s/\|\|/\+/g;
                # if formula is not valid => delete data
                if (!eval $SCAN_LOG{$alert}{$SHA}[2]) {
                    # print "formula not valid => delete $alert\n";
                    delete $SCAN_LOG{$alert}{$SHA};
                }
            }
        }

        # replace SHA
        foreach my $alert (keys %SCAN_LOG) {
            if (exists $SCAN_LOG{$alert}{$SHA}) {
                my $filename = basename($file);
                $SCAN_LOG{$alert}{$filename}{$file} = delete $SCAN_LOG{$alert}{$SHA};
            }
        }
    }
}

sub SimpleElaborateLog ($$$$) {
    my $numline = shift;
    my $mnemoID = shift;
    my $rulesDescription = shift;
    my $ruleNamePortal = shift;

    # IDE EXPORT OPTION ACTIVATED & TECHNO IS EXPECTED
    if (defined $ideOutputOpt && exists $expectedTechnosForIDE->{$technoLabelInPortal}) {
        # rule should be present in portal
        if (exists $rulesDescription->{$mnemoID}->{$technoLabelInPortal}) {

            my $SHA = getSHA();

            my $ruleLabel = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleNamePortal}->[0];
            my $urlDoc = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleNamePortal}->[1];
            # my $formula = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleNamePortal}->[2];
            my $kindIndicator = $rulesDescription->{$mnemoID}->{$technoLabelInPortal}->{$ruleNamePortal}->[3];

            # pattern detection only for a numline, position is not taken into account
            $SCAN_LOG{$ruleLabel}{$SHA}[0] .= "$numline\[\-\]\|";

            # urldoc
            $SCAN_LOG{$ruleLabel}{$SHA}[1] = $urlDoc;

            # kind (booster/blocker)
            $SCAN_LOG{$ruleLabel}{$SHA}[4] = $kindIndicator;
        }

    }
}

sub getScanLog {
    return \%SCAN_LOG;
}

sub isCommentLine($$) {
    my $techno = shift;
    my $lineWanted = shift;

    my $aggloView = CommonOutput::ManageVue::getAggloView();

    if (defined $$aggloView) {
        # get agglo view : P is a code line & C is a comment line
        my $view = $aggloView;
        my $regex = qr/(?<!P)C/;
        # /!\ avoid the following issue:
        # "strings with code points over 0xFF may not be mapped into in-memory file handles"
        #    search & replace non ascii characters (may be encountered in comments)
        $$view =~ s/[^[:ascii:]]/ /g;
        open my $fh, '<', $view;
        while (<$fh>) {
            if ($. == $lineWanted) {
                # my $match = $_;
                if ($_ =~ /$regex/) {
                    return 1;
                }
            }
        }
        return 0;
    }
    else {
        print STDERR "[ElaborateLog::isCommentLine] ERROR: agglo View not defined for $techno (comment lines are not detecting correctly)\n";
        return 1;
    }
}

sub isCommentLineNew($$) {
    my $aggloView = shift;
    my $lineWanted = shift;

    # agglo view : P is a code line & C is a comment line
    my $regex = qr/(?<!P)C/;
    # /!\ avoid the following issue:
    # "strings with code points over 0xFF may not be mapped into in-memory file handles"
    #    search & replace non ascii characters (may be encountered in comments)
    $$aggloView =~ s/[^[:ascii:]]/ /g;
    open my $fh, '<', $aggloView;
    while (<$fh>) {
        if ($. == $lineWanted) {
            # my $match = $_;
            if ($_ =~ /$regex/) {
                # it's a comment line
                return 1;
            }
        }
    }

    return 0;
}


sub getSHA() {

    # get binary view
    my $binaryView = CommonOutput::ManageVue::getBinaryView();

    if (defined $$binaryView) {
        return Lib::SHA::SHA256($binaryView);
    }
    else {
        print STDERR "[ElaborateLog::getSHA] WARNING: binary View not defined (SHA file is not computed properly)\n";
        return 0;
    }
}

sub getTechnoLabelInPortal($) {
    my $techno = shift;

    my $technoLabelInPortal;
    # techno name is not the same between CLI & portal
    if (uc($techno) eq 'CS') {
        $technoLabelInPortal = 'CSHARP'
    }
    elsif (uc($techno) eq 'VBDOTNET') {
        $technoLabelInPortal = 'VB'
    }
    else {
        $technoLabelInPortal = uc($techno);
    }

    return $technoLabelInPortal;
}

1;
