package CommonOutput::detection;

use strict;
use warnings;

my $META_DATA = undef;
my $CURRENT_FILE_NAME;
my $CURRENT_UNIT_NAME;
my $TARGETS = {
	"cloudready"	=> "CloudReady",
	"greenit"		=> "GreenIT"
};

# In case the techno being analysed is a master techno composed with several sub technos (ex CCpp that contains C and Cpp),
# this variable contain the real techno (sub techno) of the current file being analysed ...
my $CURRENT_FILE_TECHNO;

my $UNIT_MODE = 0;
# FILE_MODE targets are targets whose detections are not issued by units processing. So these targets needs the FILE_MODE management.
# Concerns only internal units mode : the files are split logically, in opposiion with external units mode, where files split in a temporary directory and each unit treat as a file.
my %FILE_MODE_TARGET = (
	"CloudReady" => 1
);

my %TARGETS_DATA = ();

# This hash aim to associate each real techno (not master techno), if any, with a list of mnemonics in order to know, in the case the mnemonic is not present for a given file,
# if it is an error or if it is normal (the real underlying techno is not concerned with the mnemo).
# This is usefull, in this cas, to decide if a n error message should be issued or not !
my %TECHNO_MNEMO_LIST = ();

sub setUnitMode() {
	$UNIT_MODE = 1;
}

sub setMetaData($) {
	$META_DATA = shift;
}

sub getFileDataValue($$) {
	my $target = shift;
	my $mnemo = shift;
	return $TARGETS_DATA{ $target }->{'FILE_DATA'}->{$mnemo};
}

sub initFileData($$) {
	my $target = shift;
	my $user_FileData = shift;
	
	$TARGETS_DATA{ $target }->{'FILE_DATA'} = $user_FileData;
}

sub init($$$;$) {
	my $target = shift;
	my $techno = shift;
	my $USER_CONTEXT = shift;
	
	my $subTechno = shift;
	
#print STDERR "---> INIT $techno/$target\n";	
	my $canonic_target = $TARGETS->{lc($target)};
	
	# Check if mnemonics are registered. If none, do no proceed to init, because the techno is not concerned by GreenIT ...
	if ( (! exists $USER_CONTEXT->{'FILE_MNEMO_LIST'}) && (! exists $TARGETS_DATA{$canonic_target}->{'FILE_MNEMO_LIST'})) {
		return 1;
	}
	
	if (defined $TARGETS_DATA{ $canonic_target }->{'activated'}) {
		print STDERR "WARNING : attempt to init target $canonic_target twice !!\n";
		return 0;
	}
	else {
		$TARGETS_DATA{ $canonic_target }->{'activated'} = 1;
	}
	
	$UNIT_MODE = 0;
	
	# Add user init data	
	#$TARGETS_DATA{ $canonic_target }->{'FILE_DATA'} = $USER_CONTEXT->{'FILE_DATA'};
	initFileData($canonic_target, $USER_CONTEXT->{'FILE_DATA'});
	$TARGETS_DATA{ $canonic_target }->{'APPLI_DATA'} = $USER_CONTEXT->{'APPLI_DATA'};
	if (exists $USER_CONTEXT->{'FILE_MNEMO_LIST'}) {
		addFileMnemoList($target, $USER_CONTEXT->{'FILE_MNEMO_LIST'}, $subTechno);
	}
	if (exists $USER_CONTEXT->{'APPLI_MNEMO_LIST'}) {
		$TARGETS_DATA{ $canonic_target }->{'APPLI_MNEMO_LIST'} = $USER_CONTEXT->{'APPLI_MNEMO_LIST'};
	}

	$TARGETS_DATA{ $canonic_target }->{'OUTPUT_FILE_HANDLER'} = $USER_CONTEXT->{'OUTPUT_FILE_HANDLER'}; # FIXME no sens
	
	
	# Open CSV and write header
	my $r_OUTPUT_FILE_HANDLER = $TARGETS_DATA{ $canonic_target }->{'OUTPUT_FILE_HANDLER'};
		
	my $OUTPUT_FILE_NAME = $TARGETS_DATA{ $canonic_target }->{'OUTPUT_FILE_NAME'};
	
	if (! defined $target) {
		print STDERR ("[CommonOutput::detection::init] ERROR : initialize target $canonic_target\n");
		return 1;
	}
	
	if (! defined $$r_OUTPUT_FILE_HANDLER) {
		if (defined $OUTPUT_FILE_NAME) {
			my $ret = open($$r_OUTPUT_FILE_HANDLER, ">:encoding(utf-8)", $OUTPUT_FILE_NAME);

			if (! defined $ret) {
				print STDERR "[CommonOutput::detection::init] ERROR : unable to open output file name $OUTPUT_FILE_NAME...\n";
			}

			# print metadat header
			if (defined $META_DATA) {
				print $$r_OUTPUT_FILE_HANDLER("#$canonic_target\n");
				print $$r_OUTPUT_FILE_HANDLER("#uuid;".$META_DATA->{"uuid"}."\n");
				print $$r_OUTPUT_FILE_HANDLER("#start_date;".$META_DATA->{"start_date"}."\n");
				print $$r_OUTPUT_FILE_HANDLER("#version_highlight;".$META_DATA->{"version_highlight"}."\n");
			}
			else {
				print STDERR "[CommonOutput::detection::init] ERROR : missing META DATA. Cannot create header for $OUTPUT_FILE_NAME\n";
			}

			# print mnemo list
			print $$r_OUTPUT_FILE_HANDLER ("FILE SECTION\n\n");
			print $$r_OUTPUT_FILE_HANDLER ("section=$techno\n");
			print $$r_OUTPUT_FILE_HANDLER ("Dat_FileName;");
			print $$r_OUTPUT_FILE_HANDLER ("Dat_AbortCause;");
			for my $mnemo (@{$TARGETS_DATA{ $canonic_target }->{'FILE_MNEMO_LIST'}}) {
				print $$r_OUTPUT_FILE_HANDLER ("$mnemo;");
			}
			print $$r_OUTPUT_FILE_HANDLER ("\n");
		}
		else {
			print "[CommonOutput::detection::init] ERROR : output file name is undefined ...\n";
		}
	}

	return 0;
}

sub addFileDetection($$$$) {
	my $target = shift;
	my $mnemo = shift;
	my $value = shift;
	my $file = shift;

	my $canonical_target = $TARGETS->{lc($target)};
	
	if (! defined $canonical_target) {
		print STDERR "[CommonOutput::detection::addFileDetection] WARNING : attempt to add mnemonic value for unknow target '$target' !\n";
		return;
	}
	$TARGETS_DATA{ $canonical_target }->{'FILE_DATA'}->{$mnemo} = $value;
}

sub addAppliDetection($$$) {
	my $target = shift;
	my $mnemo = shift;
	my $data = shift;

	my $canonical_target = $TARGETS->{lc($target)};

	if (! defined $canonical_target) {
		print STDERR "[CommonOutput::detection::addAppliDetection] WARNING : attempt to add mnemonic value for unknow target '$target' !\n";
		return;
	}

	if (!exists $TARGETS_DATA{ $canonical_target }->{'APPLI_DATA'}->{$mnemo}) {
		$TARGETS_DATA{ $canonical_target }->{'APPLI_DATA'}->{$mnemo} = [];
	}

	if (ref $data eq "SCALAR") {
		push @{$TARGETS_DATA{ $canonical_target }->{'APPLI_DATA'}->{$mnemo}}, $data;
	}
	else {
		push @{$TARGETS_DATA{ $canonical_target }->{'APPLI_DATA'}->{$mnemo}}, @$data;
	}
}

sub setFileAbortCause($) {
	my $AbortCause  = shift;

	for my $target (keys %TARGETS_DATA) {
		$TARGETS_DATA{$target}->{'FILE_DATA'}->{'Dat_AbortCause'} = $AbortCause;
	}
}

sub setUnitAbortCause($) {
	my $AbortCause  = shift;
	
	for my $target (keys %TARGETS_DATA) {
		$TARGETS_DATA{$target}->{'FILE_DATA'}->{'Dat_AbortCause'} = $AbortCause;
	}
}


sub setOutputFileName($$) {
	my $target = shift;
	my $outfile = shift;
	
	my $canonic_target = $TARGETS->{lc($target)};
	
	print "[$canonic_target] output file set to $outfile.\n";
	$TARGETS_DATA{$canonic_target}->{'OUTPUT_FILE_NAME'} = $outfile
	#my $outputDir = $outfile;
	#$outputDir =~ s/[^\\\/]*$//;
}

sub addFileMnemoList($$;$) {
	my $target = shift;
	my $input = shift;
	
	# indicate the real techno, and can be undefined if useless. Usefull only in case of master techno with subtechno (files have not the same techno, so provide the information).
	# if undefined, then the techno of the file is the techno of the analysis, given in initialisation of this module.
	my $fileTechno = shift;
	
	if (! exists $TARGETS_DATA{$target}) {
		$TARGETS_DATA{$target} = {};
	}
		
	if (! exists $TARGETS_DATA{$target}->{'FILE_MNEMO_LIST'}) {
		# tab for order conservation.
		$TARGETS_DATA{$target}->{  'FILE_MNEMO_LIST'} = [];
		# hash for rapid existance check.
		$TARGETS_DATA{$target}->{'H_FILE_MNEMO_LIST'} = {};
	}

	my $ref_fileMnemoList = $TARGETS_DATA{$target}->{'FILE_MNEMO_LIST'};
	my $H_fileMnemoList = $TARGETS_DATA{$target}->{'H_FILE_MNEMO_LIST'};
	for my $mnemo (@$input) {
		
		# record the mnemo for the csv
		if (! exists $H_fileMnemoList->{$mnemo}) {
#print STDERR "ADD mnemo $mnemo to target $target\n";
			push @{$ref_fileMnemoList}, $mnemo;
			$H_fileMnemoList->{$mnemo} = 1;
		}
#else {
#	print STDERR "ALREADY PRESENT $mnemo to target $target\n";
#}		
		
		# if any, associate the mnemo with the techno 
		if (defined $fileTechno) {
			$TECHNO_MNEMO_LIST{$fileTechno}->{$mnemo} = 1;
		}
	}
}


sub add_Targets_FileMnemoList($;$) {
	my $input = shift;
	my $techno = shift;
	
	for my $target (keys %$input) {
		
		my $canonic_target = $TARGETS->{lc($target)};
		
		# the input contains unsupported target ...
		if (! defined $canonic_target) {
			print STDERR "[CommonOutput::detection::add_Targets_FileMnemoList] WARNING : attempt to register mnemonic list for unknow target '$target' !\n";
			next;
		}
		
		addFileMnemoList($canonic_target, $input->{$target}, $techno);
	}
}

sub commit($;$) {
	my $target = shift;
	my $filename = shift;
	
	if (! defined $TARGETS_DATA{ $target }->{'activated'} ) {
		return;
	}

	my $r_handle = $TARGETS_DATA{$target}->{'OUTPUT_FILE_HANDLER'};
	
	if (! defined $r_handle) {
		print STDERR "[CommonOutput::commit] no output handler for target $target !!!\n";
		return;
	}
	
	my $FILE_DATA = $TARGETS_DATA{$target}->{'FILE_DATA'};
	
	print $$r_handle ("$filename;");
	print $$r_handle ($FILE_DATA->{'Dat_AbortCause'}.";");
	for my $mnemo (@{$TARGETS_DATA{$target}->{'FILE_MNEMO_LIST'}}) {
		if (defined $FILE_DATA->{$mnemo}) {
			print $$r_handle ($FILE_DATA->{$mnemo}.";");
		}
		else {
			print $$r_handle ("0;");

			if ((! defined $CURRENT_FILE_TECHNO) || (exists $TECHNO_MNEMO_LIST{$CURRENT_FILE_TECHNO}->{$mnemo})) {
				# the mnemo is expected for the techno. It's an error if not present in the result ...
				print "[CommonOutput::detection::commit] WARNING : file $filename has no value for mnemmo $mnemo in $target detection. Assuming 0 ...\n";
			}
		}
	}
	print $$r_handle ("\n");
}

sub resetData() {
	for my $target (keys %TARGETS_DATA) {
		$TARGETS_DATA{ $target }->{'FILE_DATA'} = {};
	}
}

sub commitFile() {

	if (! defined $CURRENT_FILE_NAME) {
		print STDERR "[CommonOutput::detection::commitFile] undefined name for source file\n";
		return;
	}
	
	# write data related to current file
	for my $target (keys %TARGETS_DATA) {
		# do no commit in UNIT MODE (there is a dedicated treatment for that) unless the target is not adressed by units management
		if ((! $UNIT_MODE) || (exists $FILE_MODE_TARGET{$target})) {
			commit($target, $CURRENT_FILE_NAME);
		}
	}
}

sub excludeFile() {
	# nothing to do actually ...
}

sub setCurrentFileTechno($) {
	my $techno = shift;
	
#print STDERR "SET CURRENT TECHNO : $techno\n";
	$CURRENT_FILE_TECHNO = $techno;
}

sub setCurrentFile($;$) {
	my $new_file = shift;
	
	$UNIT_MODE = 0; # by default

	$CURRENT_FILE_NAME = $new_file;
	
	# reset data for the new file
	resetData();
}

sub setCurrentUnit($) {
	$CURRENT_UNIT_NAME = shift;
	
	$UNIT_MODE = 1;
	
	# reset data for the new file
	resetData();
}

sub commitUnit() {
	
	if (! defined $CURRENT_UNIT_NAME) {
		print STDERR "[CommonOutput::detection::commitUnit] undefined name for unit\n";
		return;
	}

	for my $target (keys %TARGETS_DATA) {
		# do not commit if the target is not adressed by units management.
		next if (exists $FILE_MODE_TARGET{$target});
		commit($target, $CURRENT_UNIT_NAME);
	}
	
	# reset data for the new file
	for my $target (keys %TARGETS_DATA) {
		$TARGETS_DATA{ $target }->{'FILE_DATA'} = {};
	}
}

sub dumpAppliDetections($) {
	my $target = shift;
	
	my $r_handle = $TARGETS_DATA{$target}->{'OUTPUT_FILE_HANDLER'};
	
	my $APPLI_DATA = $TARGETS_DATA{$target}->{'APPLI_DATA'};
		
	print $$r_handle ("\nAPPLI SECTION\n");
	for my $mnemo (@{$TARGETS_DATA{$target}->{'APPLI_MNEMO_LIST'}}) {
	# print mnemo in column 0
		print $$r_handle ("$mnemo;");
	
		if (exists $APPLI_DATA->{$mnemo}) {
			my $numberOfFiles = scalar @{$APPLI_DATA->{$mnemo}};
			print $$r_handle ("$numberOfFiles;");
		}
		else {
			print $$r_handle ("0;");
		}

		# print data related to detections
		my $lastElmt = $APPLI_DATA->{$mnemo}->[-1];
		for my $data (@{$APPLI_DATA->{$mnemo}}) {
			if ($data ne $lastElmt) {
				print $$r_handle ("$data\,");
			}
			else {
				print $$r_handle ("$data");
			}
		}
		print $$r_handle ("\n");
	}
}

sub dumpCommonOutputDetectionResults() {
	for my $target (keys %TARGETS_DATA) {
		# FIX bug caused by the return
		# APPLI SECTION of CloudReady was not always dumped in csv
		#if (! defined $TARGETS_DATA{ $target }->{'activated'} ) {
		#	return;
		#}
		if (defined $TARGETS_DATA{ $target }->{'activated'} ) {
			# this test is at first glance useless (due to the previous one, more general) ...
			if ((exists $TARGETS_DATA{$target}->{'CloudReadyDetectionActivated'}) and (!$TARGETS_DATA{$target}->{'CloudReadyDetectionActivated'})) {
				print "[CommonOutput::detection] INFO: no $target detection results to dump...\n";
			}
			else {
				my $r_handler = $TARGETS_DATA{$target}->{'OUTPUT_FILE_HANDLER'};
				if (defined $r_handler) {
					dumpAppliDetections($target);
					close ${$TARGETS_DATA{$target}->{'OUTPUT_FILE_HANDLER'}};
				}
			}
		}
	}
}

sub dumpScanLogDetails($$$$) {
	my $techno = shift;
	my $SCAN_LOG = shift;
	my $srcDir = shift; # from --dir-source CLI parameter

	my $outdir = shift;

	if (defined $outdir) {
		my $logfile_cloud = File::Spec->catfile($outdir, "cloudDetail_$techno.csv");
		my $logfile_green = File::Spec->catfile($outdir, "greenDetail_$techno.csv");

		# GREENIT
		open(LOG_GREEN, ">:encoding(utf-8)", $logfile_green);
		print LOG_GREEN "[$techno] Scan " . localtime() . "\n";
		print LOG_GREEN "[dir project] " . File::Spec->catfile($srcDir) . "\n";
		print LOG_GREEN "\nAlert;Path;LineNumberPos;Url_doc\n";

		# BOOSTERS + BLOCKERS CLOUD
		open(LOG_CLOUD, ">:encoding(utf-8)", $logfile_cloud);
		print LOG_CLOUD "[$techno] Scan " . localtime() . "\n";
		print LOG_CLOUD "[dir project] " . File::Spec->catfile($srcDir) . "\n";
		print LOG_CLOUD "\nAlert;Path;LineNumberPos;Url_doc;Kind\n";

		# 1st sort by alerts
		foreach my $alert (sort keys %{$SCAN_LOG}) {
			# 2nd sort by filenames
			foreach my $filename (sort keys %{$SCAN_LOG->{$alert}}) {
				foreach my $path (keys %{$SCAN_LOG->{$alert}->{$filename}}) {
					my $pathLabel = File::Spec->catfile($path);
					my $numLinePos = $SCAN_LOG->{$alert}->{$filename}->{$path}->[0];
					# 3rd sort by numline & positions
					my @numLinePos = split(/\|/, $numLinePos);
					$numLinePos = "";
					foreach my $elt (sort {(split(/\[/, $a))[0] <=> (split(/\[/, $b))[0]} @numLinePos) {
						$numLinePos .= $elt . "\|";
					}
					chop $numLinePos;
					my $kind = $SCAN_LOG->{$alert}->{$filename}->{$path}->[4];
					my $urlDoc = $SCAN_LOG->{$alert}->{$filename}->{$path}->[1];
					if (defined $kind) {
						# BLOCKER CLOUD
						if ($kind eq 'BLOCKER') {
							print LOG_CLOUD "\"$alert\";$pathLabel;$numLinePos;$urlDoc;BLOCKER";
							print LOG_CLOUD "\n";
						}
						# BOOSTER CLOUD
						else {
							print LOG_CLOUD "\"$alert\";$pathLabel;$numLinePos;$urlDoc;BOOSTER";
							print LOG_CLOUD "\n";
						}
					}
					# GREENIT ($kind = undefined)
					else {
						print LOG_GREEN "\"$alert\";$pathLabel;$numLinePos;$urlDoc";
						print LOG_GREEN "\n";
					}
				}
			}
		}
		close LOG_CLOUD;
		close LOG_GREEN;
	}
}

1;
