package JS::CountLoop;
# les modules importes
use strict;
use warnings;
use Carp::Assert; # traces_filter_line
use Erreurs;
use Couples;

use Lib::NodeUtil;
use JS::JSNode;

use CommonOutput::ElaborateLog;

use GreenIT::lib::KeepPortalData;

my $While__mnemo = Ident::Alias_While();
my $Do__mnemo = Ident::Alias_Do();
my $For__mnemo = Ident::Alias_For();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $ForinLoop__mnemo = Ident::Alias_ForinLoop();
my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $InstantiationInLoop__mnemo = Ident::Alias_InstantiationInLoop();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $LoopCondComparedToNonZero__mnemo = Ident::Alias_LoopCondComparedToNonZero();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();

my $nb_Do = 0;
my $nb_For = 0;
my $nb_While = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_ForinLoop = 0;
my $nb_ConcatInLoop = 0;
my $nb_InstantiationInLoop = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_LoopCondComparedToNonZero = 0;
my $nb_ToManyNestedLoop = 0;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

sub checkNestedLoop($) {
	my $loop = shift;

	# find nested loops ...
	my @subloop = GetNodesByKindList($loop, [WhileKind, ForKind, DoKind], 1);
	for my $sloop (@subloop) {
		$nb_ToManyNestedLoop++;
		my $numline = $sloop->[5];
		CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
		Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "sub loop at line ".GetLine($sloop));
	}
}

sub CountLoop($$$)
{
	my ($file, $vue, $compteurs) = @_;

	my $ret = 0;
	$nb_Do = 0;
	$nb_For = 0;
	$nb_While = 0;
	$nb_FunctionCallInLoopTest = 0;
	$nb_ForinLoop = 0;
	$nb_LoopCondComparedToNonZero = 0;
	$nb_ToManyNestedLoop = 0;

	my $root = $vue->{'structured_code'};

	if (!defined $root) {
		$ret |= Couples::counter_add($compteurs, $While__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $Do__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $For__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $ForinLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $LoopCondComparedToNonZero__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my @While = GetNodesByKind($root, WhileKind);
	my @Do = GetNodesByKind($root, DoKind);
	my @For = GetNodesByKind($root, ForKind);

	$nb_While = scalar @While;
	$nb_Do = scalar @Do;
	$nb_For = scalar @For;

	my @Loops = (@While, @Do, @For);

	for my $loop (@Loops) {

		my $condNode = Lib::NodeUtil::GetChildren($loop)->[0];
		my $condTest = ${Lib::NodeUtil::GetXKindData($condNode, 'flatCond')};

		# Check condition
		#if (!IsKind($loop, DoKind)) {

			# get the first node if any ...
			my @calls = GetNodesByKind($condNode, FunctionCallKind, 1);
			if (scalar @calls > 0) {
				$nb_FunctionCallInLoopTest++;
				my $numline = $condNode->[5];
				# print "Avoid calling a function in a condition loop at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
				Erreurs::VIOLATION($FunctionCallInLoopTest__mnemo, "Avoid calling a function in a condition loop at line $numline");

				#print "----> FUNCTION CALL IN LOOP END TEST\n";
			}

			if (IsKind($loop, ForKind)) {
				#print "loop cond = $condTest\n";
				if ($condTest =~ /\bin\b/s) {
					$nb_ForinLoop++;
				}
			}
		#}
		
		# check simple conditions using operators <, >, <=, >=, ===, ==, !=
		my $bool_comparisonToZero;
		if ($condTest !~ /\&\&|\|\|/) {
			if ( $condTest =~ /([^<>!=\s]+)\s*(?:<=|>=|!==|!=|===|==|=|<|>)\s*([^<>!=\s]+)/ ) {
				$bool_comparisonToZero = 0;
				if (($1 =~ /(?:^|[^\.\w])0$/m) || ($2 =~ /^0(?:$|[^\.\w])/m)) {
					$bool_comparisonToZero = 1;
				}
			}
			if ((defined $bool_comparisonToZero) && ( $bool_comparisonToZero == 0)) {
				$nb_LoopCondComparedToNonZero++;
				my $numline = GetLine($condNode);
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_LoopCondComparedToNonZero, $rulesDescription, "LoopCondComparedToNonZero");
				Erreurs::VIOLATION($LoopCondComparedToNonZero__mnemo, "Loop condition is not comparison to zero at line ".GetLine($condNode));
			}
		}
		
		checkNestedLoop($loop)
	}

	$ret |= Couples::counter_add($compteurs, $While__mnemo, $nb_While);
	$ret |= Couples::counter_add($compteurs, $Do__mnemo, $nb_Do);
	$ret |= Couples::counter_add($compteurs, $For__mnemo, $nb_For);
	$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, $nb_FunctionCallInLoopTest);
	$ret |= Couples::counter_add($compteurs, $ForinLoop__mnemo, $nb_ForinLoop);
	$ret |= Couples::counter_add($compteurs, $LoopCondComparedToNonZero__mnemo, $nb_LoopCondComparedToNonZero);
	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);

	return $ret;
}

sub CountPatternInLoop($$$) {
	my ($file, $vue, $compteurs) = @_;

	my $ret = 0;

	$nb_ConcatInLoop = 0;
	$nb_InstantiationInLoop = 0;
	$nb_OpenCloseFileInLoop = 0;

	my $root = $vue->{'structured_code'};
	my $code = \$vue->{'code'};

	if (!defined $root) {
		$ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my @loops = GetNodesByKindList($root, [ WhileKind, ForKind, DoKind ], 1);
	# HL-2022 Avoid OPEN/CLOSE inside loops
	# codeBody is in lowercase
	my $regexOpen = qr/\.?\bopen\s*\(.*?\)|\bcreateReadStream\b|\bcreateWriteStream\b/i;
	my $regexClose = qr/\.(?:close|destroy)\b/i;
	for my $loop (@loops) {
		my $codeBody = Lib::NodeUtil::GetXKindData($loop, 'codeBody');
		my $bodyBeginLine = Lib::NodeUtil::GetXKindData($loop,'bodyBeginLine');

		# check presence of literal strings concaténations (variables strings cannot be detected !)
		my $lineOffset = 0;
		while ($$codeBody =~ /(\n+)|(\bCHAINE_\d+)|(\+=)|(\+)|($regexOpen)|($regexClose)|(\bnew\b)/gc) {
			if (defined $1) {
				$lineOffset += () = $1 =~ /\n/g;
			}
			elsif (defined $7) {
				$nb_InstantiationInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
				Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation in loop at line " . ($bodyBeginLine + $lineOffset));
			}
			elsif (defined $2) {
				# CHAINE_XX
				if ($$codeBody =~ /\G(\s*)\+/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
				}
			}
			elsif (defined $5 || defined $6) {
				$lineOffset += () = $5 =~ /\n/g if (defined $5);
				$lineOffset += () = $6 =~ /\n/g if (defined $6);
				$nb_OpenCloseFileInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				# print "Open/close file inside loop at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
				Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
			}
			else {
				# + first or +=
				if ($$codeBody =~ /\G(\s*)\bCHAINE_\d+(\s*)(\+)?/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));

					$lineOffset += () = $2 =~ /\n/g;
					if (defined $3) {
						$nb_ConcatInLoop++;
						my $numline = $bodyBeginLine + $lineOffset;
						CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
						Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
					}
				}
			}
		}
	}

	$ret |= Couples::dispatch($ConcatInLoop__mnemo, $nb_ConcatInLoop);
	$ret |= Couples::dispatch($InstantiationInLoop__mnemo, $nb_InstantiationInLoop);
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);

	return $ret;
}

1;
