package JS::CountOpenedFiles;

use warnings;
use strict;

use Lib::NodeUtil;
use JS::JSNode;

use GreenIT::lib::KeepPortalData;
my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $OpenedFileWithoutClose_mnemo = Ident::Alias_OpenedFileWithoutClose();

my $nb_OpenedFileWithoutClose = 0;

sub CountOpenedFiles() {
	my ($fichier, $views, $compteurs, $options) = @_;

	my $status = 0;
	$nb_OpenedFileWithoutClose = 0;

	my $ret = 0;

	my $KindsLists = $views->{'KindsLists'};

	if (!defined $KindsLists) {
		$status |= Couples::counter_add($compteurs, $OpenedFileWithoutClose_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my $kinds = [ $KindsLists->{'FunctionDeclaration'} ];

	my $regexOpen = qr/\.?\bopen\s*\(.*?\)|\bcreateReadStream\b|\bcreateWriteStream\b/i;
	my $regexClose = qr/\.(?:close|destroy)\b/i;
	for my $kind (@{$kinds}) {
		foreach my $element (@{$kind}) {
			my $elementCodeBody = Lib::NodeUtil::GetKindData($element)->{'codeBody'};
			my $bodyBeginLine = Lib::NodeUtil::GetKindData($element)->{'bodyBeginLine'};
			if (defined $elementCodeBody) {
				# elementCodeBody is in lowercase
				# HL-2022 Avoid Programs not using explicitly OPEN and CLOSE files
				my $count_open = 0;
				my @numlineOpen;
				my $numlineBody = $bodyBeginLine;
				while ($$elementCodeBody =~ /(\n+)|($regexOpen)|($regexClose)/sg) {
					if (defined $1) {
						$numlineBody += () = $1 =~ /\n/g;
					}
					elsif (defined $2) {
						$numlineBody += () = $2 =~ /\n/g;
						push(@numlineOpen, $numlineBody);
						$count_open++;
					}
					elsif (defined $3) {
						$numlineBody += () = $3 =~ /\n/g;
						pop(@numlineOpen);
						$count_open--;
					}
				}

				if ($count_open > 0) {
					$nb_OpenedFileWithoutClose = $count_open;
					foreach my $numline (@numlineOpen) {
						# print "File opened without .Close() method at line $numline\n";
						CommonOutput::ElaborateLog::SimpleElaborateLog($numline, $OpenedFileWithoutClose_mnemo, $rulesDescription, "Avoid_OpenedFileWithoutClose");
						Erreurs::VIOLATION($OpenedFileWithoutClose_mnemo, "File opened without .Close() method at line $numline");
					}
				}
			}
		}
	}

	$ret |= Couples::dispatch($OpenedFileWithoutClose_mnemo, $nb_OpenedFileWithoutClose);
}

1;
