package Java::CountInstruction;
# les modules importes
use strict;
use warnings;
use Carp::Assert; # traces_filter_line
use Erreurs;
use Couples;
use Lib::NodeUtil;

use Java::JavaNode;

my $CaseLengthAverage__mnemo = Ident::Alias_CaseLengthAverage();
my $SmallSwitchCase__mnemo = Ident::Alias_SmallSwitchCase();
my $UnconditionalJump__mnemo = Ident::Alias_UnconditionalJump();
my $MissingBreakInSwitch__mnemo = Ident::Alias_MissingBreakInSwitch();

my $caseLenghtAverage = 0;
my $nb_caseLenghtAverage = 0;
my $nb_SmallSwitchCase = 0;
my $nb_UnconditionalJump = 0;
my $nb_MissingBreakInSwitch = 0;

sub CountInstruction($$$) 
{
	my ($file, $vue, $compteurs) = @_ ;
    
	my $ret = 0;
    $nb_caseLenghtAverage = 0;
    $nb_SmallSwitchCase = 0;
    $nb_UnconditionalJump = 0;
	$nb_MissingBreakInSwitch = 0;
   
	my $KindsLists = $vue->{'KindsLists'};
  
	if ( ! defined $KindsLists )
	{
		$ret |= Couples::counter_add($compteurs, $CaseLengthAverage__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $SmallSwitchCase__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $UnconditionalJump__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $MissingBreakInSwitch__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
  
	my $caseInstr = $KindsLists->{&CaseKind};
    my $countCases;
    my $TotalCaseLenght;
    
    # 18/12/2017 HL-332 Avoid long case statements    
	for my $case (@{$caseInstr}) 
    {
		    
        my $beginLineCase = GetLine ($case);
        my $endLineCase = GetEndline ($case);    
            
        my $lenghtCase = $endLineCase - $beginLineCase;
        $TotalCaseLenght+= $lenghtCase;
        $countCases++;
	}
    
    #print 'TotalCaseLenght='.$TotalCaseLenght."\n";
    #print 'countCases='.$countCases."\n";
    if ($countCases) {
		$nb_caseLenghtAverage = int($TotalCaseLenght/$countCases);
	}
	else {
		$nb_caseLenghtAverage = 0;
	}
    
    # print 'caseLenghtAverage='.$caseLenghtAverage."\n";
    Erreurs::VIOLATION($CaseLengthAverage__mnemo, "Case length average is $nb_caseLenghtAverage");
    
    # HL-404 Avoid small switch/Case
    # number of case or default instructions (CaseKind + DefaultKind)
    # if number < 3 => violation

    my $switchInstr = $KindsLists->{&SwitchKind};
    
	for my $switch (@{$switchInstr}) 
    {    
        my $thenNode = GetChildren($switch)->[1];
		my $countCasePerSwitch = 0;
        
        #foreach my $child_niv1 (@{$ref_children_niv1})
        #{      
            
            # print "child=" . GetKind ($child) ."\n";
            my $ref_LIST_cases_defaults = GetChildren($thenNode);
            
            foreach my $case_default (@{$ref_LIST_cases_defaults})
            {                 
                #if ( GetKind ($child_niv2) eq CaseKind or GetKind ($child_niv2) eq DefaultKind )
                #{
                    # print "child=" . GetKind ($child_niv2) ."\n";
                    $countCasePerSwitch++;
#print "CASE or DEFAULT\n";
                    
                    # 10/01/2018 HL-405 Avoid unconditional jump statement 
                    my $ref_LIST_statements = GetChildren($case_default);
                    
                    if (scalar @$ref_LIST_statements) {
						my $kind = GetKind($ref_LIST_statements->[-1]);
						if (($kind ne BreakKind) && ($kind ne ReturnKind) && ($kind ne ContinueKind) && ($kind ne ThrowKind) && ($kind ne GotoKind)) {
							# the last statement is not a "break"
							Erreurs::VIOLATION($MissingBreakInSwitch__mnemo, "Missing break for case/default at line ".GetLine($case_default));
							$nb_MissingBreakInSwitch++;
						}
					}
                                   
                    foreach my $stmtNode (@{$ref_LIST_statements})
                    {        

                        if ( GetKind ($stmtNode) eq BreakKind )
                        {
                            # print '2++++++' . GetKind ($child)."\n";
                            my $nextSibling = Lib::Node::GetNextSibling($stmtNode);
                            
							if ($nextSibling) {
								$nb_UnconditionalJump++;
								Erreurs::VIOLATION($UnconditionalJump__mnemo, "Unconditional jump at line ".GetLine($stmtNode));
							}
                        }
                    }
                #}
            }
        #}
        
	    if ($countCasePerSwitch < 3) {
			$nb_SmallSwitchCase++;
			Erreurs::VIOLATION($SmallSwitchCase__mnemo, "Small switch/case at line ".GetLine($switch)."\n");
		}
    }
	
    $ret |= Couples::counter_add($compteurs, $CaseLengthAverage__mnemo, $nb_caseLenghtAverage );
    $ret |= Couples::counter_add($compteurs, $SmallSwitchCase__mnemo, $nb_SmallSwitchCase );
    $ret |= Couples::counter_update($compteurs, $UnconditionalJump__mnemo, $nb_UnconditionalJump );
    $ret |= Couples::dispatch($MissingBreakInSwitch__mnemo, $nb_MissingBreakInSwitch);

    return $ret;
    
}

1;
