package Java::CountLoop;
# les modules importes
use strict;
use warnings;
use Carp::Assert; # traces_filter_line
use Erreurs;
use Couples;
use Lib::NodeUtil;

use Java::JavaNode;
use GreenIT::lib::KeepPortalData;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $UnconditionalJump__mnemo = Ident::Alias_UnconditionalJump();
my $EqualityInLoopCondition__mnemo = Ident::Alias_EqualityInLoopCondition();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $LoopCounterModification__mnemo = Ident::Alias_LoopCounterModification();
my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $InstantiationInLoop__mnemo = Ident::Alias_InstantiationInLoop();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $LoopCondComparedToNonZero__mnemo = Ident::Alias_LoopCondComparedToNonZero();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();
my $Break__mnemo = Ident::Alias_Break();
my $MultipleBreakLoops__mnemo = Ident::Alias_MultipleBreakLoops();

my $nb_UnconditionalJump = 0;
my $nb_EqualityInLoopCondition = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_LoopCounterModification = 0;
my $nb_ConcatInLoop = 0;
my $nb_InstantiationInLoop = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_LoopCondComparedToNonZero = 0;
my $nb_ToManyNestedLoop = 0;
my $nb_BreakLoop = 0 ;
my $nb_MultipleBreakLoops = 0 ;


sub checkLoopCounterModification($$) {
	my $condNode = shift;
	my $codeBody = shift;
	
	if ( GetKind ($condNode) eq ConditionKind )
	{          
        my $inc = Lib::NodeUtil::GetKindData($condNode)->{'inc'};
        if ($inc =~ /(\w+)\s*(?:\+|=[^=])|\+\+(\w+)/) {
			my $loopCounter = $1 || $2;
			
			# check if the loop counter is modified inside the body :
			if ($$codeBody =~ /(?:$loopCounter\s*(?:=[^=]|\+=|\+\+)|\+\+$loopCounter)/) {
				$nb_LoopCounterModification++;
				Erreurs::VIOLATION($LoopCounterModification__mnemo, "Loop counter $loopCounter is modified inside the body of the loop declared at line".GetLine($condNode));
			}
		}
	}
}

sub checkLoopBody($) {
	my $thenBody = shift;

	if ( GetKind($thenBody) eq ThenKind )
	{
#print '1++++++' . GetKind ($thenBody)."\n";

		my $ref_children_niv2 = GetChildren($thenBody);
		foreach my $child_niv2 (@{$ref_children_niv2})
		{   
			my $kind = GetKind ($child_niv2);
			if ( $kind eq ReturnKind 
				or $kind eq BreakKind 
				or $kind eq ContinueKind 
				or $kind eq ThrowKind )
			{
# print '2++++++' . GetKind ($child_niv2)."\n";
				$nb_UnconditionalJump++;
				Erreurs::VIOLATION($UnconditionalJump__mnemo, "Unconditional jump at line ".GetLine($child_niv2));
			}
		}
	}
}

sub checkLoopTermination($) {
	# 15/01/2018 HL-411 Avoid equality in loop termination condition
	my $condNode = shift;

	if (GetKind($condNode) eq ConditionKind) {
		my $cond = Lib::NodeUtil::GetKindData($condNode)->{'cond'};

		# more than one condition
		while ($cond =~ /(\S+)\s*[!=]=\s*(\S+)/g) {
			if (($1 ne "null") && ($2 ne "null")) {
				# print '++++++++violation <' . $cond  .">\n";
				$nb_EqualityInLoopCondition++;
				Erreurs::VIOLATION($EqualityInLoopCondition__mnemo, "Equality in loop termination operator at line " . GetLine($condNode));
			}
		}

		# check all expressions using operators <, >, <=, >=, ==, !=
		my $bool_comparisonToZero;
		if ($cond !~ /\&\&|\|\|/) {
			if ($cond =~ /([^<>!=\s]+)\s*(?:[<>!=]=|<|>)\s*([^<>!=\s]+)/g) {
				$bool_comparisonToZero = 0;
				if (($1 =~ /(?:^|[^\.\w])0$/m) || ($2 =~ /^0(?:$|[^\.\w])/m)) {
					$bool_comparisonToZero = 1;
				}
			}
			if ((defined $bool_comparisonToZero) && ($bool_comparisonToZero == 0)) {
				$nb_LoopCondComparedToNonZero++;
				my $numline = GetLine($condNode);
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_LoopCondComparedToNonZero, $rulesDescription, "LoopCondComparedToNonZero");
				Erreurs::VIOLATION($LoopCondComparedToNonZero__mnemo, "Loop condition is not comparison to zero at line " . GetLine($condNode));
			}
		}
	}
}

sub checkLoopTerminationFunction($) {
	# 16/01/2018 HL-413 Avoid function call in loop stop condition
	my $condNode = shift;
	
	#my $ref_children_niv1 = GetChildren($loop);
  	if ( GetKind ($condNode) eq ConditionKind )
	{            
        my $cond = Lib::NodeUtil::GetKindData($condNode)->{'cond'};
        # print '++++++++first check <' . $cond . "> \n";
        
        # detect a function call
        while ( $cond =~ /(\w+)\s*\(/g ) 
        {
            # print '+++++second check <' . $1 . "> \n";
            $nb_FunctionCallInLoopTest++;
			my $numline = $condNode->[5];
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
			Erreurs::VIOLATION($FunctionCallInLoopTest__mnemo, "Function call in loop condition at line ".GetLine($condNode));
        }
    }
}

sub checkNestedLoop($) {
	my $loop = shift;

	# find nested loops ...
	my @subloop = GetNodesByKindList($loop, [WhileKind, ForKind, EForKind, DoKind], 1);
	for my $sloop (@subloop) {
		$nb_ToManyNestedLoop++;
		my $numline = $sloop->[5];
		CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
		Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "sub loop at line ".GetLine($sloop));
	}
}

sub checkBreakLoop($) {
	my $loop = shift;

	# get a breaks, but do not search inside sub-loops or switches ...
	my @breaks = GetNodesByKindList_StopAtBlockingNode($loop, [BreakKind] , [WhileKind, ForKind, EForKind, DoKind, SwitchKind]);
	
	$nb_BreakLoop += scalar @breaks;

	if (scalar @breaks > 1) {
		Erreurs::VIOLATION($MultipleBreakLoops__mnemo, "Too many breaks for loop at line ".GetLine($loop));
		$nb_MultipleBreakLoops++;
	}
}

sub CountLoop($$$)
{
	my ($file, $vue, $compteurs) = @_;

	my $ret = 0;
	$nb_UnconditionalJump = 0;
	$nb_EqualityInLoopCondition = 0;
	$nb_FunctionCallInLoopTest = 0;
	$nb_LoopCounterModification = 0;
	$nb_LoopCondComparedToNonZero = 0;
	$nb_ToManyNestedLoop = 0;
	$nb_BreakLoop = 0 ;
	$nb_MultipleBreakLoops = 0 ;

	my $KindsLists = $vue->{'KindsLists'};

	if (!defined $KindsLists) {
		$ret |= Couples::counter_add($compteurs, $UnconditionalJump__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $EqualityInLoopCondition__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $LoopCounterModification__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $LoopCondComparedToNonZero__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $Break__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $MultipleBreakLoops__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my $forLoop = $KindsLists->{&ForKind};
	# 08/01/2018 HL-405 Avoid unconditional jump statement
	for my $for (@{$forLoop}) {
		my ($cond, $thenBody) = @{GetChildren($for)};
		my $codeBody = Lib::NodeUtil::GetKindData($for)->{'codeBody'};

		checkLoopBody($thenBody);
		checkLoopTermination($cond);
		checkLoopTerminationFunction($cond);
		checkLoopCounterModification($cond, $codeBody);
		checkNestedLoop($for);
		checkBreakLoop($for);
	}

	my $whileLoop = $KindsLists->{&WhileKind};
	for my $while (@{$whileLoop}) {

		my ($cond, $thenBody) = @{GetChildren($while)};

		checkLoopBody($thenBody);
		checkLoopTermination($cond);
		checkLoopTerminationFunction($cond);
		checkNestedLoop($while);
		checkBreakLoop($while);
	}

	my $EForLoop = $KindsLists->{&EForKind};
	for my $efor (@{$EForLoop}) {
		checkLoopBody($efor);
		checkNestedLoop($efor);
		checkBreakLoop($efor);
	}

	my $doLoop = $KindsLists->{&DoKind};
	for my $do (@{$doLoop}) {
		
		my ($cond, $thenBody) = @{GetChildren($do)};
		
		checkLoopBody($do);
		checkLoopTermination($cond);
		checkLoopTerminationFunction($cond);
		checkNestedLoop($do);
		checkBreakLoop($do);
	}

	$ret |= Couples::counter_update($compteurs, $UnconditionalJump__mnemo, $nb_UnconditionalJump);
	$ret |= Couples::counter_add($compteurs, $EqualityInLoopCondition__mnemo, $nb_EqualityInLoopCondition);
	$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, $nb_FunctionCallInLoopTest);
	$ret |= Couples::counter_add($compteurs, $LoopCounterModification__mnemo, $nb_LoopCounterModification);
	$ret |= Couples::dispatch($LoopCondComparedToNonZero__mnemo, $nb_LoopCondComparedToNonZero);
	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);
	$ret |= Couples::dispatch($Break__mnemo, $nb_BreakLoop);
    $ret |= Couples::dispatch($MultipleBreakLoops__mnemo, $nb_MultipleBreakLoops);

	return $ret;
}

sub CountPatternInLoop($$$) {
	my ($file, $vue, $compteurs) = @_;

	my $ret = 0;

	$nb_ConcatInLoop = 0;
	$nb_InstantiationInLoop = 0;
	$nb_OpenCloseFileInLoop = 0;

	my $root = $vue->{'structured_code'};
	my $code = \$vue->{'code'};

	if (!defined $root) {
		$ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my @loops = GetNodesByKindList($root, [ WhileKind, ForKind, EForKind ], 1);
	# HL-2022 Avoid OPEN/CLOSE inside loops
	# codeBody is in lowercase
	my $regexOpen = qr/\bnew\s+(?:FileInputStream|FileOutputStream|InputStreamReader|OutputStreamWriter|BufferedWriter)\b/i;
	my $regexClose = qr/\.close\b/i;
	for my $loop (@loops) {
		my $codeBody = Lib::NodeUtil::GetKindData($loop)->{'codeBody'};
		my $bodyBeginLine = Lib::NodeUtil::GetKindData($loop)->{'bodyBeginLine'};

		# check presence of literal strings concaténations (variables strings cannot be detected !)
		my $lineOffset = 0;
		while ($$codeBody =~ /(\n+)|(\bCHAINE_\d+)|(\+=)|(\+)|($regexOpen)|($regexClose)|(\bnew\b)/gc) {
			if (defined $1) {
				$lineOffset += () = $1 =~ /\n/g;
			}
			elsif (defined $7) {
				$nb_InstantiationInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
				Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation in loop at line " . ($bodyBeginLine + $lineOffset));
			}
			elsif (defined $2) {
				# CHAINE_XX
				if ($$codeBody =~ /\G(\s*)\+/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
				}
			}
			elsif (defined $5 || defined $6) {
				$lineOffset += () = $5 =~ /\n/g if (defined $5);
				$lineOffset += () = $6 =~ /\n/g if (defined $6);
				$nb_OpenCloseFileInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				# print "Open/close file inside loop at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
				Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
			}
			else {
				# + first or +=
				if ($$codeBody =~ /\G(\s*)\bCHAINE_\d+(\s*)(\+)?/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));

					$lineOffset += () = $2 =~ /\n/g;
					if (defined $3) {
						$nb_ConcatInLoop++;
						my $numline = $bodyBeginLine + $lineOffset;
						CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
						Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
					}
				}
			}
		}
	}

	$ret |= Couples::dispatch($ConcatInLoop__mnemo, $nb_ConcatInLoop);
	$ret |= Couples::dispatch($InstantiationInLoop__mnemo, $nb_InstantiationInLoop);
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);

	return $ret;
}

1;
