package Kotlin::CountLoop;

use strict;
use warnings;

use Erreurs;
use Lib::NodeUtil;
use Kotlin::KotlinNode;
use GreenIT::lib::KeepPortalData;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $InstantiationInLoop__mnemo = Ident::Alias_InstantiationInLoop();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $LoopCondComparedToNonZero__mnemo = Ident::Alias_LoopCondComparedToNonZero();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();

my $nb_ConcatInLoop = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_InstantiationInLoop = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_LoopCondComparedToNonZero = 0;
my $nb_ToManyNestedLoop = 0;

sub CountPatternInLoop($$$) {
	my ($file, $vue, $compteurs) = @_;

	my $ret = 0;

	$nb_ConcatInLoop = 0;
	$nb_FunctionCallInLoopTest = 0;
	$nb_InstantiationInLoop = 0;
	$nb_OpenCloseFileInLoop = 0;
	$nb_LoopCondComparedToNonZero = 0;
	$nb_ToManyNestedLoop = 0;

	my $root = $vue->{'structured_code'};
	my $code = \$vue->{'code'};

	if (!defined $root) {
		$ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $LoopCondComparedToNonZero__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my @loops = GetNodesByKindList($root, [ WhileKind, ForKind ], 1);
	my $regexOpen = qr/(?:\b(?:forEachLine|useLines|bufferedReader|readLines|inputStream|readText|getResource|getResourceAsStream)\b)/i;

	for my $loop (@loops) {
		my $codeBody = Lib::NodeUtil::GetKindData($loop)->{'codeBody'};
		my $bodyBeginLine = Lib::NodeUtil::GetKindData($loop)->{'bodyBeginLine'};

		my $lineOffset = 0;
		while ($$codeBody =~ /(\n+)|(\bCHAINE_\d+)|(\+)|(\bobject\s+\{)|($regexOpen)/gc) {
			if (defined $1) {
				$lineOffset += () = $1 =~ /\n/g;
			}
			# HL-2679 Avoid String concatenation in loops
			elsif (defined $2) {
				# CHAINE_XX
				if ($$codeBody =~ /\G(\s*)\+/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
				}
			}
			# HL-2679 Avoid String concatenation in loops
			elsif (defined $3) {
				# + first
				if ($$codeBody =~ /\G(\s*)\bCHAINE_\d+(\s*)(\+)?/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));

					$lineOffset += () = $2 =~ /\n/g;
					if (defined $3) {
						$nb_ConcatInLoop++;
						my $numline = $bodyBeginLine + $lineOffset;
						CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
						Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
					}
				}
			}
			# HL-2681 Avoid instantiations inside loops
			elsif (defined $4) {
				$nb_InstantiationInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
				Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation in loop at line " . ($bodyBeginLine + $lineOffset));
			}
			# HL-2683 Avoid opening/closing files inside a loop
			elsif (defined $5) {
				$lineOffset += () = $5 =~ /\n/g;
				$nb_OpenCloseFileInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				# print "Open/close file inside loop at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
				Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
			}
		}

		# HL-2680 Avoid calling a function in a condition loop
		my $condNode = GetChildren($loop)->[0];

		# detect a function call
		my $condition = GetStatement($condNode);
		while ($$condition =~ /(\w+)\s*\(/g) {
			$nb_FunctionCallInLoopTest++;
			my $numline = $condNode->[5];
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
			Erreurs::VIOLATION($FunctionCallInLoopTest__mnemo, "Function call in loop condition at line " . GetLine($condNode));
		}

		# HL-2685 Prefer comparison-to-0 in loop conditions
		# check all expressions using operators <, >, <=, >=, ==, !=
		my $bool_comparisonToZero;
		if ($$condition !~ /\&\&|\|\|/) {
			if ($$condition =~ /([^<>!=\s]+)\s*(?:[<>!=]=|<|>)\s*([^<>!=\s]+)/) {
				$bool_comparisonToZero = 0;
				if (($1 =~ /(?:^|[^\.\w])0$/m) || ($2 =~ /^0(?:$|[^\.\w])/m)) {
					$bool_comparisonToZero = 1;
				}
			}
			if ((defined $bool_comparisonToZero) && ($bool_comparisonToZero == 0)) {
				$nb_LoopCondComparedToNonZero++;
				my $numline = $condNode->[5];
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_LoopCondComparedToNonZero, $rulesDescription, "LoopCondComparedToNonZero");
				Erreurs::VIOLATION($LoopCondComparedToNonZero__mnemo, "Loop condition is not comparison to zero at line " . GetLine($condNode));
			}
		}

		# HL-2686 Avoid nested loops
		# find nested loops ...
		my @subloop = GetNodesByKindList($loop, [ WhileKind, ForKind ], 1);
		for my $sloop (@subloop) {
			$nb_ToManyNestedLoop++;
			my $numline = $sloop->[5];
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
			Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "sub loop at line " . GetLine($sloop));
		}
	}

	$ret |= Couples::dispatch($ConcatInLoop__mnemo, $nb_ConcatInLoop);
	$ret |= Couples::dispatch($FunctionCallInLoopTest__mnemo, $nb_FunctionCallInLoopTest);
	$ret |= Couples::dispatch($InstantiationInLoop__mnemo, $nb_InstantiationInLoop);
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);
	$ret |= Couples::dispatch($LoopCondComparedToNonZero__mnemo, $nb_LoopCondComparedToNonZero);
	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);

	return $ret;
}

1;
