package LibInstall;
use strict;
use warnings ;

sub check_install($) {
    my $kind = shift;

    print "Running Perl $kind\n";
    print "- Perl version: $]\n";
    print "- Perl location: $^X\n";

    # Windows only
    if ($^O eq 'MSWin32') {
        eval {require Win32::LongPath};
        if ($@) {
            # automatic detection of Win32::LongPath module build version
            use File::Basename qw(dirname);
            if (substr($], 0, 5) == 5.036) {
                print "Win32::LongPath module not found => add specific build version (5.036)\n";
                if ($kind eq "Discoverer") {
                    unshift @INC, dirname($0) . "/../PerlLib/5.36";
                }
                elsif ($kind eq "Analyzer") {
                    unshift @INC, dirname($0) . "/PerlLib/5.36";
                }
            }
            elsif (substr($], 0, 5) == 5.040) {
                print "Win32::LongPath module not found => add specific build version (5.040)\n";
                if ($kind eq "Discoverer") {
                    unshift @INC, dirname($0) . "/../PerlLib/5.40";
                }
                elsif ($kind eq "Analyzer") {
                    unshift @INC, dirname($0) . "/PerlLib/5.40";
                }
            }
            else {
                print "\nerror: version of perl distribution (" . substr($], 0, 5) . ") not supported\n";
                exit;
            }
        }
        else {
            # print "perl library Win32::LongPath already installed\n";
            Win32::LongPath->import;
        }

        eval {require Win32::LongPath};
        if ($@) {
            print "error: missing perl library: Win32::LongPath\n";
            print "error: \@INC: @INC\n";
            exit;
        }
    }

    # Win + Linux: check availability of other libraries
    eval {require Digest::SHA};
    if ($@) {
        print "error: missing perl library: Digest::SHA\n";
        print "error: \@INC: @INC\n";
        exit;
    }
    eval {require XML::LibXML};
    if ($@) {
        print "error: missing perl library: XML::LibXML\n";
        print "error: \@INC: @INC\n";
        exit;
    }
    eval {require JSON};
    if ($@) {
        print "error: missing perl library: JSON\n";
        print "error: \@INC: @INC\n";
        exit;
    }
    eval {require Time::HiRes};
    if ($@) {
        print "error: missing perl library: Time::HiRes\n";
        print "error: \@INC: @INC\n";
        exit;
    }
    eval {require Math::BigInt};
    if ($@) {
        print "error: missing perl library: Math::BigInt\n";
        print "error: \@INC: @INC\n";
        exit;
    }

    print "\@INC: @INC\n";
}

1;