package PlSql::CountSQL;

use strict;
use warnings;

use PlSql::PlSqlNode;
use Lib::NodeUtil;

use Erreurs;

my $THRESHOLD_MAX_TABLES = 4;

my $SelectAll__mnemo = Ident::Alias_SelectAll();
my $MissingWhereClause__mnemo = Ident::Alias_MissingWhereClause();
my $SelectDistinct__mnemo = Ident::Alias_SelectDistinct();
my $SQLComplexWhereClause__mnemo = Ident::Alias_SQLComplexWhereClause();
my $WhereClauseWithWildcards__mnemo = Ident::Alias_WhereClauseWithWildcards();
my $OnToManyTablesQueries__mnemo = Ident::Alias_OnToManyTablesQueries();
my $NegativeComparison__mnemo = Ident::Alias_NegativeComparison();
my $WithoutPrimaryKeyTables__mnemo = Ident::Alias_WithoutPrimaryKeyTables();
my $Sql_Cursor__mnemo = Ident::Alias_Sql_Cursor();

my $nb_SelectAll = 0;
my $nb_MissingWhereClause = 0;
my $nb_SelectDistinct = 0;
my $nb_SQLComplexWhereClause = 0;
my $nb_WhereClauseWithWildcards = 0;
my $nb_OnToManyTablesQueries = 0;
my $nb_NegativeComparison = 0;
my $nb_WithoutPrimaryKeyTables = 0;
my $nb_Sql_Cursor = 0;
#-----------------------  CountSQL ---------------------#

my $REG_TABLE_NAME = qr/\w+(\s*\.\s*\w+)*/;

sub countCreateTable($$) {
	my $node = shift;
	my $tableDB = shift;

	my $statement = GetStatement($node);
		
	my $name;
	if ($statement =~ /\A\s*create\s+table\s+($REG_TABLE_NAME)/i) {
		$name = $1;
	}
	
	if (defined $name) {
		if (! exists $tableDB->{$name}) {
			$tableDB->{$name}={};
		}
		$tableDB->{$name}->{'create_table'}=1;
	}
	else {
		print STDERR "Unknow table name in CREATE TABLE statement :\n$statement\n";
	}
		
	if (defined $name) {
		if ($statement =~ /\bprimary\s+key\b/i) {
			$tableDB->{$name}->{'primary_key'} = 1;
		}
	}
}

sub countAlterTable($$) {
	my $node = shift;
	my $tableDB = shift;

	my $statement = GetStatement($node);
		
	my $name;
	if ($statement =~ /\A\s*alter\s+table\s+($REG_TABLE_NAME)/i) {
		$name = $1;
	}
	
	if (defined $name) {
		if (! exists $tableDB->{$name}) {
			$tableDB->{$name}={};
		}
	}
	else {
		print STDERR "Unknow table name in ALTER TABLE statement :\n$statement\n";
	}
		
	if (defined $name) {
		if ($statement =~ /\bprimary\s+key\b/i) {
			$tableDB->{$name}->{'primary_key'}->{'alter'} = 1;
		}
	}
}



sub CountSQL($$$)
{
    my ($fichier, $vue, $compteurs) = @_;

    my $ret = 0;
    $nb_SelectAll = 0;
    $nb_MissingWhereClause = 0;
    $nb_SelectDistinct = 0;
    $nb_SQLComplexWhereClause = 0;
    $nb_WhereClauseWithWildcards = 0;
    $nb_OnToManyTablesQueries = 0;
    $nb_NegativeComparison = 0;
    $nb_WithoutPrimaryKeyTables = 0;
    $nb_Sql_Cursor = 0;

    my $root = $vue->{'structured_code'};
    my $code = $vue->{'code'};
    my $text = $vue->{'text'};
    my $HStrings = $vue->{'HString'};

    if (!defined $root) {
        $ret |= Couples::counter_add($compteurs, $SelectAll__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $MissingWhereClause__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $SelectDistinct__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $SQLComplexWhereClause__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $WhereClauseWithWildcards__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $OnToManyTablesQueries__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $NegativeComparison__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $WithoutPrimaryKeyTables__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Sql_Cursor__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

        return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
    }

    my @elements = Lib::NodeUtil::GetNodesByKindList($root, [ BeginKind, ProcedureKind, FunctionKind, CursorKind ], 1);

    foreach my $element (@elements) {
        if (Lib::NodeUtil::GetKind($element) eq BeginKind) {
            my $children = Lib::NodeUtil::GetChildren($element);
            foreach my $child (@{$children}) {
                my $statement = Lib::NodeUtil::GetStatement($child);
                checkSQLStatement($statement, GetLine($element), $HStrings);
            }
        }
        elsif (Lib::NodeUtil::GetKind($element) eq CursorKind) {
            # HL-2229
            # print "Query with cursor in a loop at line " . GetLine($element) . "\n";
            $nb_Sql_Cursor++;
            Erreurs::VIOLATION($Sql_Cursor__mnemo, "Query with cursor in a loop at line " . GetLine($element));
        }
        else {
            my $statement = Lib::NodeUtil::GetStatement($element);
            if ($statement =~ /\bselect\s+.*\bfrom\b/i) {
                checkSQLStatement($statement, GetLine($element), $HStrings);
            }
        }
    }
    
    # get infos on tables
    my $tableDB = {};
    my @CreateTables = Lib::NodeUtil::GetNodesByKindList($root, [ TableKind ], 1);
    for my $node (@CreateTables) {
		countCreateTable($node, $tableDB);
	}
	my @AlterTables = Lib::NodeUtil::GetNodesByKindList($root, [ AlterKind ], 1);
    for my $node (@AlterTables) {
		countAlterTable($node, $tableDB);
	}
	
	
	# FIXME : no CREATE TABLE on a table mean no violation, even if a ALTER TABLE as no primary key.
	# check tables detections
    for my $table (keys %$tableDB) {
	    if ( ($tableDB->{$table}->{'create_table'}) && (! $tableDB->{$table}->{'primary_key'}) ) {
		    $nb_WithoutPrimaryKeyTables++;
		    Erreurs::VIOLATION($WithoutPrimaryKeyTables__mnemo, "Missing primary key for table $table");
		}
	}

    # search in strings scope
    checkSQLInStrings($text, $HStrings);

    $ret |= Couples::counter_add($compteurs, $SelectAll__mnemo, $nb_SelectAll);
    $ret |= Couples::counter_add($compteurs, $MissingWhereClause__mnemo, $nb_MissingWhereClause);
    $ret |= Couples::counter_add($compteurs, $SelectDistinct__mnemo, $nb_SelectDistinct);
    $ret |= Couples::counter_add($compteurs, $SQLComplexWhereClause__mnemo, $nb_SQLComplexWhereClause);
    $ret |= Couples::counter_add($compteurs, $WhereClauseWithWildcards__mnemo, $nb_WhereClauseWithWildcards);
    $ret |= Couples::counter_add($compteurs, $OnToManyTablesQueries__mnemo, $nb_OnToManyTablesQueries);
    $ret |= Couples::counter_add($compteurs, $NegativeComparison__mnemo, $nb_NegativeComparison);
    $ret |= Couples::dispatch($WithoutPrimaryKeyTables__mnemo, $nb_WithoutPrimaryKeyTables);
    $ret |= Couples::counter_add($compteurs, $Sql_Cursor__mnemo, $nb_Sql_Cursor);

    return $ret;
}

sub checkSQLStatement($$$) {
    my $statement = shift;
    my $numline = shift;
    my $HStrings = shift;

    my $statement_to_check;
    if (ref($statement) eq 'SCALAR') {
        $statement_to_check = $$statement;
    }
    else {
        $statement_to_check = $statement;
    }
    if (defined $statement) {
        # HL-2216
        if ($statement_to_check =~ /\s*\*.*\bfrom\b/si) {
            # print "Avoid using Select * at line " . GetLine($element) . "\n";
            $nb_SelectAll++;
            Erreurs::VIOLATION($SelectAll__mnemo, "Avoid using Select * at line " . $numline);
        }
        # HL-2217
        if ($statement_to_check =~ /\bfrom\b/i && $statement_to_check !~ /\bwhere\b/i) {
            # print "Query with no WHERE clause at line " . GetLine($element) . "\n";
            $nb_MissingWhereClause++;
            Erreurs::VIOLATION($MissingWhereClause__mnemo, "Query with no WHERE clause at line " . $numline);
        }
        # HL-2218
        if ($statement_to_check =~ /\bwhere\s+.*(CHAINE_[0-9]+)/i) {
            my $stringID = $1;
            if ($HStrings->{$stringID} =~ /["'][%_\[]/) {
                # print "Query with wildcards in where clause at line " . $numline . "\n";
                $nb_WhereClauseWithWildcards++;
                Erreurs::VIOLATION($WhereClauseWithWildcards__mnemo, "Query with wildcards in where clause at line " . $numline);
            }
        }
        #HL-2219
        # syntax supported
        # WHERE name_of_func(xxx)
        # WHERE fieldA | fieldB
        # WHERE fieldA & fieldB
        if ($statement_to_check =~ /\bwhere\b\s*(?:.+[&|]+|\w+\()/i) {
            # print "Query with functions in 'where' clause at line " . $numline . "\n";
            $nb_SQLComplexWhereClause++;
            Erreurs::VIOLATION($SQLComplexWhereClause__mnemo, "Query with functions in 'where' clause at line " . $numline);
        }
        # HL-2220
        if ($statement_to_check =~ /\bdistinct\b/i) {
            # print "Query with SELECT DISTINCT at line " . GetLine($element) . "\n";
            $nb_SelectDistinct++;
            Erreurs::VIOLATION($SelectDistinct__mnemo, "Query with SELECT DISTINCT at line " . $numline);
        }
        # HL-2230
        if ($statement_to_check =~ /\bfrom\b.*\,/i) {
            my $nbTables = () = $statement_to_check =~ /(.*?\,)+?/g;
            if ($nbTables > $THRESHOLD_MAX_TABLES - 1) {
                # print "Query with too many tables in 'from' clause at line $numline\n";
                $nb_OnToManyTablesQueries++;
                Erreurs::VIOLATION($OnToManyTablesQueries__mnemo, "Query with too many tables in 'from' clause at line $numline");
            }
        }
        # HL-2231
        if ($statement_to_check =~ /\<\>|\bnot\b/i) {
            # print "Query with negative comparison at line " . $numline . "\n";
            $nb_NegativeComparison++;
            Erreurs::VIOLATION($NegativeComparison__mnemo, "Query with negative comparison at line " . $numline);
        }
    }
}

sub checkSQLInStrings($$) {
    my $text = shift;
    my $HStrings = shift;

    if (defined $text && defined $HStrings) {
        foreach my $stringID (keys %{$HStrings}) {
            # SQL format
            if ($HStrings->{$stringID} =~ /\bselect\b.*\bfrom\b/i) {
                my $CHAINE_ID_pattern = $HStrings->{$stringID};
                $CHAINE_ID_pattern = quotemeta($CHAINE_ID_pattern);

                while ($text =~ /(.*?)$CHAINE_ID_pattern([^\n]+)/sg) {
                    my $endLineContent = $2;
                    my $numline = 1;
                    $numline += () = $1 =~ /\n/g;

                    # HL-2216
                    if ($HStrings->{$stringID} =~ /\bselect\s+\*.*\bfrom\b/i) {
                            # print "Avoid using Select * at line $numline\n";
                            $nb_SelectAll++;
                            Erreurs::VIOLATION($SelectAll__mnemo, "Avoid using Select * at line $numline");
                    }
                    # HL-2217
                    if ($HStrings->{$stringID} !~ /\bwhere\b/) {
                        if (defined $endLineContent && $endLineContent =~ /\bwhere\b/i) {
                            # do nothing
                        }
                        # endLineContent not defined or not contain where...
                        else {
                            # print "Query with no 'WHERE' clause at line $numline\n";
                            $nb_MissingWhereClause++;
                            Erreurs::VIOLATION($MissingWhereClause__mnemo, "Query with no 'WHERE' clause at line $numline");
                        }
                    }
                    # HL-2218
                    if ($HStrings->{$stringID} =~ /\bwhere\b/i) {
                        if ($HStrings->{$stringID} =~ /["'][%_\[]/) {
                            # print "Query with wildcards in 'where' clause at line $numline\n";
                            $nb_WhereClauseWithWildcards++;
                            Erreurs::VIOLATION($WhereClauseWithWildcards__mnemo, "Query with wildcards in 'where' clause at line $numline");
                        }
                    }
                    # HL-2219
                    # syntax supported
                    # WHERE name_of_func(xxx)
                    # WHERE fieldA | fieldB
                    # WHERE fieldA & fieldB
                    if ($HStrings->{$stringID} =~ /\bwhere\b\s*(?:.+[&|]+|\w+\()/i) {
                        # print "Query with functions in 'where' clause at line $numline\n";
                        $nb_SQLComplexWhereClause++;
                        Erreurs::VIOLATION($SQLComplexWhereClause__mnemo, "Query with functions in 'where' clause at line $numline");
                    }
                    # HL-2220
                    if ($HStrings->{$stringID} =~ /\bselect\s+distinct\b/i) {
                        # print "Query with SELECT DISTINCT at line $numline\n";
                        $nb_SelectDistinct++;
                        Erreurs::VIOLATION($MissingWhereClause__mnemo, "Query with SELECT DISTINCT at line $numline");
                    }
                    # HL-2229
                    if ($HStrings->{$stringID} =~ /\bcursor\b/i) {
                        # print "Query with cursor in a loop at line " . $numline . "\n";
                        $nb_Sql_Cursor++;
                        Erreurs::VIOLATION($Sql_Cursor__mnemo, "Query with cursor in a loop at line " . $numline);
                    }
                    # HL-2230
                    if ($HStrings->{$stringID} =~ /\bfrom\b.*\,/i) {
                        my $nbTables = () = $HStrings->{$stringID} =~ /(.*?\,)+?/g;
                        if ($nbTables > $THRESHOLD_MAX_TABLES - 1) {
                            # print "Query with too many tables in 'from' clause at line $numline\n";
                            $nb_OnToManyTablesQueries++;
                            Erreurs::VIOLATION($OnToManyTablesQueries__mnemo, "Query with too many tables in 'from' clause at line $numline");
                        }
                    }
                    # HL-2231
                    if ($HStrings->{$stringID} =~ /\<\>|\bnot\b/i) {
                        # print "Query with negative comparison at line " . $numline . "\n";
                        $nb_NegativeComparison++;
                        Erreurs::VIOLATION($NegativeComparison__mnemo, "Query with negative comparison at line " . $numline);
                    }
                }
            }
        }
    }
}

1;



