package Python::CountLoop;

use strict;
use warnings;
use Carp::Assert; # traces_filter_line
use Erreurs;
use Couples;
use Lib::NodeUtil;

use Python::PythonNode;
use Python::PythonConf;
use Lib::NodeUtil;
use GreenIT::lib::KeepPortalData;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $LoopWithElseWithoutBreak__mnemo = Ident::Alias_LoopWithElseWithoutBreak();
my $Loop__mnemo = Ident::Alias_Loop();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();

my $nb_LoopWithElseWithoutBreak = 0;
my $nb_Loop = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_ConcatInLoop = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_ToManyNestedLoop = 0;

sub checkLoop($$) {
	my $loop = shift;
	my $context = shift;
	my $children = Lib::NodeUtil::GetChildren($loop);
	
	my $elseNode;
	# Iterate on others childs than conds
	for my $child (@$children) {
		if (IsKind($child, ConditionKind)) {
			next;
		}
		elsif(IsKind($child, ElseKind)) {
			$elseNode = $child;
		}
		Lib::Node::Iterate ($child, 0, \&_cbGetAllNodesExcept, $context);
	}
	
	if (defined $elseNode) {
			my $thenNode = $children->[1];
			my @BreakLoop = GetNodesByKindList_StopAtBlockingNode($thenNode, [BreakKind], [WhileKind, ForKind]);
			if (scalar @BreakLoop == 0) {
				if ( ! Lib::NodeUtil::IsContainingKind($thenNode, ReturnKind)) {
					$nb_LoopWithElseWithoutBreak++;
					Erreurs::VIOLATION($LoopWithElseWithoutBreak__mnemo, "Loop with else statement whithout a break at line : ".GetLine($loop));
				}
			}
	}
}

sub isConcatenation($) {
	my $node = shift;
	if (${GetStatement($node)} =~ /((?:\+|\+=)\s*CHAINE_\d+\b)|(\bCHAINE_\d+\s*(?:\+|\+=))/) {
		if ((defined $1) || (defined $2)) {
			my $line = GetLine($node) || "??";
			$nb_ConcatInLoop++;
			Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation with operator + inside loop at line : $line");
			CommonOutput::ElaborateLog::SimpleElaborateLog($line, $ConcatInLoop__mnemo, $rulesDescription, "Avoid_ConcatInLoop");
		}
	}
	return 0;
}

## FIXME : do not apply to the "cond" node of the loop ...
sub _cbGetAllNodesExcept($$)
{
	my ($node, $context) = @_;
	my $ref_list = $context->[0];
	my $H_exceptKinds = $context->[1];
	my $treatmentCallbacks = $context->[2];

	my $Kind = GetKind($node);

	if (! exists $H_exceptKinds->{$Kind} ) {
		for my $cb (@$treatmentCallbacks) {
			$cb->($node);
		}
	}
	else {
		return 0;
	}

	return undef;
}

sub CountLoop($$$) {
	my ($file, $views, $compteurs) = @_;
	my $ret = 0;

	$nb_LoopWithElseWithoutBreak = 0;
	$nb_Loop = 0;
	$nb_OpenCloseFileInLoop = 0;
	$nb_ConcatInLoop = 0;
	$nb_FunctionCallInLoopTest = 0;
	$nb_ToManyNestedLoop = 0;

	my $HString = $views->{'HString'};
	my $kindLists = $views->{'KindsLists'};
	my $code = \$views->{'code'};

	if ((!defined $HString) || (!defined $kindLists) || (!defined $code)) {
		$ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $LoopWithElseWithoutBreak__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $Loop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my %HLoopKinds = (&WhileKind => 1, &ForKind => 1);

	#my @loops = GetNodesByKindList($root, \@TLoopKinds);
	my @loops = (@{$kindLists->{&WhileKind}}, @{$kindLists->{&ForKind}});
	my @KindList = (WhileKind, ForKind);

	my $nb_Loop = scalar @loops;

	my @context = ([], \%HLoopKinds, [ \&isConcatenation ]);

	# HL-2022 Avoid OPEN/CLOSE inside loops
	my $regexOpen = qr/(?:\bopen(?:\_connection)?|\bio\.BytesIO)\s*\(/i;
	my $regexClose = qr/\.close\b/i;
	for my $loop (@loops) {
		checkLoop($loop, \@context);

		my $codeBody = Lib::NodeUtil::GetKindData($loop)->{'codeBody'};
		my $bodyBeginLine = Lib::NodeUtil::GetKindData($loop)->{'bodyBeginLine'};
		my $lineOffset = 0;
		while ($$codeBody =~ /(\n+)|($regexOpen)|($regexClose)/gc) {
			if (defined $1) {
				$lineOffset += () = $1 =~ /\n/g;
			}
			elsif (defined $2 || defined $3) {
				$lineOffset += () = $2 =~ /\n/g if (defined $2);
				$lineOffset += () = $3 =~ /\n/g if (defined $3);
				$nb_OpenCloseFileInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				# print "Open/close file inside loop at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
				Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
			}
		}

		my $cond = GetStatement(GetChildren($loop)->[0]);
		if (defined $cond) {
			# in Python, FunctionCallInLoopTest diag ignores the range function
			if ($$cond =~ /\brange\s*\((.*?)\)$/sm) {
				my $testExpression = $1;
				if ($testExpression =~ /\w\s*\(/sg) {
					$nb_FunctionCallInLoopTest++;
					my $numline = $loop->[5];
					# print "FUNCTION CALL IN LOOP: <$testExpression> at line $numline\n";
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
				}
			}
			elsif ($$cond =~ /\w\s*\(/sg) {
				$nb_FunctionCallInLoopTest++;
				my $numline = $loop->[5];
				# print "FUNCTION CALL IN LOOP at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
			}
		}

		my @NestedLoopTab = GetNodesByKindList($loop, \@KindList, 0);
		if (scalar @NestedLoopTab > 0) {
			$nb_ToManyNestedLoop += scalar @NestedLoopTab;
			my $kind = $loop->[0];
			my $numline = $loop->[5];
			# print "Avoid nested loops in $kind at line $numline\n";
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
			Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "Avoid nested loops in $kind at line $numline");
		}
	}

	$ret |= Couples::dispatch($ConcatInLoop__mnemo, $nb_ConcatInLoop);
	$ret |= Couples::counter_update($compteurs, $LoopWithElseWithoutBreak__mnemo, $nb_LoopWithElseWithoutBreak);
	$ret |= Couples::counter_update($compteurs, $Loop__mnemo, $nb_Loop);
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);
	$ret |= Couples::dispatch($FunctionCallInLoopTest__mnemo, $nb_FunctionCallInLoopTest);
	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);

	return $ret;
}

1;


