package Python::CountString;

use strict;
use warnings;
use Carp::Assert; # traces_filter_line
use Erreurs;
use Couples;
use Lib::NodeUtil;

use Python::PythonNode;
use Python::PythonConf;
use GreenIT::lib::HardCodedPathNotGreen;
use GreenIT::lib::KeepPortalData;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $PercentStringFormat__mnemo = Ident::Alias_PercentStringFormat();
my $AutomaticNumberingInStringsFields__mnemo = Ident::Alias_AutomaticNumberingInStringsFields();
my $MixedStringsStyle__mnemo = Ident::Alias_MixedStringsStyle();
my $SpacesInStringReplacementFields__mnemo = Ident::Alias_SpacesInStringReplacementFields();
my $String__mnemo = Ident::Alias_String();
my $HardCodedPathNotGreen__mnemo = Ident::Alias_HardCodedPathNotGreen();

my $nb_PercentStringFormat = 0;
my $nb_AutomaticNumberingInStringsFields = 0;
my $nb_MixedStringsStyle = 0;
my $nb_SpacesInStringReplacementFields = 0;
my $nb_String = 0;
my $nb_HardCodedPathNotGreen = 0;

sub CountStrings($$$) {
	my ($file, $views, $compteurs) = @_;
	my $ret = 0;

	$nb_PercentStringFormat = 0;
	$nb_AutomaticNumberingInStringsFields = 0;
	$nb_MixedStringsStyle = 0;
	$nb_SpacesInStringReplacementFields = 0;
	$nb_String = 0;
	$nb_HardCodedPathNotGreen = 0;

	my $HString = $views->{'HString'};
	my $text = $views->{'text'};
	my $kindLists = $views->{'KindsLists'};
	my $code = \$views->{'code'};

	if ((!defined $HString) || (!defined $kindLists) || (!defined $code)) {
		$ret |= Couples::counter_add($compteurs, $PercentStringFormat__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $AutomaticNumberingInStringsFields__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $MixedStringsStyle__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $SpacesInStringReplacementFields__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $String__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	$nb_String = scalar keys %$HString;

	my $nb_quote = 0;
	my $nb_doubleQuote = 0;
	while ($$code =~ /\b(CHAINE_\d+)\b\s*(?:(\%)|(\.)\s*format\()?/g) {
		my $id = $1;
		my $percent = defined $2 ? 1 : 0;
		my $format = defined $3 ? 1 : 0;
		my $value = $HString->{$id};

		if ($percent) {
			$nb_PercentStringFormat++;
			Erreurs::VIOLATION($PercentStringFormat__mnemo, "Use of % operator in string : " . $value);
		}
		elsif ($format) {
			if ($HString->{$id} =~ /\{\s*\}/) {
				$nb_AutomaticNumberingInStringsFields++;
				Erreurs::VIOLATION($AutomaticNumberingInStringsFields__mnemo, "Automatic numbering in format replacement fields in string : " . $value);
			}
		}

		if (($percent) || ($format)) {
			while ($value =~ /\{[^}]*\s[^}]*}/sg) {
				$nb_SpacesInStringReplacementFields++;
				Erreurs::VIOLATION($SpacesInStringReplacementFields__mnemo, "Space in format replacement fields in string : " . $value);
			}
		}

		if ($value =~ /^'[^']/) {
			$nb_quote++;
		}
		elsif ($value =~ /^"[^"]/) {
			$nb_doubleQuote++;
		}
	}

	if ($nb_quote + $nb_doubleQuote) {
		if ($nb_quote > $nb_doubleQuote) {
			$nb_MixedStringsStyle = int(($nb_doubleQuote / ($nb_quote + $nb_doubleQuote) * 100));
		}
		else {
			$nb_MixedStringsStyle = int(($nb_quote / ($nb_quote + $nb_doubleQuote) * 100));
		}
	}
	Erreurs::VIOLATION($MixedStringsStyle__mnemo, "String style homogeneity : $nb_MixedStringsStyle ($nb_quote ' vs $nb_doubleQuote \")");

	$nb_HardCodedPathNotGreen += GreenIT::lib::HardCodedPathNotGreen::checkHardCodedPathNotGreen($HString, 'Python', \$text, $rulesDescription);

	$ret |= Couples::counter_update($compteurs, $PercentStringFormat__mnemo, $nb_PercentStringFormat);
	$ret |= Couples::counter_update($compteurs, $AutomaticNumberingInStringsFields__mnemo, $nb_AutomaticNumberingInStringsFields);
	$ret |= Couples::counter_update($compteurs, $MixedStringsStyle__mnemo, $nb_MixedStringsStyle);
	$ret |= Couples::counter_update($compteurs, $SpacesInStringReplacementFields__mnemo, $nb_SpacesInStringReplacementFields);
	$ret |= Couples::counter_update($compteurs, $String__mnemo, $nb_String);
	$ret |= Couples::dispatch($HardCodedPathNotGreen__mnemo, $nb_HardCodedPathNotGreen);

	return $ret;
}

1;


