package Ruby::CountCondition;

use warnings;
use strict;

use Lib::Node;
use Lib::NodeUtil;
use Ruby::RubyNode;
use Ruby::RubyConfig;
use Lib::ConditionUtil;

my $AssignmentsInConditionalExpr_mnemo = Ident::Alias_AssignmentsInConditionalExpr;
my $MissingDefaults_mnemo = Ident::Alias_MissingDefaults;
my $ComplexConditions_mnemo = Ident::Alias_ComplexConditions;
my $UselessParenthesesForConditions_mnemo = Ident::Alias_UselessParenthesesForConditions;
my $InvertedLogic_mnemo = Ident::Alias_InvertedLogic;
my $NegativeConditions_mnemo = Ident::Alias_NegativeConditions;
my $UnlessWithElse_mnemo = Ident::Alias_UnlessWithElse;


my $nb_AssignmentsInConditionalExpr = 0;
my $nb_MissingDefaults = 0;
my $nb_ComplexConditions = 0;
my $nb_UselessParenthesesForConditions = 0;
my $nb_InvertedLogic = 0;
my $nb_NegativeConditions = 0;
my $nb_UnlessWithElse = 0;

sub CountCondition($$$) {
	my ($fichier, $views, $compteurs, $options) = @_;
	
	my $status = 0;

	$nb_AssignmentsInConditionalExpr = 0;
	$nb_ComplexConditions = 0;
	$nb_UselessParenthesesForConditions = 0;
	$nb_InvertedLogic = 0;
	$nb_NegativeConditions = 0;
	
	my $root = $views->{'structured_code'};
	my $KindsLists = $views->{'KindsLists'};


	if ( ! defined $KindsLists ) {
		$status |= Couples::counter_add($compteurs, $AssignmentsInConditionalExpr_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $ComplexConditions_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $UselessParenthesesForConditions_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $InvertedLogic_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $NegativeConditions_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		
		return $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my $conditions = $KindsLists->{&ConditionKind};
	
	my $heteroLogicalCondition = 0;
	my $totalHeterop = 0;

	for my $cond ( @$conditions) {
		my $statement = GetStatement($cond);
		
		my $condsTree = Lib::ConditionUtil::parse($$statement);
#print STDERR "CONDITION : $$statement\n";
#Lib::ConditionUtil::printNode($condsTree, "  ");
		
		if ( defined $condsTree) {
			
			if ($condsTree->[0] eq "not") {
				$nb_InvertedLogic++;
				Erreurs::VIOLATION($InvertedLogic_mnemo, "Inverted logic at line ".GetLine($cond)." : $$statement");
			}
		
			if (($condsTree->[0] ne 'not') || ($condsTree->[2]->[0]->[0] ne "expr")) {
				if (Lib::ConditionUtil::getNegativeImpact($condsTree) < 0) {
					$nb_NegativeConditions++;
					Erreurs::VIOLATION($NegativeConditions_mnemo, "Condition with too many negation at line ".GetLine($cond)." : $$statement");
				}
			}
		}
		else {
			print STDERR "ERROR : Unable to parse condition $$statement at line ".GetLine($cond)."\n";
		}
		
		
		my $stmtCpy = $$statement;
		
		my $flag_enclosed_assignment = 0;
		
		# remove each enclosed expression and check if it contains an assignment
		while ($stmtCpy =~ /\(([^\(\)]*)\)/) {
			my $expr = $1;
			if ($expr =~ /[^<>=\/+\-*\/\%!|&^]=[^=]/) {
				#~ if ($expr =~ /&&|\|\||\band\b|\bor\b/) {
				    #~ # FIXME : => will be no violation for Nbr_UselessParenthesesForConditions, whereas the assignment is mixed with logical operator ...
					#~ print STDERR "PARENTHESE contains logical AND assignment at line ".GetLine($cond)."!!!\n";
				#~ }
				$flag_enclosed_assignment = 1;
			}
			else {
				$flag_enclosed_assignment = 0;
			}
			$stmtCpy =~ s/\(([^\(\)]*)\)//;
		}
		
		if ($stmtCpy !~ /\S/) {
			# fully parenthesed condition
			if (!$flag_enclosed_assignment) {
				# the aim is not to protect assignment => VIOLATION
				$nb_UselessParenthesesForConditions++;
				Erreurs::VIOLATION($UselessParenthesesForConditions_mnemo, "Unexpected parenthesed condition at line ".GetLine($cond));
			}
			
		}
		else {
			if ($stmtCpy =~ /[^<>=\/+\-*\/\%!|&^]=[^=~]/) {
				$nb_AssignmentsInConditionalExpr++;
				Erreurs::VIOLATION($AssignmentsInConditionalExpr_mnemo, "Assignment in conditional expression at line : ".GetLine($cond));
			}
		}
		
		my $nb_AND = () = $$statement =~ /(&&|\band\b)/g;
		my $nb_OR = () = $$statement =~ /(\|\||\bor\b)/g;
		
		if (($nb_AND > 0) && ($nb_OR > 0)) {
			if (($nb_AND + $nb_OR) > 2 ) {
				$nb_ComplexConditions++;
				Erreurs::VIOLATION($ComplexConditions_mnemo, "ComplexCondition at :line ".GetLine($cond));
			}
		}
		
		# FIXME : abandonned, because cannot distinguish between ordered and non ordered data ...
		#~ my $parentKind = GetKind(GetParent($cond));
		#~ if (($parentKind eq WhileKind) || ($parentKind eq UntilKind)) {	
			#~ if ($$statement =~ /==/) {
				#~ print STDERR "DEBUG : Equality inside condition '$$statement' at line ".GetLine($cond)."\n";
			#~ }
		#~ }
	}
	
	$status |= Couples::counter_add($compteurs, $AssignmentsInConditionalExpr_mnemo, $nb_AssignmentsInConditionalExpr);
	$status |= Couples::counter_add($compteurs, $ComplexConditions_mnemo, $nb_ComplexConditions);
	$status |= Couples::counter_add($compteurs, $UselessParenthesesForConditions_mnemo, $nb_UselessParenthesesForConditions);
	$status |= Couples::counter_add($compteurs, $InvertedLogic_mnemo, $nb_InvertedLogic);
	$status |= Couples::counter_add($compteurs, $NegativeConditions_mnemo, $nb_NegativeConditions);
	
	return $status;
}

sub CountSwitch($$$) {
	my ($fichier, $views, $compteurs, $options) = @_;
	
	my $status = 0;

	$nb_MissingDefaults = 0;
	
	my $KindsLists = $views->{'KindsLists'};
	
	if ( ! defined $KindsLists ) {
		$status |= Couples::counter_add($compteurs, $MissingDefaults_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		
		return $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my $cases = $KindsLists->{&CaseKind};
	
	for my $case (@$cases) {

		my $defaultPresent = 0;
		
		for my $child (@{GetChildren($case)}) {
			if (IsKind($child, ElseKind)) {
				$defaultPresent =1;
			}
		}
		if (! $defaultPresent) {
			$nb_MissingDefaults++;
			Erreurs::VIOLATION($MissingDefaults_mnemo, "Missing default statement in case at line ".GetLine($case));
		}
	}
	
	$status |= Couples::counter_add($compteurs, $MissingDefaults_mnemo, $nb_MissingDefaults);
}

sub CountUnless($$$) {
	my ($fichier, $views, $compteurs, $options) = @_;
	
	my $status = 0;

	$nb_UnlessWithElse = 0;
	
	my $KindsLists = $views->{'KindsLists'};
	
	if ( ! defined $KindsLists ) {
		$status |= Couples::counter_add($compteurs, $UnlessWithElse_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		
		return $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my $unlesses = $KindsLists->{&UnlessKind};
	
	for my $unless (@$unlesses) {

		if (defined GetChildren($unless)->[2]) {
			$nb_UnlessWithElse++;
			Erreurs::VIOLATION($UnlessWithElse_mnemo, "Unless with else at line ".GetLine($unless));
		}
	}
	
	$status |= Couples::counter_add($compteurs, $UnlessWithElse_mnemo, $nb_UnlessWithElse);
}

1;


