package Ruby::CountException;

use warnings;
use strict;

use Lib::Node;
use Lib::NodeUtil;
use Ruby::RubyNode;
use Ruby::RubyConfig;

my $EmptyCatches_mnemo = Ident::Alias_EmptyCatches;
my $GenericCatches_mnemo = Ident::Alias_GenericCatches;

my $nb_EmptyCatches = 0;
my $nb_GenericCatches = 0;

my %GENERIC_EXCEPTIONS = (
	"Exception" => 1,
	"NoMemoryError" => 1,
	"ScriptError" => 1,
	"SecurityError" => 1,
	"SignalException" => 1,
	"StandardError" => 1,
	"SystemExit" => 1,
	"SystemStackError" => 1,
	"" => 1
);

sub CountException($$$) {
	my ($fichier, $views, $compteurs, $options) = @_;
	
	my $status = 0;

	$nb_EmptyCatches = 0;
	$nb_GenericCatches = 0;
	
	my $KindsLists = $views->{'KindsLists'};
	my $agglo_index = $views->{'agglo_LinesIndex'};
	my $agglo = \$views->{'agglo'};

	if ( ! defined $KindsLists ) {
		$status |= Couples::counter_add($compteurs, $EmptyCatches_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$status |= Couples::counter_add($compteurs, $GenericCatches_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		
		return $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my $Rescues = $KindsLists->{&RescueKind};
	
	for my $rescue (@$Rescues) {
		
		my $statement = GetStatement($rescue);
		
		my ($ExceptionClasses) = $$statement =~ /\A\s*([^=]*)/;
		$ExceptionClasses =~ s/\s*//g;
		
		my @classes = split ",", $ExceptionClasses;
		push @classes, "" if scalar @classes ==0;
		for my $class ( @classes ) {

			if (($class eq "") || (exists $GENERIC_EXCEPTIONS{$class})) {
				$nb_GenericCatches++;
				Erreurs::VIOLATION($GenericCatches_mnemo, "rescue with generic exception $class at line : ".GetLine($rescue));
			}
		}
		
		if (scalar @{GetChildren($rescue)} == 0) {
			my $beginLine = GetLine($rescue);
			my $endLine = GetEndline($rescue);
			my $begin_index = $agglo_index->[$beginLine];
			my $end_index = $agglo_index->[$endLine];
			
			if (substr ($$agglo, $begin_index, ($end_index-$begin_index)) !~ /C/) {
				$nb_EmptyCatches++;
				Erreurs::VIOLATION($EmptyCatches_mnemo, "Empty rescue at line : ".GetLine($rescue));
			}
		}
	}

	
		
	$status |= Couples::counter_add($compteurs, $EmptyCatches_mnemo, $nb_EmptyCatches);
	$status |= Couples::counter_add($compteurs, $GenericCatches_mnemo, $nb_GenericCatches);
	
	return $status;
}

1;


