package Ruby::ParseRuby;
# les modules importes
use strict;
use warnings;

use Erreurs;

use CountUtil;

use Lib::Node; # qw( Leaf Node Append );
use Lib::NodeUtil ;
use Lib::NodeUtil qw( SetName SetStatement SetLine GetLine SetEndline GetEndline); 
use Lib::ParseUtil;

use Ruby::RubyNode;
use CountUtil;

my $DEBUG = 1;

my @rootContent = ( 
			#\&parseSimple_Stmt,
			#\&parseCompound_Stmt,
			\&parseClass,
			\&parseModule,
			\&parseMethod,
			\&parseIf,
			\&parseUnless,
			\&parseWhile,
			\&parseFor,
			\&parseLoop,
			\&parseUntil,
			\&parseCase,
			\&parseBegin,
			\&parseBEGIN_END,
			\&parseAlias,
);

my @moduleContent = (
			\&parseMethod,
			\&parseClass,
			\&parseModule,
			\&parseAlias,
			\&parseIf,
			\&parseUnless,
			\&parseWhile,
			\&parseFor,
			\&parseLoop,
			\&parseUntil,
			\&parseCase,
			\&parseBegin,
);

my @classContent = (
			\&parseMethod,
			\&parseAccessor,
			\&parseVisibility,
			\&parseClass,
			\&parseModule,
			\&parseIncludeExtendPrepend,
			\&parseAlias,
			\&parsePrivateClassMethod,
			\&parseIf,
			\&parseUnless,
			\&parseWhile,
			\&parseFor,
			\&parseLoop,
			\&parseUntil,
			\&parseCase,
			\&parseBegin,
);

my @statementContent = (
			\&parseIf,
			\&parseUnless,
			\&parseWhile,
			\&parseFor,
			\&parseLoop,
			\&parseUntil,
			\&parseCase,
			\&parseMethod,
			\&parseClass,
			\&parseModule,
			\&parseBegin,
			\&parseAlias,
			\&parseReturn,
			\&parseYield,
			\&parseRescue,
			\&parseEnsure
);

# A closable (or atomic) expression is a self coherent expression not implying to be followed by an Lvalue.
# This kind of expression can be followed by additional syntaxes
# - reversed controle syntaxes. Ex: <statement expression> if <condition>
# - blocs in parameters.  Ex: next if references.any? do |reference| ...
#                                  -> do is an argument of the method .any?, and belongs to the condition of the if !!!

# use this context if the next statement follows an atomic expression ...
my @atomicExpressionContent = (
			\&parseIf,
			\&parseUnless,
			\&parseLambda,
			\&parseBegin,
			\&parseCase,
			#\&parseBloc,
);

# the next statement is hardly consistent with the previous statement ...
my @complexExpressionContent = (
			\&parseTrailingConditions,
			\&parseBloc,
);


my @StmtContent = (\&parseSimple_Stmt, \&parseCompound_Stmt);

my @Simple_StmtContent = ();

my @Compound_StmtContent = (
			#\&parseFunction,
			#\&parseIf,
			#\&parseWhile,
			#\&parseFor,
			#\&parseTry,
			#\&parseWith,
			#\&parseClass,
			#\&parseDecoration,
			#\&parseAsync,
			#\&parseBreak,
			#\&parseContinue,
			#\&parseReturn,
			#\&parseImport,
			#\&parseFrom,
			#\&parseRaise,
			#\&parseYield,
			#\&parsePass,
);

my $NullString = '';

my $StringsView = undef;

my $BINARY_OPS = qr/(?:\+|-|\/|\*|\%|=|<|>|&|\||,|\.|[^:]\b(?:and|or|not|in)\b)/;

# operators that require a left value (operator that can not begin an expression, that means that wath is preceding belongs to the same expression)
my $LEFT_VALUE_OPS = qr/(?:\?)/;

# operators that require a right value (operator that can not end an expression, that means that wath is following belongs to the same expression)
# NOTE : ! preceded with \w, ), ] or } is not the "not" operator.
my $RIGHT_VALUE_OPS = qr/(?:[^\w\]\)\}]\s*!|not)/;

 
########################## CONTEXT ###################

use constant STATEMENT => 0;
use constant EXPRESSION => 1;

my @context = ();

sub initContext() {
	@context = (STATEMENT);
}

sub pushContext($) {
	push @context, shift;
}

sub popContext() {
	if (scalar @context) {
		pop @context;
	}
	else {
		print "PARSE ERROR : context underflow !\n";
	}
}

sub getContext() {
	return $context[-1];
}

sub purgeEndOfStatement() {
	while ((defined nextStatement()) && ((${nextStatement()} eq "\n") || (${nextStatement()} eq ";"))) {
		getNextStatement();
	}
}

sub purgeNewLines() {
	while ((defined nextStatement()) && ((${nextStatement()} eq "\n"))) {
		getNextStatement();
	}
}

sub setIndentation($;$) {
	my $node = shift;
	my $item = shift;
	
	if ((defined $item) && ($item =~ /\A\s*(?:CHAINE_\d+|__REGEXP__|__ARRAY_OF_SYMBOLS__)/)) {
		my $indentation = Lib::ParseUtil::getIndentation();
		my $line_indentation = Lib::ParseUtil::getLineIndentation();
		my $blanked = ${Lib::ParseUtil::getBlankedIndentation2()};

		# these elements replace original item of the code source, but with an additional space => do not count this space  !!!
		$indentation =~ s/ $//m;
		$blanked =~ s/ $//m;
		$line_indentation =~ s/ $//m;
		
		setRubyKindData($node, 'indentation', $indentation);
		setRubyKindData($node, 'line_indentation', $line_indentation);
		setRubyKindData($node, 'blanked_indentation', $blanked);
	}
	else {
		setRubyKindData($node, 'indentation', Lib::ParseUtil::getIndentation());
		setRubyKindData($node, 'line_indentation', Lib::ParseUtil::getLineIndentation());
		setRubyKindData($node, 'blanked_indentation', ${Lib::ParseUtil::getBlankedIndentation2()});
	}
}

sub setNextIndentation($) {
	my $node = shift;
	setRubyKindData($node, 'indentation', Lib::ParseUtil::getNextIndentation());
	setRubyKindData($node, 'blanked_indentation', ${Lib::ParseUtil::getNextBlankedIndentation()});
	setRubyKindData($node, 'line_indentation', Lib::ParseUtil::getLineIndentation());
}

sub parseParenthesis($) {
	my $open = shift;
	my $close;
	my $reg;
	
	my $parenthLine = getStatementLine();
	
	if ($open eq "{") {
		$close = "}";
		$reg = qr/([\{\}])/;
	}
	elsif ($open eq "[") {
		$close = "]";
		$reg = qr/([\[\]])/;
	}
	else {
		$open = "(";
		$close = ")";
		$reg = qr/([()])/;
	}

	# get the openning
	my $statement = ${getNextStatement()};
	my $opened = 1;
	
	while ($opened) {
		if (defined nextStatement()) {
			my $next = ${nextStatement()};
			my @items = grep {$_ ne ''} split /$reg/, $next;
			my $nb_used=0;
			for my $item (@items) {
				$nb_used++;
				if ($item eq $open) { $opened++};
				if ($item eq $close) { $opened--};
				
				$statement .= $item;
				
				if (! $opened) {
					last;
				}
			}
			
			if ($nb_used < scalar @items) {
				# closing has been found and there are remaining statement parts ...
				# so insert them in current position in the statement list.
				#splice @items, 0, $nb;
				#Lib::ParseUtil::insertStatements(undef, \@items);
				
				# closing has been found and there are remaining statement after it ...
				# So replace the statement with the corresponding split ...
				Lib::ParseUtil::replaceCurrentStatementWithList(\@items);
				# ... and trash the items used.
				for (my $i=0; $i<$nb_used; $i++) {
					getNextStatement();
				}
			}
			else {
				# trash the statement that has been used.
				getNextStatement();
			}
		}
		else {
			print "PARSE ERROR : missing \"$close\", openned at line $parenthLine\n";
			last;
		}
	}
	return $statement;
}

sub isNextClosingBrace() {
	if ( ${nextStatement()} eq '}' ) {
		return 1;
	}  
	return 0;
}

sub isNextClosingParenthese() {
	if ( ${nextStatement()} eq ')' ) {
		return 1;
	}  
	return 0;
}

sub isNextClosingBracket() {
	if ( ${nextStatement()} eq ']' ) {
		return 1;
	}  
	return 0;
}

my %H_CLOSING_CB = (
	'(' => \&isNextClosingParenthese,
	'{' => \&isNextClosingBrace,
	'[' => \&isNextClosingBracket,
);

my %H_CLOSING = (
	'(' => ')',
	'{' => '}',
	'[' => ']',
);

sub parseEnclosedExpression($) {
	my $parent = shift;;
	
	#my $enclosedNode = Node(UnknowKind, createEmptyStringRef());

	my $r_statement = GetStatement($parent);
	
	# get opening
	my $opening = ${getNextStatement()};
	my $openingLine = getStatementLine();

	$$r_statement .= $opening;
	
	#my $callbacks = [ $H_CLOSING_CB{$opening}, \&isNextQuestionMark, \&isNextColon ];
	my $callbacks = [ $H_CLOSING_CB{$opening} ];
	
	parseExpression($parent, $callbacks );
	
	# do several calls to parseExpression, because only closing pattern can stop.
	while ((defined nextStatement()) && (${nextStatement()} ne $H_CLOSING{$opening})) {
		
		#if (${nextStatement()} eq "?") {
		#	$bool_Ternary = 1;
		#}
		#elsif (${nextStatement()} eq ":") {
			
		#}
		# get item (certainly ";") that ended unexpectedly the expression in the previous parseExpression() call.
		$$r_statement .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
		parseExpression($parent, $callbacks);
	}
					
	# get closing
	if (defined nextStatement()) {
		$$r_statement .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
	}
	else {
		Lib::Log::ERROR("Missing closing for $opening opened at line $openingLine");
	}

	#my $parentStatement = GetStatement($parent);
	#$$parentStatement .= $$r_statement;
}

sub isPreviousOpExpectingRValue($) {
	my $previous = shift;
	
	if ($$previous =~ /(?:${BINARY_OPS}|${RIGHT_VALUE_OPS})\s*\z/) {
		return 1;
	}
	
	# ":" is not an opérator, but before a indentifier it defines a symbol, and is therefore expecting an identifier. And after an indentifier, it can be :
	# - a symbol used as a hash key (ex {:a => 123} can be writed {a: 123}
	# - a keyword argument declaration
	# - a keayword argument passing
	#      => in all its use-cases, colon ":" is never ending an expression, even followed with a new line
	if ($$previous =~ /(?::)\s*\z/) {
		return 1;
	}
	
	return 0;
}

sub isNextOpExpectingLValue($) {
	my $next = shift;
	
	if ($$next =~ /\A\s*(?:${BINARY_OPS}|${LEFT_VALUE_OPS})/) {
		return 1;
	}
	return 0;
}

# atomic means self-coherent : the expression could end at this point, and that would be syntaxically OK. But this does not mean that the expression is finished.
# atomic means :
# - end with \w, ), }.   Eventually followed by ! or ?
# - is empty
sub isPreviousAtomic($) {
	my $previous = shift;
	
	if ($$previous =~ /\A\s*\z|[\w\)\]\}]\s*([!\?]\s*)?\z/) {
		return 1;
	}
	return 0;
}

sub nextTokenIsEndingInstruction($) {
	my $r_previousToken = shift;
	
	my $r_previousBlanks = Lib::ParseUtil::getSkippedBlanks();
	
	if (isPreviousOpExpectingRValue($r_previousToken)) {
		return 0;
	}
	
	if (defined nextStatement()) {

		# INSTRUCTIONS SEPARATOR
		# ----------------------
		# a ';' or '}' always ends a statement expression.
		my $next = nextStatement();
		
		my $idx = 0;
		while ((defined nextStatement($idx)) && (${nextStatement($idx)} !~ /\S/)) {
			$idx++;
		}
		my $nextNonBlank = nextStatement($idx);
		
		return 1 if (! defined $nextNonBlank);
		
		if (($$nextNonBlank eq ';') || ($$nextNonBlank eq '}') || ($$nextNonBlank eq ')') || ($$nextNonBlank eq ']') || ($$nextNonBlank eq 'end')) {
			return 1;
		}
		
		if (isNextOpExpectingLValue($nextNonBlank)) {
			return 0;
		}
		
		if ($$next eq "\n") {
			# no continuation pattern have been detected before, so end of line is ending expression
			return 1;
		}
	}
	else {
		# if next token is NOT DEFINED, then return true because another try to
		# retrieve the next token will not provide a new statement.
		return 1;
	}

	return 0;
}

sub parseExpression($;$$);
sub parseExpression($;$$) {
	my $parent = shift;
	my $cb_end = shift;
	my $indentation_reference = shift;
	if (! defined $indentation_reference) {
		$indentation_reference = "line_indentation";
	}

	pushContext(EXPRESSION);

    # true if the keyword 'new' is encountered.
    # In this case, when encountering a "<" it is certainely a template openning, and not a comparison operator.
    my $new_context = 0;

	#enterContext(EXPRESSION);

	my $endOfStatement = 0;
	
	my $r_statement = createEmptyStringRef();
	my $parentStatement = GetStatement($parent);
	# during treatment, parent will have a local statement, that will be added to previous statement at end of function.
	SetStatement($parent, $r_statement);
	
	# parse the statement until the end is encountered.
	while ((defined nextStatement()) && (! $endOfStatement)) {

		# check if the next statement correspond to a token that ends the expression.
		if (defined $cb_end) {
			for my $cb (@{$cb_end}) {
				if ($cb->()) {
					$endOfStatement = 1;
					last;
				}
			}
		}

		if (! $endOfStatement) {

			my $skippedBlanks = ${Lib::ParseUtil::getSkippedBlanks()};

			my $subNode;

			#~ if (${nextStatement()} eq 'end') {
				#~ print STDERR "[ParseRuby::parseExpression] ERROR : unexpected keyword 'end' inside expression at line ".getNextStatementLine()."\n";
				#~ print STDERR "[ParseRuby::parseExpression] ----> current statement = $$r_statement\n";
			#~ }

			#if (($$r_statement =~ /[=\(\[\{]\s*$/m) && ${nextStatement()} =~ "{") {
			if ((${Lib::ParseUtil::getLastNonBlankStatement()}) && ${nextStatement()} =~ "{") {
				# it's a hash literal, becaue a bloc can not be assigned nor used after (, { or [
				# Do nothing .. (just to prevent from trying to parse a bloc !!!)
			}
			elsif (isPreviousAtomic($r_statement)) {
				# r_statement is atomic, means what is following is not an Lvalue !!!
				$subNode = Lib::ParseUtil::tryParse(\@complexExpressionContent, $r_statement);
			}
			else {
				# Next token belongs to the expression. Parse it.
				$subNode = Lib::ParseUtil::tryParse(\@atomicExpressionContent, $r_statement);
			}

			if (defined $subNode) {
				# *** SYNTAX has been recognized by CALLBACK
				if (ref $subNode eq "ARRAY") {
					Append($parent, $subNode);
					#if (GetLine($subNode) == GetLine($parent)) {
						# if child and parent are on the same line, indentation of child is indentation of the parent ...
						my $expression_indentation = Ruby::RubyNode::getRubyKindData($parent, $indentation_reference);
						if (! defined $expression_indentation) {
							print STDERR "WARNING: unable to find indentation reference of '$parent->[0]' for expresion.";
						}
						Ruby::RubyNode::setRubyKindData($subNode, 'expression_indentation', $expression_indentation);
					#}
					$$r_statement .= $skippedBlanks . Ruby::RubyNode::nodeLink($subNode);
				}
				else {
					$$r_statement .= $$subNode;
				}
			}
			else {
				my $next = nextStatement();
				
				if (($$next eq "(") || ($$next eq "{") || ($$next eq "[")) {
					$$r_statement .= $skippedBlanks;
					parseEnclosedExpression($parent);
				}
		
				else {
					# get the next statement, as planed ...
					$$r_statement .= $skippedBlanks . ${getNextStatement()};
				}
			}
		}

		if ( nextTokenIsEndingInstruction($r_statement)) {
#print "--> CHECK END EXPRESSION\n";
			last;
		}
	}

	$$parentStatement .= $$r_statement;
	SetStatement($parent, $parentStatement);

#print "EXPRESSION : $$parentStatement\n";

	popContext();

	#leaveContext();
	return $r_statement;
}

# test cases :
# - statement followed by \n
# - statement followed by :
# - statement followed by \\\n
# - statement with multiline (), [] or {}
sub parseUnknow() {
	my $node = Node(UnknowKind, createEmptyStringRef());
	SetLine($node, getNextStatementLine());
	
	my $indentation = Lib::ParseUtil::getNextIndentation();
	my $blanked = ${Lib::ParseUtil::getNextBlankedIndentation()};
	my $line_indentation = Lib::ParseUtil::getLineIndentation();
	#~ if (${nextStatement()} =~ /\A\s*(?:CHAINE_\d+|__REGEXP__|__ARRAY_OF_SYMBOLS__)/) {
		#~ # these elements replace original item of the code source, but with an additional space => do not count this space  !!!
		#~ $indentation =~ s/ $//m;
		#~ $blanked =~ s/ $//m;
		#~ $line_indentation =~ s/ $//m;
	#~ }
	setRubyKindData($node, 'indentation', $indentation);
	setRubyKindData($node, 'blanked_indentation', $blanked);
	setRubyKindData($node, 'line_indentation', $line_indentation);

	parseExpression($node);

	purgeNewLines();

	if (${GetStatement($node)} =~ /\A\s*end\b/) {
		print STDERR "WARNING : unexpected end keyword at line ".GetLine($node)." (means certainly parsing error)\n";
	}

	return $node;
}

################# MAGIC NUMBERS #########################

my %H_MagicNumbers;

sub initMagicNumbers($) {
  my $view = shift;

  %H_MagicNumbers = ();
  $view->{HMagic} = \%H_MagicNumbers;
}

sub declareMagic($) {
   my $magic = shift;

   my $artiKey = Lib::ParseUtil::getCurrentArtifactKey();
   if (! exists $H_MagicNumbers{$artiKey}) {
     my %Hash = ();
     $H_MagicNumbers{$artiKey} = \%Hash;
   }

   if (! exists $H_MagicNumbers{$artiKey}->{$magic}) {
     $H_MagicNumbers{$artiKey}->{$magic}=1;
   }
   else {
     $H_MagicNumbers{$artiKey}->{$magic}++;
   }
#print "---> MAGIC = $magic\n";
}

sub getMagicNumbers($$) {
  my $r_expr = shift;
  my $isVarInitContext = shift;

  # reconnaissance des magic numbers :
  # 1) identifiants commencant forcement par un chiffre decimal.
  # 2) peut contenir des '.' (flottants)
  # 3) peut contenir des 'E' ou 'e' suivis eventuellement de '+/-' pour les flottants
  while ( $$r_expr =~ /(?:^|[^\w])((?:\d|\.\d)(?:[e][+-]?|[\d\w\.])*)/sg )
  {
    my $magic = $1;
    
    if ($isVarInitContext) {
      # if the expression is composed only with the magic number that is a 
      # variable initialisation at declaration, then consider as a "constant",
      # not a magic number...
      my $quotedMagic = quotemeta $magic;
      if ( $$r_expr =~ /^\s*$quotedMagic\s*$/sm) {
        return;
      }
      else {
	# do not make this test if the expression contains other magics.
        $isVarInitContext = 0;
      }
    }
     declareMagic($magic);
  }
}

##################################################################
#                   GETTERS
##################################################################

my %peer = ('(' => ')', '{' => '}', '[' => '\]');

sub getUntilPeer($$) {
	my $stmt = shift;
	my $open = shift;
	my $close = $peer{$open};
	if (!defined $close) {
		print "[ParseRuby] ERROR : no closing peer for $open !!!";
		return "";
	}
	
	my $openned = 1;
	my $expr = "";
	
	while ($$stmt =~ /\G([^$open$close]*)([$open$close])/sg) {
		$expr .= $1.$2;
		if ($2 eq $open) {
			$openned++;
		}
		else {
			$openned--;
			if (! $openned) {
				last;
			}
		}
	}
	return $expr;
}

sub getDefaultValue($) {
	my $stmt = shift;
	my $default = "";
	#my $lastPos = pos($$stmt);
	while ($$stmt =~ /\G([^,()\[\{]*)(.)/sg) {
		$default .= $1;
		if (($2 eq '(') || ($2 eq '{') || ($2 eq '[')) {
			$default .= $2;
			$default .= getUntilPeer($stmt, $2);
			if (! defined pos($$stmt)) {
				# end of function proto has been encountered before the end of the default value expression.
				return $default;
			}
		}
		else {
			# a character ',' or ')' has been encountered, this is the end of the default value expression.
			pos($$stmt)--;
			return $default;
		}
		#$lastPos = pos($$stmt);
	}
	# after the while has ended whithout matching regexpr, the pos is set to undef by the perl regexp motor.
	# so me have to restaure the last valid value in order to pursue matching later.
	#pos($$stmt) = $lastPos;
	return $default;
}

#~ sub getArguments($) {
	#~ my $artifactNode = shift;
	#~ my %argList = ();
	
	#~ my $stmt = GetStatement($artifactNode);
	#~ my $line = 0;
	#~ # stop after the first "(" or the last "\n" before the first non blank!
	#~ if ( $$stmt !~ /\A[^(]*\(/gc) {
		#~ return {};
	#~ }
	
	#~ if ( $$stmt =~ /\G([ \t]*\n)+/gc) {
		#~ # new line encountered, count them ...
		#~ $line += () = $1 =~ /\n/g;
	#~ }
	
	#~ my $indent;
	#~ my $name;
	#~ my $argLine;
	#~ my $default;
	#~ while ($$stmt =~ /\G(?:(\s*)(\w+)|(\s*=\s*)|(\s*(,|\))([ \t]*\n)?))/sg) {
		#~ # parse argument name + indentation
		#~ if (defined $2) {
			#~ $indent = $1;
			#~ $name = $2;
			
			#~ $line += () = $indent =~ /\n/g;
			#~ $indent =~ s/.*\n//sg;
			#~ $argLine = $line;
			#~ $default = undef;
#~ print STDERR "ARGUMENT : $name, indent = >$indent<, line = $argLine\n";
		#~ }
		#~ # parse default value
		#~ elsif (defined $3) {
			#~ $line += () = $3 =~ /\n/g;
			#~ $default = getDefaultValue($stmt);
			#~ $line += () = $default =~ /\n/g;
			#~ # remove trailing blanks
			#~ $default =~ s/\s*$//m;
#~ #print "-->default value = $default\n";
			#~ if (! defined pos($$stmt)) {
				#~ print STDERR "[ParseRuby] Syntax error in argument default value parsing : $default\n";
				#~ $argList{$name} = [$argLine, $indent, $default];
				#~ last;
			#~ }
		#~ }
		#~ # next argument or 
		#~ else {
			#~ if (defined $name) {
				#~ $argList{$name} = [$argLine, $indent, $default];
			#~ }
			#~ if ($5 eq ",") {
				#~ $line += () = $4 =~ /\n/g;
			#~ }
			#~ else {
				#~ last;
			#~ }
		#~ }
	#~ }
	
	#~ return \%argList;
#~}

##################################################################
#              GENERIC
##################################################################

sub ParseGeneric($) {
	my $kind = shift;

	my $statement = "";
	my $node = Node($kind, \$statement);
	my $proto = '';
	
	# consumes the keyword
	#getNextStatement();
	
	SetLine($node, getStatementLine());
	#setRubyKindData($node, 'indentation', Lib::ParseUtil::getIndentation());
	setIndentation($node);
	
	# do not parse expression if the keyword is followed with a \n
	#if (${nextStatement()} ne "\n") {
	
	# do not parse expression if the keyword is followed with an empty expression
	if (! nextTokenIsEndingInstruction(\$statement)) {
		parseExpression($node);
	}
	
	return $node;
}

##################################################################
#              YIELD
##################################################################

sub isNextYield() {
	if ( ${nextStatement()} eq 'yield' ) {
		return 1;
	}  
	return 0;
}

sub parseYield() {
	if (isNextYield()) {
		getNextStatement();
		return ParseGeneric(YieldKind);
	}
	return undef;
}


##################################################################
#              RAISE
##################################################################

sub isNextRaise() {
	if ( ${nextStatement()} eq 'raise' ) {
		return 1;
	}  
	return 0;
}

sub parseRaise() {
	if (isNextRaise()) {
		getNextStatement();
		return ParseGeneric(RaiseKind);
	}
	return undef;
}

##################################################################
#              RETURN
##################################################################

sub isNextReturn() {
	if ( ${nextStatement()} eq 'return' ) {
		return 1;
	}  
	return 0;
}

sub parseReturn() {
	if (isNextReturn()) {
		getNextStatement();
		return ParseGeneric(ReturnKind);
	}
	return undef;
}

##################################################################
#              CONTINUE
##################################################################

sub isNextContinue() {
	if ( ${nextStatement()} eq 'continue' ) {
		return 1;
	}  
	return 0;
}

sub parseContinue() {
	if (isNextContinue()) {
		getNextStatement();
		return ParseGeneric(ContinueKind);
	}
	return undef;
}

##################################################################
#              BREAK
##################################################################

sub isNextBreak() {
	if ( ${nextStatement()} eq 'break' ) {
		return 1;
	}  
	return 0;
}

sub parseBreak() {
	if (isNextBreak()) {
		getNextStatement();
		return ParseGeneric(BreakKind);
	}
	return undef;
}

##################################################################
#              ALIAS
##################################################################

sub parseAlias() {
	if (${nextStatement()} =~ /\A\s*(alias(_method)?)\b/gc) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		my $statement = $1;
		my $aliasNode = Node(AliasKind, \$statement);
		SetLine($aliasNode, getStatementLine());
		#setRubyKindData($aliasNode, 'indentation', Lib::ParseUtil::getIndentation());
		setIndentation($aliasNode);
		
		# NOTE : there can be a space between ":" and \w+, when \w+ matches CHAINE_XX ...
		# use \$?\w+\?? to match variable name ...
		#if (${nextStatement()} =~ /\A(\s*:?\s*\$?\w+\??)/gc) {
		if (${nextStatement()} =~ /\A\s*(:\s+CHAINE_\d+|[^\s,]+)/gc) {
			Lib::ParseUtil::splitAndFocusNextStatementOnPos();
			$statement .= ${Lib::ParseUtil::getSkippedBlanks()}.$1;
			
			if (${nextStatement()} =~ /\A\s*,/gc) {
				# trash comma if any ...
				Lib::ParseUtil::splitAndFocusNextStatementOnPos();
			}
			
			if (${nextStatement()} =~ /\A(\s*\S+)/gc) {
				Lib::ParseUtil::splitAndFocusNextStatementOnPos();
				$statement .= ${Lib::ParseUtil::getSkippedBlanks()}.$1;
			}
			else {
				print STDERR "ERROR : unknow syntax for second alias parameter at line ".getNextStatementLine()." : ".${nextStatement()}."\n";
			}
		}
		else {
			print STDERR "ERROR : unknow syntax for first alias parameter at line ".getNextStatementLine()." : ".${nextStatement()}."\n";
			
		}
		
		purgeEndOfStatement();
		
		return $aliasNode;
	}
	return undef;
}

##################################################################
#              INCLUDE / EXTEND / PREPEND
##################################################################

sub parseIncludeExtendPrepend() {
	if ( ${nextStatement()} =~ /\A\s*(include|extend|prepend)/gc ) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		my $kind = ($1 eq 'include' ? IncludeKind : ($1 eq 'extend' ? ExtendKind : PrependKind));
		return ParseGeneric($kind);
	}
	return undef;
}

##################################################################
#              BEGIN { ... }   END { ... }
##################################################################

sub parseBEGIN_END() {
	if (${nextStatement()} =~ /\A\s*(BEGIN|END)\b/gc) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		my $kind = ($1 eq "BEGIN" ? BEGINKind : ENDKind);
		my $beginNode = Node($kind, createEmptyStringRef());
		SetLine($beginNode, getStatementLine());
		#setRubyKindData($beginNode, 'indentation', Lib::ParseUtil::getIndentation());
		setIndentation($beginNode);
		
		purgeNewLines();
		
		my $bloc = parseBloc();
		if (defined $bloc) {
			Append($beginNode, $bloc);
		}
		
		purgeNewLines();
		
		return $beginNode;
	}
	return undef;
}

##################################################################
#              CONDITION (IF / WHILE / FOR ...)
##################################################################

sub isEndOfCondition($) {
	my $previous = shift;
	
	# get next non nul statement ...
	my $next = nextStatement();
	my $idx=1;
	while ($$next !~ /\S/) {
		$next = nextStatement($idx++);
	}
	
	# check next non nul statement ...
	if ( ($$next eq "then") || ($$next eq "do") || ($$next =~ /^\s*(\?)/gmc)) {
		return 1;
	}
	
	if ($$previous =~ /${BINARY_OPS}\s*\z/) {
		return 0;
	}
	
	if ($$next =~ /\A\s*${BINARY_OPS}/) {
		return 0;
	}
	
	return 1;
}

sub isNextDo() {
	if (${nextStatement()} eq "do") {
		return 1;
	}
	return 0;
}

sub isNextThen() {
	if (${nextStatement()} eq "then") {
		return 1;
	}
	return 0;
}

sub isNextEnd() {
	if (${nextStatement()} eq "end") {
		return 1;
	}
	return 0;
}

sub parseCondition(;$) {
	
	my $kind = shift;
	
	my $condition = '';
	my $condNode = Node(ConditionKind, \$condition);
	
	setNextIndentation($condNode);
	
	SetLine($condNode, getStatementLine());
	
	# Condition ends :
	# - on "do" or "then"  	(ex: if <condition> then ... // while <condition> do ...)
	# - on "end" or "}"    	(ex: 5.times { |i| break if i==0 }    -- or same with do ... end syntax ! )
	# - on ")" 				(ex: @department = (department_parts.join( CHAINE_2 ).to_sym unless department_parts.empty?)
	# - on "\n" if context is ending the expression ....
	#~ my $stmt;
	#~ while ((defined ($stmt = nextStatement())) && (($$stmt ne "then") && ($$stmt ne "do") && ($$stmt ne "end") && ($$stmt ne "}") && ($$stmt ne ")"))) {
		
		#~ $condition .= ${Lib::ParseUtil::getSkippedBlanks()};

		#~ if ($$stmt eq "\n") {
			#~ if (isEndOfCondition(\$condition)) {
				#~ last;
			#~ }
			#~ else {
				#~ $condition .= ${getNextStatement()};
			#~ }
		#~ }
		#~ elsif ($$stmt eq "(") {
			#~ $condition .= parseParenthesis('(');
		#~ }
		#~ elsif ($$stmt eq "[") {
			#~ $condition .= parseParenthesis('[');
		#~ }
		#~ elsif ($$stmt eq "{") {
			#~ $condition .= parseParenthesis('{');
		#~ }
		#~ else { 
			#~ $condition .= ${getNextStatement()};
		#~ }
	#~ }
	
	my $endCallbacks = [ \&isNextThen, \&isNextEnd];
	if ((defined $kind) && (($kind eq WhileKind) || ($kind eq ForKind) || ($kind eq UnlessKind))) {
		push @$endCallbacks, \&isNextDo;
	}
	
	# NOTE : indentation of code inside the condition should have the blanked indentation of the control ...
	parseExpression($condNode, $endCallbacks, "blanked_indentation");
#print "CONDITION : $condition\n";

	# if condition ends with "then" or "do" => trash these statements ...
	if ((defined nextStatement()) && ((${nextStatement()} eq 'then') || (${nextStatement()} eq 'do')) ) {
		# consumes the keyword
		getNextStatement();
	}
	
	return $condNode;
}

##################################################################
#              TRAILING CONDITION
##################################################################

my $TRAILING_KINDS = {
	"if"		=> TrailingIfKind,
	"unless"	=> TrailingUnlessKind,
	"until"		=> TrailingUntilKind,
	"while"		=> TrailingWhileKind,
};

sub parseTrailingConditions() {
	if (${nextStatement()} =~ /\A\s*\b(if|while|unless|until)\b/gc) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		my $node = Node($TRAILING_KINDS->{$1}, createEmptyStringRef());
		setIndentation($node);
		
		SetLine($node, getStatementLine());
		
		purgeNewLines();
		
		my $condNode = parseCondition();
		if (defined $condNode) {
			Append($node, $condNode);
		}
		else {
			# no cond node means no more statement... useless to pursue
			print STDERR "[ParseRuby::parseTrailingConditions] ERROR : missing condition at line ".getNextStatementLine().": ".${nextStatement()}."\n";
			return undef;
		}

		# do not purge new lines because if the bloc belongs to an expression, new line could be significant to detect the end of the expression.

		return $node;
	}
	return undef;
}

##################################################################
#              IF
##################################################################


sub isNextIf() {
	if ( ${nextStatement()} eq 'if' ) {
		return 1;
	}  
	return 0;
}

sub parseIf() {
	if (isNextIf()) {
		getNextStatement();
		return parseControle(IfKind, ['elsif', 'else']);
	}
	return undef;
}

sub parseUnless() {
	if (${nextStatement()} eq 'unless') {
		getNextStatement();
		return parseControle(UnlessKind, ['else']);
	}
	return undef;
}

##################################################################
#              LOOPS
##################################################################

sub isNextWhile() {
	if ( ${nextStatement()} eq 'while' ) {
		return 1;
	}  
	return 0;
}

sub parseWhile() {
	if (isNextWhile()) {
		getNextStatement();
		return parseControle(WhileKind, []);
	}
	return undef;
}

sub isNextFor() {
	if ( ${nextStatement()} eq 'for' ) {
		return 1;
	}  
	return 0;
}

sub parseFor() {
	if (isNextFor()) {
		getNextStatement();
		return parseControle(ForKind, []);
	}
	return undef;
}

sub parseLoop() {
	if (${nextStatement()} =~ /\A\s*loop\b/gc) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		return parseControle(LoopKind, [], 0);
	}
	return undef;
}

sub parseUntil() {
	if (${nextStatement()} eq "until") {
		getNextStatement();
		return parseControle(UntilKind, []);
	}
	return undef;
}

##################################################################
#              CASE
##################################################################

sub parseCase() {
	if (${nextStatement()} eq "case") {
		getNextStatement();
		return parseControle(CaseKind, ['when', 'else'], 2); # (2 is for parse condition, but don't create cond node)
	}
	return undef;
}

##################################################################
#              BEGIN
##################################################################

sub parseBegin() {
	if (${nextStatement()} eq "begin") {
		getNextStatement();
		return parseControle(BeginKind, ['rescue', 'else', 'ensure'], 0, 1); # (0 for no condition to parse, but 1 to force "then" level ...)
	}
	return undef;
}

sub parseEmbeddedBegin() {
	if (${nextStatement()} eq "begin") {
		getNextStatement();
		return parseControle(BeginKind, ['rescue', 'else', 'ensure'], 0, 1, 1); # (0 for no condition to parse, 1 to force "then" level, 1 for expression context)
	}
	return undef;

}

##################################################################
#              BLOC
##################################################################

sub parseInstructions($$) {
	my $parentNode = shift;
	my $stopStmt = shift;
	
	my $stmt;
	while ((defined ($stmt = nextStatement())) && ($$stmt ne "end")) {
		for my $stop (@$stopStmt) {
			if ($$stmt eq $stop) {
				return;
			}
		}
		my $node = Lib::ParseUtil::tryParse_OrUnknow(\@statementContent);
		Append($parentNode, $node);
		purgeEndOfStatement();
	}
}
	
sub parseBloc(;$) {
	my $blocIsArtifactBody = shift;
	
	if ((${nextStatement()} eq "do") || (${nextStatement()} eq "{")) {
		my $begin = ${getNextStatement()};
		my $blocNode = Node(BlocKind, createEmptyStringRef());
		SetLine($blocNode, getStatementLine());
		#setRubyKindData($blocNode, 'indentation', Lib::ParseUtil::getLineIndentation());
		setIndentation($blocNode);
		
		purgeNewLines();
		
		if (${nextStatement()} =~ /\A\s*\|/gc) {
			my $rawParameters = "";
			Lib::ParseUtil::splitAndFocusNextStatementOnPos();
			my $insideParams = 1;
			while ($insideParams) {
				if (${nextStatement()} =~ /\A([^\|]*)\|/gc) {
					Lib::ParseUtil::splitAndFocusNextStatementOnPos();
					$rawParameters = $1;
					$insideParams = 0;
				}
				else {
					$rawParameters .= ${getNextStatement()};
				}
			}

			my $params = parseParameters(\$rawParameters);
			setRubyKindData($blocNode, "parameters", $params)
		}
		else {
			setRubyKindData($blocNode, "parameters", []);
		}

		purgeNewLines();
		
		my $previousLastArtifactUpdateState = Lib::ParseUtil::getLastArtifactUpdateState();
		
		Lib::ParseUtil::setLastArtifactUpdateState(1) if $blocIsArtifactBody;
		
		#parse instructions until "end" or "}"
		parseInstructions($blocNode, ["}"]);
		
		Lib::ParseUtil::setLastArtifactUpdateState($previousLastArtifactUpdateState) if $blocIsArtifactBody;

		if ((defined nextStatement()) && ((${nextStatement()} eq "end") || (${nextStatement()} eq "}"))) {
			getNextStatement();
			setRubyKindData($blocNode, "end_indentation", Lib::ParseUtil::getLineIndentation());
			SetEndline($blocNode, getStatementLine());
		}
		else {
			print STDERR "[ParseRuby::parseBloc] ERROR : missing bloc closing, opened at line ".GetLine($blocNode)."\n";
		}
		
		# do not purge new lines because if the bloc belongs to an expression, new line could be significant to detect the end of the expression.
		
		return $blocNode;
	}
	return undef;
}

##################################################################
#              LAMBDA
##################################################################

sub parseLambda() {
	if ((${nextStatement()} eq "lambda") || (${nextStatement()} eq "->")) {
	
		my $next2 = ${Lib::ParseUtil::nextNonBlank()};

		#if (($next2 ne "(") && ($next2 ne "do") && ($next2 ne "{")) {
		if ( $next2 =~ /\A[ \t]*[^\w\(\{ \t]/) {
			# it's not a lambda
			return undef;
		}
		
		my $statement = "";
		my $lambdaNode = Node(LambdaKind, \$statement);
		
		my $artiKey = Lib::ParseUtil::newArtifact('lambda', getNextStatementLine(), LambdaKind);

		setRubyKindData($lambdaNode, 'artifact_key', $artiKey);
		
		Lib::ParseUtil::setLastArtifactUpdateState(0);

		$statement .= ${getNextStatement()};
		my $line = getStatementLine();
		SetLine($lambdaNode, $line);

		my $rawParameters = "";
		while ((${nextStatement()} ne '{') && (${nextStatement()} ne 'do')){
			$rawParameters .= ${Lib::ParseUtil::getSkippedBlanks()}.${getNextStatement()};
		}
		$statement .= $rawParameters;
		
		my $params = parseParameters(\$rawParameters);
		setRubyKindData($lambdaNode, "parameters", $params);

		# parse bloc. Param 1 mean the function has to manage the bloc as an artifact body ...
		my $blocNode = parseBloc(1);
		Lib::ParseUtil::setLastArtifactUpdateState(1);
		
		if (defined $blocNode) {
			Append($lambdaNode, $blocNode);
		}
		else {
			print STDERR "[ParseRuby::parseLambda] ERROR : missing bloc for lambda statement at line $line\n";
		}
		
		Lib::ParseUtil::endArtifact($artiKey);
		setRubyKindData($lambdaNode, 'codeBody', Lib::ParseUtil::getArtifact($artiKey));		
		
		return $lambdaNode;
	}
	return undef;
}

##################################################################
#              CONTROLE
##################################################################

# FIXME ????
sub getEndBlocLine() {
	# return the (line-1) of the next instruction if any.
	# If the end of the program is encountered (no next instruction),
	# then the next statement line corresponds to the end of the program, and so the end of the bloc.
	if (defined nextStatement()) {
		return getNextStatementLine()-1;
	}
	return getNextStatementLine();
}

my $SUB_CONTROL_DEF = {
	# keyword	=>	[ kind, 		condition, 		cond mandatory]
	"else" 		=> 	[ ElseKind, 	undef, 			0],
	"elsif"		=> 	[ ElsifKind,	"NODE",			1],
	"case"		=>	[ CaseKind, 	"STATEMENT", 	1],
	"when"		=>	[ WhenKind, 	"STATEMENT", 	1],
	"then"		=>	[ ThenKind, 	undef,			0],
	"begin"		=>	[ BeginKind, 	undef,			0],
	"rescue"	=>	[ RescueKind, 	"STATEMENT",	0],
	"ensure"	=>	[ EnsureKind, 	undef,			0],
};

sub parseSubControle($$;$) {
	my $keyword = shift;
	my $stopStmt = shift;
	
	my $kind = $SUB_CONTROL_DEF->{$keyword}->[0];
	my $cond = $SUB_CONTROL_DEF->{$keyword}->[1];
	my $condMandatory = $SUB_CONTROL_DEF->{$keyword}->[2];
	
	my $statement = "";
	my $subControlNode = Node($kind, \$statement);
	SetLine($subControlNode, getStatementLine());
	#setRubyKindData($subControlNode, 'indentation', Lib::ParseUtil::getIndentation());
	#setRubyKindData($subControlNode, 'blanked_indentation', ${Lib::ParseUtil::getBlankedIndentation()});
	setIndentation($subControlNode);

	# PARSE EXPRESSION (condition) that follow the keyword  
	if ($condMandatory || (${nextStatement()} ne "\n")) {
		# there is a condition to parse, that can be on next line if mandatory => purge new line (if any) to access the consition ...
		purgeNewLines();
		
		if ((defined $cond) && ($cond ne "NONE")) {
		
			if ($cond eq "NODE") {
				my $condNode = parseCondition();
				if (defined $condNode) {
					Append($subControlNode, $condNode);
				}
				else {
					# no cond node means no more statement... useless to pursue
					print STDERR "[ParseRuby::parseSubControle] ERROR : missing condition at line ".getNextStatementLine().": ".${nextStatement()}."\n";
					return undef;
				}
			}
			elsif ($cond eq "STATEMENT") {
				parseExpression($subControlNode, [\&isNextThen]);
			}
		}
	}
	
	purgeEndOfStatement();
	
	if (${nextStatement()} eq "then") {
		getNextStatement();
		purgeEndOfStatement();
	}
	
	# PARSE INSTRUCTION
	parseInstructions($subControlNode, $stopStmt);

	SetEndline($subControlNode, getStatementLine());
	return $subControlNode;
}

sub parseControle($$;$$$) {
	my $kind = shift;
	my $subPartsKeywords = shift;
	my $parseCond = shift;
	my $thenLevel = shift;
	my $expressionContext = shift;
	
	pushContext(STATEMENT);
	
	# 0 : no cond to parse
	# 1 : parse cond and create a node (default)
	# 2 : parse cond, but don't create node
	$parseCond = (!defined $parseCond ? 1 : $parseCond);
	
	# if the condition is not on the same line than the keyword, this means there is no condition !!!
	# example : while !scan.eos?
	#				case
	#				when str = scan.scan(FLOAT)         then emit(:FLOAT, pos, scan.pos, meta, str)
	#				end
	#			end
	if (${nextStatement()} eq "\n") {
		$parseCond = 0;
	}
	
	if (! defined $thenLevel) {
		# "then" level only if there is a condition node.
		$thenLevel = (($parseCond == 1) ? 1 : 0);
	}

	my $statement = "";
	my $controlNode = Node($kind, \$statement);

	#trash 'if, while, for ...' keyword
	#getNextStatement();

	my $line = getStatementLine();
	SetLine($controlNode, $line);
	#setRubyKindData($controlNode, 'indentation', Lib::ParseUtil::getIndentation());
	#setRubyKindData($controlNode, 'blanked_indentation', ${Lib::ParseUtil::getBlankedIndentation()});
	setIndentation($controlNode);

	# CONDITION
	#-------------------------------------------------------------------
	if ($parseCond) {
		my $condNode = parseCondition($kind);
		if (defined $condNode) {
			if ($parseCond == 1) {
				Append($controlNode, $condNode);
			}
			else {
				$statement .= ${GetStatement($condNode)};
			}
		}
		else {
			# no cond node means no more statement... useless to pursue
			print STDERR "[ParseRuby::parseControle] ERROR : missing condition at line ".getNextStatementLine().": ".${nextStatement()}."\n";
			popContext();
			return undef;
		}
	}
	else {
		# loop kind has no condition but is followed by a "do"
		if ($kind eq LoopKind) {
			if (${nextStatement()} eq "do") {
				getNextStatement();
			}
		}
	}

	purgeEndOfStatement();

	my $artiKey = Lib::ParseUtil::newUnmodalArtifact('control_at_line', getStatementLine(), "ControlKind");
	setRubyKindData($controlNode, 'artifact_key', $artiKey);
	setRubyKindData($controlNode, 'bodyBeginLine', getStatementLine());
	
	# CONTENT
	#-------------------------------------------------------------------
	if ($thenLevel) {
#print STDERR "CONTROL NODE = $kind at line $line\n";
		my $thenNode = parseSubControle("then", $subPartsKeywords);
		Append($controlNode, $thenNode);
	}
	# no "then" level AND no sub parts ==> all instructions in the same bloc
	elsif (scalar @$subPartsKeywords == 0) {
		# parse instruction until encountering "end" keyword.
		parseInstructions($controlNode, []);
	}
	
	# if "end" keyword not encountered, parse sub parts ...
	my $next;
	while ((defined ($next = nextStatement())) && ($$next ne "end")) {
		
		my $subControlNode;
		for my $keyword (@$subPartsKeywords) {
			if ( $$next eq $keyword) {
				getNextStatement();
				#purgeNewLines();
				$subControlNode = parseSubControle($keyword, $subPartsKeywords);
				Append($controlNode, $subControlNode);
			}
		}
		
		# case unexpected statement ...
		if (! defined $subControlNode) {
			print STDERR "ERROR : unexpected statement (".${nextStatement()}.") inside controle structure '$kind' at line ".getNextStatementLine().": ".${nextStatement()}."\n";
			my $node = Lib::ParseUtil::tryParse_OrUnknow(\@statementContent);
			Append($controlNode, $node);
		}
		purgeEndOfStatement();
	}

	# restore encompassing context. (should be done here to know if purgeEndOfStatement should be called ...)
	popContext();

	setRubyKindData($controlNode, 'codeBody', Lib::ParseUtil::getArtifact($artiKey));
	Lib::ParseUtil::endUnmodalArtifact($artiKey);

	if (defined nextStatement()) {
		# trash "end"
		getNextStatement();
		setRubyKindData($controlNode, "end_indentation", Lib::ParseUtil::getLineIndentation());
#print STDERR "END keyword of $kind at line ".getStatementLine()."\n";

		if (getContext() != EXPRESSION) {
			
			# Following is to manage trailing condition after the "end". For example :
			#     begin
			#     ...
			#     end if <condition>
			if (defined nextStatement()) {
				my $node = parseTrailingConditions();
				if (defined $node) {
					print STDERR "WARNING : found condition statement applying to $kind at line $line, but forget it ...!!\n";
				}
			}
			
			# if the controle is embedded inside an expression, do not purge end of statement because it is needed to the parseExpression() function to recognize the end of the expression.
			# ex:
			# def toto()
			#   @options[:formatters] ||= begin
			#     ...
			#   end 
			# end
			# => parseExpression() will call parseBegin(). But if parseBegin() purge the \n after the end, then parseExpression() could not check the presence of the \n 
			# and then will include the following item (here the second "end") inside de expression, whereas it's a new statement !
			purgeEndOfStatement();
		}
	}

	SetEndline($controlNode, getEndBlocLine());
	
	return $controlNode;
}

##################################################################
#              CLASS
##################################################################

sub parseVisibility() {
	if (${nextStatement()} =~ /^\s*(private|public|protected)\b/gc) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		my $contextNode = Node("CONTEXT", createEmptyStringRef());
		SetLine($contextNode, getStatementLine());
		setIndentation($contextNode);
	
		my $data = {};
		
		setRubyKindData($contextNode, "data", $data);
		
		if (${nextStatement()} eq "\n") {
			$data->{"visibility_zone"} = $1;
		}
		elsif (${nextStatement()} eq "def") {
			my $methodNode = parseMethod();
			setRubyKindData($methodNode, "visibility", $1);
			return $methodNode;
		}
		else {
			my $symbols_list = "";
			while ((defined nextStatement()) && (${nextStatement()} ne "\n")) {
				$symbols_list .= ${getNextStatement()};
			}
			
			purgeEndOfStatement();
			
			$symbols_list =~ s/\A\s*\[//;
			$symbols_list =~ s/\]\s*\z//;
			$symbols_list =~ s/\s*//g;
			my @symbols = split(/,/, $symbols_list);
			for my $symb (@symbols) {
				$data->{"symbols"}->{$symb} = $1;
			}
		}
		return $contextNode;
	}
	return undef;
}

sub parseAccessor() {
	
	if (${nextStatement()} =~ /^\s*attr(?:_(reader|writer|accessor))?\b/gc) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		my $kind = AttrAccessorKind;
		if (defined $1) {
			if ($1 eq 'reader') {
				$kind = AttrReaderKind;
			}
			elsif ($1 eq 'writer') {
				$kind = AttrWriterKind;
			}
		}
		else {
			# since Ruby 1.9 attr is depreceted and is just an alias for attr_reader
			$kind = AttrReaderKind; 
		}
		my $accessNode = Node($kind, createEmptyStringRef());
		SetLine($accessNode, getStatementLine());
		#setRubyKindData($accessNode, 'indentation', Lib::ParseUtil::getIndentation());
		setIndentation($accessNode);
		
		purgeNewLines();
		my $moreItems = 1;
		while ($moreItems) {
			
			if (${nextStatement()} =~ /^\s*:(\w+)\s*/mgc) {
				Lib::ParseUtil::splitAndFocusNextStatementOnPos();
				my $attrNode = Node(AttributeKind, createEmptyStringRef());
				SetLine($attrNode, getStatementLine());
				#setRubyKindData($attrNode, 'indentation', Lib::ParseUtil::getIndentation());
				setIndentation($attrNode);
				SetName($attrNode, $1);
				Append($accessNode, $attrNode);
				purgeNewLines();
				if (${nextStatement()} =~ /^\s*,\s*/mgc) {
					Lib::ParseUtil::splitAndFocusNextStatementOnPos();
					purgeNewLines();
				}
				else {
					# item not followed by "," : it was the last item !
					$moreItems = 0;
				}
			}
			else {
				print STDERR "[ParseRuby::parseAcessor] ERROR : unknow syntax for variable at line ".getStatementLine()." : ".${nextStatement()}."\n";
				$moreItems = 0;
			}
		}
		
		return $accessNode;
	}
	return undef;
}

my $NAMESPACE_PATTERN = qr/(?:[\w\.]+::)*/;
my $IDENTIFIER_PATTERN = qr/(?:[\w\.]+)/;

sub parseMember($;$) {
	my $contextContent = shift;
	my $CONTEXT = shift;
	
	my $memberNode = Lib::ParseUtil::tryParse($contextContent);
	
	if (defined $memberNode) {
		if (IsKind($memberNode, "CONTEXT")) {
			my $data = Ruby::RubyNode::getRubyKindData($memberNode, "data");
			if (defined $CONTEXT) {
				# update default visibility
				if (exists $data->{'visibility_zone'}) {
					$CONTEXT->{'visibility_zone'} = $data->{'visibility_zone'};
				}
				
				# retrieve symbol declared behind visibility
				if (exists $data->{'symbols'}) {
					for my $symb (keys %{$data->{'symbols'}}) {
						$CONTEXT->{'symbols'}->{$symb} = $data->{'symbols'}->{$symb};
					}
				}
			}
		}
	}
	#~ elsif (! defined $memberNode) {
		#~ # try to parse attribute
		#~ if (${nextStatement()} =~ /\A\s*\@?\@?(${IDENTIFIER_PATTERN})/gc) {
			#~ Lib::ParseUtil::splitAndFocusNextStatementOnPos();
			#~ $memberNode = Node(AttributeKind, createEmptyStringRef());
			#~ my $item = $1;
			#~ SetLine($memberNode, getStatementLine());
			#~ #setRubyKindData($memberNode, 'indentation', Lib::ParseUtil::getLineIndentation());
			#~ setIndentation($memberNode, $item);
			
			#~ if (${nextStatement()} =~ /\A\s*=/gc) {
				#~ # it's an attribute with init ...
				#~ SetName($memberNode, $item);
				#~ Lib::ParseUtil::splitAndFocusNextStatementOnPos();
				#~ #my $statement = "=".Lib::ParseUtil::getSkippedBlanks();
				#~ my $statement = "";
				#~ my $initNode = Node(InitKind, \$statement);
				#~ #setRubyKindData($initNode, 'indentation', Ruby::RubyNode::getRubyKindData($memberNode, 'indentation'));
				#~ setIndentation($initNode);
				#~ setRubyKindData($initNode, "expression_indentation", Ruby::RubyNode::getRubyKindData($memberNode, 'line_indentation'));
				
				#~ Append($memberNode, $initNode);
				#~ parseExpression($initNode);
			#~ }
			#~ else {
				#~ if (! nextTokenIsEndingInstruction(\$item)) {
					#~ # it's an unknow statement
					#~ SetStatement($memberNode, \$item);
					#~ parseExpression($memberNode);
					#~ SetKind($memberNode, UnknowKind);
				#~ }
				#~ else {
					#~ # it's an attribute without init
					#~ SetName($memberNode, $item);
				#~ }
			#~ }
			#~ purgeEndOfStatement();
		#~ }
	#~ }
	
	if (defined $CONTEXT-> {'visibility_zone'}) {
		if (! defined Ruby::RubyNode::getRubyKindData($memberNode, "visibility")) {
			# node has NOT his own visibility => give it the one from context ...
			Ruby::RubyNode::setRubyKindData($memberNode, "visibility", $CONTEXT-> {'visibility_zone'});
		}
	}
	
	return $memberNode;
}

sub isNextClass() {
	if ( ${nextStatement()} eq 'class' ) {
		return 1;
	}  
	return 0;
}

sub parseClass() {
	if (isNextClass()) {
		my $classNode = Node(ClassKind, createEmptyStringRef());

		#Lib::ParseUtil::setArtifactUpdateState(0);

		my $artiKey = Lib::ParseUtil::newArtifact('class_at_line_', getNextStatementLine(), ClassKind);
		setRubyKindData($classNode, 'artifact_key', $artiKey);

		#trash 'class' keyword
		getNextStatement();
		
		my $statementLine = getStatementLine();
		SetLine($classNode, $statementLine);
		
		#setRubyKindData($classNode, 'indentation', Lib::ParseUtil::getLineIndentation());
		setIndentation($classNode);

		my $context = {"visibility_zone" => "default"};
		
		my $isSingleton = 0;
		my $proto = '';
		my $next;
		while ((defined ($next=nextStatement())) && ($$next ne "\n") && ($$next ne ";")) {
			if ( ${nextStatement()} =~ /(^[^<]*<+)/gc ) {
				$proto .= ${Lib::ParseUtil::getSkippedBlanks()}.$1;
				Lib::ParseUtil::splitAndFocusNextStatementOnPos();
				if (${nextStatement()} eq "\n") {
					$proto .= ${Lib::ParseUtil::getSkippedBlanks()}.${getNextStatement()};
				}
			}
			else {
				$proto .= ${Lib::ParseUtil::getSkippedBlanks()}.${getNextStatement()};
			}
		}		
		# consumes one or more "\n"
		purgeEndOfStatement();
		
		my $name;
		if ($proto =~ /\A\s*<<\s*(.*)/) {
			$name = "__$1_singleton_class_line_${statementLine}__";
			$isSingleton = 1;
		}
		else {
			($name) = $proto =~ /^\s*([\w:]+)/sm;
		}
		
		SetStatement($classNode, \$proto);
		SetName($classNode, $name);
		my $lines_in_proto = () = $proto =~ /\n/g;
		setRubyKindData($classNode, 'lines_in_proto', $lines_in_proto);
		
		#Lib::ParseUtil::setArtifactUpdateState(1);
		
		my $CONTEXT = {};
		my $H_ClassVariable = {};
		my $H_InstanceVariable = {};
		my $H_LocalVariable = {};
		my $H_ConstantVariable = {};
		while ((defined nextStatement()) && (${nextStatement()} ne "end")) {
			my $memberNode = parseMember(\@classContent, $CONTEXT);
			if (defined $memberNode) {
				if (! IsKind($memberNode, "CONTEXT")) {
					Append($classNode, $memberNode);
				}
				purgeEndOfStatement();
			}
			else {
				#print STDERR "[ParseRuby::parseClass] ERROR : unknow member syntax at line ".getNextStatementLine().": ".${nextStatement()}."\n";
				#my $unknowMember = Node(UnknowKind, getNextStatement());
				#SetLine($unknowMember, getStatementLine());
				my $unknowMember = parseUnknow();
				
				# check variables
				my $stmt = GetStatement($unknowMember);
								
				Append($classNode, $unknowMember);
				purgeNewLines();
			}
		}
		
		if (defined nextStatement()) {
			# trash "end" keyword
			getNextStatement();
			setRubyKindData($classNode, "end_indentation", Lib::ParseUtil::getLineIndentation());
			
#print STDERR "END keyword of class ($name) at line ".getStatementLine()."\n";
			
			if (defined nextStatement()) {
				my $node = parseTrailingConditions();
				if (defined $node) {
					print STDERR "WARNING : found condition statement applying to class $name, but forget it ...!!\n";
				}
			}

			purgeEndOfStatement();
		}
		else {
			print STDERR "ERROR : missing end keyword for class at line $statementLine\n";
		}

		
		if ($isSingleton) {
			Lib::ParseUtil::endArtifact($artiKey, $artiKey);
		}
		else {
			Lib::ParseUtil::endArtifact($artiKey);
		}

		return $classNode;
	}
	return undef;
}

##################################################################
#              MODULE
##################################################################
sub isNextModule() {
	if ( ${nextStatement()} =~ /^\s*module\b\s*/mgc ) {
		return 1;
	}  
	return 0;
}

sub parseModule() {
	if (isNextModule()) {
		my $moduleNode = Node(ModuleKind, createEmptyStringRef());

		my $artiKey = Lib::ParseUtil::newArtifact('module_at_line_', getNextStatementLine(), ModuleKind);
		setRubyKindData($moduleNode, 'artifact_key', $artiKey);
		
		# trash "module" keyword ...
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		
		my $statementLine = getStatementLine();
		SetLine($moduleNode, $statementLine);
		
		#setRubyKindData($moduleNode, 'indentation', Lib::ParseUtil::getLineIndentation());
		setIndentation($moduleNode);
		
		my $proto = '';
		while ((defined nextStatement()) && (${nextStatement()} ne "\n") ) {
			$proto .= ${Lib::ParseUtil::getSkippedBlanks()}.${getNextStatement()};
		}
		
		# consumes one or more "\n"
		purgeNewLines();
		
		my ($name) = $proto =~ /^\s*([\w:]+)/sm;
		
		SetStatement($moduleNode, \$proto);
		SetName($moduleNode, $name);
		my $lines_in_proto = () = $proto =~ /\n/g;
		setRubyKindData($moduleNode, 'lines_in_proto', $lines_in_proto);
		
		while ((defined nextStatement()) && (${nextStatement()} ne "end")) {
			#my $memberNode = Lib::ParseUtil::tryParse(\@moduleContent);
			my $memberNode = parseMember(\@moduleContent);
			if (defined $memberNode) {
				Append($moduleNode, $memberNode);
			}
			else {
				#~ #print STDERR "[ParseRuby::parseModule] ERROR : unknow member syntax for module at line ".getNextStatementLine().": ".${nextStatement()}."\n";
				#~ my $unknowMember = Node(UnknowKind, getNextStatement());
				#~ SetLine($unknowMember, getStatementLine());
				#~ #setRubyKindData($unknowMember, 'indentation', Lib::ParseUtil::getIndentation());
				#~ setIndentation($unknowMember);
				my $unknowMember = parseUnknow();
				Append($moduleNode, $unknowMember);
				purgeNewLines();
			}
		}
		
		if (defined nextStatement()) {
			# trash "end" keyword
			getNextStatement();
			setRubyKindData($moduleNode, "end_indentation", Lib::ParseUtil::getLineIndentation());
			
			#print STDERR "END keyword of module at line ".getStatementLine()."\n";
			
			if (defined nextStatement()) {
				my $node = parseTrailingConditions();
				if (defined $node) {
					print STDERR "WARNING : found condition statement applying to module $name, but forget it ...!!\n";
				}
			}
			purgeEndOfStatement();
		}
		else {
			print STDERR "ERROR : missing end keyword for module at line $statementLine\n";
		}

		Lib::ParseUtil::endArtifact($artiKey);

		return $moduleNode;
	}
	return undef;
}

##################################################################
#              FUNCTION
##################################################################

sub parseParameters($) {
	my $parameters = shift;

	my @T_params = ();

	$$parameters =~ s/\A\s*\(//;
	$$parameters =~ s/\)\s*\z//;
	
	for my $param (split /,/, $$parameters) {
		if ($param =~ /\A\s*(\*|\*\*|\&|\.\.\.)?(?:(\w+)(:)?\s*=?\s*(.*))?\s*\z/) {
			my $type = $1||"";
			my $name = $2||"";
			my $mode = (defined $3 ? "named" : "position");
			my $init = $4||"";
			
#print STDERR "PARAMETER : TYPE=$type NAME=$name INIT=$init\n";
			push @T_params, [$name, $init, $type, $mode];
		}
		else {
			print STDERR "Unknow parameter syntax at line ".getStatementLine()." : $param\n";
		}
	}
	
	return \@T_params;
}

sub isNextMethod() {
	if ( ${nextStatement()} eq 'def' ) {
		return 1;
	}  
	return 0;
}



# METHOD SYNTAX:
# def method_name [( [arg [= default]]...[, * arg [, &expr ]])]
#    expr..
# end
sub parseMethod() {
	if (isNextMethod()) {
		my $methodNode = Node(MethodKind, createEmptyStringRef());

		my $artiKey = Lib::ParseUtil::newArtifact('func_at_line_', getNextStatementLine(), MethodKind);
		setRubyKindData($methodNode, 'artifact_key', $artiKey);

		# temporary desactive artifact updating: the prototype of the function should not appears in the encompassing (parent) artifact.
		# This to prevent some argument with default values (xxx = ) to be considered as  parent's variable while greping variable in parent's body.
		Lib::ParseUtil::setLastArtifactUpdateState(0);

		#trash 'def' keyword
		getNextStatement();

		my $statementLine = getStatementLine();
		SetLine($methodNode, $statementLine);
		#setRubyKindData($methodNode, 'indentation', Lib::ParseUtil::getLineIndentation());
		setIndentation($methodNode);

		my $proto = '';
		my $flag_endlessMethod = 0;
		my $stmt;
		Lib::ParseUtil::splitNextStatementOnPattern("=");

		while ((defined ($stmt = nextStatement())) && ($$stmt ne "\n") && ($$stmt ne ";") && ($$stmt ne "end")) {
			
			#if (($$stmt eq "=") && ($proto !~ /(?:\)\s*|\S)$/m)) {
			if ($$stmt eq "=") {
				#if (
				#	# proto has parentheses for parameters
				#	($proto =~ /\)\s*$/m) ||
				#	#~ # proto has no parentheses but previous indentifier before "=" is a parameter:
				#	#~ # 	ex : def toto param=default
				#   #~ ($proto =~ /[^\s]\s+[\w]+\s*$/m) ||
				#    #~ # 	ex : def titi p1,p2=default
				#    #~ ($proto =~ /,\s*[\w]+\s*$/m)){
				#    ($proto =~ /\A\s*(?:[^\s]+\s*|[^\s=<>!])$/m)) {
				
				if (($proto =~ /\)\s+$/m) || ($proto =~ /\A\s*\S+\s+$/m)) {
					# "=" is for endless method syntax ==> stop proto parsing
					last;
				}
				else {
					# "=" is for default init of argument ==> continue proto parsing
					$proto .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
				}
			}
			else {
			
				Lib::ParseUtil::splitNextStatementOnPattern("=");

				$proto .= ${Lib::ParseUtil::getSkippedBlanks()};
				setRubyKindData($methodNode, 'bodyBeginLine', getStatementLine());
				if ($$stmt eq '(') {
					$proto .= ${Lib::ParseUtil::parseRawOpenClose()};
					
					Lib::ParseUtil::splitNextStatementOnPattern("=");
					getNextStatement() while ${nextStatement()} !~ /\S/;
					
					# The parameters have been parsed, so this is the end of the prototype.
					$stmt = nextStatement();
					last
				}
				else {
					$proto .= ${getNextStatement()};
				}
			}
		}

		# consumes the "\n" or ";"
		purgeEndOfStatement();
#print STDERR "PROTO = $proto\n";
		my ($name, $parameters) = $proto =~ /^\A\s*([^\s\(]*)\s*(.*)/sm;
		
		my $params = parseParameters(\$parameters);
		setRubyKindData($methodNode, "parameters", $params);
		my %H_params;
		for my $param (@$params) {
			$H_params{$param->[0]} = $param;
		}
		setRubyKindData($methodNode, "H_parameters", \%H_params);
		
		if ($$stmt eq "=") {
			$proto .= ${getNextStatement()};
			$flag_endlessMethod = 1;
		}
		
		SetStatement($methodNode, \$proto);
		SetName($methodNode, $name);
		my $lines_in_proto = () = $proto =~ /\n/g;
		setRubyKindData($methodNode, 'lines_in_proto', $lines_in_proto);
		
		Lib::ParseUtil::setLastArtifactUpdateState(1);
		
		while ((defined nextStatement()) && (${nextStatement()} ne "end")) {
			my $node = Lib::ParseUtil::tryParse_OrUnknow(\@statementContent);
			Append($methodNode, $node);
			purgeEndOfStatement();
			# quit loop after the first instruction if the method is endless (assigned with "=") ...
			last if $flag_endlessMethod;
		}

		if (! $flag_endlessMethod) {
			if (! defined nextStatement()) {
				print STDERR "ERROR : missing end keyword for method at line $statementLine\n";
			}
			else {
				# trash "end" keyword
				Lib::ParseUtil::setLastArtifactUpdateState(0);
				getNextStatement();
				Lib::ParseUtil::setLastArtifactUpdateState(1);
				setRubyKindData($methodNode, "end_indentation", Lib::ParseUtil::getLineIndentation());
				SetEndline($methodNode, getStatementLine());
#print STDERR "END keyword of method ($name) at line ".getStatementLine()."\n";
			}
		}

		purgeEndOfStatement();
		
		Lib::ParseUtil::endArtifact($artiKey);
		setRubyKindData($methodNode, 'codeBody', Lib::ParseUtil::getArtifact($artiKey));
#		setRubyKindData($methodNode, 'arguments', getArguments($methodNode));
		
		return $methodNode;
	}
	return undef;
}

sub parsePrivateClassMethod() {
	if ((defined nextStatement()) and (${nextStatement()} =~ /\A\s*private_class_method\b/gc)) {
		Lib::ParseUtil::splitAndFocusNextStatementOnPos();
		if (${nextStatement()} eq "def") {
			return parseMethod();
		}
		else {
			# FIXME : should really return an unknown node ? 
			my $statement = "private_class_method ";
			my $node = Node(UnknowKind, \$statement);
			#setRubyKindData($node, 'indentation', Lib::ParseUtil::getIndentation());
			setIndentation($node);
			parseExpression($node);
			
			purgeEndOfStatement();
			return $node
		}
	}
	return undef;
}

##################################################################
#              RESCUE / ENSURE
##################################################################

# methods are just named "begin/end", so sub-statement of begin/end apply to methods too...
# In other words, ensure and rescue are "special statement" that we can parse as we parse them in begin/end (ie by using parseControle())...

sub parseRescue() {
	if (${nextStatement()} eq "rescue") {
		getNextStatement();
		return parseSubControle("rescue", ["rescue", "ensure"]);
	}
	return undef;
}

sub parseEnsure() {
	if (${nextStatement()} eq "ensure") {
		getNextStatement();
		return parseSubControle("ensure", ["rescue", "ensure"]);
	}
	return undef;
}

##################################################################
#              STATEMENT
##################################################################

sub parseSimple_Stmt($) {
	return Lib::ParseUtil::tryParse(\@Simple_StmtContent);
}

sub parseCompound_Stmt($) {
	return Lib::ParseUtil::tryParse(\@Compound_StmtContent);
}

sub parseSeveralInstrOnSameline($) {
	my $parent = shift;
	
	################# WILL NOT WORK BECAUSE THE parseUnknow eat the "\n"
	if ((defined nextStatement()) && (${nextStatement()} ne "\n")) {
		my $node = Lib::ParseUtil::tryParseOrUnknow(\@Simple_StmtContent);
		Append($parent, $node);
	}
}

##################################################################
#              ROOT
##################################################################
sub parseRoot() {
	my $root = Node(RootKind, \$NullString);

	SetName($root, 'root');

	my $artiKey = Lib::ParseUtil::newArtifact('root', 1, RootKind);
	SetLine($root, getNextStatementLine());
	setRubyKindData($root, 'artifact_key', $artiKey);

	while ( defined nextStatement() ) {
		my $subNode = Lib::ParseUtil::tryParse_OrUnknow(\@rootContent);
		if (defined $subNode) {
			Append($root, $subNode);
		}
	}

	Lib::ParseUtil::endArtifact($artiKey);

	return $root;
}

#
# Split a JS buffer into statement separated by structural token
# 

my $SPECIAL_SYMBOLS = qr/:(?:\.\.|\||\^|&|<=>|===|==|=~|>=|<=|\/|\%|\*\*|\*|<<|>>|<|>|~|\+\@|\-\@|\[\]=|\[\]|\+|-)/;

sub replaceSymbol($) {
	my $behind = shift;
	my $symbol = shift;
	if ($behind =~ /\w/) {
		return $behind.$symbol;
	}
	else {
		$behind."__SPECIAL_SYMBOL__";
	}
}

sub splitRuby($) {
   my $r_view = shift;

	$$r_view =~ s/(.)($SPECIAL_SYMBOLS)/&replaceSymbol($1, $2)/sge;

   # /!\ for keywords :
   # - parse a dot before or after if any ... keywords used in namespace are just identifiers ...  		regexp = \.[\.\w]+
   # - parse a @     before if any ... in case keywords are used as identifier @keyword> ...			regexp = \@\w+
   # - parse a colon before if any ... in case keywords are use for symbols ...							regexp = :\w+
   # - parse a colon after  if any ... in case keywords are use for symbols ...							regexp = <keyword>:
   # - parse a question mark after if any because trailing question mark belongs to identifier syntax	regexp = <keyword>?
   my  @statements = split /(\\\n|\n|;|->|\(|\{|\[|\)|\}|\]|:\w+|\@\@?\w+|\.[\.\w]+|\b(?:class|def|lambda|begin|end|if|then|do|else|elsif|while|unless|until|for|do|break|next|when|case|return|yield|rescue|ensure)\b\??:?)/sm, $$r_view;
   
   if (scalar @statements > 0) {
    # if the last statement is a Null statement, it should be removed
    # because it is not significant.
    if ($statements[-1] !~ /\S/ ) {
      pop @statements ;
    }
  }
   return \@statements;
}


sub ParseRuby($) {
  my $r_view = shift;
  
  my $r_statements = splitRuby($r_view);

  # only spaces and tabs will be considered as blanks in a statement (\n are considered as SEPARATORS)
  Lib::ParseUtil::setBlankRegex('[ \t]');

  Lib::ParseUtil::InitParser($r_statements);
  
  # pass all beginning empty lines
  while ((defined nextStatement()) && (${nextStatement()} eq "\n")) {
	getNextStatement();
  }

  initContext();

  # Mode ALL for artifact consolidation.
  Lib::ParseUtil::setArtifactMode(0);
  
  my $root = parseRoot();
  my $Artifacts = Lib::ParseUtil::getArtifacts();

  return ($root, $Artifacts);
}

###################################################################################
#              MAIN
###################################################################################

sub preComputeListOfKinds($$$) {
	my $node = shift;
	my $views = shift;
	my $kinds = shift;
  
	my %H_KindsLists = ();
  
	for my $kind (@$kinds) {
		my @NodesList = GetNodesByKind($node, $kind );
		$H_KindsLists{$kind}=\@NodesList;
	}

	$views->{'KindsLists'} = \%H_KindsLists;
}

sub getExpancedCode($) {
	my $string = shift;
	my $output = "";
	while ($string =~ /(\#\{[^\}]*\})/g) {
		$output .= $1;
	}
	return $output;
}

sub postParsingOrganization($$) {
	my $root = shift;
	my $views = shift;
	
	my @selfClasses = GetNodesByKind($root, ClassKind);
	for my $class (@selfClasses) {
		if (GetName($class) =~ /\A__self_singleton_class/) {
			my $parent = GetParent($class);
			my $brotherwood = GetChildren($parent);
			
			my $idx = 0;
			while ($brotherwood->[$idx] != $class) {
				$idx++;
			}

			# replace self singleton by its childrens.
			my $children = GetChildren($class);
			for my $child (@$children) {
				Lib::Node::SetParent($child, $parent);
				setRubyKindData($child, "self_singleton_method", 1);
			}
			
			# replace self singleton with its children ...
			splice @$brotherwood, $idx, 1, @$children;
			
			# In the parent class, expand the "link" with the content of the singleton class (that belongs to the parent, logically speaking ) ...
			# by this way, greping into parent class, also greps into inner class.
			my $singleton_key = Ruby::RubyNode::getRubyKindData($class, "artifact_key");
			my $singleton_artifact = Lib::ParseUtil::getArtifact($singleton_key);
			
			my $parent_key = Ruby::RubyNode::getRubyKindData($parent, "artifact_key");
			my $parent_artifact = Lib::ParseUtil::getArtifact($parent_key);
			
			$$parent_artifact =~ s/$singleton_key/$$singleton_artifact/;
		}
	}
	
	# for each method, create a view with strings expansed ...
	my $HString = $views->{'HString'};
	for my $method( GetNodesByKind($root, MethodKind)) {
		my $codeWithStrings = ${Ruby::RubyNode::getRubyKindData($method, "codeBody")};
		$codeWithStrings =~ s/(\bCHAINE_\d+\b)/getExpancedCode($HString->{$1})/ge;
		setRubyKindData($method, "codeWithStrings", \$codeWithStrings);
	}
	
	# Do the same for root ...
	my $codeWithStrings = Lib::ParseUtil::getArtifact("root_1");
	$codeWithStrings =~ s/(\bCHAINE_\d+\b)/getExpancedCode($HString->{$1})/ge;
	setRubyKindData($root, "codeWithStrings", \$codeWithStrings);
}

# description: Ruby parse module Entry point.
sub Parse($$$$)
{
	my ($fichier, $vue, $couples, $options) = @_;
	my $status = 0;

	initMagicNumbers($vue);

	$StringsView = $vue->{'HString'};

	Lib::ParseUtil::registerParseUnknow(\&parseUnknow);

	# Methods are present inside class artifact beacause it is needed to
	# - grep class variable => sometimes it appears only inside a method body ...
	# - grep private methods usage ...
	
	Lib::ParseUtil::setArtifact_v2();
	
	Lib::ParseUtil::addUpdateNestingRule(MethodKind, undef, 0);
	Lib::ParseUtil::addUpdateNestingRule(ClassKind, undef, 0);
	Lib::ParseUtil::addUpdateNestingRule(ModuleKind, undef, 0);

    my $statements =  $vue->{'statements_with_blanks'} ;

	# launch first parsing pass : strutural parse.
	my ($RubyNode, $Artifacts) = ParseRuby(\$vue->{'code'});

	postParsingOrganization($RubyNode, $vue);

#      print "Buffered routines = ".join(', ', keys %H_ROUTINE_BUFFER)."\n";
#      print $H_ROUTINE_BUFFER{'IsValidUser'}."\n------\n";

	if (defined $options->{'--print-tree'}) {
		Lib::Node::Dump($RubyNode, *STDERR, "ARCHI") if ($DEBUG);
	}
	if (defined $options->{'--print-test-tree'}) {
		print STDERR ${Lib::Node::dumpTree($RubyNode, "ARCHI")} ;
	}

	$vue->{'structured_code'} = $RubyNode;
	$vue->{'artifact'} = $Artifacts;

	# pre-compute some list of kinds :
	preComputeListOfKinds($RubyNode, $vue, [MethodKind, LambdaKind, ModuleKind, ClassKind, ConditionKind, WhileKind, ForKind, IfKind, BeginKind, RescueKind, CaseKind, UnlessKind]);

	#TSql::ParseDetailed::ParseDetailed($vue);
	if (defined $options->{'--print-artifact'}) {
		for my $key ( keys %{$vue->{'artifact'}} ) {
			print "-------- $key -----------------------------------\n";
			print  $vue->{'artifact'}->{$key}."\n";
		}
	}

	return $status;
}

1;


