
package Ruby::RubyNode ;

use strict;
use warnings;

use Lib::NodeUtil;

use Erreurs;


use Exporter 'import'; # gives you Exporter’s import() method directly

my @kinds =  qw( RootKind ModuleKind IndentKind
		ClassKind MethodKind WhileKind ForKind LoopKind UntilKind CaseKind WhenKind IfKind UnlessKind ThenKind ElsifKind ElseKind ConditionKind
		TrailingIfKind TrailingUnlessKind TrailingWhileKind TrailingUntilKind BlocKind LambdaKind
		AttributeKind AttrReaderKind AttrWriterKind AttrAccessorKind
		BreakKind ContinueKind ReturnKind RaiseKind YieldKind PassKind BEGINKind ENDKind BeginKind RescueKind EnsureKind InitKind
		IncludeKind ExtendKind PrependKind AliasKind
		setRubyKindData getPythonKindData
	 );

# Symboles pour lesquels l'utilisateur n'aura pas a specifier l'espace de nom ObjCNode:
our @EXPORT_OK = (@kinds);  # symbols to export on request

# Importe automatiquement sans intervention du package utilisateur.
our @EXPORT = (@kinds);

sub setRubyKindData($$$) {
	my $node = shift;
	my $kindName = shift;
	my $kindData = shift;
	
	$node->[7]->{$kindName} = $kindData;
}

sub getRubyKindData($$) {
	my $node =shift;
	my $kindName = shift;

	return $node->[7]->{$kindName};
}

use constant RootKind		=> 'root';
use constant ModuleKind		=> 'module';
use constant ClassKind		=> 'class';
use constant MethodKind		=> 'meth';
use constant AttributeKind	=> 'attr';
use constant AttrReaderKind	=> 'reader';
use constant AttrWriterKind	=> 'writer';
use constant AttrAccessorKind	=> 'access';
use constant CaseKind		=> 'case';
use constant WhenKind		=> 'when';
use constant IfKind			=> 'if';
use constant UnlessKind		=> 'unless';
use constant WhileKind		=> 'while';
use constant ForKind		=> 'for';
use constant LoopKind		=> 'loop';
use constant UntilKind		=> 'until';
use constant TrailingWhileKind	=> 't_while';
use constant TrailingUntilKind	=> 't_until';
use constant ThenKind		=> 'then';
use constant ElsifKind		=> 'elsif';
use constant ElseKind		=> 'else';
use constant TrailingIfKind	=> 't_if';
use constant TrailingUnlessKind	=> 't_unless';
use constant BlocKind		=> 'bloc';
use constant LambdaKind		=> 'lambda';
use constant IndentKind		=> 'indent';
use constant ConditionKind	=> 'cond';
use constant BreakKind		=> 'break';
use constant ContinueKind	=> 'cont';
use constant ReturnKind		=> 'ret';
use constant RaiseKind		=> 'raise';
use constant YieldKind		=> 'yield';
use constant PassKind		=> 'pass';
use constant BEGINKind		=> 'BEGIN';
use constant ENDKind		=> 'END';
use constant BeginKind		=> 'begin';
use constant RescueKind		=> 'rescue';
use constant EnsureKind		=> 'ensure';
use constant InitKind		=> 'init';
use constant IncludeKind	=> 'include';
use constant ExtendKind		=> 'extend';
use constant PrependKind	=> 'prepend';
use constant AliasKind		=> 'Alias';


# return a string that can identify the related node.
sub nodeLink($) {
  my $node = shift;

  if (IsKind($node, IfKind)) {
      return "__if_expression__";
  }
  elsif (IsKind($node, UnlessKind)) {
      return "__unless_expression__";
  }
  elsif (IsKind($node, TrailingIfKind)) {
      return "__T_if__";
  }
  elsif (IsKind($node, LambdaKind)) {
      return "__lambda__";
  }
  elsif (IsKind($node, BeginKind)) {
      return "__begin_end__";
  }
  elsif (IsKind($node, BlocKind)) {
      return "__bloc__";
  }
  elsif (IsKind($node, CaseKind)) {
      return "__case__";
  }

  return "";
}

1;


