package Scala::CountLoop;
# les modules importes
use strict;
use warnings;

use Carp::Assert; # traces_filter_line
use Erreurs;
use Couples;

use Lib::NodeUtil;
use Scala::ScalaNode;
use GreenIT::lib::KeepPortalData;
use GreenIT::lib::HardCodedPathNotGreen;
use CommonOutput::ElaborateLog;

my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $DEBUG = 0;

my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $InstantiationInLoop__mnemo = Ident::Alias_InstantiationInLoop();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $LoopCondComparedToNonZero__mnemo = Ident::Alias_LoopCondComparedToNonZero();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();

my $nb_ConcatInLoop = 0;
my $nb_InstantiationInLoop = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_LoopCondComparedToNonZero = 0;
my $nb_ToManyNestedLoop = 0;

sub checkNestedLoop($) {
	my $loop = shift;

	# find nested loops ...
	my @subloop = GetNodesByKindList($loop, [WhileKind, ForKind, DoWhileKind], 1);
	for my $sloop (@subloop) {
		$nb_ToManyNestedLoop++;
		my $numline = $sloop->[5];
		CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
		Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "sub loop at line ".GetLine($sloop));
	}
}

sub CountLoop($$$) {
	my ($file, $vue, $compteurs) = @_;

	my $ret = 0;
	$nb_ConcatInLoop = 0;
	$nb_InstantiationInLoop = 0;
	$nb_FunctionCallInLoopTest = 0;
	$nb_LoopCondComparedToNonZero = 0;
	$nb_ToManyNestedLoop = 0;

	my $root = $vue->{'structured_code'};

	if (!defined $root) {
		$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $LoopCondComparedToNonZero__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		
		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my @loops = GetNodesByKindList($root, [ WhileKind, ForKind, DoWhileKind ]);

	for my $loop (@loops) {
		my $cond;

		$cond = GetChildren($loop)->[0];

		my $condExpr = ${GetStatement($cond)};
		# replace "to" with "->", because should not be confused with a user identifier, captured with "\w+"
		$condExpr =~ s/\bto\b/->/g;

		if ($condExpr =~ /\w+(?:\s+\w+|\s*\()/) {
			$nb_FunctionCallInLoopTest++;
			my $numline = $cond->[5];
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
			Erreurs::VIOLATION($FunctionCallInLoopTest__mnemo, "Function call in loop test at line " . GetLine($loop));
		}
		
		# check all expressions using operators <, >, <=, >=, ==, !=
		if (! IsKind($loop, ForKind)) {
			my $bool_comparisonToZero;
			if ($condExpr !~ /\&\&|\|\|/) {
				if ($condExpr =~ /([^<>!=\s]+)\s*(?:[<>!=]=|<|>)\s*([^<>!=\s]+)/g) {
					$bool_comparisonToZero = 0;
					if (($1 =~ /(?:^|[^\.\w])0$/m) || ($2 =~ /^0(?:$|[^\.\w])/m)) {
						$bool_comparisonToZero = 1;
					}
				}
				if ((defined $bool_comparisonToZero) && ($bool_comparisonToZero == 0)) {
					$nb_LoopCondComparedToNonZero++;
					my $numline = GetLine($cond);
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_LoopCondComparedToNonZero, $rulesDescription, "LoopCondComparedToNonZero");
					Erreurs::VIOLATION($LoopCondComparedToNonZero__mnemo, "Loop condition is not comparison to zero at line " . GetLine($cond));
				}
			}
		}
		checkNestedLoop($loop);
	}

	$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, $nb_FunctionCallInLoopTest);
	$ret |= Couples::dispatch($LoopCondComparedToNonZero__mnemo, $nb_LoopCondComparedToNonZero);
	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);
}

sub CountPatternInLoop($$$)
{
    my ($file, $vue, $compteurs) = @_;

    my $ret = 0;
    $nb_ConcatInLoop = 0;
    $nb_InstantiationInLoop = 0;
	$nb_OpenCloseFileInLoop = 0;

    my $root = $vue->{'structured_code'};

    if (! defined $root) {
        $ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
    }
    
    my @loops = GetNodesByKindList($root, [ WhileKind, ForKind, DoWhileKind ], 1);
	# HL-2022 Avoid OPEN/CLOSE inside loops
	# codeBody is in lowercase
	my $regexOpen = qr/\.fromFile\b/i;
	my $regexClose = qr/\.close\b/i;
	for my $loop (@loops) {
		my $codeBody = Lib::NodeUtil::GetKindData($loop)->{'codeBody'};
		my $bodyBeginLine = Lib::NodeUtil::GetKindData($loop)->{'bodyBeginLine'};

		# check presence of literal strings concaténations (variables strings cannot be detected !)
		my $lineOffset = 0;
		while ($$codeBody =~ /(\n+)|(\bCHAINE_\d+)|(\+=)|(\+)|($regexOpen)|($regexClose)|(\bnew\b)/gc) {
			if (defined $1) {
				$lineOffset += () = $1 =~ /\n/g;
			}
			elsif (defined $7) {
				$nb_InstantiationInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
				Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation in loop at line " . ($bodyBeginLine + $lineOffset));
			}
			elsif (defined $2) {
				# CHAINE_XX
				if ($$codeBody =~ /\G(\s*)\+/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
				}
			}
			elsif (defined $5 || defined $6) {
				$lineOffset += () = $5 =~ /\n/g if (defined $5);
				$lineOffset += () = $6 =~ /\n/g if (defined $6);
				$nb_OpenCloseFileInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				# print "Open/close file inside loop at line $numline\n";
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
				Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
			}
			else {
				# + first or +=
				if ($$codeBody =~ /\G(\s*)\bCHAINE_\d+(\s*)(\+)?/gc) {
					$lineOffset += () = $1 =~ /\n/g;
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));

					$lineOffset += () = $2 =~ /\n/g;
					if (defined $3) {
						$nb_ConcatInLoop++;
						my $numline = $bodyBeginLine + $lineOffset;
						CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
						Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
					}
				}
			}
		}
		
		# Count instantiations without "new" keyword
		# : <obj> = <obj> (
		# : <obj> = <obj> {
		# : <obj> = <obj>.apply(
		pos($$codeBody) = 0;
		$lineOffset = 0;
		while ($$codeBody =~ /:\s*(\w+)\s*(=\s*\1)\s*(?:\(|\{|\.apply\()|(\n)/g) {
			if (defined $3) {
				$lineOffset++;
			}
			else {
				$lineOffset += () = $1 =~ /\n/g;
				$nb_InstantiationInLoop++;
				Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation of $1 in loop at line " . ($bodyBeginLine + $lineOffset));
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
			}
		}
	}

    $ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, $nb_ConcatInLoop);
    $ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, $nb_InstantiationInLoop);
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);

    return $ret;
}

1;
