package StripAspDotNet ;

use strict;
use warnings;
use Carp::Assert;                                                               # traces_filter_line
use Timing;                                                                     # timing_filter_line
use StripUtils;

use StripUtils qw(
			garde_newlines
			warningTrace
			configureLocalTraces
			StringStore
			dumpVueStrings
);
use Prepro;
use AnaUtils;
use Vues;

use Lib::ParseUtil;
use AspDotNet::AspDotNetScriptNode;
use StripCS;
use StripVbDotNet;

# AspDotNet openning and closing tags.

my $STD_AspDotNet_DIRECTIVE = '<%\s*\@';
my $STD_AspDotNet_TAG = '<%[=:]?';
my $XML_SCRIPT = '<script\b';
my $STD_AspDotNet_EXPR = '<%=';
my $XML_AspDotNet_TAG = '<asp:Xml\b';
my $AspDotNet_TAG_LIB = '<\w+:\w+';
my $HTML_COMMENT = '<!--';
my $AspDotNet_COMMENT = '<%--';
my $NET_COMMENT = '\/\/|\/\*|\'';
my $CDATA = '<!\[CDATA\[';
my $closing_STD_AspDotNet_TAG = '%>';
my $closing_XML_SCRIPT = '<\/script>';
my $closing_AspDotNet_TAG_LIB = '<\/\w+:\w+\s*>';
my $closing_XML_AspDotNet_TAG = '<\/asp:Xml\s*>';
my $closing_HTML_COMMENT = '-->';
my $closing_AspDotNet_COMMENT = '--%>';
my $closing_NET_COMMENT = '\n|\*\/';
my $closing_CDATA = '\]\]>';
my $ASPNET_TECHNO = "";

my $string = '\\\\?(?:"|\')';
my $esc_string = '\\\\"|\\\\\'';

# esc_escape aims to match \\ in a string. 
# So it should be \\\\ in the regular expression !!
# so it should be \\\\\\\\ in the string ($esc_escape) expanded in the regular expression !!!!
my $esc_escape = '\\\\\\\\';

my $openServerTags = "$AspDotNet_COMMENT|$STD_AspDotNet_DIRECTIVE|$STD_AspDotNet_TAG|$XML_SCRIPT|$XML_AspDotNet_TAG|$AspDotNet_TAG_LIB";
my $closeServerTags = "$closing_AspDotNet_COMMENT|$closing_STD_AspDotNet_TAG|$closing_XML_SCRIPT|$closing_AspDotNet_TAG_LIB";#'%>|\/>|<\/\w+:\w+>';

sub appendAsNetComment($$;$) {
	my ($context, $vues, $updateMixBloc) = @_;

	my $element = $context->{'element'};
	my $blanked_element = $context->{'blanked_element'};

	$vues->append('comment_a', $element);

	# Sometime, comment items written in MixBloc view are not the same than in comment view.
	# In this case do not update MixBloc, it is the responsibiity of the calling routine.
	if ((!defined $updateMixBloc) || ($updateMixBloc == 1)) {
		$vues->append('MixBloc_a', $element);
	}

	# 'script' view contains the Net comments just as Net code .
	$vues->append('script_a', $element);
	$vues->append('script_html_a', $element);

	# other views related to AspDotNet script do not contain Net comments.
	#Lib::ParseUtil::appendAsBlanked($context, $vues);
	$vues->append('tag_comment_a', $blanked_element);
	$vues->append('html_a', $blanked_element);

	$context->{'blanked_comment'} .= $context->{'blanked_element'};
}

sub appendItemToNetComment($$) {
	my ($item, $vues) = @_;

	$vues->append('comment_a', $item);
}

sub appendItemToMixBloc($$) {
	my ($item, $vues) = @_;

	$vues->append('MixBloc_a', $item);
}

# additional callback to be called each time a element will be considered as Script item.
sub cb_appendAsScript($$) {
	my ($context, $vues) = @_;

	$vues->append('MixBloc_a', $context->{'element'});
}

# additional callback to be called each time a element will be considered as html item.
sub cb_appendAsHtml($$) {
	my ($context, $vues) = @_;

	$vues->append('MixBloc_a', $context->{'blanked_element'});
}

#---------------------- HTML_COMMENT ---------------------------------------
sub trigger_HTML_COMMENT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	Lib::ParseUtil::enterState($context, 'HTML_COMMENT');
	Lib::ParseUtil::appendAsHTML($context, $vues);
}

sub cb_HTML_COMMENT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	if (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	else {
		Lib::ParseUtil::appendAsHTML($context, $vues);
	}
}

sub closing_HTML_COMMENT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	Lib::ParseUtil::appendAsHTML($context, $vues);
	Lib::ParseUtil::leaveCurrentState($context);
}

#---------------------- STRING ---------------------------------------
sub trigger_AspDotNet_STRING($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	$context->{'string_buffer'} = $element;

	# Option 0 will prevent to add a AspDotNet_STRING node to the tree ...
	Lib::ParseUtil::enterState($context, 'AspDotNet_STRING');

	# the triggering pattern " or ' will be the expected closing pattern !
	Lib::ParseUtil::setStateClosingPattern($context, $element);

}

sub cb_STRING($$$$) {
	my ($context, $vues, $STRING_VIEW_NAME, $TAG_NAME) = @_;

	my $element = $context->{'element'};

	if (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	elsif (Lib::ParseUtil::isStateClosingPattern($context, $element)) {
		# this is the end of the string
		my $string_buffer = $context->{'string_buffer'};
		$string_buffer .= $element;

		# store in the string view and get the key
		my $string_id = StringStore($context->{$STRING_VIEW_NAME}, $string_buffer, $TAG_NAME);

		# keep the number of line for a multiline string...
		my $nb = () = $string_buffer =~ /\n/g;
		my $newlines = "\n" x $nb;

		Lib::ParseUtil::appendAsScriptEndString($context, $vues, $string_id . $newlines);
		# Back to the state containing the string.
		Lib::ParseUtil::leaveCurrentState($context);

		# append to the statement of the node containing te string.
		my $str_replace = ' ' . $string_id . $newlines . ' ';
		Lib::ParseUtil::appendNodeStatement($context, \$str_replace);
	}
	else {
		# it's any element of the string. Record it ...
		$context->{'string_buffer'} .= $element;

		my $node = $context->{'state'}->[$Lib::ParseUtil::STATE_NODE];
		detectTechnoAspNet($node, $element);

		Lib::ParseUtil::appendAsScriptString($context, $vues);
	}
}

sub cb_AspDotNet_STRING($$) {
	my ($context, $vues) = @_;
	# unfortunately, the name of the string vue is hardcoded in ParseUtil.pm ... cannot change ...
	cb_STRING($context, $vues, "JSP_string_context", "AspDotNet_CHAINE_");
}

sub cb_NET_STRING($$) {
	my ($context, $vues) = @_;
	# tag for Net strings is "CHAINE_" in compliance with Net analyzer.
	cb_STRING($context, $vues, "NET_string_context", "CHAINE_");
}

sub trigger_NET_STRING($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	$context->{'string_buffer'} = $element;

	# Option 0 will prevent to add a NET_STRING node to the tree ...
	Lib::ParseUtil::enterState($context, 'NET_STRING');

	# the triggering pattern " or ' will be the expected closing pattern !
	Lib::ParseUtil::setStateClosingPattern($context, $element);
}

#---------------------- STD directives ---------------------------------------
sub trigger_STD_AspDotNet_DIRECTIVE($$) {
	my ($context, $vues) = @_;
	
	# ROBUSTNESS: if there is blank betwwen <% and @ int the directive tagn remove it ... 
	$context->{'element'} =~ s/\A<%\s*\@/<%@/;
	
	Lib::ParseUtil::enterState($context, 'STD_AspDotNet_DIRECTIVE');

	Lib::ParseUtil::appendAsScript($context, $vues);
}

#---------------------- STD tags ---------------------------------------
sub trigger_STD_AspDotNet_TAG($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	Lib::ParseUtil::enterState($context, 'STD_AspDotNet_TAG');

	Lib::ParseUtil::appendAsScript($context, $vues);
}

sub cb_STD_AspDotNet_TAG($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	# an end of tag ?
	if ($element eq '%>') {
		Lib::ParseUtil::appendAsScript($context, $vues);
		Lib::ParseUtil::leaveCurrentState($context);
	}
	# a string ... ?
	elsif (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	# a scriptlet content !
	else {
		Lib::ParseUtil::appendAsScript($context, $vues);
		#$context->{'appendScriptlet'}->($context, $vues);
	}
}
#---------------------- tags lib ---------------------------------------
sub trigger_AspDotNet_TAG_LIB($$) {
	my ($context, $vues) = @_;

	Lib::ParseUtil::enterState($context, 'AspDotNet_TAG_LIB');

	Lib::ParseUtil::appendAsScript($context, $vues);

	my $tagName = $context->{'element'};
	$tagName =~ s/<//s;
	Lib::ParseUtil::setStateClosingPattern($context, "</$tagName>");
}

sub cb_AspDotNet_TAG_LIB($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	if ($element eq '/>') {
		Lib::ParseUtil::appendAsScript($context, $vues);
		# The tag is closed, back to HTML.
		Lib::ParseUtil::leaveCurrentState($context);
	}
	elsif ($element eq '>') {
		Lib::ParseUtil::appendAsScript($context, $vues);
		# Back to the HTML state, but the tag is not closed. 
		Lib::ParseUtil::enterState($context, 'HTML');
	}
	elsif (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	else {
		# its anything else inside the tag
		Lib::ParseUtil::appendAsScript($context, $vues);
	}
}

sub leaving_AspDotNet_TAG_LIB($$) {
	my ($context, $vues) = @_;

	if (Lib::ParseUtil::restoreOpenningTagState($context, $context->{'element'})) {
		# if a matching openning tag has been found then consider the closing pattern as AspDotNet code
		Lib::ParseUtil::appendAsScript($context, $vues);
		# leave the state.
		Lib::ParseUtil::leaveCurrentState($context);
	}
	else {
		# before openning and closing tag lib, we are in HTML zone.
		# if the encountered "closing tag lib" has no corresponding opening tag lib,
		# then it is ignore:
		# - consider the closing tag lib as an HTML element.
		# - stay in HTML state.
		Lib::ParseUtil::appendAsHTML($context, $vues);
	}
}
#---------------------- XML tags ---------------------------------------
sub trigger_XML_AspDotNet_TAG($$) {
	my ($context, $vues) = @_;

	Lib::ParseUtil::enterState($context, 'XML_AspDotNet_TAG');

	Lib::ParseUtil::appendAsScript($context, $vues);

	my $tagName = $context->{'element'};
	$tagName =~ s/<//s;
	Lib::ParseUtil::setStateClosingPattern($context, "</$tagName>");
}

sub cb_XML_AspDotNet_TAG($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	if ($element eq '/>') {
		Lib::ParseUtil::appendAsScript($context, $vues);
		# The tag is closed, back to HTML.
		Lib::ParseUtil::leaveCurrentState($context);
	}
	elsif ($element eq '>') {
		Lib::ParseUtil::appendAsScript($context, $vues);

		if (Lib::NodeUtil::GetName($context->{'state'}->[$Lib::ParseUtil::STATE_NODE]) =~ /asp:(?:scriptlet|declaration|expression)\b/) {
			Lib::ParseUtil::enterState($context, 'XML_AspDotNet_TEXT');
		}
		else {
			Lib::ParseUtil::enterState($context, 'HTML');
		}

	}
	elsif (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	else {
		# its anything else inside the tag
		Lib::ParseUtil::appendAsScript($context, $vues);
	}
}

#---------------------- XML tag script ---------------------------------------

sub trigger_XML_SCRIPT($$) {
	my ($context, $vues) = @_;

	Lib::ParseUtil::enterState($context, 'XML_SCRIPT');

	Lib::ParseUtil::appendAsScript($context, $vues);

	Lib::ParseUtil::setStateClosingPattern($context, "</script>");
}

sub cb_XML_SCRIPT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	# an end of tag ?
	if ($element eq '</script>') {
		Lib::ParseUtil::appendAsScript($context, $vues);
		
		detectTechno_in_XML_SCRIPT($context->{'state'}->[$Lib::ParseUtil::STATE_NODE]);
		
		Lib::ParseUtil::leaveCurrentState($context);
	}
	elsif (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	# a scriptlet content !
	else {
		Lib::ParseUtil::appendAsScript($context, $vues);
	}
}

#---------------------- XML tag text ---------------------------------------
sub cb_XML_AspDotNet_TEXT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	# FIXME : may not work in every case, its not the entire tag !!
	if ($element =~ /<\//) {
		# an end tag is encountered. Try to retrieves the corresponding
		# openning tag and leaves the corresponding state.
		Lib::ParseUtil::restoreOpenningTagState($context, $element);
		Lib::ParseUtil::appendAsScript($context, $vues);
		Lib::ParseUtil::leaveCurrentState($context);
	}
	elsif (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	else {
		Lib::ParseUtil::appendAsScript($context, $vues);
	}
}

#---------------------- CDATA ---------------------------------------

sub trigger_CDATA($$) {
	my ($context, $vues) = @_;

	Lib::ParseUtil::enterState($context, 'CDATA');

	Lib::ParseUtil::appendAsScript($context, $vues);
}

sub cb_CDATA($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	if ($element =~ /\]\]>/) {
		Lib::ParseUtil::appendAsScript($context, $vues);
		Lib::ParseUtil::leaveCurrentState($context);
	}
	elsif (Lib::ParseUtil::checkRegexpTriggeringToken($context, $vues)) {
	}
	else {
		Lib::ParseUtil::appendAsScript($context, $vues);
	}
}

#---------------------- AspDotNet COMMENT ---------------------------------------

my $NB_OPENNED_AspDotNet_TAGS = 0;

sub trigger_AspDotNet_COMMENT($$) {
	my ($context, $vues) = @_;

	$NB_OPENNED_AspDotNet_TAGS = 0;

	Lib::ParseUtil::enterState($context, 'AspDotNet_COMMENT');

	Lib::ParseUtil::appendAsComment($context, $vues);
}

sub cb_AspDotNet_COMMENT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	if ($element eq $closing_AspDotNet_COMMENT) {
		Lib::ParseUtil::appendAsComment($context, $vues);
		appendItemToMixBloc($context->{'blanked_element'}, $vues);
		Lib::ParseUtil::leaveCurrentState($context);
	}
	else {
		# check presence of AspDotNet TAG inside a AspDotNet comment
		if ($element =~ /$STD_AspDotNet_TAG|$STD_AspDotNet_DIRECTIVE|$STD_AspDotNet_EXPR/) {
			$NB_OPENNED_AspDotNet_TAGS++;
		}

		# end of embedded AspDotNet TAG ? 
		if ($element eq $closing_STD_AspDotNet_TAG) {
			if ($NB_OPENNED_AspDotNet_TAGS > 0) {
				# YES ... it's ok
				$NB_OPENNED_AspDotNet_TAGS--;
			}
			else {
				# NO ... arf ! it's a confusing syntax !!!!
				print "[WARNING] %> assumed to be the end of a AspDotNet comment at " . $context->{'line'} . "\n";
				# Memorize 1 additional violation.
				$context->{'AmbigousClosing'}++;
				Lib::ParseUtil::appendAsComment($context, $vues);
				Lib::ParseUtil::leaveCurrentState($context);
			}
		}
		Lib::ParseUtil::appendAsCommentContent($context, $vues);
		appendItemToMixBloc($context->{'blanked_element'}, $vues);
	}
}

#---------------------- Net COMMENT ---------------------------------------
sub trigger_NET_COMMENT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	$context->{'blanked_comment'} = '';

	# Option 0 will prevent to add a NET_STRING node to the tree ...
	Lib::ParseUtil::enterState($context, 'NET_COMMENT', 0);

	if ($element eq '//') {
		Lib::ParseUtil::setStateClosingPattern($context, "\n");
		appendAsNetComment($context, $vues, 0); # 0 means do not update MixBloc
		appendItemToMixBloc("/*", $vues);

	}
	else {
		Lib::ParseUtil::setStateClosingPattern($context, '*/');
		appendAsNetComment($context, $vues);
	}
}

sub leave_NET_COMMENT($) {
	my $context = shift;
	# Back to the state containing the Net comment.
	Lib::ParseUtil::leaveCurrentState($context);

	# append the blanked comment in the code statement in ordre to conserve the lines return.
	# (code items that are not in the same line should not be concatenated on the same line !)
	Lib::ParseUtil::appendNodeStatement($context, \$context->{'blanked_comment'});
}

sub cb_NET_COMMENT($$) {
	my ($context, $vues) = @_;

	my $element = $context->{'element'};

	if (Lib::ParseUtil::isStateClosingPattern($context, $element)) {
		# append as blank element for all "script" views ...
		if ($element ne "\n") {
			appendAsNetComment($context, $vues);
		}
		else {
			appendAsNetComment($context, $vues, 0); # 0 means do not update MixBloc
			appendItemToMixBloc("*/\n", $vues);
		}

		leave_NET_COMMENT($context);
	}

	# if we are in a // comment (ie. closing pattern is '\n') and a end scriptlet is encountered, then leave the Net comment state
	elsif (($element eq '%>') && (Lib::ParseUtil::getStateClosingPattern($context) eq "\n")) {
		my $previous = Lib::ParseUtil::getPreviousState(1);

		# check if the %> is really closing a scriptlet ...
		if ($previous->[$Lib::ParseUtil::STATE_ID] eq 'STD_AspDotNet_TAG') {

			# close the MixBloc comment with */
			appendItemToMixBloc("*/", $vues);

			# Cannot close the comment int he view 'comment', because for that we should go to line, whereas this is not the case in real code.
			#appendItemToNetComment("\n", $vues);


			# Back to the STD_AspDotNet_TAG state containing the Net comment.
			leave_NET_COMMENT($context);

			# treat the %> with the appropriate callback.
			cb_STD_AspDotNet_TAG($context, $vues);
		}
		else {
			# OK, the %> was not closing a scriptlet !
			appendAsNetComment($context, $vues);
		}
	}
	# if an end of line is encountered that is not the end of the comment, 
	# then replace the "\n" with "*/\n/*" in the MixBloc view
	elsif ($element eq "\n") {
		appendAsNetComment($context, $vues, 0); # 0 means do not update MixBloc
		appendItemToMixBloc("*/\n/*", $vues);
	}
	else {
		appendAsNetComment($context, $vues);
	}
}



# NOTE : if a state is not associated with anay triggering token, this
#		 means the callback of the state implements all treatments.

# STRUCTURE for each state: 
#         <STATE> => [ 	<callback>,
#						<token extractor regexpr>, 
#						<triggers>
#					 ]

my %STATES = (
	'HTML'	=> 
		[	undef,			# use default treatment for HTML
			qr/\G($AspDotNet_COMMENT|$STD_AspDotNet_DIRECTIVE|$STD_AspDotNet_TAG|$XML_SCRIPT|$XML_AspDotNet_TAG|$AspDotNet_TAG_LIB|$closing_AspDotNet_TAG_LIB|$HTML_COMMENT|<|[^<]*)/sm,
			#triggers
			[	[$AspDotNet_COMMENT, 			\&trigger_AspDotNet_COMMENT],
				[$STD_AspDotNet_DIRECTIVE, 		\&trigger_STD_AspDotNet_DIRECTIVE],
				[$STD_AspDotNet_TAG, 			\&trigger_STD_AspDotNet_TAG],
				[$XML_SCRIPT, 					\&trigger_XML_SCRIPT],
				[$XML_AspDotNet_TAG, 			\&trigger_XML_AspDotNet_TAG],
				[$AspDotNet_TAG_LIB, 			\&trigger_AspDotNet_TAG_LIB],
				[$closing_AspDotNet_TAG_LIB,	\&leaving_AspDotNet_TAG_LIB],
				[$HTML_COMMENT, 		\&trigger_HTML_COMMENT],
			],
		],
	# inside an html comment
	'HTML_COMMENT'	=>
		[	\&cb_HTML_COMMENT,
			qr/\G($AspDotNet_COMMENT|$STD_AspDotNet_DIRECTIVE|$STD_AspDotNet_TAG|$XML_AspDotNet_TAG|$AspDotNet_TAG_LIB|$closing_AspDotNet_TAG_LIB|$closing_HTML_COMMENT|<|-|[^<\-]*)/sm,
			# triggers
			[	[$AspDotNet_COMMENT, 			\&trigger_AspDotNet_COMMENT],
				[$STD_AspDotNet_DIRECTIVE, 	\&trigger_STD_AspDotNet_DIRECTIVE],
				[$STD_AspDotNet_TAG, 			\&trigger_STD_AspDotNet_TAG],
				[$XML_AspDotNet_TAG, 			\&trigger_XML_AspDotNet_TAG],
				[$AspDotNet_TAG_LIB, 			\&trigger_AspDotNet_TAG_LIB],
				[$closing_AspDotNet_TAG_LIB, 	\&leaving_AspDotNet_TAG_LIB],
				[$closing_HTML_COMMENT,	\&closing_HTML_COMMENT],
			],
		],
		
	# inside a standard tag <%@ ... %>
	'STD_AspDotNet_DIRECTIVE'	=>
		# NOTE : it's not an error, callback is the same than for state STD_AspDotNet_TAG
		[	\&cb_STD_AspDotNet_TAG,
			qr/\G(\%>|\%|$string|"|'|\\|>|[^\%>"'\\]*)/sm,
			#triggers
			[	[$string, 			\&trigger_AspDotNet_STRING],],
		],
	# inside a standard tag <% ... %>
	'STD_AspDotNet_TAG'	=>
		[	\&cb_STD_AspDotNet_TAG,
			# Add \\ in the list, because $string can begin by \\ sometimes ...
			qr/\G(\%>|\%|$NET_COMMENT|\/|$string|"|'|\\|>|[^\%>"'\/\\]*)/sm,
			#triggers
			# NOTE : all strings in this state are assumed to be Net strings !
			[	
				# DO NOT STRIP strings and comments in .Net code (will be by dedicated analyzer)
				#[$string, 			\&trigger_NET_STRING],
				#[$NET_COMMENT,		\&trigger_NET_COMMENT],
			],
		],
		# inside a standard tag <script >... </script>>
	'XML_SCRIPT'	=>
		[	\&cb_XML_SCRIPT,
			qr/\G(<\/script>|<script\b|$AspDotNet_COMMENT|$STD_AspDotNet_TAG|<|[^<]*)/sm,
			#triggers
			[	#[$XML_SCRIPT, 			\&trigger_XML_SCRIPT],
				[$AspDotNet_COMMENT, 	\&trigger_AspDotNet_COMMENT],      # script for javascript can contain <%-- ... --%>
				[$STD_AspDotNet_TAG, 	\&trigger_STD_AspDotNet_TAG],],
		],
	# inside an XML tag <asp:xxx ... > or <asp:xxx ... />
	'XML_AspDotNet_TAG'	=>
		[	\&cb_XML_AspDotNet_TAG,
			qr/\G(\/>|$string|"|'|\\|>|\/|[^\/>"'\\]*)/sm,
			#triggers
			[	[$string, 			\&trigger_AspDotNet_STRING],],
		],
	# inside a tag lib <xxx:yyy ... > or <xxx:yyy ... />
	'AspDotNet_TAG_LIB'	=>
		[	\&cb_AspDotNet_TAG_LIB,
			qr/\G(\/>|$string|"|'|\\|>|\/|$STD_AspDotNet_TAG|<|[^<\/>"'\\]*)/sm,
			#triggers
			[	[$string, 			\&trigger_AspDotNet_STRING],
				[$STD_AspDotNet_TAG, 	\&trigger_STD_AspDotNet_TAG],	],
		],
	# between openning and closing xml tag : <asp:xxx ...> AspDotNet text </asp:xxx>
	#      (only for xxx = scriptlet|declaration|expression) !!!!!!
	'XML_AspDotNet_TEXT'	=>
		[	\&cb_XML_AspDotNet_TEXT,
			qr/\G($closing_XML_AspDotNet_TAG|$CDATA|$NET_COMMENT|\/|$string|"|'|\\|<|[^<"'\/\\]*)/sm,
			#triggers
			[	[$string, 			\&trigger_NET_STRING],
				[$NET_COMMENT,		\&trigger_NET_COMMENT],
				[$CDATA, 			\&trigger_CDATA],	],
		],
	# inside a CDATA
	'CDATA'	=>
		[	\&cb_CDATA,
			qr/\G($closing_CDATA|$NET_COMMENT|\/|$string|"|'|\\|\]|[^\]"'\/\\]*)/sm,
			#triggers
			[	[$string, 			\&trigger_NET_STRING],
				[$NET_COMMENT,		\&trigger_NET_COMMENT],],
		],
	# inside a string
	'AspDotNet_STRING'	=>
		[	\&cb_AspDotNet_STRING,
			qr/\G($esc_escape|$esc_string|$string|"|'|\\|$STD_AspDotNet_EXPR|<|[^"'\\<]*)/sm,
			#triggers
			[	[$STD_AspDotNet_EXPR, 			\&trigger_STD_AspDotNet_TAG],	],
		],
	# inside a string
	'NET_STRING'	=>
		[	\&cb_NET_STRING,
			qr/\G($esc_escape|$esc_string|$string|"|'|\\|[^"'\\]*)/sm,
			undef						# No triggering tokens
		],
	# inside a comment 
	'AspDotNet_COMMENT'	=>
		[	\&cb_AspDotNet_COMMENT,
			qr/\G($closing_AspDotNet_COMMENT|-|$STD_AspDotNet_TAG|$STD_AspDotNet_DIRECTIVE|$STD_AspDotNet_EXPR|<|$closing_STD_AspDotNet_TAG|%|[^\-<%]*)/sm,
			undef						# No triggering tokens
		],
	# inside a Net comment 
	'NET_COMMENT'	=>
		[	\&cb_NET_COMMENT,
			qr/\G($closing_NET_COMMENT|\*|\n|$closing_STD_AspDotNet_TAG|\%|[^*\n\%]*)/sm,
			undef						# No triggering tokens
		],
);

sub extractFromTagText($) {
	my $node = shift;

	my $children = Lib::NodeUtil::GetChildren($node);

	if (scalar @$children > 0) {
		# the first child of the node shoud be an XML_AspDotNet_TEXT
		my $child = $children->[0];

		if (Lib::NodeUtil::GetKind($child) eq 'XML_AspDotNet_TEXT') {
			# FIXME case of a CDATA
			return Lib::NodeUtil::GetStatement($child);
		}
		else {
			print "[WARNING] missing XML_AspDotNet_TEXT in a XML_AspDotNet_TAG\n";
		}
	}
	return Lib::NodeUtil::createEmptyStringRef();
}

sub cb_extractDeclaration($$$) {
	my ($node, $context, $level) = @_;
	my $type = Lib::NodeUtil::GetName($node);
	
	my $virtualClass  = $context->[0];
	
	if ($type eq '<%!') {
		${Lib::NodeUtil::GetStatement($node)} =~ /\A<\%!(.*)\%>/sg;
		if (defined $1) {
			$$virtualClass .= $1 . "\n";
		}
	}
	elsif ($type eq '<asp:declaration') {
		$$virtualClass .= ${extractFromTagText($node)} . "\n";
		return 0; # do not pursue the iteration inside the node.
	}
	return undef;
}

sub cb_extractCode($$$) {
	my ($node, $context, $level) = @_;
	my $kind = Lib::NodeUtil::GetKind($node);
	my $type = Lib::NodeUtil::GetName($node);

	my $virtualClass  = $context->[0];
	my $virtualMethod = $context->[1];

	if (($kind eq "STD_AspDotNet_TAG") || ($kind eq "STD_AspDotNet_DIRECTIVE")) {
		# regexp match the tags <%, <%=, <%: but not <%@ ! (directives will not be in the Net code)
		${Lib::NodeUtil::GetStatement($node)} =~ /\A($STD_AspDotNet_TAG)([^@].*)\%>/sg;
		if (defined $2) {
			if (($1 eq "<%=") || ($1 eq "<%:")) {
				$$virtualMethod .= "    expression = $2";
			}
			else {
				$$virtualMethod .= "    $2";
			}
			if ($ASPNET_TECHNO eq "C#") {
				$$virtualMethod .= ";";
			}
			
			$$virtualMethod .= "\n";
		}
	}
	elsif ($kind eq "XML_AspDotNet_TAG") {
		if ($type =~ /<asp:(?:expression|scriptlet)\b/) {
			$$virtualMethod .= ${extractFromTagText($node)} . "\n";
		}
	}
	elsif ($kind eq "XML_SCRIPT") {
		${Lib::NodeUtil::GetStatement($node)} =~ /\A($XML_SCRIPT[^>]*>)(.*)<\/script>/sg;
		my $content = $2;
		if (defined $content) {
			# ignore HTML tag without any attribute, because they are supposed to be for javascript, VBscript ...
			# ignore HTML tag with "type" attribute (related to text/javascript and so on ...)
			if (($1 ne '<script>') && ($1 !~ /type\s*=/)) {
				$$virtualClass .= $content;
			}
		}
	}
	return undef;
}

my %CLASS_BEGIN = (
	"C#" => "class Scriptlet {\n",
	"VB" => "Public Class Scriptlet\n",
);

my %CLASS_END = (
	"C#" => "\n}\n",
	"VB" => "\nEnd Class\n",
);

my %METHOD_BEGIN = (
	"C#" => "  int myScriptlet() {\n",
	"VB" => "  Public Sub myMethod()\n",
);

my %METHOD_END = (
	"C#" => "\n  \}\n",
	"VB" => "  End Sub\n",
);

sub buildScriptletCode($) {
	my $views = shift;
	
	my $classCode = "";
	my $methodCode = "";
	
	if (! exists $CLASS_BEGIN{$ASPNET_TECHNO}) {
		print STDERR "[ERROR] unknow inline code techno ($ASPNET_TECHNO). Cannot build code view !\n";
		return \$classCode;
	}

	# extract declarations :
	Lib::Node::Iterate($views->{'tag_tree'}, 0, \&cb_extractDeclaration, [ \$classCode, \$methodCode ]);
	# Extract code :
	Lib::Node::Iterate($views->{'tag_tree'}, 0, \&cb_extractCode, [ \$classCode, \$methodCode ]);
	
	$views->{"strip_info"}->{"addedLinesOfCode"} = 0;
	
	if ($methodCode ne "") {
		$methodCode = $METHOD_BEGIN{$ASPNET_TECHNO} . $methodCode . $METHOD_END{$ASPNET_TECHNO};
		$views->{"strip_info"}->{"addedLinesOfCode"} += () = $METHOD_BEGIN{$ASPNET_TECHNO} =~ /^\s*\S/mg;
		$views->{"strip_info"}->{"addedLinesOfCode"} += () = $METHOD_END{$ASPNET_TECHNO} =~ /^\s*\S/mg;
	}
	
	if (($methodCode ne "") || ($classCode ne "")) {
		$classCode = $CLASS_BEGIN{$ASPNET_TECHNO} . $classCode . "\n".$methodCode . $CLASS_END{$ASPNET_TECHNO};
		$views->{"strip_info"}->{"addedLinesOfCode"} += () = $CLASS_BEGIN{$ASPNET_TECHNO} =~ /^\s*\S/mg;
		$views->{"strip_info"}->{"addedLinesOfCode"} += () = $CLASS_END{$ASPNET_TECHNO} =~ /^\s*\S/mg;
	}


	
	

	# Some Net code in scriptlet have missing closing braces because (AspDotNet parser add implicit closing braces at end of file !!)
	# Detect missing braces ...
	my $openning = () = $classCode =~ /\{/g;
	my $closing = () = $classCode =~ /}/g;

	# ... and if any, add them !!
	if ($closing < $openning) {
		my $nb = $openning - $closing;
		$classCode .= "}" x ($nb);
		StripUtils::storeStripInfo($views, "ImplicitClosing", $nb);
		print STDERR "[WARNING] found $nb implicit closing braces.\n";
	}
	# computing and applying the num line offset in the code view
	my $pattern_code_view;
	if ($classCode =~ /^class Scriptlet \{\n+(.*)$/m) {
		$pattern_code_view = quotemeta $1;
		my $numlineOffset = 0;
		if ($views->{'script'} =~ /(.*)$pattern_code_view/s) {
			my $prefix = $1;
			$numlineOffset = () = $prefix =~ /\n/g;
		}
		if ($numlineOffset > 0) {
			my $nbReturn = "\n" x $numlineOffset;
			$classCode =~ s/^class Scriptlet \{\n+/${nbReturn}class Scriptlet \{\t\t/m;
		}
	}
	return \$classCode;
}

sub _cb_PreSearchKind($$) {
	my ($node, $context) = @_;

	my $kind = Lib::NodeUtil::GetKind($node);

	# check if $kind if in the list of searched kinds
	if (exists $context->[1]->{$kind}) {
		# if yes, record the node in the corresponding list.
		if (!exists $context->[0]->{$kind}) {
			$context->[0]->{$kind} = [ $node ];
		}
		else {
			push @{$context->[0]->{$kind}}, $node;
		}
	}
	return undef;
}

# XML_AspDotNet_TAG allways have a XML_AspDotNet_TEXT tag node that contains the AspDotNet code between opening and closing tag.
# If this AspDotNet code is empty (no statement and no child), then the node should be removed.
sub cb_excludeEmptyTags($$) {
	my $childNode = shift;
	my $context = shift;
	if (defined $childNode) {
		if (Lib::NodeUtil::GetKind($childNode) eq XML_AspDotNet_TEXT) {
			my $stmt = Lib::NodeUtil::GetStatement($childNode);
			my $children = Lib::Node::GetChildren($childNode);
			# if it's an xml AspDotNet tag node without children and whose statement is empty...
			if (($$stmt !~ /\S/s) && (scalar @$children == 0)) {
				#	... then remove it.
				Lib::Node::Detach($childNode);
			}
		}
	}
}

# detect the embedded techno of .aspx file
# <script> tags are not supported
# files with the following syntax <script language="C#"...> are not parsed because "C#" here defines "Visual C#"
# which is a language specific to Microsoft, and for instance: other languages as VBScript, Jscript can be used
sub detectTechnoAspNet($$) {
	my $node = shift;
	my $element = shift;
	
	return if ($ASPNET_TECHNO ne "");
	
	my $parent = Lib::NodeUtil::GetParent($node);
	my $kind = Lib::NodeUtil::GetKind($parent);
	my $statement = Lib::NodeUtil::GetStatement($parent);

	if ($ASPNET_TECHNO eq '') {
		# Example of line directive to analyse ...
		# <%@ Page Title="Phone Number" Language="C#" MasterPageFile="~/Site.Master" AutoEventWireup="true" CodeBehind="AddPhoneNumber.aspx.cs" Inherits="SRSOnline.Account.AddPhoneNumber" %>
		if ($kind eq 'STD_AspDotNet_DIRECTIVE' && $$statement =~ /\<\%\@\s*[^\%]*\bLanguage\s*\=/i) {
			$ASPNET_TECHNO = uc($element);
			print "[ASPNET] detected language = " . $ASPNET_TECHNO . "\n";
		}
		elsif ($kind eq 'XML_SCRIPT' && $$statement =~ /<script\b.*Language\s*\=/i)  {
			$ASPNET_TECHNO = uc($element);
			print "[ASPNET] detected language = " . $ASPNET_TECHNO . "\n";
		}
	}
}

sub detectTechno_in_XML_SCRIPT($$) {
	my $node = shift;
	
	return if ($ASPNET_TECHNO ne "");

	my $statement = Lib::NodeUtil::GetStatement($node);

	# Example of line directive to analyse ...
	# <script language="c#" runat="server"> ....
		
	if ($$statement =~ /\A\s*<script\b[^>]*\bLanguage\s*\=\s*"([^"]+)"/i)  {
		$ASPNET_TECHNO = uc($1);
		print "[ASPNET] detected language = " . $ASPNET_TECHNO . "\n";
	}	
}

sub getTechnoAspNet() {
	return $ASPNET_TECHNO;
}
#-------------------------------------------------------------------------------
# DESCRIPTION: provide following views:
#
# tag_comment	: BUFFER, 	containing AspDotNet tags comment (excluding HTML & Net comments )
# script_string : HASH,		containing strings found in AspDotNet tags
# script		: BUFFER, 	containing non HTML code + emptied comments + encoded strings.
# html			: BUFFER, 	containing HTML code.
# script_html	: BUFFER, 	containing TEXT + emptied AspDotNet comments + encoded AspDotNet & Net strings (same as "script" but with HTML elements in addition !)
# tag_tree		: TREE, 	containing AspDotNet tree.
# MixBloc2		: BUFFER,	contains script tags, Net code and Net comments. Net comments are normalized to /* ... */ and not multiline !
#
# NOTE: for Net, views names are those used in the Net analyzer !!!
# comment		: BUFFER, 	containing Net comments
# code			: BUFFER,	containing Net code (packaged in a analyzable class for the Highlight Net analyzer!!)
# HString		: HASH,		containing strings found in embedded Net code.
# MixBloc		: BUFFER,	contains Net code and Net comments and Net strings.
#
# kinds         : HASH,		containing lists of pre-searched 'kind' nodes.
#-------------------------------------------------------------------------------
sub StripAspDotNet($$$$)
{
	my ($filename, $vue, $options, $couples) = @_;
	my $status = 0;

	my $b_TraceInconsistent = (exists $options->{'--TraceInconsistent'} ? 1 : 0); # traces_filter_line
	my $b_assert = (exists $options->{'--TraceDetect'} ? 1 : 0);                  # traces_filter_line

	{
		my $message = 'Launching de StripAspDotNet::StripAspDotNet';
		Erreurs::LogInternalTraces('verbose', undef, undef, 'StripAspDotNet', $message);
	}

	my $b_timing_strip = ((defined Timing->isSelectedTiming('Strip')) ? 1 : 0);
	#print STDERR join ( "\n", keys ( %{$options} ) );                            # traces_filter_line
	configureLocalTraces('StripAspDotNet', $options); # traces_filter_line
	my $stripAspDotNetTiming = new Timing('StripAspDotNet', Timing->isSelectedTiming('Strip'));
	$stripAspDotNetTiming->markTimeAndPrint('--init--') if ($b_timing_strip); # timing_filter_line

	localTrace('verbose', "working with  $filename \n"); # traces_filter_line

	# Init parsing context
	my %hContext = ();
	my $context = \%hContext;

	# Data for string management
	my %strings_values = ();
	my %strings_counts = ();
	my %NET_strings_context = (
		'nb_distinct_strings' => 0,
		'strings_values'      => \%strings_values,
		'strings_counts'      => \%strings_counts);

	# will be use by states callbacks to acces datas about Net strings
	$context->{'NET_string_context'} = \%NET_strings_context;


	# View to memorize data produced by strip phase ... if needed ...
	$context->{'AmbigousClosing'} = 0;

	#my $ref_sep = separer_code_commentaire_chaine(\$vue->{'text'}, $options);
	my $err = Lib::ParseUtil::parseMarkup(\$vue->{'text'}, # input text view
		$vue,                                              # views output set
		$options,                                          # command line options
		\%STATES,                                          # state definition with associated callbacks
		[ 'comment', 'MixBloc' ],                          # additional buffered views (to the default sripts views) to be declared by parseMarkup().
		[ \&cb_appendAsScript, undef, \&cb_appendAsHtml ],
		# addiditional treatments for respectively "appendAsScript" and "appendAsComment"
		\&cb_excludeEmptyTags, # additional empty tag excluding treatments.
		$openServerTags,       # for preliminary split
		$closeServerTags,      # for preliminary split
		$context);


	# View to memorize data produced by strip phase ... if needed ...
	$vue->{'strip_info'} = {};
	StripUtils::storeStripInfo($vue, 'AmbigousClosing', $context->{'AmbigousClosing'});

	# record the Net strings (in compliance with the Net analyzer !!)
	$vue->{'HString'} = \%strings_values;

	$stripAspDotNetTiming->markTimeAndPrint('separer_code_commentaire_chaine') if ($b_timing_strip); # timing_filter_line

	$vue->{'agglo'} = "";
	#StripUtils::agglomerate_C_Comments($MixBloc, \$vue->{'agglo'});

	if ($ASPNET_TECHNO ne "") {
		# build the 'code' view corresponding to the Net code inside scriptlet, declaration and expressions.
		$vue->{'code'} = ${buildScriptletCode($vue)};
	}
	else {
		$vue->{'code'} = "";
	}

	# ASP MixBloc view will be modified by techno strip ... save it in MixBloc2 !
	$vue->{'MixBloc2'} = $vue->{'MixBloc'};

	# invoke real Strip for techno-behind
	my $technoView->{'text'} = $vue->{'code'};
	if ($ASPNET_TECHNO eq "C#") {
		$status |= StripCS::StripCS($filename, $technoView, $options, $couples);
		$vue->{'code'} = $technoView->{'code'};
		$vue->{'comment'} = $technoView->{'comment'};
		$vue->{'HString'} = $technoView->{'HString'};
		$vue->{'code_with_prepro'} = $technoView->{'code_with_prepro'} = "";
		$vue->{'MixBloc'} = $technoView->{'MixBloc'};
		
		$vue->{'agglo'} = $technoView->{'agglo'};
		$vue->{'MixBloc_LinesIndex'} = $technoView->{'MixBloc_LinesIndex'};
		$vue->{'agglo_LinesIndex'} = $technoView->{'agglo_LinesIndex'};
		$vue->{'code_with_prepro'} = $technoView->{'code_with_prepro'};
		
		#~ $vue->{'agglo'} = "";
		#~ StripUtils::agglomerate_C_Comments(\$vue->{'MixBloc'}, \$vue->{'agglo'});
		#~ $vue->{'MixBloc_LinesIndex'} = StripUtils::createLinesIndex(\$vue->{'MixBloc'});
		#~ $vue->{'agglo_LinesIndex'} = StripUtils::createLinesIndex(\$vue->{'agglo'});
		#~ $vue->{'code_with_prepro'} = $technoView->{'code_with_prepro'};
	}
	elsif ($ASPNET_TECHNO eq "VB") {
		$status |= StripVbDotNet::StripVbDotNet($filename, $technoView, $options);
		# computing specific views for VB.Net parser
		$vue->{'code_lc'} = $technoView->{'code_lc'} ;
		$vue->{'code'} = $technoView->{'code'} ;
		$vue->{'MixBloc'} = $technoView->{'MixBloc'} ;
		$vue->{'comment'} = $technoView->{'comment'} ;
		$vue->{'string'} = $technoView->{'string'} ;
		$vue->{'sans_prepro'} = $technoView->{'sans_prepro'} ;
		$vue->{'agglo'} = $technoView->{'agglo'} ;
		$vue->{'comment_sans_tag'} = $technoView->{'comment_sans_tag'} ;
		
		
		#~ my $comment_sans_tag_buffer = $technoView->{'comment'};
		#~ $comment_sans_tag_buffer =~ s/[<][^>]*(?:[>]|\n)/<>/smg ;
		#~ $vue->{'comment_sans_tag'} = $comment_sans_tag_buffer ;
		#~ $vue->{'sansprepro'} = $technoView->{'code'} ;
		#~ $vue->{'sansprepro'} =~ s/^[ \t]*#.*$//gm ;
	}

	$vue->{'strip_info'}->{'techno'} = $ASPNET_TECHNO;

	# PRE-SEARCH BUILDING
	#----------------------
	# build pre-searched kinds lists.
	my %kinds;
	my @context = (\%kinds, { 'XML_AspDotNet_TEXT' => 1 });
	Lib::Node::Iterate($vue->{'tag_tree'}, 0, \&_cb_PreSearchKind, \@context);
	$vue->{'kinds'} = \%kinds;

	if (defined $options->{"--AspDotNet-tree"}) {
		#print STDERR ${Lib::Node::dumpTree($root_HTML, "ARCHI")} ;
		Lib::Node::Dump($vue->{'tag_tree'}, *STDOUT, "ALL");
	}

	if ($err gt 0) {
		my $message = 'Fatal error in code/comment/string separation';
		Erreurs::LogInternalTraces('erreur', undef, undef, 'STRIP // ABORT_CAUSE_SYNTAX_ERROR', 'Erreur fatale dans la separation des chaines et des commentaires');
		#return $status | ErrStripError(1, Erreurs::ABORT_CAUSE_SYNTAX_ERROR, $couples)  if ( $err gt 0);
		$status |= Erreurs::FatalError(Erreurs::ABORT_CAUSE_SYNTAX_ERROR, $couples, $message);
		return $status;
	}

	my $debug = 0; # traces_filter_line

	if (defined $options->{'--dumpstrings'}) {
		dumpVueStrings($vue->{'HString'}, $STDERR);
		dumpVueStrings($vue->{'script_string'}, $STDERR);
	}

	$stripAspDotNetTiming->dump('StripAspDotNet') if ($b_timing_strip); # timing_filter_line

	print STDERR "StripAspDotNet end:$status\n" if ($b_TraceInconsistent); # traces_filter_line
	$stripAspDotNetTiming->finish();                                       # timing_filter_line
	return $status;
}

1;
