package StripRuby;
use strict;
use warnings ;
use Carp::Assert; # traces_filter_line
use Timing ; # timing_filter_line
use StripUtils;
use CommonOutput::ManageVue;

# prototypes publics
sub StripRuby($$$$);

use StripUtils qw(
                  garde_newlines
                  warningTrace
                  configureLocalTraces
                  StringStore
                  dumpVueStrings
                 );

StripUtils::init('StripRuby', 0);

#-----------------------------------------------------------------------

my $blank = \&garde_newlines;

my $TEXT;
my $CODE="";
my $COMMENT = "";
my $MIX = "";
my $DATA = "";

my $STRING_CONTEXT = { 
	'nb_distinct_strings' => 0,
	'strings_values' => {},
	'strings_counts' => {}
} ;
my $RT_STRINGS = $STRING_CONTEXT->{'strings_values'};


# COMMENTS FEATURE :
#-------------------
# kinds :
#   - Simple line : # ..... \n
#   - Multiline   : =begin ..... =end

sub parseLineComment($) {
	my $el = shift;
	my $newLines = $blank->($el);
	$CODE .= $newLines;
	$COMMENT .= $el;
	$MIX .= '/*';
	
	if ( $$TEXT =~ /\G([^\n]+)/gc) {
		$newLines = $blank->($1);
		$CODE .= $blank->($1);
		$COMMENT .= $1;
		$MIX .= $1;
	}
	$MIX .= '*/';
	return 0;
}

sub parseMultilineLineComment($) {
	my $el = shift;
	my $newLines = $blank->($el);
	$CODE .= $newLines;
	$COMMENT .= $el;
	$MIX .= $el;
	
	while ( $$TEXT =~ /\G(=end|\=|[^=]*)/gc) {
		$newLines = $blank->($1);
		if ($1 eq '=end') {
			$CODE .= $blank->($1);
			$COMMENT .= $1;
			$MIX .= $1;
			return 0;
		}
		else {
			$CODE .= $blank->($1);
			$COMMENT .= $1;
			$MIX .= $1;
		}
	}
	print STDERR "Unterminated comment !!!\n";
	return 1;
}

# STRINGS FEATURES
#-----------------
# kinds: 
#  -  Simple quote string  ''
#  -  Double quote string  ""
#  -  Template string      %{  }
#                     or   '   '\   
#                     or   "   "\   
#                     or   <<-TEXT TEXT
#
# Escape :
#     respectively \' \"
#
# Line Continuation :
#     allowed for all

# DOUBLE QUOTE
sub parseDblString($) {
    my $el = shift;
    my $newLines = $blank->($el);
    $COMMENT .= $newLines;
    $MIX .= $newLines;
    my $string_buffer = $1;
    my $CODE_padding = "";

    while ($$TEXT =~ /\G(\\"|\"\"|"|\\#|#\{|#|\\\\|\\|\\\n|[^\\"#]*)/gc) {
        $newLines = $blank->($1);
        $CODE_padding .= $newLines;
        $COMMENT .= $newLines;
        # $MIX .= $newLines;

        if ($1 eq '"') {
            $string_buffer .= $1;
            
            my $separator_padding = "";
            if ($CODE =~ /\w\z/) {
				$separator_padding = " ";
			}
            
            my $string_id = StringStore($STRING_CONTEXT, $string_buffer);
            $CODE .= $separator_padding . "$string_id " . $CODE_padding;
            $MIX  .= $separator_padding . "$string_id " . $CODE_padding;
#print STDERR "DOUBLE STRING = $string_buffer\n";
            return 0;
        }
        elsif ($1 eq '#{') {
            # parse_interpolation
            $string_buffer .= parseInterpolation($TEXT);
        }
        elsif ($1 ne "\\\n") {
            # not a continuing line => update $string_buffer
            $string_buffer .= $1;
        }
    }
    print STDERR "Unterminated string for sub parseDblString !!!\n";
    return 1;
}

# HEREDOC
sub parseHeredoc($$;$$);
sub parseHeredoc($$;$$) {
	#required
	my $openingSymbol = shift;
    my $heredocsPattern = shift;
    
    #optional
    my $string_buffer = shift;
    my $CODE_padding = shift;
    
    my $flag_newString;
    
    if (! defined $string_buffer) {
		my $emptyString = "";
		$string_buffer = \$emptyString;
		$flag_newString = 1;
	}
	else {
		$flag_newString = 0;
	}

	if (! defined $CODE_padding) {
		my $emptyString = "";
		$CODE_padding = \$emptyString;
	}

    my $newLines = $blank->($openingSymbol);
	$COMMENT .= $newLines;

    my $firstLetterHeredocsPattern;
    #my $endOfFirstLine;
    
		
	#($endOfFirstLine) = $$TEXT =~ /\G([^\n]*)/gc;
	
	if ($heredocsPattern =~ /^(\w)/) {
		$$string_buffer .= $openingSymbol;
		$firstLetterHeredocsPattern = $1;
	}

    my $regexp_TplString;
    if ($heredocsPattern) {
        $regexp_TplString = qr /\G((<<(?:[\-'"]?)([a-zA-Z_]+))|<|\\\\|\\|\\#|#\{|#|(?<=\n|\s)$heredocsPattern\b|$firstLetterHeredocsPattern|\n|\\\n|[^$firstLetterHeredocsPattern\<#]*)/;
    }
    
    my $item;
	while ( $$TEXT =~ /$regexp_TplString/gc) {
		$item = $1;
		$newLines = $blank->($1);
		$COMMENT .= $newLines;
		$$CODE_padding .= $newLines;
#print STDERR "ITEM = $item \n";
        if (defined $2 and defined $3 and ($$string_buffer !~ /(\bclass\s*|^[ \t]*#[^\n]*)\z/m)) {
			$$string_buffer .= $2;
			$heredocsPattern = $3;
			
			my $err = parseHeredoc($2, $3, $string_buffer, $CODE_padding);
			
            #~ if ($heredocsPattern and $heredocsPattern =~ /^(\w)/) {
                #~ $string_buffer = $openingSymbol;
                #~ $firstLetterHeredocsPattern = $1;
            #~ }
            
            #~ # print 'discovering new pattern ' . $heredocsPattern."\n";
            #~ $regexp_TplString = qr /\G((<<(?:[\-'"]?)([a-zA-Z_]+))|<|(?<=\n|\s)$heredocsPattern\b|$firstLetterHeredocsPattern|[^$firstLetterHeredocsPattern\<]*)/;

		}        
        #~ elsif ($heredocsPattern and $1 eq $heredocsPattern) {
        if (($item eq $heredocsPattern) && ($$string_buffer =~ /(\A|\n)\s*\z/)) {
			$$string_buffer .= $item;
			
			if ($flag_newString) {
				my $string_id = StringStore( $STRING_CONTEXT, $$string_buffer );
			
				#$CODE .= " $string_id ".$endOfFirstLine.$$CODE_padding;
				#$MIX .= " $string_id ".$endOfFirstLine.$$CODE_padding;
				$CODE =~ s/__UNPARSED_HEREDOC__/$string_id/g;
				$MIX =~ s/__UNPARSED_HEREDOC__/$string_id/g;
				$CODE .= $$CODE_padding;
				$MIX .= $$CODE_padding;
#print STDERR "HEREDOC = $$string_buffer\n";
			}
			return 0;
		}
		else {
			$$string_buffer .= $item;
		}
	}
	print STDERR "Unterminated string for sub parseHeredoc : $$string_buffer\n";
	return 1;
}

my %OPEN_TO_CLOSE = (
	'{' => '}',
	'(' => ')',
	'[' => ']',
	'<' => '>',
);

my $OPEN_TO_REGEXP = {};

sub parseString($;$) {
	my $openingSymbol = shift;
	my $letter = shift;

	my $closing = $OPEN_TO_CLOSE{$openingSymbol};
	if (!defined $closing) {
		$closing = $openingSymbol;
	}


	my $regexp_TplString;
	if (defined $OPEN_TO_REGEXP->{$openingSymbol}) {
		$regexp_TplString = $OPEN_TO_REGEXP->{$openingSymbol};
	}
	else {
		my $escaped_closing = quotemeta $closing;
		my $escaped_opening = quotemeta $openingSymbol;
		if ($closing eq $openingSymbol) {
			$regexp_TplString = qr /\G(#\{|\\#|\\\\|\\${escaped_closing}|${escaped_closing}|#|\\|\n|\\\n|[^${escaped_closing}\n\\#]*)/;
		}
		else {
			$regexp_TplString = qr /\G(#\{|\\#|\\\\|\\${escaped_opening}|${escaped_opening}|\\${escaped_closing}|${escaped_closing}|#|\\|\n|\\\n|[^${escaped_opening}${escaped_closing}\n\\#]*)/;
		}
		$OPEN_TO_REGEXP->{$openingSymbol} = $regexp_TplString;
	}

    my $newLines = $blank->($openingSymbol);
	$COMMENT .= $newLines;
	my $string_buffer = $1;
	my $CODE_padding = "";
    
    # should take into account the nested level. Here is an example of that :
    # 			TOTO = %i[[] [   fetch]].freeze
    # whose result is, if we dump the variable TOTO
    #			[:[], :"[", :"fetch]"]
    my $level = 0;
    
	while ( $$TEXT =~ /$regexp_TplString/gc) {
		$newLines = $blank->($1);
		$COMMENT .= $newLines;
		$CODE_padding .= $newLines;

		if ($1 eq $closing) {
			
			if ($level<1) {
				$string_buffer .= $1;
				
				my $separator_padding = "";
				if ($CODE =~ /\w\z/) {
					$separator_padding = " ";
				}
				
				# try to respect the indentation for the item coming after on the same line :
				if( $string_buffer =~ /\n/) {
					my ($indentation) = $string_buffer =~ /\n([^\n\S]*)[^\n]*\z/;
					$CODE_padding .= $indentation;
				}
				else {
					my $indentation_padding = length($string_buffer) - length("__ARRAY_OF_SYMBOLS__ ") - length($separator_padding);
					if ($indentation_padding >=0 ) {
						$CODE_padding .= " "x$indentation_padding;
					}
				}
				
				if ((defined $letter) and (($letter eq "i") or ($letter eq "I"))) {
					# WARNING : when changing __ARRAY_OF_SYMBOLS__ into anything else, don't forget to modify Ruby::ParseRuby::ParseUnknow() too !!!!!
					$CODE .= $separator_padding . "__ARRAY_OF_SYMBOLS__ ".$CODE_padding;
					$MIX  .= $separator_padding . "__ARRAY_OF_SYMBOLS__ ".$CODE_padding;
#print STDERR "ARRAY OF SYMBOLS : $string_buffer\n";
				}
				else {
					my $string_id = StringStore( $STRING_CONTEXT, $string_buffer );
					$CODE .= $separator_padding . "$string_id ".$CODE_padding;
					$MIX  .= $separator_padding . "$string_id ".$CODE_padding;
				}
#print STDERR "STRING : $string_buffer\n";
				return 0;
			}
			
			else {
				$level--;
				$string_buffer .= $1;
			}
		}
		elsif ($1 eq $openingSymbol) {
			$level++;
			$string_buffer .= $1;
		}
        # remove backtick from string syntax because this syntax is used for command system and still be in the code view
        # elsif ($1 eq '`' and $openingSymbol eq '`') {
		# 	$string_buffer .= $1;
		# 	my $string_id = StringStore( $STRING_CONTEXT, $string_buffer );
		# 	$CODE .= " $string_id ".$CODE_padding;
		# 	$MIX .= " $string_id ".$CODE_padding;
		# 	return 0;
		# }
        elsif ($1 eq '#{') {
            # parse_interpolation
            $string_buffer .= parseInterpolation($TEXT);
        }
		elsif ($1 eq "\n") {
			$string_buffer .= $1;
		}
		elsif ($1 ne "\\\n") {
			# not a continuing line => update $string_buffer
			$string_buffer .= $1;
		}
	}
	print STDERR "Unterminated string for sub parseString : $string_buffer\n";
	return 1;
}

# consumes interpolation block into string_buffer
sub parseInterpolation
{
	my $textInput = shift;
    
    my $buff = '#{';
    my $level = 1;
    while ($$textInput =~ /\G(\}|#\{|#|\{|[^}#\{]+)/gc) {

        if ($1 eq '}') {
			$level--;
            $buff .= $1;
            if (! $level) {
				return $buff;
			}
        }
        elsif ($1 eq '{') {
			$level++;
			$buff .= $1;
		}
        elsif ($1 eq '#{') {
            $buff .= parseInterpolation($textInput);
        }
        else {
            $buff .= $1;
        }
    }
    
    print STDERR "[parseInterpolation] Missing closing }\n";
    return $buff;
}

sub parseCode($$$) {
	$TEXT = shift;
	my $options = shift;
       
	my $err = 0;
    
	# Init data
	$CODE = "";
    $COMMENT = "";
    $MIX = "";
    $STRING_CONTEXT = { 
		'nb_distinct_strings' => 0,
		'strings_values' => {},
		'strings_counts' => {}
	};
	$RT_STRINGS = $STRING_CONTEXT->{'strings_values'};
    
    my $keywordsRegex = 
    {
        'if' => 1,
        'elsif' => 1,
        'scan' => 1,
        'split' => 1,
        'unless' => 1,
        'until' => 1,
        'when' => 1,
        'while' => 1,
    };
    my $stringKeywords = '';
    $stringKeywords .= '\s*\b(?:'.join("|", keys %$keywordsRegex).')\s*';

    my $prefixRegexKeyword =  qr /$stringKeywords/;
    my $prefixRegexR =  qr /\%r/;
    my $prefixRegexOperators =  qr /=~|\~|\,|\!|\||\(|\=|\&|not|and|or|\[/;

	my $flag_heredoc_pending = undef;
	my $nb_lineContinuation = 0;

	# special variable $", $', $`
	# special ?" and ?'...
    # method name notation  :name_method
    # symbol  :symbol
    # symbol  :"symbol"
    # symbol  :'symbol'
    # symbol unicode value  :"\u0000"
	while ( $$TEXT =~ /
            \G
            (
				\n
				# line continuation
				|\\\n
                |($prefixRegexKeyword\s*(\/))
                |($prefixRegexR\s*([\(\[\|\!\{'\/]))
                |($prefixRegexOperators\s*(\/))
                #regex suffix .match 
                |(\/.*\/\.match)\b                  
                |\/
                |=begin
                |(<<(?:[\-'"~]*)([a-zA-Z_]+)['"]?)
                |=
                |\#\{
                |\#
                #|%\{
                #|(%[qQiIwWxs]?([\[\/\-{\("<]))
                |%([qQiIwWxs]?)([^\w\s])
                |__END__
                |\$[^\s] # predefined variable
                |\$
                |\?[^\s] # unary question mark => single character string ...
                |\?
                |\_
                |"
                |'
                |`
                |%
                |<
                |\=
                |\|
                |\,
                |\~
                |\!
                |\(
                |\{
                |\&
                |\[
                |:"
                |:'
                |:\$?(?:[^\s]) # needed to capture symbol :\/ in order \/ would not be considered as a regexp delimiter !!
                |:
                |not
                |and
                |or
                |n
                |a
                |o
                |g
                |s
                |w
                |i
                |e
                |u
                # |[^"'`=\#\%<\$\=\,\|\~naogswieu\!\{\(\?\_\&\/\[\:]+
                |[^"'`=\#\%<\$\=\,\|\~naogswieu\!\{\(\?\_\&\/\[\:\n\\]+
            )
            /xgc) {

        my $newLines = $blank->($1);

        my $pattern = $1;
        my $KeywordRegexBegin = $2;
        my $prefixKeywordRegex = $3;  
        my $RRegexBegin = $4;
        my $prefixRRegex = $5;  
        my $simpleRegexBegin = $6;
        my $prefixLineRegex = $7;
        
        
        my $heredocPattern = $9;
        my $heredocName = $10;
        #~ if ((defined $9) && ($CODE !~ /(\bclass\s*|^[ \t]*#[^\n]*)\z/m)) {
			#~ # NOTES :
			#~ # 	1) << should not be preceded by "class" (in this cas, it is an inheritage syntax)
			#~ # 	2) the line containing << should not be a comment line beginning with "#" ... 
			#~ $heredocPattern = $9;
			#~ $heredocName = $10;
		#~ }
        
        my $stringLetter = $11;
        my $stringOpenning = $12;
        
        # print "code_fragment: <" . $pattern.">\n" if (defined $pattern) ;
        # print "KeywordRegexBegin: <" . $KeywordRegexBegin.">\n" if (defined $KeywordRegexBegin) ;
        # print "prefixKeywordRegex: <" . $prefixKeywordRegex.">\n" if (defined $prefixKeywordRegex) ;
        # print "RRegexBegin: <" . $RRegexBegin.">\n" if (defined $RRegexBegin) ;
        # print "prefixRRegex: <" . $prefixRRegex.">\n" if (defined $prefixRRegex) ;
        # print "simpleRegexBegin: <" . $simpleRegexBegin.">\n" if (defined $simpleRegexBegin) ;
        # print "prefixLineRegex: <" . $prefixLineRegex.">\n" if (defined $prefixLineRegex) ;
       
        if ($pattern eq '#') {
			$err = parseLineComment($pattern);
		}
		elsif ($pattern eq "\n") {
			if (defined $flag_heredoc_pending) {
				# $flag_heredoc_pending->[0] contains the whole captured pattern
				# $flag_heredoc_pending->[1] contains the heredoc name
				$err = parseHeredoc($flag_heredoc_pending->[0], $flag_heredoc_pending->[1]);
			
				$flag_heredoc_pending = undef;
				$CODE .= $pattern;
				$COMMENT .= $newLines;
				$MIX .= $pattern;
			}
			elsif ($nb_lineContinuation) {
				# add the current \n to the previous due to line continuation ...
				$nb_lineContinuation++;
				
				my $padding = "\n" x $nb_lineContinuation;
				$CODE .= $padding;
				$COMMENT .= $padding;
				$MIX .= $padding;
				$nb_lineContinuation= 0;
			}
			else {
				$CODE .= "\n";
				$COMMENT .= "\n";
				$MIX .= "\n";
			}
		}
		elsif ($pattern eq "\\\n") {
			$nb_lineContinuation++;
		}
		elsif ($pattern eq '=begin') {
			$err = parseMultilineLineComment($pattern);
		}
        elsif (defined $prefixLineRegex 
        or defined $prefixKeywordRegex
            or defined $prefixRRegex) {
            if (defined $simpleRegexBegin) # regex
                                            # ... =~ /.../
                                            # or ... ~ /.../ 
                                            # or ... , /.../ 
                                            # or ... | /.../
                                            # or ... ( /.../
                                            # or ... ! /.../
                                            # or ... = /.../
            {
				# remove the / in the detection pattern. It should not be present in the code, it belongs to the regexp ...
				$simpleRegexBegin =~ s/\///;
                $CODE .= $simpleRegexBegin;
                $MIX .= $simpleRegexBegin;

                $err = parseRegExp($prefixLineRegex,$TEXT);
            }
            elsif (defined $RRegexBegin) # regex %r({/!['|...|']!/})
            {
                $err = parseRegExp($prefixRRegex,$TEXT, $RRegexBegin);                
            
            }
            else # regex (g)sub(!)/.../ 
                    # or (g)sub(!)(%r{},...)
                    # or scan (/.../)
                    # or when /.../
                    # or if /.../
                    # or split (/.../)            
            {
				# remove the / in the detection pattern. It should not be present in the code, it belongs to the regexp ...
				$KeywordRegexBegin =~ s/\///;
                $CODE .= $KeywordRegexBegin;
                $MIX .= $KeywordRegexBegin;

                $err = parseRegExp($prefixKeywordRegex,$TEXT);
            }
        }
        elsif ($pattern eq "/") {
			# last chance to recognize regexp ...
			
			# check specific "false" cases
			# ----------------------------
			
			# check for /= operator :
			# - followed with =
			# - preceded by <ident>, ) or ]
			#~ if (($$TEXT =~ /\G(?=[=])/gc) &&($CODE =~ /([\w]\??|[\)\]])\s*\z/gc)) {
				#~ $CODE .= $pattern;
                #~ $COMMENT .= $newLines;
                #~ $MIX .= $pattern;
			#~ }
			
			# check what is before the /
			# ----------------------------
			# - Ternary "?" can be followed by regexp. And "?" is considered as ternary op if it is not glued to \w.( ex: n.begin_type? ? /.+/ : source_to_regexp(n.source)) 
			# - "/" following ":" is systematically a regexp ...
			# - \n : assume that if the "/" is preceded with nothing on the same line, it is a regex.
			#                  ex : regexp =
			#							if RUBY_ENGINE == 'jruby'
			#							  /no such file to load -- unknownlibrary/
			#							else
			#							  /cannot load such file -- unknownlibrary/
			#							end
			if ($CODE =~ /(\breturn\b|\(|,|\[|:|=>|\n|(?:[^\w]|\s)\?)\s*\z/gc) {
#print STDERR "ITEM 1\n";
				$err = parseRegExp("/",$TEXT);
			}
			#elsif ($CODE =~ /([\w]\??|[\)\]])\s*\z/gc) {
			elsif ($CODE =~ /[\)\]]\s*\z/gc) {
				$CODE .= $pattern;
                $COMMENT .= $newLines;
                $MIX .= $pattern;
			}
			# check what is after the /
			# ---------------------------
			elsif ($$TEXT =~ /\G(?=\s*[^=\@\w\(\.\s])/gc) {
#print STDERR "ITEM 2\n";
				$err = parseRegExp("/",$TEXT);
			}
			else {
				$CODE .= $pattern;
                $COMMENT .= $newLines;
                $MIX .= $pattern;
			}
		}           
		elsif ($pattern eq '"' or $pattern eq ':"') {
            
            if ($pattern eq ':"') {
                $CODE .= ':';
                $COMMENT .= $newLines;
                $MIX .= ':';
            }
       
			#$err = parseDblString($pattern);
			$err = parseString('"');
            #~ if ($err == 1) {
                #~ $err = 0;
                #~ $err = parseHeredoc($pattern);
            #~ }
        }
		elsif ($pattern eq "'" or $pattern eq ":'") {
            if ($pattern eq ':"') {
                $CODE .= ':';
                $COMMENT .= $newLines;
                $MIX .= ':';
            }

            $err = parseString("'");
            #~ if ($err == 1) {
                #~ $err = 0;
                #~ $err = parseHeredoc($pattern);
            #~ }
		}
        elsif ($pattern eq '`') {
            $err = parseString($pattern);
		}
		#elsif ($pattern eq '%{') {
        #    $err = parseHeredoc($pattern);
		#}
        elsif (defined $8) {
            my $position = pos ($$TEXT) - length($8) + 1;           
            pos ($$TEXT) = $position;
            $err = parseRegExp('/',$TEXT);
		}	          
        elsif (defined $heredocPattern and defined $heredocName and (($CODE !~ /(\bclass\s*|^[ \t]*#[^\n]*)\z/m))) {
            $flag_heredoc_pending = [$heredocPattern, $heredocName];
            $CODE .= " __UNPARSED_HEREDOC__ ";
            $MIX .= " __UNPARSED_HEREDOC__ ";
		}
		
		elsif (defined $stringOpenning) {
			
			# %q[ ] 	Non-interpolated String (except for \\ \[ and \])
            # %Q[ ] 	Interpolated String (default)
            # %r[ ] 	Interpolated Regexp (flags can appear after the closing delimiter)
            # %i[ ] 	Non-interpolated Array of symbols, separated by whitespace (after Ruby 2.0)
            # %I[ ] 	Interpolated Array of symbols, separated by whitespace (after Ruby 2.0)
            # %w[ ] 	Non-interpolated Array of words, separated by whitespace
            # %W[ ] 	Interpolated Array of words, separated by whitespace
            # %x[ ] 	Interpolated shell command
            # %s[ ] 	Non-interpolated symbol
            
            # $11 is the letter
            # $12 is the opening character
            
            # print 'String interpolated discovered !!!! '."=<$pattern>\n";
            if (($stringLetter eq "q") or ($stringLetter eq "Q") or ($stringLetter eq "") or ($stringLetter eq "w") or ($stringLetter eq "W") or ($stringLetter eq "i") or ($stringLetter eq "I")) {
				$err = parseString($stringOpenning, $stringLetter);          
			}
			else {
				$CODE .= "%".$stringLetter.$12;
				$COMMENT .= $newLines;
				$MIX .= "%".$stringLetter.$12;
			}
		}	   
        elsif ($pattern eq '__END__') {
            $DATA = $CODE;
            $DATA =~ s/.*\n/\n/g;
            
            my ($data) = $$TEXT =~ /\G(.*)/sgc;
            $DATA .= $data;
            $CODE .= $pattern;
            $COMMENT .= $newLines;
            $MIX .= $pattern;
            return 0;
		}
		else {
			$CODE .= $pattern;
			$COMMENT .= $newLines;
            $MIX .= $pattern;
		}
		
		if ($err gt 0) {
			return 1;
		}
	}
	
	return  0;
}

my %CLOSING = (
	'(' => ')',
	'/' => '/',
	'[' => ']',
	'|' => '|',
	'!' => '!',
	'{' => '}',
	"'" => "'",
);

sub parseRegExp($$;$) 
{
    my $openingSymbol = shift;
    my $textInput = shift;
    my $RRegexBegin = shift;
    
    my $regexp;
    if ( defined $RRegexBegin) {
		$regexp = $RRegexBegin;
	}
	else {
		$regexp = $openingSymbol;
	}
  
    #~ $CODE .= $openingSymbol;
    #~ $MIX .= $openingSymbol;

    my $closingSymbol = $CLOSING{$openingSymbol};
    
    my $matchClosingSymbol = "\\".$closingSymbol;
    my $matchOpeningSymbol = "\\".$openingSymbol;
    my $matchOpeningEscapedSymbol = "\\\\\\".$openingSymbol;
    my $matchClosingEscapedSymbol = "\\\\\\".$closingSymbol;
    # print 'openingSymbol=<'.$openingSymbol.">\n";
    # print 'closingSymbol=<'.$closingSymbol.">\n";
    # print 'matchOpeningSymbol='.$matchOpeningSymbol."\n";
    # print 'matchClosingSymbol='.$matchClosingSymbol."\n";
    # print 'matchOpeningEscapedSymbol='.$matchOpeningEscapedSymbol."\n";
    # print 'matchClosingEscapedSymbol='.$matchClosingEscapedSymbol."\n";
    
    my $countSymbol = 0;
    
    while ($$textInput =~ /
    \G(
        $matchOpeningEscapedSymbol
        |$matchClosingEscapedSymbol
        |$matchOpeningSymbol
        |$matchClosingSymbol
        |\\\\
        |\\\/
        |\\
        |\/
        |[^\/$matchOpeningSymbol$matchClosingSymbol\\]+
    )/xgc) {
#print STDERR "REGITEM = $1\n";
        if ($1 eq $closingSymbol and $countSymbol == 0) {
			
			$regexp .= $1;
			
			my $separator_padding = "";
			if ($CODE =~ /\w\z/) {
				$separator_padding = " "
			}
			
			# parse modifiers if any ...
			if ($$textInput =~ /\G(\w+)/gc) {
				$regexp .= $1;
			}
			
			my $nb = () = $regexp =~ /\n/g;
			# WARNING : when changing __REGEXP__ into anything else, don't forget to modify Ruby::ParseRuby::ParseUnknow() too !!!!!
            $CODE .= $separator_padding . "__REGEXP__ ".("\n"x$nb);
            $MIX  .= $separator_padding . "__REGEXP__ ".("\n"x$nb);
            
#print STDERR 'REGEXP '.$regexp."\n";
            return 0;
        }
        elsif ($RRegexBegin and $1 eq $openingSymbol) {
        # print '++countSymbol' ."\n";
            #~ $CODE .= $1;
            #~ $MIX .= $1;
            $regexp .= $1;
            $countSymbol++;            
        }
        elsif ($RRegexBegin and $1 eq $closingSymbol) {
        # print '--countSymbol' ."\n";
            #~ $CODE .= $1;
            #~ $MIX .= $1;
            $regexp .= $1;
            $countSymbol--;            
        }
        else {
            #~ $CODE .= $1;
            #~ $MIX .= $1;
            $regexp .= $1;
       }
    }
    
    print STDERR "[parseRegExp] Missing closing delimiter $closingSymbol in $regexp !!!\n";
    return 1;
}

# analyse du fichier
sub StripRuby($$$$)
{
    my ($filename, $vue, $options, $couples) = @_;
    my $b_timing_strip = ((defined Timing->isSelectedTiming ('Strip'))? 1 : 0);

    configureLocalTraces('StripRuby', $options); # traces_filter_line
    my $stripRubyTiming = new Timing ('StripRuby', Timing->isSelectedTiming ('Strip'));
    $stripRubyTiming->markTimeAndPrint('--init--') if ($b_timing_strip); # timing_filter_line
    
    my $err = parseCode(\$vue->{'text'}, $options, $vue);
    
    $stripRubyTiming->markTimeAndPrint('separer_code_commentaire_chaine') if ($b_timing_strip); # timing_filter_line

    $vue->{'comment'} = $COMMENT;
    $vue->{'HString'} = $RT_STRINGS;
    $vue->{'code_with_prepro'} = "";
    $vue->{'MixBloc'} = $MIX;
    $vue->{'agglo'} = "";                       
    StripUtils::agglomerate_C_Comments(\$MIX, \$vue->{'agglo'});
    $vue->{'MixBloc_LinesIndex'} = StripUtils::createLinesIndex(\$MIX);
    $vue->{'agglo_LinesIndex'} = StripUtils::createLinesIndex(\$vue->{'agglo'});

    if ( $err gt 0) {
      my $message = 'Fatal error when separating code/comments/strings';
      Erreurs::LogInternalTraces ('erreur', undef, undef, 'STRIP // ABORT_CAUSE_SYNTAX_ERROR', $message);
      return Erreurs::FatalError( Erreurs::ABORT_CAUSE_SYNTAX_ERROR, $couples, $message);
    }
    
    $vue->{'code'} = $CODE;
    $vue->{'data'} = $DATA;
    if (defined $options->{'--dumpstrings'}) {
      dumpVueStrings(  $RT_STRINGS , $STDERR );
    }

    CommonOutput::ManageVue::setVue($vue);

    $stripRubyTiming->dump('StripRuby') if ($b_timing_strip); # timing_filter_line
    return 0;
}

1; # Le chargement du module est okay.

