package StripTypescript;
use strict;
use warnings ;
use Carp::Assert; # traces_filter_line
use Timing ; # timing_filter_line
use StripUtils;

# prototypes publics
sub StripTypescript($$$$);

use StripUtils qw(
                  garde_newlines
                  warningTrace
                  configureLocalTraces
                  StringStore
                  dumpVueStrings
                 );

StripUtils::init('StripTypescript', 0);
use CommonOutput::ManageVue;

#-----------------------------------------------------------------------

my $blank = \&garde_newlines;

my $TEXT;
my $CODE="";
my $COMMENT = "";
my $MIX = "";

my $STRING_CONTEXT = { 
	'nb_distinct_strings' => 0,
	'strings_values' => {},
	'strings_counts' => {}
} ;
my $RT_STRINGS = $STRING_CONTEXT->{'strings_values'};


# COMMENTS FEATURE :
#-------------------
# kinds :
#   - Simple line : // ..... \n
#   - Multiline   : /* ..... */
sub parseLineComment($) {
	my $el = shift;
	my $newLines = $blank->($el);
	$CODE .= $newLines;
	$COMMENT .= $el;
	$MIX .= '/*';
	
	while ( $$TEXT =~ /\G(\n|[^\n]*)/gc) {
		$newLines = $blank->($1);
		if ($1 eq "\n") {
			$CODE .= $blank->($1);
			$COMMENT .= $1;
			$MIX .= "*/\n";
			return 0;
		}
		else {
			$CODE .= $blank->($1);
			$COMMENT .= $1;
			$MIX .= $1;
		}
	}
	print STDERR "Unterminated comment !!!\n";
	
	return 1;
}

sub parseMultilineLineComment($) {
	my $el = shift;
	my $newLines = $blank->($el);
	$CODE .= $newLines;
	$COMMENT .= $el;
	$MIX .= $el;
	
	while ( $$TEXT =~ /\G(\n|\*\/|\*|[^*\n]*)/gc) {
		$newLines = $blank->($1);
		if ($1 eq "*/") {
			$CODE .= $blank->($1);
			$COMMENT .= $1;
			$MIX .= $1;
			return 0;
		}
		elsif( $1 eq "\n") {
			$CODE .= $blank->($1);
			$COMMENT .= "\*\/\n\/\*";
			$MIX .= "\*\/\n\/\*";
		}
		else {
			$CODE .= $blank->($1);
			$COMMENT .= $1;
			$MIX .= $1;
		}
	}
	print STDERR "Unterminated comment !!!\n";
	return 1;
}

# STRINGS FEATURES
#-----------------
# kinds: 
#  -  Simple quote string  ''
#  -  Double quote string  ""
#  -  Template string      ``
#
# Escape :
#     respectively \' \" \`
#
# Line Continuation :
#     allowed for all

# DOUBLE QUOTE
sub parseDblString($) {
	my $el = shift;
	my $newLines = $blank->($el);
	$COMMENT .= $newLines;
	$MIX .= $newLines;
	my $string_buffer = $1;
	my $CODE_padding = "";
	
	while ( $$TEXT =~ /\G(\\"|\"\"|"|\\\\|\\|\\\n|[^\\"]*)/gc) {
		$newLines = $blank->($1);
		$CODE_padding .= $newLines;
		$COMMENT .= $newLines;
		# $MIX .= $newLines;
		
		if ($1 eq '"') {
			$string_buffer .= $1;
			my $string_id = StringStore( $STRING_CONTEXT, $string_buffer );
			$CODE .= " $string_id ".$CODE_padding;
			$MIX .= " $string_id ".$CODE_padding;
			return 0;
		}
		elsif ($1 ne "\\\n") {
			# not a continuing line => update $string_buffer
			$string_buffer .= $1;
		}
	}
	print STDERR "Unterminated string !!!\n";
	print STDERR "$string_buffer\n";
	return 1;
}

# SIMPLE QUOTE
sub parseSplString($) {
	my $el = shift;
	my $newLines = $blank->($el);
	$COMMENT .= $newLines;
	$MIX .= $newLines;
	my $string_buffer = $1;
	my $CODE_padding = "";
	
	while ( $$TEXT =~ /\G(\\'|\'\'|'|\\\\|\\|\\\n|[^\\']*)/gc) {
		$newLines = $blank->($1);
		$CODE_padding .= $newLines;
		$COMMENT .= $newLines;
		# $MIX .= $newLines;
		
		if ($1 eq "'") {
			$string_buffer .= $1;
			my $string_id = StringStore( $STRING_CONTEXT, $string_buffer );
			$CODE .= " $string_id ".$CODE_padding;
			$MIX .= " $string_id ".$CODE_padding;
			return 0;
		}
		elsif ($1 ne "\\\n") {
			# not a continuing line => update $string_buffer
			$string_buffer .= $1;
		}
	}
	print STDERR "Unterminated string !!!\n";
	return 1;
}

# TEMPLATE
sub parseTplString($) {
	my $el = shift;
	my $newLines = $blank->($el);
	$COMMENT .= $newLines;
	my $string_buffer = $1;
	my $CODE_padding = "";
	
	my $inside_code = 0;
	
	while ( $$TEXT =~ /\G(\\`|`|\\\\|\\|\n|\\\n|\$\{|\$|\{|\}|[^\\`\n\$\{\}]*)/gc) {
		$newLines = $blank->($1);
		$COMMENT .= $newLines;
		$CODE_padding .= $newLines;
		
		if ($inside_code) {
			if ($1 eq '${') {
				$inside_code++;
			}
			elsif ($1 eq '{') {
				$inside_code++;
			}
			elsif ($1 eq '}') {
				$inside_code--;
			}
			$string_buffer .= $1;
		}
		elsif ($1 eq '${') {
			$inside_code++;
		}
		elsif ($1 eq "`") {
			$string_buffer .= $1;
			my $string_id = StringStore( $STRING_CONTEXT, $string_buffer );
			$CODE .= " $string_id ".$CODE_padding;
			$MIX .= " $string_id ".$CODE_padding;
#print "TEMPLATE STRING : $string_buffer\n";
			return 0;
		}
		elsif ($1 eq "\n") {
			$string_buffer .= $1;
		}
		elsif ($1 ne "\\\n") {
			# not a continuing line => update $string_buffer
			$string_buffer .= $1;
		}
		else {
			# $string_buffer not updated
		}
	}
	print STDERR "Unterminated string !!!\n";
	return 1;
}

sub parseRegExp($) {
	my $textInput = shift;
	my $openingSymbol = '/';
	my $closingSymbol = $openingSymbol;
	my $potentialRegexp = $openingSymbol;
	my $matchClosingSymbol = "\\" . $closingSymbol;
	my $matchOpeningSymbol = "\\" . $openingSymbol;
	my $matchOpeningEscapedSymbol = "\\\\\\" . $openingSymbol;
	my $matchClosingEscapedSymbol = "\\\\\\" . $closingSymbol;
	while ($$textInput =~ /
		\G(
			$matchOpeningEscapedSymbol
			|$matchClosingEscapedSymbol
			|$matchOpeningSymbol
			|$matchClosingSymbol
			|\\\\
			|\\\/
			|\\\[
			|\\\]
			|\\
			|\/
			|\[
			|\]
			|[^\[\]\/\\$matchOpeningSymbol$matchClosingSymbol]+
		)/xgc) {
		if ($1 eq $closingSymbol) {
			$potentialRegexp .= $1;
			# consume modifiers/tags of regex :
			if ($$textInput =~ /\G([gimsyu])/gc) {
				$potentialRegexp .= $1;
			}
			if ($potentialRegexp =~ /\n/) {
				# new line is detected it's not a regex but maybe an html part of file.
				# regex with /x tag is not supported with JS/TS, so the regex must be written only on one line
				$CODE .= $potentialRegexp;
				$MIX .= $potentialRegexp;
			}
			else {
				my $nb_spaces = () = $potentialRegexp =~ /\s+/;
				$nb_spaces = $nb_spaces + length($potentialRegexp) - 9; # 9 = length (REGEXP_XX)
				
				if ($nb_spaces > 0) {
					$CODE .= "REGEXP_XX" . " " x $nb_spaces;
					$MIX .= "REGEXP_XX" . " " x $nb_spaces;
				}
				else {
					$CODE .= "REGEXP_XX";
					$MIX .= "REGEXP_XX";
				}
				# print 'Regexp detected \n' . $potentialRegexp . "\n";
			}
			# no error
			return 0;
		}
		else {
			$potentialRegexp .= $1;
		}
	}
	print STDERR "[parseRegExp] Missing closing delimiter $closingSymbol \n";
	return 1;
}

sub parseAnnotation($) {
	my $pattern = shift;
	
	if ($$TEXT =~ /\G\s*\(/gc) {
		my $level = 1;
		while ($$TEXT =~ /\G([^\(\)\n]*)([\(\)\n])/gc) {
			if (defined $2) {
				if ($2 eq "(") {
					$level++;
				}
				elsif ($2 eq ")") {
					$level--;
					return 0 if ($level == 0);
				}
				else {
					$CODE .= "\n";
					$COMMENT .= "\n";
					$MIX .= "\n";
				}
			}
		}
	}
	return 1;
}

sub parseCode($$) {
	$TEXT = shift;
	my $options = shift;

	my $err = 0;

	# Init data
	$CODE = "";
	$COMMENT = "";
	$MIX = "";
	$STRING_CONTEXT = {
		'nb_distinct_strings' => 0,
		'strings_values'      => {},
		'strings_counts'      => {}
	};
	$RT_STRINGS = $STRING_CONTEXT->{'strings_values'};

	# characters that can be found before the "/" of a regexp ...
	my $prefixRegexOperators = qr/\=|\(|\[|\||\&|\!|:|>|,/;
	my $previousItem = "";
	my $regexp_possible = 0;

	while ($$TEXT =~ /
    \G
    (
        ($prefixRegexOperators)
        |\/\/
        |\/\*
        |\/
        |\=
        |\(
        |\[
        |"
        |'
        |`
        |\|
        |\&
        |\!
        |:
        |>
        |,
        |\@\w+
        |\@
        |[^"'\/`\=\(\[\|\&\!:>,\@]*
    )
    /xgc) {
		my $newLines = $blank->($1);

		my $pattern = $1;
		my $RegexBegin = $2; # pattern that can be followed by a regexp
		#print "ITEM = $1 --- $regexp_possible\n";
		if ($pattern eq '//') {
			$err = parseLineComment($1);
		}
		elsif ($pattern eq '/*') {
			$err = parseMultilineLineComment($1);
		}
		elsif ($pattern eq '"') {
			$err = parseDblString($1);
			$regexp_possible = 0;
		}
		elsif ($pattern eq "'") {
			$err = parseSplString($1);
			$regexp_possible = 0;
		}
		elsif ($pattern eq "`") {
			$err = parseTplString($1);
			$regexp_possible = 0;
		}
		 elsif (defined $RegexBegin) {
			$regexp_possible = 1;
			if ($RegexBegin eq "!") {
				my $CaptureLength = length $pattern;
				my $charBeforeBang = substr($$TEXT, pos($$TEXT) - $CaptureLength - 1, 1);
				if ($charBeforeBang =~ /\w/) {
					# do not parse a regexp if the ! sign is the "defined assertion" applying to a data.
					$regexp_possible = 0;
				}
			}
			$CODE .= $pattern;
			$COMMENT .= $newLines;
			$MIX .= $pattern;
		}
 		elsif ($pattern eq "/") {
			# capture next non blank character in $TEXT
			my $pos = pos($$TEXT);
			my ($nextNonBlank) = $$TEXT =~ /\G\s*(\S)/g;
			pos($$TEXT) = $pos;
			if (! defined $nextNonBlank) {
				$nextNonBlank = "";
			}
				
			if ($regexp_possible ||
				($previousItem =~ /[^\w\)\s]\s*$/ &&                 # "/" is not preceded by an identifier or a ")" (would mean it is the / or /= arithmetic operator)
					($previousItem =~ /(?:\breturn|\+|-|\*)\s*\z/ || # and "/" is preceded by these items ...
						$$TEXT =~ /(?=\G\s*[^\s\(\w\$])/gc) &&		 #   or "/" is not followed by "(", \w or $ ...
					!($previousItem =~ /<\s*$/m &&					 # and "/" is not between < and > (closing tag of concise syntax of react fragment : <> ... </>)
					  $nextNonBlank eq ">"))        			
			) {
				# err = 0 (no error)
				# err = 1 (code error)
				$err = parseRegExp($TEXT);
				$regexp_possible = 0;
			}
			else {
				$CODE .= $pattern;
				$COMMENT .= $newLines;
				$MIX .= $pattern;
			}
		}
		elsif ($pattern =~ /\@\w/) {
			$err = parseAnnotation($pattern);
		}
		else {
			$CODE .= $pattern;
			$COMMENT .= $newLines;
			$MIX .= $pattern;

			# the pattern was not able to introduce a regex => reset flag
			if ($pattern =~ /\S/) {
				$regexp_possible = 0;
			}
		}

		# return a code error 1 (0 = no error)
		if ($err gt 0) {
			return 1;
		}

		$previousItem = $pattern;
	}

	return 0;
}

# analyse du fichier
sub StripTypescript($$$$)
{
    my ($filename, $vue, $options, $couples) = @_;
    my $b_timing_strip = ((defined Timing->isSelectedTiming ('Strip'))? 1 : 0);

    configureLocalTraces('StripTypescript', $options); # traces_filter_line
    my $stripTypescriptTiming = new Timing ('StripTypescript', Timing->isSelectedTiming ('Strip'));
    $stripTypescriptTiming->markTimeAndPrint('--init--') if ($b_timing_strip); # timing_filter_line
    
    my $err = parseCode(\$vue->{'text'}, $options);
    
    $stripTypescriptTiming->markTimeAndPrint('separer_code_commentaire_chaine') if ($b_timing_strip); # timing_filter_line

    $vue->{'comment'} = $COMMENT;
    $vue->{'HString'} = $RT_STRINGS;
    $vue->{'code_with_prepro'} = "";
    $vue->{'MixBloc'} = $MIX;
    $vue->{'agglo'} = "";                       
    StripUtils::agglomerate_C_Comments(\$MIX, \$vue->{'agglo'});
    $vue->{'MixBloc_LinesIndex'} = StripUtils::createLinesIndex(\$MIX);
    $vue->{'agglo_LinesIndex'} = StripUtils::createLinesIndex(\$vue->{'agglo'});    
    $vue->{'code'} = $CODE;
    
    if ( defined $options->{'--dumpstrings'})
    {
      dumpVueStrings(  $RT_STRINGS , $STDERR );
    }
    
	# if (($filename ne AnaUtils::DUMMYFILENAME) and defined $options->{'--strip'})
	# {
		# Vues::dump_vues( $filename, $vue, $options);
	# }
	
    if ( $err gt 0) {
      my $message = 'Fatal error when separating code/comments/strings';
      Erreurs::LogInternalTraces ('erreur', undef, undef, 'STRIP // ABORT_CAUSE_SYNTAX_ERROR', $message);
      return Erreurs::FatalError( Erreurs::ABORT_CAUSE_SYNTAX_ERROR, $couples, $message);
    }

	CommonOutput::ManageVue::setVue($vue);

	$stripTypescriptTiming->dump('StripTypescript') if ($b_timing_strip); # timing_filter_line
    return 0;
}

1; # Le chargement du module est okay.

