package TSql::CountLoop;

use strict;
use warnings;

use Lib::NodeUtil;

use TSql::Node;
use TSql::TSqlNode;

use Erreurs;

use Ident;

my $DEBUG=1;

sub CountSelect($$$);

my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();
my $QueriesInsideLoop__mnemo = Ident::Alias_QueriesInsideLoop(); 

my $nb_ToManyNestedLoop = 0;
my $nb_QueriesInsideLoop = 0;

sub checkNestedLoop($) {
	my $loop = shift;

	# find nested loops ...
	my @subloop = GetNodesByKindList($loop, [LoopKind], 1);

	for my $sloop (@subloop) {
		$nb_ToManyNestedLoop++;
		Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "Sub loop at line ".GetLine($sloop));
	}
}

sub checkSelectOutsideLoop($);
sub checkSelectOutsideLoop($) {
	my $node = shift;
	
	if ($node->[0] eq SelectKind) {
		$nb_QueriesInsideLoop++;
		Erreurs::VIOLATION($QueriesInsideLoop__mnemo, "Query inside loop at line ".GetLine($node));
	}
	elsif ($node->[0] eq LoopKind) {
		# in case of a loop, work only with the condition of the loop (first child of the loop).
		$node = GetChildren($node)->[0];
	}
	
	for my $child (@{GetChildren($node)}) {
		checkSelectOutsideLoop($child);
	}
}

sub checkQueryInLoop($) {
	my $loop = shift;

	my $children = GetChildren($loop);
	
	for my $child (@$children) {
		
		# skip condition of the loop (it can contain queries, but do not take into account, because not "inside" the loop body)
		next if ($child->[0] eq ConditionKind);
		
		checkSelectOutsideLoop($child);
	}
}

sub CountLoop($$$) {
	my ($fichier, $vue, $compteurs) = @_ ;

	my $ret = 0;
	$nb_ToManyNestedLoop = 0;
	$nb_QueriesInsideLoop = 0;
 
	my $root =  $vue->{'structured_code'} ;

	if ( ! defined $root ) {
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $QueriesInsideLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );

		$ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
		return $ret;
	}

	my @loops = GetNodesByKind($root, LoopKind);
	
	for my $loop (@loops) {
		my $line = GetLine($loop);

		my $cond = GetChildren($loop)->[0];
		my $condStmt = ${GetStatement($cond)};

		checkNestedLoop($loop);
		checkQueryInLoop($loop);
	}

	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop );
	$ret |= Couples::dispatch($QueriesInsideLoop__mnemo, $nb_QueriesInsideLoop );

	return $ret;
}


1;



