package TSql::CountSQL;
# Module du comptage TDB: Nombre de procedures internes non referencees.

use strict;
use warnings;

use TSql::Node;
use TSql::TSqlNode;
use TSql::Identifier;
use Lib::NodeUtil;

use Erreurs;

use Ident;

use CountUtil;

my $DEBUG=1;

sub analyze_Transaction_SQL($;$$$);
sub countUnsafeSQL($$);
sub checkFinalReturn($);
sub CountRoutine($$$);
sub CheckComplexArtifact($$);

my $Nbr_InsertWithoutColumnsList__mnemo = Ident::Alias_InsertWithoutColumnsList();
my $Nbr_WithMultiExprPerLine_Where__mnemo = Ident::Alias_WithMultiExprPerLine_Where();
my $Nbr_BadQualifiedDatabaseObject__mnemo = Ident::Alias_BadQualifiedDatabaseObject();
my $Nbr_DBObjectUsingRef__mnemo = Ident::Alias_DBObjectUsingRef();
my $Nbr_NumericalOrderBy__mnemo = Ident::Alias_NumericalOrderBy();
my $Nbr_WithTooManyColumnsTables__mnemo = Ident::Alias_WithTooManyColumnsTables();
my $Nbr_OnToManyTablesQueries__mnemo = Ident::Alias_OnToManyTablesQueries();
my $Nbr_WithoutPrimaryKeyTables__mnemo = Ident::Alias_WithoutPrimaryKeyTables();
my $Nbr_UnCommentedColumns__mnemo = Ident::Alias_UnCommentedColumns();
my $Nbr_TotalColumns__mnemo = Ident::Alias_TotalColumns();
my $SelectAll__mnemo = Ident::Alias_SelectAll();
my $MissingWhereClause__mnemo = Ident::Alias_MissingWhereClause();
my $SelectDistinct__mnemo = Ident::Alias_SelectDistinct();
my $SQLComplexWhereClause__mnemo = Ident::Alias_SQLComplexWhereClause();
my $WhereClauseWithWildcards__mnemo = Ident::Alias_WhereClauseWithWildcards();
my $NegativeComparison__mnemo = Ident::Alias_NegativeComparison();
my $Sql_Cursor__mnemo = Ident::Alias_Sql_Cursor();

my $nb_InsertWithoutColumnsList = 0;
my $nb_WithMultiExprPerLine_Where = 0;
my $nb_BadQualifiedDatabaseObject = 0;
my $nb_DBObjectUsingRef = 0;
my $nb_NumericalOrderBy = 0;
my $nb_WithTooManyColumnsTables = 0;
my $nb_OnToManyTablesQueries = 0;
my $nb_WithoutPrimaryKeyTables = 0;
my $nb_UnCommentedColumns = 0;
my $nb_TotalColumns = 0;
my $nb_SelectAll = 0;
my $nb_MissingWhereClause = 0;
my $nb_SelectDistinct = 0;
my $nb_SQLComplexWhereClause = 0;
my $nb_WhereClauseWithWildcards = 0;
my $nb_NegativeComparison = 0;
my $nb_Sql_Cursor = 0;

my $THRESHOLD_MAX_COLUMN = 20;
my $THRESHOLD_MAX_TABLES = 4;

# Number of line before a "column" in "create table" in which a comment line should be found.
my $COMMENT_ZONE_BEFORE_COLUMN = 2;

# Pattern for the from list :<table> [[AS] <alias>], ... n
 my $FROM_LIST = '[\w\.\#]+\s*(?:(?:AS\s+)?(?:\w+\s*))?(?:,\s*[\w\.\#]+\s*(?:(?:AS\s+)?(?:\w+\s*))?)*';


sub ptr_checkUnCommentedColumns($) {
  my $agglo = shift;
#print "Here's the callback (ptr_checkUnCommentedColumns) !!!!\n";
  if (! CountUtil::IsCommentBeforeAndAt($agglo)) {
    $nb_UnCommentedColumns++;
#print "=======> FOUND a violement !!!\n";
  }
}

sub getTableNameAtPos($) {
	my $buf = shift;
	my $name;
	# table name database_name.schema_name.table_name | schema_name.table_name | table_name
	# example : #mytable
	#           [dbo].[ORGN]
	#           [dbo].my_schema.my_table
	if ($$buf =~ /\G\s*((?:[\[\]\w#]+)(?:\s*\.[\[\]\w#]+)*)/) {
		$name = $1;
		$name =~ s/\s*//g;
	}
	return $name;
}

sub countCreateTable($$$$) {
  my $r_buf = shift;
  my $r_artifact = shift;
  my $r_TabToCheck = shift;
  my $tableDB = shift;

  if ( $$r_buf =~ /\A\s*create\s+table\b/igc ) {

	my $name = getTableNameAtPos($r_buf);
	if (defined $name) {
		if (! exists $tableDB->{$name}) {
			$tableDB->{$name}={};
		}
		$tableDB->{$name}->{'create_table'}=1;
	}
	else {
		print STDERR "Unknow table name in ALTER TABLE statement :\n $$r_buf\n";
	}

    my $artifactLine=undef;

    if ( $$r_artifact =~ /-LINE-(\d+)-/ ) {
      $artifactLine = $1;
    }

    my ($r_columns, $r_rest) = CountUtil::splitAtPeer($r_buf, "(", ")", 1);
    my $nb_columns = 0;
    my $nb_primary = 0;

    # Suppress beginning of the artifact to keep only the columns between parenthesis,
    # and count the corresponding lines deleted.
    my $CutPattern='\A[^(]*\(';
    my $nb_ln = () = $$r_columns =~ /${CutPattern}/s ;
    my $currentLine = $artifactLine + $nb_ln;
    $$r_columns =~ s/${CutPattern}//s ;
    $$r_columns =~ s/,?\s*\).*//s ;

    my @Columns = split (',', $$r_columns);

    # ANALYZE EACH COLUMN ...
    for my $column (@Columns) {

      # Is a primary key defined ?
      if ( $column =~ /\bprimary\s+key\b/si) {
        $nb_primary++;
      }
#print "COLUMN = $column\n";
      # Is it a column (or a table constraint) ?
      if ( $column !~ /\A\s*constraint\b/si) {
        $nb_columns++;
        $nb_TotalColumns++;
        # Check if commented...
        if ( $nb_UnCommentedColumns != Erreurs::COMPTEUR_ERREUR_VALUE) {
          my ($BeginningPadding) = $column =~ /\A(\s*)\S/s ;
          my $currentColLine = $currentLine;
          $currentColLine += () = $BeginningPadding =~ /(\n)/g ;


          my $beginLine = $currentColLine-$COMMENT_ZONE_BEFORE_COLUMN;
          if ($beginLine < $artifactLine) {
            $beginLine = $artifactLine;
          }

          push @{$r_TabToCheck}, [$beginLine, $currentColLine,  \&ptr_checkUnCommentedColumns ];

        }
      }

      $currentLine += () = $column =~ /(\n)/g ;
    }

    #~ if ( $nb_primary == 0) {
      #~ $nb_WithoutPrimaryKeyTables ++;
    #~ }
    #~ else {
    if ( $nb_primary > 0) {
      $tableDB->{$name}->{'primary_key'} = 1;
	}

    if ($nb_columns > $THRESHOLD_MAX_COLUMN) {
      $nb_WithTooManyColumnsTables ++;
    }
  }

}

sub countAlterTable($$$) {
	my $r_buf = shift;
	my $r_artifact = shift;
	my $tableDB = shift;

	if ( $$r_buf =~ /\A\s*alter\s+table\b/igc ) {
		
		my $name = getTableNameAtPos($r_buf);
		if (defined $name) {
			if (! exists $tableDB->{$name}) {
				$tableDB->{$name}={};
			}
		}
		else {
			print STDERR "Unknow table name in ALTER TABLE statement :\n$$r_buf\n";
		}

		my $artifactLine=undef;

		if ( $$r_artifact =~ /-LINE-(\d+)-/ ) {
			$artifactLine = $1;
		}
		
		if (defined $name) {
			if ($$r_buf =~ /\bprimary\s+key\b/i) {
				$tableDB->{$name}->{'primary_key'} = 1;
			}
		}
	}
}

sub CountNumericalOrderBy($) {
  my $r_buf = shift;

  if ( $$r_buf =~ /\border\s+by\b(.*?)(?:\bwhere\b|\bgroup\s+by\b|\bhaving\b)?$/sig) {
#print "ITEM : $1\n";
    my @items = split ',', $1;

    for my $item (@items) {
      if ( $item =~ /\A\s*\d+\s*$/s ) {
#print "---> violement\n";
        return 1;
      }
    }
  }
  return 0;
}

sub is_CanonicalObjectIdent($) {
  my $r_item = shift;

  if ( $$r_item =~ /\A\s*#/s ) {
    # rule doesn't apply to temporary objects => return no violation
    return 1;
  }

  my $element = '[^\.]+';
  if ( $$r_item =~ /\[/ ) {
    $element = '\[[^\]]+\]'
  }

  if ( $$r_item =~ /\A\s*$element\s*\.\s*$element/s ) {
    return 1;
  }
  else {
    return 0;
  }
}

sub hasToManyTable($) {
  my $artifact = shift;

  if (TSql::ParseDetailed::get_NbObjects($artifact) > $THRESHOLD_MAX_TABLES) {
    return 1;
  }
  else {
    return 0;
  }
}

sub count_FROM_NonCanonicalObjectIdent($) {
  my $r_buf = shift;
  my $nb_nonCanonical = 0;

  my @tab_clause = split /\b(from|join)\b/i, $$r_buf ;
  my $i = 0;
  my $imax = scalar @tab_clause - 2;
  while ($i <= $imax) {
    if ( $tab_clause[$i] =~ /\b(?:from|join)\b/i) {
      $i++;
      if (defined $tab_clause[$i] && ($tab_clause[$i] =~ /\s+(${FROM_LIST})/isg)) {
	my @items=split ",", $1;
	for my $item (@items) {
          $nb_DBObjectUsingRef++;
          if (! is_CanonicalObjectIdent(\$item)) {
            $nb_nonCanonical++;
          }
        }
      }
    }
    $i++;
  }
  return $nb_nonCanonical;
}

sub count_TSQL_NonCanonicalObjectIdent($) {
  my $r_buf = shift ;
  my $nb_nonCanonical = 0;

  my $modifiable_buffer = $$r_buf ;

  # Replace grant instructions because they can contain pattern of searched intructions, but
  # we don't want to recognize it because they belong to the grant instruction.
  $modifiable_buffer =~ s/\bgrant\b.*?\bto\b/GRANT_REPLACED /sig;

  #while ( $$r_buf =~ /\b(?:into|(?:create|drop|truncate)\s+(?:table|view|trigger|procedure|proc|function))\b\s+((?:\.\.|[#\w]+)?\w+\b[^\.])/isg ) {
  while ( $modifiable_buffer =~ /\b(?:into|(?:create|drop|truncate)\s+(?:table|view|trigger|procedure|proc|function))\b\s+($ROUTINE_NAME_PATTERN)/isg ) {
    my $item = $1 ;
    $nb_DBObjectUsingRef++;
#print "OBJECT REF = $item\n";
    if (! is_CanonicalObjectIdent(\$item)) {
#print " ---> non canonical\n";
      $nb_nonCanonical++;
    }
  }
  return $nb_nonCanonical;
}


sub has_WHERE_MultiExpressionPerLine($) {
  my $r_WhereClause = shift;

  while ( $$r_WhereClause =~ /^(.*)$/mg ) {
    my $line = $1;
    # Search for
    #   1 - two or more operators on same line
    #   2 - one operator with both right and left expression on the same line.
    if ( ( scalar ( () = $line =~  /\b(and|or)\b/ig ) > 1 ) ||
	 ($line =~  /[^\n\t ][ \t]*\b(and|or)\b[ \t]*[^\n\t ]/ig ) ) {
      return 1;
    }
  }
  return 0;
}




#-----------------------  CountSQL ---------------------#
sub CountSQL($$$)
{
    my ($fichier, $vue, $compteurs) = @_;

    my $ret = 0;
    $nb_InsertWithoutColumnsList = 0;
    $nb_WithMultiExprPerLine_Where = 0;
    $nb_BadQualifiedDatabaseObject = 0;
    $nb_DBObjectUsingRef = 0;
    $nb_WithTooManyColumnsTables = 0;
    $nb_OnToManyTablesQueries = 0;
    $nb_WithoutPrimaryKeyTables = 0;
    $nb_UnCommentedColumns = 0;
    $nb_TotalColumns = 0;
    $nb_NumericalOrderBy = 0;
    $nb_SelectAll = 0;
    $nb_MissingWhereClause = 0;
    $nb_SelectDistinct = 0;
    $nb_SQLComplexWhereClause = 0;
    $nb_WhereClauseWithWildcards = 0;
    $nb_NegativeComparison = 0;
    $nb_Sql_Cursor = 0;

    if (!defined $vue->{'code_without_directive'}) {
        $ret |= Couples::counter_add($compteurs, $Nbr_BadQualifiedDatabaseObject__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        # Nbr_NonCanonicalObjectIdent can not be completely computed without this view. So if it is
        # no available, the counter will not be computed.
        $nb_BadQualifiedDatabaseObject = Erreurs::COMPTEUR_ERREUR_VALUE;
        $nb_DBObjectUsingRef = Erreurs::COMPTEUR_ERREUR_VALUE;
    }

    if (!defined $vue->{'agglo'}) {
        $ret |= Couples::counter_add($compteurs, $Nbr_UnCommentedColumns__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        # Counters can not be completely computed without this view. So if it is
        # no available, the counter will not be computed.
        $nb_UnCommentedColumns = Erreurs::COMPTEUR_ERREUR_VALUE;
    }

    my $NomVueCode = 'routines';
    my $ArtifactsView = $vue->{$NomVueCode};
    if (!defined $ArtifactsView) {
        $ret |= Couples::counter_add($compteurs, $Nbr_InsertWithoutColumnsList__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_WithMultiExprPerLine_Where__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_BadQualifiedDatabaseObject__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_DBObjectUsingRef__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_NumericalOrderBy__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_WithTooManyColumnsTables__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_WithoutPrimaryKeyTables__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_OnToManyTablesQueries__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_UnCommentedColumns__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Nbr_TotalColumns__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $MissingWhereClause__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $SelectDistinct__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $SQLComplexWhereClause__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $WhereClauseWithWildcards__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $NegativeComparison__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Sql_Cursor__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
        return $ret;
    }

    my @TabToCheck = ();
	my $tableDB = {};
	
    for my $artifact (keys %{$ArtifactsView}) {
        #**************** Artifact_SqlWithSelect *************

        # Artifact_SqlWithSelect is the category of SQL artifact that can contain a select.
        # The Insert instruction belongs to this category.
        if ($artifact =~ /Artifact_SQL_/) {
            if ($ArtifactsView->{$artifact} =~ /\binsert\s+(.*?)\b(?:select|values|exec|execute|default\s+values)\b/is) {
                my $tmp_stat = $1;
                # Removing of Parenthesis imbrication
                while ($tmp_stat =~ s/\([^\)]*\([^\(\)]*\)/\(/) {};

                # Removing of parasitic patterns ("top" and "with") !!
                $tmp_stat =~ s/top\s*\([^()]*\)\s*percent\b|with\s*\([^()]*\)//i;

                # check if the remaining expression ends with (...), that are necessarily columns list :
                if ($tmp_stat !~ /\)\s*$/) {
                    $nb_InsertWithoutColumnsList++;
                }
            }

            if ($ArtifactsView->{$artifact} =~ /\bwhere\s+(.*?)(?:\border\s+by\b|\bgroup\s+by\b|\bhaving\b)?$/is) {
                my $where_clause = $1;
                if (has_WHERE_MultiExpressionPerLine(\$where_clause)) {
                    $nb_WithMultiExprPerLine_Where++;
                }
            }

            if ($nb_BadQualifiedDatabaseObject != -1) {
                $nb_BadQualifiedDatabaseObject += count_FROM_NonCanonicalObjectIdent(\$ArtifactsView->{$artifact});
            }
			
            #countCreateTable(\$ArtifactsView->{$artifact}, \$artifact, \$vue->{'agglo'});
            countCreateTable(\$ArtifactsView->{$artifact}, \$artifact, \@TabToCheck, $tableDB);
            countAlterTable(\$ArtifactsView->{$artifact}, \$artifact, $tableDB);
        }
		
        #**************** Artifact_Select *************
        if ($artifact =~ /Artifact_select_/) {
            if ($ArtifactsView->{$artifact} =~ /\bwhere\s+(.*?)(?:\border\s+by\b|\bgroup\s+by\b|\bhaving\b)?$/is) {
                my $where_clause = $1;
                if (has_WHERE_MultiExpressionPerLine(\$where_clause)) {
                    $nb_WithMultiExprPerLine_Where++;
                }
            }

            $nb_NumericalOrderBy += CountNumericalOrderBy(\$ArtifactsView->{$artifact});

            $nb_OnToManyTablesQueries += hasToManyTable($artifact);

            if ($nb_BadQualifiedDatabaseObject != -1) {
                $nb_BadQualifiedDatabaseObject += count_FROM_NonCanonicalObjectIdent(\$ArtifactsView->{$artifact});
            }
        }
    }

	# check tables detections
    for my $table (keys %$tableDB) {
		if ( ($tableDB->{$table}->{'create_table'}) && (! $tableDB->{$table}->{'primary_key'}) ) {
		    $nb_WithoutPrimaryKeyTables++;
		    Erreurs::VIOLATION($Nbr_WithoutPrimaryKeyTables__mnemo, "Missing primary key for table $table\n");
		}
	}

    # Check comments in the pre-defined zones ...
    CountUtil::checkComment(\$vue->{'agglo'}, \@TabToCheck);

    if ($nb_BadQualifiedDatabaseObject != -1) {
        $nb_BadQualifiedDatabaseObject += count_TSQL_NonCanonicalObjectIdent(\$vue->{'code_without_directive'});
    }

    my $root = $vue->{'structured_code'};
    my $code = $vue->{'code'};
    my $text = $vue->{'text'};
    my $HStrings = $vue->{'HString'};

    if (!defined $root) {
        $ret |= Couples::counter_add($compteurs, $SelectAll__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $MissingWhereClause__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $SelectDistinct__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $SQLComplexWhereClause__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $WhereClauseWithWildcards__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $NegativeComparison__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        $ret |= Couples::counter_add($compteurs, $Sql_Cursor__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);
        return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
    }

    my @elements = Lib::NodeUtil::GetNodesByKindList($root, [ SelectKind, ProcedureKind, FunctionKind, CursorDeclarationKind ]);

    foreach my $element (@elements) {
        if (Lib::NodeUtil::GetKind($element) eq SelectKind) {
            my $children = Lib::NodeUtil::GetChildren($element);
            foreach my $child (@{$children}) {
                my $statement = Lib::NodeUtil::GetStatement($child);
                checkSQLStatement($statement, GetLine($element), $HStrings);
            }
        }
        elsif (Lib::NodeUtil::GetKind($element) eq CursorDeclarationKind) {
            # HL-2229
            # print "Query with cursor in a loop at line " . GetLine($element) . "\n";
            $nb_Sql_Cursor++;
            Erreurs::VIOLATION($Sql_Cursor__mnemo, "Query with cursor in a loop at line " . GetLine($element));
        }
        else {
            my $statement = Lib::NodeUtil::GetStatement($element);
            if ($statement =~ /\bselect\s+.*\bfrom\b/i) {
                checkSQLStatement($statement, GetLine($element), $HStrings);
            }
        }
    }

    # search in strings scope
    checkSQLInStrings($text, $HStrings);

    $ret |= Couples::counter_add($compteurs, $Nbr_InsertWithoutColumnsList__mnemo, $nb_InsertWithoutColumnsList);
    $ret |= Couples::counter_add($compteurs, $Nbr_WithMultiExprPerLine_Where__mnemo, $nb_WithMultiExprPerLine_Where);
    $ret |= Couples::counter_add($compteurs, $Nbr_BadQualifiedDatabaseObject__mnemo, $nb_BadQualifiedDatabaseObject);
    $ret |= Couples::counter_add($compteurs, $Nbr_DBObjectUsingRef__mnemo, $nb_DBObjectUsingRef);
    $ret |= Couples::counter_add($compteurs, $Nbr_NumericalOrderBy__mnemo, $nb_NumericalOrderBy);
    $ret |= Couples::counter_add($compteurs, $Nbr_WithTooManyColumnsTables__mnemo, $nb_WithTooManyColumnsTables);
    $ret |= Couples::counter_add($compteurs, $Nbr_WithoutPrimaryKeyTables__mnemo, $nb_WithoutPrimaryKeyTables);
    $ret |= Couples::counter_add($compteurs, $Nbr_OnToManyTablesQueries__mnemo, $nb_OnToManyTablesQueries);
    $ret |= Couples::counter_add($compteurs, $Nbr_UnCommentedColumns__mnemo, $nb_UnCommentedColumns);
    $ret |= Couples::counter_add($compteurs, $Nbr_TotalColumns__mnemo, $nb_TotalColumns);
    $ret |= Couples::counter_add($compteurs, $SelectAll__mnemo, $nb_SelectAll);
    $ret |= Couples::counter_add($compteurs, $MissingWhereClause__mnemo, $nb_MissingWhereClause);
    $ret |= Couples::counter_add($compteurs, $SelectDistinct__mnemo, $nb_SelectDistinct);
    $ret |= Couples::counter_add($compteurs, $SQLComplexWhereClause__mnemo, $nb_SQLComplexWhereClause);
    $ret |= Couples::counter_add($compteurs, $WhereClauseWithWildcards__mnemo, $nb_WhereClauseWithWildcards);
    $ret |= Couples::counter_add($compteurs, $NegativeComparison__mnemo, $nb_NegativeComparison);
    $ret |= Couples::counter_add($compteurs, $Sql_Cursor__mnemo, $nb_Sql_Cursor);

    return $ret;
}

sub checkSQLStatement($$$) {
    my $statement = shift;
    my $numline = shift;
    my $HStrings = shift;

    my $statement_to_check;
    if (ref($statement) eq 'SCALAR') {
        $statement_to_check = $$statement;
    }
    else {
        $statement_to_check = $statement;
    }
    if (defined $statement) {
        # HL-2216
        if ($statement_to_check =~ /\s*\*.*\bfrom\b/si) {
            # print "Avoid using Select * at line " . GetLine($element) . "\n";
            $nb_SelectAll++;
            Erreurs::VIOLATION($SelectAll__mnemo, "Avoid using Select * at line " . $numline);
        }
        # HL-2217
        if ($statement_to_check =~ /\bfrom\b/i && $statement_to_check !~ /\bwhere\b/i) {
            # print "Query with no WHERE clause at line " . $numline . "\n";
            $nb_MissingWhereClause++;
            Erreurs::VIOLATION($MissingWhereClause__mnemo, "Query with no WHERE clause at line " . $numline);
        }
        # HL-2218
        if ($statement_to_check =~ /\bwhere\s+.*(CHAINE_[0-9]+)/i) {
            my $stringID = $1;
            if ($HStrings->{$stringID} =~ /["'][%_\[]/) {
                # print "Query with wildcards in where clause at line " . $numline . "\n";
                $nb_WhereClauseWithWildcards++;
                Erreurs::VIOLATION($WhereClauseWithWildcards__mnemo, "Query with wildcards in where clause at line " . $numline);
            }
        }
        #HL-2219
        # syntax supported
        # WHERE name_of_func(xxx)
        # WHERE fieldA | fieldB
        # WHERE fieldA & fieldB
        if ($statement_to_check =~ /\bwhere\b\s*(?:.+[&|]+|\w+\()/i) {
            # print "Query with functions in where clause at line " . $numline . "\n";
            $nb_SQLComplexWhereClause++;
            Erreurs::VIOLATION($SQLComplexWhereClause__mnemo, "Query with functions in where clause at line " . $numline);
        }
        # HL-2220
        if ($statement_to_check =~ /\bdistinct\b/i) {
            # print "Query with SELECT DISTINCT at line " . $numline . "\n";
            $nb_SelectDistinct++;
            Erreurs::VIOLATION($SelectDistinct__mnemo, "Query with SELECT DISTINCT at line " . $numline);
        }
        # HL-2231
        if ($statement_to_check =~ /\<\>|\bnot\b/i) {
            # print "Query with negative comparison at line " . $numline . "\n";
            $nb_NegativeComparison++;
            Erreurs::VIOLATION($NegativeComparison__mnemo, "Query with negative comparison at line " . $numline);
        }
    }
}

sub checkSQLInStrings($$) {
    my $text = shift;
    my $HStrings = shift;

    if (defined $text && defined $HStrings) {
        foreach my $stringID (keys %{$HStrings}) {
            # SQL format
            if ($HStrings->{$stringID} =~ /\bselect\b.*\bfrom\b/i) {
                my $CHAINE_ID_pattern = $HStrings->{$stringID};
                $CHAINE_ID_pattern = quotemeta($CHAINE_ID_pattern);

                while ($text =~ /(.*?)$CHAINE_ID_pattern([^\n]+)/sg) {
                    my $endLineContent = $2;
                    my $numline = 1;
                    $numline += () = $1 =~ /\n/g;

                    # HL-2216
                    if ($HStrings->{$stringID} =~ /\bselect\s+\*.*\bfrom\b/i) {
                        # print "Avoid using Select * at line $numline\n";
                        $nb_SelectAll++;
                        Erreurs::VIOLATION($SelectAll__mnemo, "Avoid using Select * at line $numline");
                    }
                    # HL-2217
                    if ($HStrings->{$stringID} !~ /\bwhere\b/) {
                        if (defined $endLineContent && $endLineContent =~ /\bwhere\b/i) {
                            # do nothing
                        }
                        # endLineContent not defined or not contain where...
                        else {
                            # print "Query with no WHERE clause at line $numline\n";
                            $nb_MissingWhereClause++;
                            Erreurs::VIOLATION($MissingWhereClause__mnemo, "Query with no WHERE clause at line $numline");
                        }
                    }
                    # HL-2218
                    if ($HStrings->{$stringID} =~ /\bwhere\b/i) {
                        if ($HStrings->{$stringID} =~ /["'][%_\[]/) {
                            # print "Query with wildcards in where clause at line $numline\n";
                            $nb_WhereClauseWithWildcards++;
                            Erreurs::VIOLATION($WhereClauseWithWildcards__mnemo, "Query with wildcards in where clause at line $numline");
                        }
                    }
                    # HL-2219
                    # syntax supported
                    # WHERE name_of_func(xxx)
                    # WHERE fieldA | fieldB
                    # WHERE fieldA & fieldB
                    if ($HStrings->{$stringID} =~ /\bwhere\b\s*(?:.+[&|]+|\w+\()/i) {
                        # print "Query with functions in where clause at line $numline\n";
                        $nb_SQLComplexWhereClause++;
                        Erreurs::VIOLATION($SQLComplexWhereClause__mnemo, "Query with functions in where clause at line $numline");
                    }
                    # HL-2220
                    if ($HStrings->{$stringID} =~ /\bselect\s+distinct\b/i) {
                        # print "Query with SELECT DISTINCT at line $numline\n";
                        $nb_SelectDistinct++;
                        Erreurs::VIOLATION($MissingWhereClause__mnemo, "Query with SELECT DISTINCT at line $numline");
                    }
                    # HL-2229
                    if ($HStrings->{$stringID} =~ /\bcursor\b/i) {
                        # print "Query with cursor in a loop at line " . $numline . "\n";
                        $nb_Sql_Cursor++;
                        Erreurs::VIOLATION($Sql_Cursor__mnemo, "Query with cursor in a loop at line " . $numline);
                    }
                    # HL-2230
                    if ($HStrings->{$stringID} =~ /\bfrom\b.*\,/i) {
                        my $nbTables = () = $HStrings->{$stringID} =~ /(.*?\,)+?/g;
                        if ($nbTables > $THRESHOLD_MAX_TABLES - 1) {
                            # print "Query with too many tables in from clause at line $numline\n";
                            $nb_OnToManyTablesQueries++;
                            Erreurs::VIOLATION($Nbr_OnToManyTablesQueries__mnemo, "Query with too many tables in from clause at line $numline");
                        }
                    }
                    # HL-2231
                    if ($HStrings->{$stringID} =~ /\<\>|\bnot\b/i) {
                        # print "Query with negative comparison at line " . $numline . "\n";
                        $nb_NegativeComparison++;
                        Erreurs::VIOLATION($NegativeComparison__mnemo, "Query with negative comparison at line " . $numline);
                    }
                }
            }
        }
    }
}

1;



