
package TSql::Identifier;

use strict;
use warnings;

use Exporter  'import';

our @ISA = qw(Exporter);
our @EXPORT = qw( $ROUTINE_NAME_PATTERN );

my $IDENT_CHAR='[\w$#@]';
my $DELIMITED_IDENT = '\[[^\]]+\]';

our $ROUTINE_NAME_PATTERN = "(?:(?:$IDENT_CHAR+(?:\\s*\\.\\s*$IDENT_CHAR+)?)|(?:$DELIMITED_IDENT(?:\\s*\\.\\s*$DELIMITED_IDENT)?))";



# For information :
#
# http://msdn.microsoft.com/fr-fr/library/ms175874.aspx
#
#Il existe deux classes d'identificateurs :
#
#Identificateurs rguliers
#
#    Les identificateurs rguliers respectent les rgles relatives au format des identificateurs.Ils ne sont pas dlimits lorsqu'ils sont utiliss dans des instructions Transact-SQL.
#
#    SELECT *
#    FROM TableX
#    WHERE KeyCol = 124
#
#Identificateurs dlimits
#
#    Les identificateurs dlimits sont mis entre guillemets (") ou entre crochets ([ ]).Les identificateurs qui respectent les rgles relatives au format des identificateurs peuvent ne pas tre dlimits.Exemple :
#
#    SELECT *
#    FROM [TableX]         --Delimiter is optional.
#    WHERE [KeyCol] = 124  --Delimiter is optional.
#
#    Ceux qui ne respectent pas ces rgles ne peuvent tre utiliss dans une instruction Transact-SQL qu'en tant dlimits.Exemple :
#
#    SELECT *
#    FROM [My Table]      --Identifier contains a space and uses a reserved keyword.
#    WHERE [order] = 10   --Identifier is a reserved keyword.
#
#Qu'ils soient rguliers ou dlimits, les identificateurs doivent contenir de 1  128 caractres.Dans le cas des tables temporaires locales, l'identificateur peut contenir jusqu' 116 caractres.
#
#
#--------------------------------------
#Rgles pour identificateurs rguliers
#--------------------------------------
#
#Les rgles relatives au format des identificateurs rguliers dpendent du niveau de compatibilit de la base de donnes.Ce niveau peut tre dfini au moyen de ALTER DATABASE.Lorsque le niveau de compatibilit est 100, les rgles applicables sont les suivantes :
#
#    Le premier caractre doit tre l'un des suivants :
#
#        une des lettres dfinies par Unicode Standard 3.2.Elles incluent les caractres latins de a  z et de A  Z ainsi que des caractres alphabtiques d'autres langues.
#
#        Les symboles trait de soulignement (_), arobase (@) ou dise (#).
#
#        Certains symboles au dbut d'un identificateur ont une signification particulire dans SQL Server.Un identificateur rgulier qui commence par le signe arobase (@) dnote toujours une variable ou un paramtre local et ne peut pas tre utilis comme le nom d'un autre type d'objet.Un identificateur commenant par un symbole numro indique un objet temporaire (table ou procdure).Un identificateur commenant par le double signe # (##) indique un objet temporaire global.Bien que les symboles dise (#) et double dise (##) puissent tre utiliss pour commencer les noms d'autres types d'objets, nous ne recommandons pas cette pratique.
#
#        Le nom de certaines fonctions Transact-SQL commence par un double arobas (@@).Pour viter toute confusion avec ces fonctions, n'utilisez pas de noms commenant par @@.
#
#    Les caractres suivants peuvent inclure les lments suivants :
#
#        Des lettres dfinies dans Unicode Standard 3.2.
#
#        Des nombres dcimaux de Basic Latin ou d'autres scripts nationaux.
#
#        L'arobase, le symbole dollar ($), le symbole dise ou le trait de soulignement.
#
#    L'identificateur ne doit pas tre un mot rserv Transact-SQL.SQL Server conserve les majuscules et les minuscules des mots rservs.
#
#    Les espaces incorpors ou les caractres spciaux ne sont pas autoriss.
#
#    L'utilisation de caractres supplmentaires n'est pas autorise.
#
#Un identificateur qui ne respecte pas toutes ces rgles doit toujours tre dlimit par des crochets ou des guillemets doubles lors de son utilisation dans une instruction Transact-SQL.
#RemarqueRemarque
#
#Les noms de variables, de fonctions et de procdures stockes doivent toujours respecter les rgles portant sur les identificateurs Transact-SQL.


1;



