package VbDotNet::CountLoop;

use warnings;
use strict;

use Lib::NodeUtil;
use VbDotNet::VbDotNetNode;

use GreenIT::lib::KeepPortalData;
my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $ConcatInLoop__mnemo = Ident::Alias_ConcatInLoop();
my $InstantiationInLoop__mnemo = Ident::Alias_InstantiationInLoop();
my $FunctionCallInLoopTest__mnemo = Ident::Alias_FunctionCallInLoopTest();
my $OpenCloseFileInLoop__mnemo = Ident::Alias_OpenCloseFileInLoop();
my $LoopCondComparedToNonZero__mnemo = Ident::Alias_LoopCondComparedToNonZero();
my $ToManyNestedLoop__mnemo = Ident::Alias_ToManyNestedLoop();


my $nb_ConcatInLoop = 0;
my $nb_InstantiationInLoop = 0;
my $nb_FunctionCallInLoopTest = 0;
my $nb_OpenCloseFileInLoop = 0;
my $nb_LoopCondComparedToNonZero = 0;
my $nb_ToManyNestedLoop = 0;

sub checkLoopTerminationCondition($) {
	my $loopNode = shift;
	my $condNode = GetChildren($loopNode)->[0];

	# check presence of function call unless it's a FOR loop ...
  	if ( GetKind ($loopNode) ne ForKind ) {            
        my $cond = GetStatement($condNode);
        
        # detect a function call
        while ( $$cond =~ /(\w+)\s*\(/g ) {
            $nb_FunctionCallInLoopTest++;
			my $numline = $condNode->[5];
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_FunctionCallInLoopTest, $rulesDescription, "Avoid_FunctionCallInLoopTest");
			Erreurs::VIOLATION($FunctionCallInLoopTest__mnemo, "Function call in loop condition at line ".GetLine($condNode));
        }
        
       	#~ # check all expressions using operators <, >, <=, >=, =, <>
		my $bool_comparisonToZero;
		# no comparison to zero expected for compound conditions
		if ($$cond !~ /\b(?:and|or|xor)\b/) {
			if ( $$cond =~ /([^<>=\s]+)\s*(?:<>|<=|>=|=|<|>)\s*([^<>=\s]+)/ ) {
				$bool_comparisonToZero = 0;
				if (($1 =~ /(?:^|[^\.\w])0$/m) || ($2 =~ /^0(?:$|[^\.\w])/m)) {
					$bool_comparisonToZero = 1;
				}
			}
			if ((defined $bool_comparisonToZero) && ( $bool_comparisonToZero == 0)) {
				$nb_LoopCondComparedToNonZero++;
				my $numline = $condNode->[5];
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_LoopCondComparedToNonZero, $rulesDescription, "LoopCondComparedToNonZero");
				Erreurs::VIOLATION($LoopCondComparedToNonZero__mnemo, "Loop condition is not comparison to zero at line ".GetLine($condNode));
			}
		}
    }
}

sub checkNestedLoop($) {
	my $loop = shift;

	# find nested loops ...
	my @subloop = GetNodesByKindList($loop, [WhileKind, ForKind, DoKind], 1);
	for my $sloop (@subloop) {
		$nb_ToManyNestedLoop++;
		my $numline = $sloop->[5];
		CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ToManyNestedLoop, $rulesDescription, "Avoid_ToManyNestedLoop");
		Erreurs::VIOLATION($ToManyNestedLoop__mnemo, "sub loop at line ".GetLine($sloop));
	}
}

# check first level loops
sub checkLoop($) {
	my $loop = shift;

	my $codeBody = Lib::NodeUtil::GetKindData($loop)->{'codeBody'};
	my $bodyBeginLine = Lib::NodeUtil::GetKindData($loop)->{'bodyBeginLine'};

	# HL-2022 Avoid OPEN/CLOSE inside loops
	my $regexOpen = qr/(?:\bFile\.(?:Open|Create)\b|\bFileOpen\b|\bOpenTextFileWriter\b|\bNew\s+(?:System\.IO\.)?(?:FileStream|StreamReader|StreamWriter)\b)/i;
	my $regexClose = qr/\.Close\b|\bFileClose\b/i;
	# check presence of literal strings concaténations (variables strings cannot be detected !)
	my $lineOffset = 0;
	while ($$codeBody =~ /(\n+)|("ch\d+")|(\+=)|(\+)|($regexOpen)|($regexClose)|(\bnew\b)|(\&)/gc) {
		if (defined $1) {
			$lineOffset += () = $1 =~ /\n/g;
		}
		elsif (defined $7) {
			$nb_InstantiationInLoop++;
			my $numline = $bodyBeginLine + $lineOffset;
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_InstantiationInLoop, $rulesDescription, "Avoid_InstantiationInLoop");
			Erreurs::VIOLATION($InstantiationInLoop__mnemo, "Instantiation in loop at line " . ($bodyBeginLine + $lineOffset));
		}
		elsif (defined $2) {
			# "chxx"
			if ($$codeBody =~ /\G(\s*)\+/gc) {
				$lineOffset += () = $1 =~ /\n/g;
				$nb_ConcatInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
				Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
			}
		}
		elsif (defined $8) {
			$nb_ConcatInLoop++;
			my $numline = $bodyBeginLine + $lineOffset;
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
			Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation (op. &) inside loop at line " . ($bodyBeginLine + $lineOffset));
		}
		elsif (defined $5 || defined $6) {
			$lineOffset += () = $5 =~ /\n/g if (defined $5);
			$lineOffset += () = $6 =~ /\n/g if (defined $6);
			$nb_OpenCloseFileInLoop++;
			my $numline = $bodyBeginLine + $lineOffset;
			# print "Open/close file inside loop at line $numline\n";
			CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_OpenCloseFileInLoop, $rulesDescription, "Avoid_OpenCloseFileInLoop");
			Erreurs::VIOLATION($OpenCloseFileInLoop__mnemo, "Open/close file inside loop at line $numline");
		}
		else {
			# + first or +=
			if ($$codeBody =~ /\G(\s*)"ch\d+"(\s*)(\+)?/gc) {
				$lineOffset += () = $1 =~ /\n/g;
				$nb_ConcatInLoop++;
				my $numline = $bodyBeginLine + $lineOffset;
				CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
				Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));

				$lineOffset += () = $2 =~ /\n/g;
				if (defined $3) {
					$nb_ConcatInLoop++;
					my $numline = $bodyBeginLine + $lineOffset;
					CommonOutput::ElaborateLog::SimpleElaborateLog($numline, Ident::Alias_ConcatInLoop, $rulesDescription, "Avoid_ConcatInLoop");
					Erreurs::VIOLATION($ConcatInLoop__mnemo, "String concatenation inside loop at line " . ($bodyBeginLine + $lineOffset));
				}
			}
		}
	}

	checkLoopTerminationCondition($loop);
	checkNestedLoop($loop);
}

sub checkInnerLoop($) {
	my $loop = shift;
	
	my @loops = GetNodesByKindList($loop, [ WhileKind, ForKind, DoKind]);
	
	# check inner loops
	for my $loop (@loops) {
		checkLoopTerminationCondition($loop);
		checkNestedLoop($loop);
	}
}



sub CountLoop() {
	my ($fichier, $views, $compteurs, $options) = @_;
    
	$nb_ConcatInLoop = 0;
	$nb_InstantiationInLoop = 0;
	$nb_FunctionCallInLoopTest = 0;
	$nb_OpenCloseFileInLoop = 0;
	$nb_LoopCondComparedToNonZero = 0;
	$nb_ToManyNestedLoop = 0;

	my $ret = 0;
	
	my $root = $views->{'structured_code'};
	my $KindsLists = $views->{'KindsLists'};

	if ( ! defined $KindsLists ) {
		$ret |= Couples::counter_add($compteurs, $ConcatInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $InstantiationInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $FunctionCallInLoopTest__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $OpenCloseFileInLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $LoopCondComparedToNonZero__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );
		$ret |= Couples::counter_add($compteurs, $ToManyNestedLoop__mnemo, Erreurs::COMPTEUR_ERREUR_VALUE );

		return $ret |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}
	
	my @firstLevelLoops = GetNodesByKindList($root, [ WhileKind, ForKind, DoKind], 1);

	for my $loop (@firstLevelLoops) {
		checkLoop($loop);
		checkInnerLoop($loop);
	}

	$ret |= Couples::dispatch($ConcatInLoop__mnemo, $nb_ConcatInLoop);
	$ret |= Couples::dispatch($InstantiationInLoop__mnemo, $nb_InstantiationInLoop);
	$ret |= Couples::dispatch($FunctionCallInLoopTest__mnemo, $nb_FunctionCallInLoopTest);
	$ret |= Couples::dispatch($OpenCloseFileInLoop__mnemo, $nb_OpenCloseFileInLoop);
	$ret |= Couples::dispatch($LoopCondComparedToNonZero__mnemo, $nb_LoopCondComparedToNonZero);
	$ret |= Couples::dispatch($ToManyNestedLoop__mnemo, $nb_ToManyNestedLoop);
}

1;
