package VbDotNet::CountOpenedFiles;

use warnings;
use strict;

use Lib::NodeUtil;
use VbDotNet::VbDotNetNode;

use GreenIT::lib::KeepPortalData;
my $rulesDescription = GreenIT::lib::KeepPortalData::keepPortalData();

my $OpenedFileWithoutClose_mnemo = Ident::Alias_OpenedFileWithoutClose();

my $nb_OpenedFileWithoutClose = 0;

sub CountOpenedFiles() {
	my ($fichier, $views, $compteurs, $options) = @_;

	my $status = 0;
	$nb_OpenedFileWithoutClose = 0;

	my $ret = 0;

	my $root = $views->{'structured_code'};
	my $KindsLists = $views->{'KindsLists'};

	if (!defined $KindsLists) {
		$status |= Couples::counter_add($compteurs, $OpenedFileWithoutClose_mnemo, Erreurs::COMPTEUR_ERREUR_VALUE);

		return $status |= Erreurs::COMPTEUR_STATUS_VUE_ABSENTE;
	}

	my $kinds = [ $KindsLists->{&NamespaceKind}, $KindsLists->{&SubKind}, $KindsLists->{&ClassKind} ];

	my $regexOpen = qr/(?:\bFile\.(?:Open|Create)\b|\bFileOpen\b|\bOpenTextFileWriter\b|\bNew\s+(?:System\.IO\.)?(?:FileStream|StreamReader|StreamWriter)\b)/i;
	my $regexClose = qr/\.Close\b|\bFileClose\b/i;
	for my $kind (@{$kinds}) {
		foreach my $element (@{$kind}) {
			my $elementCodeBody = Lib::NodeUtil::GetKindData($element)->{'codeBody'};
			my $bodyBeginLine = Lib::NodeUtil::GetKindData($element)->{'bodyBeginLine'};
			if (defined $elementCodeBody) {

				# elementCodeBody is in lowercase
				# HL-2022 Avoid Programs not using explicitly OPEN and CLOSE files
				my $count_open = 0;
				my @numlineOpen;
				my $numlineBody = $bodyBeginLine;

				while ($$elementCodeBody =~ /(.*?)(?:($regexOpen)|($regexClose))/sg) {
					#if (defined $1) {
					#	$numlineBody++;
					#}
					#elsif (defined $2) {
					if (defined $2) {
						my $previousStmt = $1;
						$numlineBody += () = $1 =~ /\n/g;
						if ($previousStmt =~ /\bUsing\b.*/i) {
							# exception for using statement: do nothing
							# for instance
							# Using fs As IO.FileStream = IO.File.Create(myFilePath)
							# does not need a .Close() method
						}
						else {
							push(@numlineOpen, $numlineBody);
							$count_open++;
						}
					}
					elsif (defined $3) {
						$numlineBody += () = $1 =~ /\n/g;
						pop(@numlineOpen);
						$count_open--;
					}
				}

				if ($count_open > 0) {
					$nb_OpenedFileWithoutClose = $count_open;
					foreach my $numline (@numlineOpen) {
						# print "File opened without .Close() method at line $numline\n";
						CommonOutput::ElaborateLog::SimpleElaborateLog($numline, $OpenedFileWithoutClose_mnemo, $rulesDescription, "Avoid_OpenedFileWithoutClose");
						Erreurs::VIOLATION($OpenedFileWithoutClose_mnemo, "File opened without .Close() method at line $numline");
					}
				}
			}
		}
	}

	$ret |= Couples::dispatch($OpenedFileWithoutClose_mnemo, $nb_OpenedFileWithoutClose);
}

1;
