package VbDotNet::ParseVbDotNet;

use warnings;
use strict;

use Erreurs;

use Lib::Node; # qw( Leaf Node Append );
use Lib::NodeUtil ;
use Lib::NodeUtil qw( SetName SetStatement SetLine GetLine SetEndline GetEndline);
use Lib::ParseUtil;

use VbDotNet::VbDotNetNode;

########################## CONTEXT ###################

my @rootContent = (
	\&parseSub,
	\&parseFunction,
	\&parseFor,
	\&parseWhile,
	\&parseDo,
	\&parseClass,
	\&parseNamespace,
	\&parseModule,
);

my @classContent = ( 
);

my @methodContent = (
);

use constant CTX_LEAVE => -1;
use constant CTX_ROOT => 0;
use constant CTX_CLASS => 1;
use constant CTX_METHOD => 2;

my $DEFAULT_CONTEXT = CTX_ROOT;

my @context = (CTX_ROOT);

my %ContextContent = (
	&CTX_ROOT() => \@rootContent,
	&CTX_CLASS() => \@classContent,
	&CTX_METHOD() => \@methodContent,
	);

sub getCurrentContextContent() {
	return $ContextContent{ $context[-1] };
}

sub initContext() {
	@context = (CTX_ROOT);
	$DEFAULT_CONTEXT = CTX_ROOT;
}

sub popContext() {
	if (scalar @context > 1) {
		pop @context;
	}
	else {
		print "PARSE ERROR : context underflow !\n";
	}
}

sub sendContextEvent($;$) {
	my $event = shift;

	if ($event == CTX_LEAVE) {
		popContext();
	}
	else {
		push @context, $event;
	}
}

sub getContext() {
	if (scalar @context) {
		return $context[-1];
	}
	else {
		print "PARSE ERROR: empty context !";
		return $DEFAULT_CONTEXT;
	}
}

sub purgeNewLine() {
	while ((defined nextStatement()) && ((${nextStatement()} eq "\n"))) {
		getNextStatement();
	}
}

sub purgeInstructionSeparator() {
	while ((defined nextStatement()) && ((${nextStatement()} eq "\n") || (${nextStatement()} eq ":"))) {
		getNextStatement();
	}
}

sub isNextClosingParenthese() {
	if (${nextStatement()} eq ")") {
		return 1;
	}
	return 0;
}

##################################################################
#              UNKNOW
##################################################################
sub parseUnknow() {
	# be sure that there is no ore newline before next item
	purgeNewLine();
	
	my $line = getNextStatementLine();
	
	my $unkNode = Node(UnknowKind, createEmptyStringRef());
	my $statement = GetStatement($unkNode);
	
	my $stmt;
	
	while ( (defined ($stmt=nextStatement())) && ($$stmt ne "\n") && ($$stmt ne ":")) {
		
		# because "modifiers" are not parsed, sub, functions and classes cannot be recognized at line beginning.
		# so we try to recognize them inside unknow statement.
		
		if (($$stmt eq "sub") && ($$statement !~ /\bexit\s+$/m)){
			# we finally recognize a sub ...
			my $node = parseSub();
			return $node;
		}
		elsif ($$stmt eq "function") {
			# we finally recognize a function ...
			my $node = parseFunction();
			return $node;
		}
		elsif ($$stmt eq "class") {
			# we finally recognize a class ...
			my $node = parseClass();
			return $node;
		}
		else {
			$$statement .= ${getNextStatement()};
		}
	}
#print STDERR "UNKNOW ($line) : $$statement\n";
	purgeInstructionSeparator();
	
	SetLine($unkNode, $line);

	return $unkNode;
}

sub parseRawExpression(;$$$);
sub parseRawExpression(;$$$) {
	my $end = shift || "";
	my $keepClosing = shift || 0;
	my $shouldPurgeNewLine =  shift;
	
	if (! defined $shouldPurgeNewLine) {
		$shouldPurgeNewLine = 1; # default
	}
	
	my $statement = "";
	my $level = 0;
	
	my $stmt;
	while (($level > 0) || (defined ($stmt=nextStatement())) && ($$stmt ne "\n") && ($$stmt ne ":") && ($$stmt ne $end)) {
		if ($$stmt eq '(') {
			$statement .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
			$statement .= ${parseRawExpression(")", 0, 0)}; # do not purge new line, because it is the stop criteria !!!
		}
		elsif ($$stmt eq '{') {
			$statement .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
			$statement .= ${parseRawExpression("}", 0, 0)};
		}
		elsif ($$stmt eq '[') {
			$statement .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
			$statement .= ${parseRawExpression("]", 0, 0)};
		}
		else {
			$statement .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
		}
	}
	
	if (defined $stmt) {
		if ($$stmt eq $end) {
			if (! $keepClosing) {
				$statement .= ${Lib::ParseUtil::getSkippedBlanks()} . ${getNextStatement()};
			}
		}
		else {
			getNextStatement();
		}
	}
	if ($shouldPurgeNewLine) {
		purgeNewLine();
	}
	
	return \$statement;
}

sub parseBloc($$;$) {
	my $parent = shift;
	my $cb_end = shift;
	my $recordBody = shift;
	
	
	while ((defined nextStatement()) && (! $cb_end->())) {
		my $subNode = Lib::ParseUtil::tryParse_OrUnknow(getCurrentContextContent());
		Append($parent, $subNode);
	}
	
	purgeInstructionSeparator();
}

sub parseControl($$$;$) {
	my $kind = shift;
	my $cb_end = shift;
	my $parseCond = shift;
	my $recordBody = shift;
			
	my $ControlNode = Node($kind, createEmptyStringRef());
	my $line = getStatementLine();
	SetLine($ControlNode, $line);
		
	# get Condition
	if ($parseCond) {
		my $expression = parseRawExpression();
		my $CondNode = Node(ConditionKind, $expression);
		SetLine($CondNode, $line);
		Append($ControlNode, $CondNode);
	}
		
	my $ThenNode = Node(ThenKind, createEmptyStringRef());
	Append($ControlNode, $ThenNode);
	
	my $artiKey;
	if ($recordBody) {
		# USE non modal (nested control do not interrupt capture for upper levels) code capture for controls ...
		$artiKey = Lib::ParseUtil::newUnmodalArtifact($kind, $line);
		setVbDotNetKindData($ControlNode, 'artifact_key', $artiKey);
		setVbDotNetKindData($ControlNode, 'bodyBeginLine', getNextStatementLine());
	}

	parseBloc($ThenNode, $cb_end);

	if ($artiKey) {
		Lib::ParseUtil::endUnmodalArtifact($artiKey);
		setVbDotNetKindData($ControlNode, 'codeBody', Lib::ParseUtil::getArtifact($artiKey));
	}
	
	if (defined nextStatement()) {
		# trash end statement
		getNextStatement();
	}
	purgeInstructionSeparator();
	
	return $ControlNode;
}

#------------------- FOR -----------------------------------------------
sub isNextNext() {
	if (${nextStatement()} eq "next") {
		return 1;
	}
	return 0;
}

sub parseFor() {
	if (${nextStatement()} eq "for") {
		getNextStatement();
		return parseControl(ForKind, \&isNextNext, 1, 1); # with condition, recordbody
	}
	return undef;
}

#------------------- While -----------------------------------------------
sub isNextEndWhile() {
	if (${nextStatement()} =~ /\Aend\s+while\b/) {
		return 1;
	}
	return 0;
}

sub parseWhile() {
	if (${nextStatement()} eq "while") {
		getNextStatement();
		return parseControl(WhileKind, \&isNextEndWhile, 1, 1); # with condition, recordbody
	}
	return undef;
}

#------------------- Do -----------------------------------------------
sub isNextLoop() {
	if (${nextStatement()} =~ /\Aloop\b/) {
		return 1;
	}
	return 0;
}

sub parseDo() {
	if (${nextStatement()} eq "do") {
		getNextStatement();

		my $statement = "";
		my $conditionFirst = 0;

		my $next = nextStatement();
		if (( $$next eq "while") || ( $$next eq "until")) {
			$statement .= ${getNextStatement()} . "<statement>";
			$conditionFirst = 1;
		}

		my $doNode = parseControl(DoKind, \&isNextLoop, $conditionFirst, 1); # with recordBody
		
		if (! $conditionFirst) {
		
			my $next = nextStatement();
			if (( $$next eq "while") || ( $$next eq "until")) {
				$statement .= "<statement>" . ${getNextStatement()};
				$conditionFirst = 1;
			}
			
			my $line = getStatementLine();
			my $condition = parseRawExpression();
			my $CondNode = Node(ConditionKind, $condition);
			SetLine($CondNode, $line);
			my $children = GetChildren($doNode);
			$children->[1] = $children->[0];
			$children->[0] = $CondNode;
		}
		 
		return $doNode;
	}
	return undef;
}

#------------------- routine -----------------------------------------------
sub parseRoutine($$) {
	my $kind = shift;
	my $cb_end = shift;
	
	getNextStatement();
	my $subNode = Node($kind, createEmptyStringRef());
	my $line = getStatementLine();
	SetLine($subNode, $line);
	
	# get Name
	my $stmt;
	my $name = "";
	while ( (defined ($stmt=nextStatement())) && ($$stmt ne "(") ) {
		$name .= ${getNextStatement()};
	}
	$name =~ s/^\s*//m;
	$name =~ s/\s*$//m;
	SetName($subNode, $name);
		
	# get parameters
	if (defined nextStatement() && ${nextStatement()} eq "(") {
		getNextStatement();  # trash "("
		my $params = parseRawExpression(\&isNextClosingParenthese);
	}
	else {
		Lib::Log::ERROR("Missing parameter for $kind at line $line");
	}

	my $artiKey;
	# USE non modal (nested control do not interrupt capture for upper levels) code capture for controls ...
	$artiKey = Lib::ParseUtil::newUnmodalArtifact($kind, $line);
	setVbDotNetKindData($subNode, 'artifact_key', $artiKey);
	setVbDotNetKindData($subNode, 'bodyBeginLine', getNextStatementLine());

	parseBloc($subNode, $cb_end);

	if ($artiKey) {
		Lib::ParseUtil::endUnmodalArtifact($artiKey);
		setVbDotNetKindData($subNode, 'codeBody', Lib::ParseUtil::getArtifact($artiKey));
	}

	if (defined nextStatement()) {
		# trash end statement
		getNextStatement();
	}
	purgeInstructionSeparator();
	
	return $subNode;
}

sub isNextEndSub() {
	if (${nextStatement()} =~ /\Aend\s+sub\b/) {
		return 1;
	}
	return 0;
}

sub parseSub() {
	if (${nextStatement()} eq "sub") {
		return parseRoutine(SubKind, \&isNextEndSub);
	}
	return undef;
}

sub isNextEndFunction() {
	if (${nextStatement()} =~ /\Aend\s+function\b/) {
		return 1;
	}
	return 0;
}

sub parseFunction() {
	if (${nextStatement()} eq "function") {
		return parseRoutine(FunctionKind, \&isNextEndFunction);
	}
	return undef;
}

#------------------- NAMED BLOC ----------------------------------------

sub parseNamedBloc($$) {
	my $kind = shift;
	my $cb_end = shift;

	getNextStatement();
	my $subNode = Node($kind, createEmptyStringRef());
	my $line = getStatementLine();
	SetLine($subNode, $line);
	
	my $expr = parseRawExpression();
	
	# get Name
	my ($name) = $$expr =~ /\s*(\w+)/;
	SetName($subNode, $name);

	my $artiKey;
	# USE non modal (nested control do not interrupt capture for upper levels) code capture for controls ...
	$artiKey = Lib::ParseUtil::newUnmodalArtifact($kind, $line);
	setVbDotNetKindData($subNode, 'artifact_key', $artiKey);
	setVbDotNetKindData($subNode, 'bodyBeginLine', getNextStatementLine());

	parseBloc($subNode, $cb_end);

	if ($artiKey) {
		Lib::ParseUtil::endUnmodalArtifact($artiKey);
		setVbDotNetKindData($subNode, 'codeBody', Lib::ParseUtil::getArtifact($artiKey));
	}

	if (defined nextStatement()) {
		# trash end statement
		getNextStatement();
	}
	purgeInstructionSeparator();
	
	return $subNode
}

sub isNextEndClass() {
	if (${nextStatement()} =~ /\Aend\s+class\b/) {
		return 1;
	}
	return 0;
}

sub parseClass() {
	if (${nextStatement()} eq "class") {
		return parseNamedBloc(ClassKind, \&isNextEndClass);
	}
	return undef;
}

sub isNextEndNamespace() {
	if (${nextStatement()} =~ /\Aend\s+namespace\b/) {
		return 1;
	}
	return 0;
}

sub parseNamespace() {
	if (${nextStatement()} eq "namespace") {
		return parseNamedBloc(NamespaceKind, \&isNextEndNamespace);
	}
	return undef;
}

sub isNextEndModule() {
	if (${nextStatement()} =~ /\Aend\s+module\b/) {
		return 1;
	}
	return 0;
}

sub parseModule() {
	if (${nextStatement()} eq "module") {
		return parseNamedBloc(ModuleKind, \&isNextEndModule);
	}
	return undef;
}
##################################################################
#              ROOT
##################################################################
sub parseRoot() {
	my $root = Node(RootKind, createEmptyStringRef());

	SetName($root, 'root');

	while ( defined nextStatement() ) {
		my $subNode = Lib::ParseUtil::tryParse_OrUnknow(\@rootContent);
		
		if (defined $subNode) {
			Append($root, $subNode);
		}
	}

	return $root;
}

sub splitVbDotNet($) {
   my $r_view = shift;

   my @statements = split /(\n|\(|\{|\[|\)|\}|\]|:=|:|\b(?:sub|function|class|namespace|module|for|next|while|until|end\s+(?:while|sub|function|class|namespace|module)|do|loop)\b)/sm, $$r_view;

   if (scalar @statements > 0) {
    # if the last statement is a Null statement, it should be removed
    # because it is not significant.
    if ($statements[-1] !~ /\S/ ) {
      pop @statements ;
    }
  }
   return \@statements;
}

sub ParseVbDotNet($) {
	my $r_view = shift;
  
	my $r_statements = splitVbDotNet($r_view);

	Lib::ParseUtil::setBlankRegex('[ \t]');

	Lib::ParseUtil::InitParser($r_statements);

	initContext();

	# Mode LAST (no inclusion) for artifact consolidation.
	Lib::ParseUtil::setArtifactMode(1);
  
	my $root = parseRoot();
	my $Artifacts = Lib::ParseUtil::getArtifacts();

	return ($root, $Artifacts);
}

###################################################################################
#              MAIN
###################################################################################

sub preComputeListOfKinds($$$) {
	my $node = shift;
	my $views = shift;
	my $kinds = shift;
  
	my %H_KindsLists = ();
  
	for my $kind (@$kinds) {
		my @NodesList = GetNodesByKind($node, $kind );
		$H_KindsLists{$kind}=\@NodesList;
	}

	$views->{'KindsLists'} = \%H_KindsLists;
}

sub Parse($$$$)
{
	my ($fichier, $vue, $couples, $options) = @_;
	
	my $status = 0;

	Lib::ParseUtil::registerParseUnknow(\&parseUnknow);

	my ($root, $Artifacts) = ParseVbDotNet(\$vue->{'code_lc'});

#      print "Buffered routines = ".join(', ', keys %H_ROUTINE_BUFFER)."\n";
#      print $H_ROUTINE_BUFFER{'IsValidUser'}."\n------\n";


	if (defined $options->{'--print-tree'}) {
		Lib::Node::Dump($root, *STDERR, "ARCHI");
	}
	if (defined $options->{'--print-test-tree'}) {
		print STDERR ${Lib::Node::dumpTree($root, "ARCHI")} ;
	}

	$vue->{'structured_code'} = $root;
	$vue->{'artifact'} = $Artifacts;

	# pre-compute some list of kinds :
	preComputeListOfKinds($root, $vue, [ForKind, WhileKind, DoKind, ClassKind, SubKind, NamespaceKind]);

	#TSql::ParseDetailed::ParseDetailed($vue);
	if (defined $options->{'--print-artifact'}) {
		for my $key ( keys %{$vue->{'artifact'}} ) {
			print "-------- $key -----------------------------------\n";
			print  $vue->{'artifact'}->{$key}."\n";
		}
	}

	return $status;
}

1;

