package framework::Conan;

use strict;
use warnings;

sub cb_conan($$;$) {

    my $path = shift;
    my $conanDB = shift;
    my $H_DatabaseName = shift;

    my @projects = ();
    my $proj;

    if ($path =~ /conan\.lock$/m) {
        $proj = parse_conan_lock($path, $conanDB);
    }
    elsif ($path =~ /conanfile\.txt$/m) {
        $proj = parse_conan_txt($path, $conanDB);
    }
    elsif ($path =~ /conanfile\.py$/m) {
        $proj = parse_conan_py($path, $conanDB);
    }

    push @projects, $proj;

    my @itemDetections = ();

    # For each project ...
    for my $proj (@projects) {
        # ... try to recognize each dependency and create a item detection...
        for my $dep (@{$proj->{'dependencies'}}) {
            my $env = 'Conan';

            my $item = framework::detections::getEnvItem(
                $dep->[0], # name
                $dep->[1], # version
                $conanDB,
                $env,
                $proj->{'filename'},
                $H_DatabaseName);

            if (defined $item) {
                push @itemDetections, $item;
            }
        }
    }

    return \@itemDetections;
}

sub parse_conan_lock($$) {
    my $conanFile = shift;
    my $conanDB = shift;

    print "file= $conanFile\n";

    my %proj = ('dependencies' => [], 'filename' => $conanFile, 'projectReference' => []);

    my $buffText;
    my $status = open(FILE, "<", $conanFile);
    if ($status) {
        local $/ = undef;
        $buffText = <FILE>;
        close FILE;
    }

    #"ref": "binutils/2.37#7e1081ddda1762c3f97d47399c4b1691",
    while ($buffText =~ /\"ref\"\s*\:\s*\"([^\/]+)\/([^#]+)/g) {
        my $name = $1;
        my $version = $2;
        # print "package=<$name> version=<$version>\n";
        push @{$proj{'dependencies'}}, [ $name, $version ];
    }

    return \%proj;
}

sub parse_conan_txt($$) {
    my $conanFile = shift;
    my $conanDB = shift;

    print "file= $conanFile\n";

    my %proj = ('dependencies' => [], 'filename' => $conanFile, 'projectReference' => []);

    my $buffText;
    my $status = open(FILE, "<", $conanFile);
    if ($status) {
        local $/ = undef;
        $buffText = <FILE>;
        close FILE;
    }

    if ($buffText =~ /\[requires\]/g) {
        # for each lines
        while ($buffText =~ /(.+)$/mg) {
            my $line = $1;
            if ($line =~ /\s*([^\/]+)\/([^\/\n]+)|(^\[)/m) {
                # end of list components
                last if (defined $3);

                if (defined $1 && defined $2) {
                    my $name = $1;
                    my $version = $2;

                    $version = format_package_version($version);
                    # print "package=<$name> version=<$version>\n";
                    push @{$proj{'dependencies'}}, [ $name, $version ];
                }
            }
        }
    }

    return \%proj;
}

sub parse_conan_py($$) {
    my $conanFile = shift;
    my $conanDB = shift;

    print "file= $conanFile\n";

    my %proj = ('dependencies' => [], 'filename' => $conanFile, 'projectReference' => []);

    my $buffText;
    my $status = open(FILE, "<", $conanFile);
    if ($status) {
        local $/ = undef;
        $buffText = <FILE>;
        close FILE;
    }

    while ($buffText =~ /\brequires\s*([=(])\s*(\[)*/g) {
        # my $op1 = $1;
        # my $op2 = $2;
        my $component_list;

        # for dependencies the 2 parameters: override / private are supported
        # tuple such as: requires = [("component1/version"), ("component2/version"), ("component3/version")]
        if (defined $2 && $2 eq "[") {
            if ($buffText =~ /([^\]]+)/g) {
                $component_list = $1;

                while ($component_list =~ /\"([^"]+)\"(?:\s*\,\s*\"(?:override|private)\")*/g) {
                    if (defined $1) {
                        my $component = $1;
                        if ($component =~ /\s*([^\/]+)\/([^\/\n]+)/) {
                            my $name = $1;
                            my $version = $2;

                            $version = format_package_version($version);
                            # print "package=<$name> version=<$version>\n";
                            push @{$proj{'dependencies'}}, [ $name, $version ];
                        }
                    }
                }
            }
        }
        # requires = "component1/version", "component2/version"
        # or
        # requires = "component1/version",\
        # "component2/version"
        # or
        # requires("component1/version", ("component2/version"))
        elsif (defined $1) {
            while ($buffText =~ /\"([^"]+)\"(?:\s*\,\s*\"(?:override|private)\")*\s*([,\\])*/g) {
                # my $comp1 = $1;
                # my $comp2 = $2;

                if (defined $1) {
                    my $component = $1;
                    if ($component =~ /\s*([^\/]+)\/([^\/\n]+)/) {
                        my $name = $1;
                        my $version = $2;

                        $version = format_package_version($version);
                        # print "package=<$name> version=<$version>\n";
                        push @{$proj{'dependencies'}}, [ $name, $version ];
                    }
                }

                # last element of list
                last if (!defined $2);
            }
        }
    }

    return \%proj;
}

sub format_package_version($) {
    my $version = shift;

    # delete brackets used for delimiting range version
    $version =~ s/\[//;
    $version =~ s/\]//;

    # delete author version if exists
    $version =~ s/\@[^\s]+//g;

    return $version;
}

1;